@@ -3,9 +3,9 @@
 void f(void) { mpn_gcdext(NULL,NULL, NULL, NULL, 0, NULL, 0); }
 int main()
 {
-  if (sizeof(mp_limb_t) == sizeof(long))
-    printf("%s", gmp_version);
+  if (sizeof(mp_limb_t) == sizeof(long *))
+    printf("%s\n", gmp_version);
   else
-    printf("unsupported");
+    printf("unsupported\n");
   return 0;
 }
