@@ -1,9 +1,10 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-main() {
+int main() {
   struct rlimit rip;
   getrlimit(RLIMIT_STACK, &rip);
   setrlimit(RLIMIT_STACK, &rip);
+  return 0;
 }
 
