@@ -288,11 +288,11 @@ MultiLift(GEN f, GEN a, GEN T, GEN p, long e0, long flag)
         else
           Td = FpX_red(T, peold);
         Tnew = FpX_red(T, penew);
-        ZpXQ_RecTreeLift(link, v, w, Td, Tnew, pd, peold, penew, f, lgpol(v),
+        ZpXQ_RecTreeLift(link, v, w, Td, Tnew, pd, peold, penew, f, lg(v)-2,
                          (flag == 0 && mask == 1));
       }
       else
-        ZpX_RecTreeLift(link, v, w, pd, peold, penew, f, lgpol(v),
+        ZpX_RecTreeLift(link, v, w, pd, peold, penew, f, lg(v)-2,
                         (flag == 0 && mask == 1));
       if (DEBUGLEVEL > 3) timer_printf(&Ti, "lifting to prec %ld", enew);
     }
@@ -316,14 +316,31 @@ MultiLift(GEN f, GEN a, GEN T, GEN p, long e0, long flag)
 /* Q list of (coprime, monic) factors of pol mod (T,p). Lift mod p^e = pe.
  * T may be NULL */
 GEN
-ZpX_liftfact(GEN pol, GEN Q, GEN T, GEN p, long e, GEN pe)
+ZpX_liftfact(GEN pol, GEN Q, GEN pe, GEN p, long e)
 {
   pari_sp av = avma;
+  pol = FpX_normalize(pol, pe);
+  if (lg(Q) == 2) return mkvec(pol);
+  return gerepilecopy(av, MultiLift(pol, Q, NULL, p, e, 0));
+}
+
+GEN
+ZpXQX_liftfact(GEN pol, GEN Q, GEN T, GEN pe, GEN p, long e)
+{
+  pari_sp av = avma;
+  pol = FpXQX_normalize(pol, T, pe);
   if (lg(Q) == 2) return mkvec(pol);
-  pol = FqX_normalize(pol, T, pe);
   return gerepilecopy(av, MultiLift(pol, Q, T, p, e, 0));
 }
 
+GEN
+ZqX_liftfact(GEN f, GEN a, GEN T, GEN pe, GEN p, long e)
+{ return T ? ZpXQX_liftfact(f, a, T, pe, p, e): ZpX_liftfact(f, a, pe, p, e); }
+GEN
+ZqX_liftroot(GEN f, GEN a, GEN T, GEN p, long e)
+{ return T ? ZpXQX_liftroot(f, a,T , p, e): ZpX_liftroot(f, a, p, e); }
+
+
 /* U = NULL treated as 1 */
 static void
 BezoutPropagate(GEN link, GEN v, GEN w, GEN pe, GEN U, GEN f, long j)
@@ -361,7 +378,7 @@ bezout_lift_fact(GEN pol, GEN Q, GEN p, long e)
   link = gel(E,2);
   v    = gel(E,3);
   w    = gel(E,4);
-  BezoutPropagate(link, v, w, pe, NULL, pol, lgpol(v));
+  BezoutPropagate(link, v, w, pe, NULL, pol, lg(v)-2);
   E = cgetg(k+1, t_VEC);
   for (i = 1; i <= 2*k-2; i++)
   {
@@ -401,12 +418,74 @@ polhensellift(GEN pol, GEN L, GEN p, long N)
       gel(L,i) = scalar_ZX_shallow(gel(L,i), varn(pol));
     RgX_check_ZXX(gel(L,i), "polhensellift");
   }
-  return gerepilecopy(av, ZpX_liftfact(pol, L, T, p, N, powiu(p,N)));
+  return gerepilecopy(av, ZqX_liftfact(pol, L, T, powiu(p,N), p, N));
+}
+
+static GEN
+FqV_roots_from_deg1(GEN x, GEN T, GEN p)
+{
+  long i,l = lg(x);
+  GEN r = cgetg(l,t_VEC);
+  for (i=1; i<l; i++) { GEN P = gel(x,i); gel(r,i) = Fq_neg(gel(P,2), T, p); }
+  return r;
+}
+
+static GEN
+ZpX_liftroots_full(GEN f, GEN S, GEN q, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN y = ZpX_liftfact(f, deg1_from_roots(S, varn(f)), q, p, e);
+  return gerepileupto(av, FqV_roots_from_deg1(y, NULL, q));
+}
+
+GEN
+ZpX_roots(GEN F, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN q = powiu(p, e);
+  GEN f = FpX_normalize(F, p);
+  GEN g = FpX_normalize(FpX_split_part(f, p), p);
+  GEN S;
+  if (degpol(g) < degpol(f))
+  {
+    GEN h = FpX_div(f, g, p);
+    F = gel(ZpX_liftfact(F, mkvec2(g, h), q, p, e), 1);
+  }
+  S = FpX_roots(g, p);
+  return gerepileupto(av, ZpX_liftroots_full(F, S, q, p, e));
+}
+
+static GEN
+ZpXQX_liftroots_full(GEN f, GEN S, GEN T, GEN q, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN y = ZpXQX_liftfact(f, deg1_from_roots(S, varn(f)), T, q, p, e);
+  return gerepileupto(av, FqV_roots_from_deg1(y, T, q));
+}
+
+GEN
+ZpXQX_roots(GEN F, GEN T, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN q = powiu(p, e);
+  GEN f = FpXQX_normalize(F, T, p);
+  GEN g = FpXQX_normalize(FpXQX_split_part(f, T, p), T, p);
+  GEN S;
+  if (degpol(g) < degpol(f))
+  {
+    GEN h = FpXQX_div(f, g, T, p);
+    F = gel(ZpXQX_liftfact(F, mkvec2(g, h), T, q, p, e), 1);
+  }
+  S = FpXQX_roots(g, T, p);
+  return gerepileupto(av, ZpXQX_liftroots_full(F, S, T, q, p, e));
+}
+
+GEN
+ZqX_roots(GEN F, GEN T, GEN p, long e)
+{
+  return T ? ZpXQX_roots(F, T, p, e): ZpX_roots(F, p, e);
 }
 
-/*************************************************************************/
-/*                             rootpadicfast                             */
-/*************************************************************************/
 /* SPEC:
  * p is a t_INT > 1, e >= 1
  * f is a ZX with leading term prime to p.
@@ -437,22 +516,33 @@ ZpX_liftroot(GEN f, GEN a, GEN p, long e)
   }
 }
 
+GEN
+ZpX_liftroots(GEN f, GEN S, GEN p, long e)
+{
+  long i, n = lg(S)-1, d = degpol(f);
+  GEN r;
+  if (n == d) return ZpX_liftroots_full(f, S, powiu(p, e), p, e);
+  r = cgetg(n+1, typ(S));
+  for (i=1; i <= n; i++)
+    gel(r,i) = ZpX_liftroot(f, gel(S,i), p, e);
+  return r;
+}
+
 GEN
 ZpXQX_liftroot_vald(GEN f, GEN a, long v, GEN T, GEN p, long e)
 {
-  pari_sp av = avma, av2, lim;
-  GEN pv = p, q, qv, W, df, Tq, fr, dfr;
+  pari_sp av = avma, av2;
+  GEN pv = p, q, W, df, Tq, fr, dfr;
   ulong mask;
   a = Fq_red(a, T, p);
   if (e <= v+1) return a;
   df = RgX_deriv(f);
-  if (v) { pv = powiu(p,v); qv = mulii(pv,p); df = ZXX_Z_divexact(df, pv); }
-  else qv = p;
+  if (v) { pv = powiu(p,v); df = ZXX_Z_divexact(df, pv); }
   mask = quadratic_prec_mask(e-v);
-  Tq = FpXT_red(T, qv); dfr = FpXQX_red(df, Tq, p);
+  Tq = FpXT_red(T, p); dfr = FpXQX_red(df, Tq, p);
   W = Fq_inv(FqX_eval(dfr, a, Tq, p), Tq, p); /* 1/f'(a) mod (T,p) */
   q = p;
-  av2 = avma; lim = stack_lim(av2, 2);
+  av2 = avma;
   for (;;)
   {
     GEN u, fa, qv, q2v, q2, Tq2;
@@ -470,7 +560,7 @@ ZpXQX_liftroot_vald(GEN f, GEN a, long v, GEN T, GEN p, long e)
     dfr = FpXQX_red(df, Tq, q);
     u = ZX_Z_divexact(FpX_Fp_sub(Fq_mul(W,FqX_eval(dfr,a,Tq,q),Tq,q),gen_1,q),q2);
     W = Fq_sub(W,ZX_Z_mul(Fq_mul(u,W,Tq2,q2),q2),Tq,q);
-    if (low_stack(lim, stack_lim(av2,2)))
+    if (gc_needed(av2,2))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"ZpXQX_liftroot, e = %ld", e);
       gerepileall(av2, 3, &a, &W, &q);
@@ -481,82 +571,18 @@ ZpXQX_liftroot_vald(GEN f, GEN a, long v, GEN T, GEN p, long e)
 GEN
 ZpXQX_liftroot(GEN f, GEN a, GEN T, GEN p, long e) { return ZpXQX_liftroot_vald(f,a,0,T,p,e); }
 
-/* Apply ZpX_liftroot to all roots in S and trace trick.
- * Elements of S must be distinct simple roots mod p for all p|q. */
-GEN
-ZpX_liftroots(GEN f, GEN S, GEN q, long e)
-{
-  long i, d, l = lg(S), n = l-1;
-  GEN y = cgetg(l, typ(S));
-  if (!n) return y;
-  for (i=1; i<n; i++)
-    gel(y,i) = ZpX_liftroot(f, gel(S,i), q, e);
-  d = degpol(f);
-  if (n != d) /* not totally split*/
-    gel(y,n) = ZpX_liftroot(f, gel(S,n), q, e);
-  else
-  { /* totally split: use trace trick */
-    pari_sp av = avma;
-    GEN z = gel(f, d+1);/* -trace(roots) */
-    for(i=1; i<n;i++) z = addii(z, gel(y,i));
-    z = modii(negi(z), powiu(q,e));
-    gel(y,n) = gerepileuptoint(av,z);
-  }
-  return y;
-}
-
-/* Same as ZpX_liftroot for the polynomial X^2-b */
-GEN
-Zp_sqrtlift(GEN b, GEN a, GEN p, long e)
-{
-  pari_sp ltop=avma;
-  GEN q, w;
-  ulong mask;
-
-  if (e == 1) return icopy(a);
-  mask = quadratic_prec_mask(e);
-  w = Fp_inv(modii(shifti(a,1), p), p);
-  q = p;
-  for(;;)
-  {
-    q = sqri(q);
-    if (mask & 1) q = diviiexact(q, p);
-    mask >>= 1;
-    if (lgefint(q) == 3)
-    {
-      ulong Q = (ulong)q[2];
-      ulong A = umodiu(a, Q);
-      ulong B = umodiu(b, Q);
-      ulong W = umodiu(w, Q);
-      A = Fl_sub(A, Fl_mul(W, Fl_sub(Fl_sqr(A,Q), B, Q), Q), Q);
-      a = utoi(A);
-      if (mask == 1) break;
-      W = Fl_sub(Fl_add(W,W,Q), Fl_mul(Fl_sqr(W,Q), Fl_add(A,A,Q),Q), Q);
-      w = utoi(W);
-    }
-    else
-    {
-      a = modii(subii(a, mulii(w, subii(Fp_sqr(a,q),b))), q);
-      if (mask == 1) break;
-      w = subii(shifti(w,1), Fp_mul(Fp_sqr(w,q), shifti(a,1),q));
-    }
-  }
-  return gerepileuptoint(ltop,a);
-}
-/* Same as ZpX_liftroot for the polynomial X^n-b
- * TODO: generalize to sparse polynomials. */
+/* Same as ZpX_liftroot for the polynomial X^n-b*/
 GEN
 Zp_sqrtnlift(GEN b, GEN n, GEN a, GEN p, long e)
 {
   pari_sp ltop=avma;
   GEN q, w, n_1;
   ulong mask;
-
-  if (equalii(n, gen_2)) return Zp_sqrtlift(b,a,p,e);
+  long pis2 = equalii(n, gen_2)? 1: 0;
   if (e == 1) return icopy(a);
   n_1 = subis(n,1);
   mask = quadratic_prec_mask(e);
-  w = Fp_inv(Fp_mul(n,Fp_pow(a,n_1,p), p), p);
+  w = Fp_inv(pis2 ? shifti(a,1): Fp_mul(n,Fp_pow(a,n_1,p), p), p);
   q = p;
   for(;;)
   {
@@ -565,7 +591,7 @@ Zp_sqrtnlift(GEN b, GEN n, GEN a, GEN p, long e)
     mask >>= 1;
     if (lgefint(q) == 3 && lgefint(n) == 3)
     {
-      ulong Q = (ulong)q[2], N = (ulong)n[2];
+      ulong Q = uel(q,2), N = uel(n,2);
       ulong A = umodiu(a, Q);
       ulong B = umodiu(b, Q);
       ulong W = umodiu(w, Q);
@@ -583,12 +609,34 @@ Zp_sqrtnlift(GEN b, GEN n, GEN a, GEN p, long e)
       a = modii(subii(a, mulii(w, subii(Fp_pow(a,n,q),b))), q);
       if (mask == 1) break;
       /* w += w - w^2 n a^(n-1)*/
-      w = subii(shifti(w,1), Fp_mul(Fp_sqr(w,q), mulii(n,Fp_pow(a,n_1,q)), q));
+      w = subii(shifti(w,1), Fp_mul(Fp_sqr(w,q),
+                           pis2? shifti(a,1): mulii(n,Fp_pow(a,n_1,q)), q));
     }
   }
   return gerepileuptoint(ltop,a);
 }
 
+
+/* Same as ZpX_liftroot for the polynomial X^2-b */
+GEN
+Zp_sqrtlift(GEN b, GEN a, GEN p, long e)
+{
+  return Zp_sqrtnlift(b, gen_2, a, p, e);
+}
+
+GEN
+Zp_sqrt(GEN x, GEN p, long e)
+{
+  pari_sp av;
+  GEN z;
+  if (absequaliu(p,2)) return Z2_sqrt(x,e);
+  av = avma;
+  z = Fp_sqrt(Fp_red(x, p), p);
+  if (!z) return NULL;
+  if (e > 1) z = Zp_sqrtlift(x, z, p, e);
+  return gerepileuptoint(av, z);
+}
+
 /* Compute (x-1)/(x+1)/p^k */
 static GEN
 ZpXQ_log_to_ath(GEN x, long k, GEN T, GEN p, long e, GEN pe)
@@ -597,7 +645,7 @@ ZpXQ_log_to_ath(GEN x, long k, GEN T, GEN p, long e, GEN pe)
   long vT = get_FpX_var(T);
   GEN bn, bdi;
   GEN bd = ZX_Z_add(x, gen_1);
-  if (equaliu(p,2)) /*For p=2, we need to simplify by 2*/
+  if (absequaliu(p,2)) /*For p=2, we need to simplify by 2*/
   {
     bn = ZX_shifti(x,-(k+1));
     bdi= ZpXQ_invlift(ZX_shifti(bd ,-1), pol_1(vT), T, p, e);
@@ -616,7 +664,7 @@ ZpXQ_log(GEN a, GEN T, GEN p, long N)
 {
   pari_sp av = avma;
   pari_timer ti;
-  long is2 = equaliu(p,2);
+  long is2 = absequaliu(p,2);
   ulong pp = is2 ? 0: itou_or_0(p);
   double lp = is2 ? 1: pp ? log2(pp): expi(p);
   long k = maxss(1 , (long) .5+pow((double)(N>>1)/(lp*lp), 1./3));
@@ -687,13 +735,13 @@ gen_ZpX_Newton(GEN x, GEN p, long n, void *E,
                       GEN eval(void *E, GEN f, GEN q),
                       GEN invd(void *E, GEN V, GEN v, GEN q, long M))
 {
-  pari_sp ltop = avma, av, st_lim;
+  pari_sp ltop = avma, av;
   long N = 1, N2, M;
   long mask;
   GEN q = p;
   if (n == 1) return gcopy(x);
   mask = quadratic_prec_mask(n);
-  av = avma; st_lim = stack_lim(av, 1);
+  av = avma;
   while (mask > 1)
   {
     GEN qM, q2, v, V;
@@ -713,7 +761,7 @@ gen_ZpX_Newton(GEN x, GEN p, long n, void *E,
     v = eval(E, x, q);
     V = ZX_Z_divexact(gel(v,1), q2);
     x = FpX_sub(x, ZX_Z_mul(invd(E, V, v, qM, M), q2), q);
-    if (low_stack(st_lim, stack_lim(av, 1)))
+    if (gc_needed(av, 1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"gen_ZpX_Newton");
       gerepileall(av, 2, &x, &q);
@@ -767,6 +815,49 @@ ZpXQ_inv(GEN a, GEN T, GEN p, long e)
   return gerepileupto(av, ZpXQ_invlift(a, ai, T, p, e));
 }
 
+GEN
+ZpXQ_div(GEN a, GEN b, GEN T, GEN q, GEN p, long e)
+{
+  return FpXQ_mul(a, ZpXQ_inv(b, T, p, e), T, q);
+}
+
+GEN
+ZpXQX_divrem(GEN x, GEN Sp, GEN T, GEN q, GEN p, long e, GEN *pr)
+{
+  pari_sp av = avma;
+  GEN S = get_FpXQX_mod(Sp);
+  GEN b = leading_coeff(S), bi;
+  GEN S2, Q;
+  if (typ(b)==t_INT) return FpXQX_divrem(x, Sp, T, q, pr);
+  bi = ZpXQ_inv(b, T, p, e);
+  S2 = FqX_Fq_mul_to_monic(S, bi, T, q);
+  Q = FpXQX_divrem(x, S2, T, q, pr);
+  if (pr==ONLY_DIVIDES && !Q) { avma = av; return NULL; }
+  if (pr==ONLY_REM || pr==ONLY_DIVIDES) return gerepileupto(av, Q);
+  Q = FpXQX_FpXQ_mul(Q, bi, T, q);
+  gerepileall(av, 2, &Q, pr);
+  return Q;
+}
+
+GEN
+ZpXQX_digits(GEN x, GEN B, GEN T, GEN q, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN b = leading_coeff(B), bi;
+  GEN B2, P, V, W;
+  long i, lV;
+  if (typ(b)==t_INT) return FpXQX_digits(x, B, T, q);
+  bi = ZpXQ_inv(b, T, p, e);
+  B2 = FqX_Fq_mul_to_monic(B, bi, T, q);
+  V = FpXQX_digits(x, B2, T, q);
+  lV = lg(V)-1;
+  P = FpXQ_powers(bi, lV-1, T, q);
+  W = cgetg(lV+1, t_VEC);
+  for(i=1; i<=lV; i++)
+    gel(W, i) = FpXQX_FpXQ_mul(gel(V,i), gel(P, i), T, q);
+  return gerepileupto(av, W);
+}
+
 struct _ZpXQ_sqrtn
 {
   GEN T, a, n, ai;
@@ -798,3 +889,94 @@ ZpXQ_sqrtnlift(GEN a, GEN n, GEN x, GEN T, GEN p, long e)
   d.ai = ZpXQ_inv(ZX_Z_mul(a, n),T,p,(e+1)>>1);
   return gen_ZpX_Newton(x, p, e, &d, _sqrtn_eval, _sqrtn_invd);
 }
+
+static GEN
+to_ZX(GEN a, long v) { return typ(a)==t_INT? scalarpol(a,v): a; }
+
+GEN
+Zq_sqrtnlift(GEN a, GEN n, GEN x, GEN T, GEN p, long e)
+{
+  return T? ZpXQ_sqrtnlift(to_ZX(a,varn(T)), n, to_ZX(x,varn(T)), T, p, e)
+          : Zp_sqrtnlift(a, n, x, p, e);
+}
+
+GEN
+ZpXQ_sqrt(GEN a, GEN T, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN z = FpXQ_sqrt(FpX_red(a, p), T, p);
+  if (!z) return NULL;
+  if (e <= 1) return gerepileupto(av, z);
+  return gerepileupto(av, ZpXQ_sqrtnlift(a, gen_2, z, T, p, e));
+}
+
+GEN
+ZpX_ZpXQ_liftroot_ea(GEN P, GEN S, GEN T, GEN p, long n, void *E,
+                     int early(void *E, GEN x, GEN q))
+{
+  pari_sp ltop = avma, av;
+  long N, r;
+  long mask;
+  GEN q2, q, W, Q, Tq2, Tq, Pq;
+  pari_timer ti;
+  T = FpX_get_red(T, powiu(p, n));
+  if (n == 1) return gcopy(S);
+  mask = quadratic_prec_mask(n);
+  av = avma;
+  q2 = p; q = sqri(p); mask >>= 1; N = 2;
+  if (DEBUGLEVEL > 3) timer_start(&ti);
+  Tq = FpXT_red(T,q);
+  Tq2 = FpXT_red(Tq,q2);
+  Pq = FpX_red(P,q);
+  W = FpXQ_inv(FpX_FpXQ_eval(FpX_deriv(P,q2), S, Tq2, q2), Tq2, q2);
+  Q  = ZX_Z_divexact(FpX_FpXQ_eval(Pq, S, Tq, q), q2);
+  r = brent_kung_optpow(degpol(P), 4, 3);
+  for (;;)
+  {
+    GEN H, Sq, Wq, Spow, dP, qq, Pqq, Tqq;
+    H  = FpXQ_mul(W, Q, Tq2, q2);
+    Sq = FpX_sub(S, ZX_Z_mul(H, q2), q);
+    if (DEBUGLEVEL > 3)
+      timer_printf(&ti,"ZpX_ZpXQ_liftroot: lift to prec %ld",N);
+    if (mask==1 || (early && early(E, Sq, q)))
+      return gerepileupto(ltop, Sq);
+    qq = sqri(q); N <<= 1;
+    if (mask&1UL) { qq = diviiexact(qq, p); N--; }
+    mask >>= 1;
+    Pqq  = FpX_red(P, qq);
+    Tqq  = FpXT_red(T, qq);
+    Spow = FpXQ_powers(Sq, r, Tqq, qq);
+    Q  = ZX_Z_divexact(FpX_FpXQV_eval(Pqq, Spow, Tqq, qq), q);
+    dP = FpX_FpXQV_eval(FpX_deriv(Pq, q), FpXV_red(Spow, q), Tq, q);
+    Wq = ZX_Z_divexact(FpX_Fp_sub(FpXQ_mul(W, dP, Tq, q), gen_1, q), q2);
+    Wq = ZX_Z_mul(FpXQ_mul(W, Wq, Tq2, q2), q2);
+    Wq = FpX_sub(W, Wq, q);
+    S = Sq; W = Wq; q2 = q; q = qq; Tq2 = Tq; Tq = Tqq; Pq = Pqq;
+    if (gc_needed(av, 1))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"ZpX_ZpXQ_Newton");
+      gerepileall(av, 8, &S, &W, &Q, &Tq2, &Tq, &Pq, &q, &q2);
+    }
+  }
+}
+
+GEN
+ZpX_ZpXQ_liftroot(GEN P, GEN S, GEN T, GEN p, long n)
+{
+  return ZpX_ZpXQ_liftroot_ea(P, S, T, p, n, NULL, NULL);
+}
+
+GEN
+ZpX_Frobenius(GEN T, GEN p, long e)
+{
+  return ZpX_ZpXQ_liftroot(get_FpX_mod(T), FpX_Frobenius(T, p), T, p, e);
+}
+
+GEN
+ZpXQM_prodFrobenius(GEN M, GEN T, GEN p, long e)
+{
+  pari_sp av = avma;
+  GEN xp = ZpX_Frobenius(T, p, e);
+  GEN z = FpXQM_autsum(mkvec2(xp, M), get_FpX_degree(T), T, powiu(p,e));
+  return gerepilecopy(av, gel(z,2));
+}
