@@ -48,7 +48,7 @@ ZX_divides_i(GEN x, GEN y, GEN B)
       p1 = dvmdii(p1,y_lead, &r);
       if (r != gen_0) return NULL;
     }
-    if (B && absi_cmp(p1, B) > 0) return NULL;
+    if (B && abscmpii(p1, B) > 0) return NULL;
     p1 = gerepileuptoint(av, p1);
     gel(z,i-dy) = p1;
   }
@@ -106,7 +106,7 @@ static GEN
 Mignotte_bound(GEN S)
 {
   long i, d = degpol(S);
-  GEN C, N2, t, binlS, lS = leading_term(S), bin = vecbinome(d-1);
+  GEN C, N2, t, binlS, lS = leading_coeff(S), bin = vecbinome(d-1);
 
   N2 = sqrtr(RgX_fpnorml2(S,DEFAULTPREC));
   binlS = is_pm1(lS)? bin: ZC_Z_mul(bin, lS);
@@ -143,7 +143,7 @@ Beauzamy_bound(GEN S)
   /* s = [S]_2^2 */
   C = powruhalf(stor(3,prec), 3 + 2*d); /* 3^{3/2 + d} */
   C = divrr(mulrr(C, s), mulur(4*d, mppi(prec)));
-  lS = absi(leading_term(S));
+  lS = absi(leading_coeff(S));
   return mulir(lS, sqrtr(C));
 }
 
@@ -184,7 +184,7 @@ cmbf(GEN pol, GEN famod, GEN bound, GEN p, long a, long b,
   GEN fa       = cgetg(lfamod+1, t_VEC);
 
   *pmaxK = cmbf_maxK(lfamod);
-  lc = absi(leading_term(pol));
+  lc = absi(leading_coeff(pol));
   if (is_pm1(lc)) lc = NULL;
   lcpol = lc? ZX_Z_mul(pol, lc): pol;
 
@@ -209,11 +209,11 @@ cmbf(GEN pol, GEN famod, GEN bound, GEN p, long a, long b,
         T1 = Fp_mul(lc, T1, pa);
         T2 = Fp_mul(lc2,T2, pa);
       }
-      trace1[i] = itou(diviiround(T1, pb));
-      trace2[i] = itou(diviiround(T2, pb));
+      uel(trace1,i) = itou(diviiround(T1, pb));
+      uel(trace2,i) = itou(diviiround(T2, pb));
     }
-    spa_b   = (ulong)  pa_b[2]; /* < 2^31 */
-    spa_bs2 = (ulong)pa_bs2[2]; /* < 2^31 */
+    spa_b   = uel(pa_b,2); /* < 2^31 */
+    spa_bs2 = uel(pa_bs2,2); /* < 2^31 */
   }
   degsofar[0] = 0; /* sentinel */
 
@@ -240,8 +240,8 @@ nextK:
       ulong t;
 
       /* d - 1 test */
-      for (t=trace1[ind[1]],i=2; i<=K; i++)
-        t = Fl_add(t, trace1[ind[i]], spa_b);
+      for (t=uel(trace1,ind[1]),i=2; i<=K; i++)
+        t = Fl_add(t, uel(trace1,ind[i]), spa_b);
       if (t > spa_bs2) t = spa_b - t;
       if (t > Sbound)
       {
@@ -249,8 +249,8 @@ nextK:
         goto NEXT;
       }
       /* d - 2 test */
-      for (t=trace2[ind[1]],i=2; i<=K; i++)
-        t = Fl_add(t, trace2[ind[i]], spa_b);
+      for (t=uel(trace2,ind[1]),i=2; i<=K; i++)
+        t = Fl_add(t, uel(trace2,ind[i]), spa_b);
       if (t > spa_bs2) t = spa_b - t;
       if (t > Sbound)
       {
@@ -263,11 +263,11 @@ nextK:
       y = lc;
       for (i=1; i<=K; i++)
       {
-        GEN q = constant_term(gel(famod,ind[i]));
+        GEN q = constant_coeff(gel(famod,ind[i]));
         if (y) q = mulii(y, q);
         y = centermodii(q, pa, pas2);
       }
-      if (!signe(y) || remii(constant_term(lcpol), y) != gen_0)
+      if (!signe(y) || remii(constant_coeff(lcpol), y) != gen_0)
       {
         if (DEBUGLEVEL>3) err_printf("T");
         avma = av; goto NEXT;
@@ -295,15 +295,15 @@ nextK:
       gel(fa,cnt++) = y;
       /* fix up pol */
       pol = q;
-      if (lc) pol = Q_div_to_int(pol, leading_term(y));
+      if (lc) pol = Q_div_to_int(pol, leading_coeff(y));
       for (i=j=k=1; i <= lfamod; i++)
       { /* remove used factors */
         if (j <= K && i == ind[j]) j++;
         else
         {
           gel(famod,k) = gel(famod,i);
-          trace1[k] = trace1[i];
-          trace2[k] = trace2[i];
+          uel(trace1,k) = uel(trace1,i);
+          uel(trace2,k) = uel(trace2,i);
           deg[k] = deg[i]; k++;
         }
       }
@@ -312,7 +312,7 @@ nextK:
       if (lfamod < 2*K) goto END;
       i = 1; curdeg = deg[ind[1]];
       bound = factor_bound(pol);
-      if (lc) lc = absi(leading_term(pol));
+      if (lc) lc = absi(leading_coeff(pol));
       lcpol = lc? ZX_Z_mul(pol, lc): pol;
       if (DEBUGLEVEL>3)
         err_printf("\nfound factor %Ps\nremaining modular factor(s): %ld\n",
@@ -335,7 +335,7 @@ END:
   *done = 1;
   if (degpol(pol) > 0)
   { /* leftover factor */
-    if (signe(leading_term(pol)) < 0) pol = ZX_neg(pol);
+    if (signe(leading_coeff(pol)) < 0) pol = ZX_neg(pol);
     if (lfamod >= 2*K) *done = 0;
 
     setlg(famod, lfamod+1);
@@ -347,23 +347,6 @@ END:
   setlg(fa, cnt); return mkvec2(fa, listmod);
 }
 
-void
-factor_quad(GEN x, GEN res, long *ptcnt)
-{
-  GEN a = gel(x,4), b = gel(x,3), c = gel(x,2), d, u, z1, z2, t;
-  GEN D = subii(sqri(b), shifti(mulii(a,c), 2));
-  long v, cnt = *ptcnt;
-
-  if (!Z_issquareall(D, &d)) { gel(res,cnt++) = x; *ptcnt = cnt; return; }
-
-  t = shifti(negi(addii(b, d)), -1);
-  z1 = gdiv(t, a); u = denom(z1);
-  z2 = gdiv(addii(t, d), a);
-  v = varn(x);
-  gel(res,cnt++) = gmul(u, gsub(pol_x(v), z1)); u = diviiexact(a, u);
-  gel(res,cnt++) = gmul(u, gsub(pol_x(v), z2)); *ptcnt = cnt;
-}
-
 /* recombination of modular factors: van Hoeij's algorithm */
 
 /* Q in Z[X], return Q(2^n) */
@@ -383,13 +366,13 @@ shifteval(GEN Q, long n)
 static GEN
 root_bound(GEN P0)
 {
-  GEN Q = leafcopy(P0), lP = absi(leading_term(Q)), x,y,z;
+  GEN Q = leafcopy(P0), lP = absi(leading_coeff(Q)), x,y,z;
   long k, d = degpol(Q);
 
   /* P0 = lP x^d + Q, deg Q < d */
   Q = normalizepol_lg(Q, d+2);
   for (k=lg(Q)-1; k>1; k--) gel(Q,k) = absi(gel(Q,k));
-  k = (long)(cauchy_bound(P0) / LOG2);
+  k = (long)(fujiwara_bound(P0));
   for (  ; k >= 0; k--)
   {
     pari_sp av = avma;
@@ -412,27 +395,6 @@ root_bound(GEN P0)
   return y;
 }
 
-GEN
-special_pivot(GEN x)
-{
-  GEN t, perm, H = ZM_hnfperm(x,NULL,&perm);
-  long i,j, l = lg(H), h = lgcols(H);
-  for (i=1; i<h; i++)
-  {
-    int fl = 0;
-    for (j=1; j<l; j++)
-    {
-      t = gcoeff(H,i,j);
-      if (signe(t))
-      {
-        if (!is_pm1(t) || fl) return NULL;
-        fl = 1;
-      }
-    }
-  }
-  return rowpermute(H, perm_inv(perm));
-}
-
 GEN
 chk_factors_get(GEN lt, GEN famod, GEN c, GEN T, GEN N)
 {
@@ -451,13 +413,13 @@ chk_factors(GEN P, GEN M_L, GEN bound, GEN famod, GEN pa)
   long i, r;
   GEN pol = P, list, piv, y, ltpol, lt, paov2;
 
-  piv = special_pivot(M_L);
+  piv = ZM_hnf_knapsack(M_L);
   if (!piv) return NULL;
-  if (DEBUGLEVEL>7) err_printf("special_pivot output:\n%Ps\n",piv);
+  if (DEBUGLEVEL>7) err_printf("ZM_hnf_knapsack output:\n%Ps\n",piv);
 
   r  = lg(piv)-1;
   list = cgetg(r+1, t_VEC);
-  lt = absi(leading_term(pol));
+  lt = absi(leading_coeff(pol));
   if (is_pm1(lt)) lt = NULL;
   ltpol = lt? ZX_Z_mul(pol, lt): pol;
   paov2 = shifti(pa,-1);
@@ -473,8 +435,8 @@ chk_factors(GEN P, GEN M_L, GEN bound, GEN famod, GEN pa)
 
     if (lt)
     {
-      pol = ZX_Z_divexact(pol, leading_term(y));
-      lt = absi(leading_term(pol));
+      pol = ZX_Z_divexact(pol, leading_coeff(y));
+      lt = absi(leading_coeff(pol));
       ltpol = ZX_Z_mul(pol, lt);
     }
     else
@@ -529,10 +491,10 @@ LLL_cmbf(GEN P, GEN famod, GEN p, GEN pa, GEN bound, long a, long rec)
   double logp = log((double)itos(p)), LOGp2 = LOG2/logp;
   double b0 = log((double)dP*2) / logp, logBr;
   GEN lP, Br, Bnorm, Tra, T2, TT, CM_L, m, list, ZERO;
-  pari_sp av, av2, lim;
+  pari_sp av, av2;
   long ti_LLL = 0, ti_CF  = 0;
 
-  lP = absi(leading_term(P));
+  lP = absi(leading_coeff(P));
   if (is_pm1(lP)) lP = NULL;
   Br = root_bound(P);
   if (lP) Br = mulii(lP, Br);
@@ -543,7 +505,7 @@ LLL_cmbf(GEN P, GEN famod, GEN p, GEN pa, GEN bound, long a, long rec)
   Bnorm = dbltor(n0 * (C*C + N0*n0/4.) * 1.00001);
   ZERO = zeromat(n0, N0);
 
-  av = avma; lim = stack_lim(av, 1);
+  av = avma;
   TT = cgetg(n0+1, t_VEC);
   Tra  = cgetg(n0+1, t_MAT);
   for (i=1; i<=n0; i++)
@@ -572,7 +534,7 @@ LLL_cmbf(GEN P, GEN famod, GEN p, GEN pa, GEN bound, long a, long rec)
       a = (long)next2pow((ulong)a);
 
       pa = powiu(p,a);
-      famod = ZpX_liftfact(P,famod,NULL,p,a,pa);
+      famod = ZpX_liftfact(P, famod, pa, p, a);
       for (i=1; i<=n0; i++) TT[i] = 0;
     }
     for (i=1; i<=n0; i++)
@@ -665,7 +627,7 @@ AGAIN:
       if (DEBUGLEVEL>2) err_printf("LLL_cmbf: chk_factors failed");
     }
     CM_L = gerepilecopy(av2, CM_L);
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"LLL_cmbf");
       gerepileall(av, 5, &CM_L, &TT, &Tra, &famod, &pa);
@@ -683,11 +645,13 @@ cmbf_precs(GEN q, GEN A, GEN B, long *pta, long *ptb, GEN *qa, GEN *qb)
   long a,b,amin,d = (long)(31 * LOG2/gtodouble(glog(q,DEFAULTPREC)) - 1e-5);
   int fl = 0;
 
-  b = logint(B, q, qb);
+  b = logintall(B, q, qb) + 1;
+  *qb = mulii(*qb, q);
   amin = b + d;
   if (gcmp(powiu(q, amin), A) <= 0)
   {
-    a = logint(A, q, qa);
+    a = logintall(A, q, qa) + 1;
+    *qa = mulii(*qa, q);
     b = a - d; *qb = powiu(q, b);
   }
   else
@@ -714,13 +678,13 @@ combine_factors(GEN target, GEN famod, GEN p, long klim)
 
   A = factor_bound(target);
 
-  la = absi(leading_term(target));
+  la = absi(leading_coeff(target));
   B = mului(n, sqri(mulii(la, root_bound(target)))); /* = bound for S_2 */
 
   (void)cmbf_precs(p, A, B, &a, &b, &pa, &pb);
 
   if (DEBUGLEVEL>2) timer_start(&T);
-  famod = ZpX_liftfact(target,famod,NULL,p,a,pa);
+  famod = ZpX_liftfact(target, famod, pa, p, a);
   if (DEBUGLEVEL>2) timer_printf(&T, "Hensel lift (mod %Ps^%ld)", p,a);
   L = cmbf(target, famod, A, p, a, b, klim, &maxK, &done);
   if (DEBUGLEVEL>2) timer_printf(&T, "Naive recombination");
@@ -788,71 +752,54 @@ pick_prime(GEN a, long fl, pari_timer *T)
   avma = av; return chosenp;
 }
 
-/* Assume pol squarefree mod p; return vector of rational roots of a */
+/* Assume A a squarefree ZX; return the vector of its rational roots */
 static GEN
 DDF_roots(GEN A)
 {
-  GEN p, Ap, lc, lcpol, z, pe, pes2, bound;
-  long i, m, e, lz, v = varn(A);
+  GEN p, lc, lcpol, z, pe, pes2, bound;
+  long i, m, e, lz;
   ulong pp;
-  pari_sp av, lim;
+  pari_sp av;
   pari_timer T;
 
   if (DEBUGLEVEL>2) timer_start(&T);
   pp = pick_prime(A, 1, &T);
   if (!pp) return cgetg(1,t_VEC); /* no root */
   p = utoipos(pp);
-  lc = leading_term(A);
+  lc = leading_coeff(A);
   if (is_pm1(lc))
   { lc = NULL; lcpol = A; }
   else
-  { lc = absi(lc); lcpol = ZX_Z_mul(A, lc); }
-  Ap = Flx_normalize(ZX_to_Flx(A, pp), pp);
-  bound = root_bound(A);
-  if (lc) bound = mulii(lc, bound);
-  e = logint(addiu(shifti(bound, 1), 1), p, &pe);
+  { lc = absi_shallow(lc); lcpol = ZX_Z_mul(A, lc); }
+  bound = root_bound(A); if (lc) bound = mulii(lc, bound);
+  e = logintall(addiu(shifti(bound, 1), 1), p, &pe) + 1;
+  pe = mulii(pe, p);
   pes2 = shifti(pe, -1);
   if (DEBUGLEVEL>2) timer_printf(&T, "Root bound");
-
-  av = avma; lim = stack_lim(av,2);
-  z = Flx_roots(Ap, pp);
-  lz = lg(z)-1;
-  if (lz > (degpol(A) >> 2))
-  { /* many roots */
-    GEN Bp = Flx_div(Ap, Flv_roots_to_pol(z, pp, v), pp);
-    z = Flv_to_ZV(z);
-    z = shallowconcat(deg1_from_roots(z, v), Flx_to_ZX(Bp));
-    z = ZpX_liftfact(A, z, NULL, p, e, pe);
-  }
-  else
-  {
-    z = Flv_to_ZV(z);
-    z = ZpX_liftroots(A, z, p, e);
-    z = deg1_from_roots(z, v);
-  }
+  av = avma;
+  z = ZpX_roots(A, p, e); lz = lg(z);
+  z = deg1_from_roots(z, varn(A));
   if (DEBUGLEVEL>2) timer_printf(&T, "Hensel lift (mod %lu^%ld)", pp,e);
-
-  for (m=1, i=1; i <= lz; i++)
+  for (m=1, i=1; i < lz; i++)
   {
     GEN q, r, y = gel(z,i);
     if (lc) y = ZX_Z_mul(y, lc);
     y = centermod_i(y, pe, pes2);
     if (! (q = ZX_divides(lcpol, y)) ) continue;
 
-    lcpol = A = q;
-    r = negi( constant_term(y) );
+    lcpol = q;
+    r = negi( constant_coeff(y) );
     if (lc) {
       r = gdiv(r,lc);
-      A = Q_primpart(A);
-      lc = absi( leading_term(A) );
-      if (is_pm1(lc)) lc = NULL; else lcpol = ZX_Z_mul(A, lc);
+      lcpol = Q_primpart(lcpol);
+      lc = absi_shallow( leading_coeff(lcpol) );
+      if (is_pm1(lc)) lc = NULL; else lcpol = ZX_Z_mul(lcpol, lc);
     }
     gel(z,m++) = r;
-    if (low_stack(lim, stack_lim(av,2)))
+    if (gc_needed(av,2))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"DDF_roots, m = %ld", m);
-      gerepileall(av, lc? 4:2, &z, &A, &lc, &lcpol);
-
+      gerepileall(av, lc? 3:2, &z, &lcpol, &lc);
     }
   }
   if (DEBUGLEVEL>2) timer_printf(&T, "Recombination");
@@ -895,7 +842,7 @@ ZX_DDF(GEN x)
 {
   GEN L;
   long m;
-  x = RgX_deflate_max(x, &m);
+  x = ZX_deflate_max(x, &m);
   L = DDF(x);
   if (m > 1)
   {
@@ -927,7 +874,7 @@ ZX_squff(GEN f, GEN *ex)
   GEN T, V, P, e;
   long i, k, n, val;
 
-  if (signe(leading_term(f)) < 0) f = gneg_i(f);
+  if (signe(leading_coeff(f)) < 0) f = gneg_i(f);
   val = ZX_valrem(f, &f);
   n = 1 + degpol(f); if (val) n++;
   e = cgetg(n,t_VECSMALL);
@@ -960,7 +907,7 @@ ZX_squff(GEN f, GEN *ex)
   setlg(e,i); *ex = e; return P;
 }
 
-GEN
+static GEN
 fact_from_DDF(GEN fa, GEN e, long n)
 {
   GEN v,w, y = cgetg(3, t_MAT);
@@ -1036,7 +983,7 @@ nfrootsQ(GEN x)
   if (typ(x)!=t_POL) pari_err_TYPE("nfrootsQ",x);
   if (!signe(x)) pari_err_ROOTS0("nfrootsQ");
   x = Q_primpart(x);
-  if (!RgX_is_ZX(x)) pari_err_TYPE("nfrootsQ",x);
+  RgX_check_ZX(x,"nfrootsQ");
   val = ZX_valrem(x, &x);
   (void)ZX_gcd_all(x, ZX_deriv(x), &x);
   z = DDF_roots(x);
@@ -1051,8 +998,18 @@ int
 ZX_is_squarefree(GEN x)
 {
   pari_sp av = avma;
-  GEN d = ZX_gcd(x,ZX_deriv(x));
-  int r = (lg(d) == 3); avma = av; return r;
+  GEN d;
+  long m;
+  int r;
+  if (lg(x) == 2) return 0;
+  m = ZX_deflate_order(x);
+  if (m > 1)
+  {
+    if (!signe(gel(x,2))) return 0;
+    x = RgX_deflate(x, m);
+  }
+  d = ZX_gcd(x,ZX_deriv(x));
+  r = (lg(d) == 3); avma = av; return r;
 }
 
 #if 0
@@ -1067,7 +1024,7 @@ maxnorm(GEN p)
   for (i=0; i<n; i++)
   {
     x = gel(p,i);
-    if (absi_cmp(x,m) > 0) m = x;
+    if (abscmpii(x,m) > 0) m = x;
   }
   m = divii(m, gel(p,n));
   return gerepileuptoint(av, addis(absi(m),1));
@@ -1081,7 +1038,8 @@ ZX_gcd_all(GEN A, GEN B, GEN *Anew)
   GEN R, a, b, q, H, Hp, g, Ag, Bg;
   long m, n, valX, valA, vA = varn(A);
   ulong p;
-  pari_sp ltop, av, avlim;
+  int small;
+  pari_sp ltop, av;
   forprime_t S;
 
   if (!signe(A)) { if (Anew) *Anew = pol_0(vA); return ZX_copy(B); }
@@ -1091,7 +1049,7 @@ ZX_gcd_all(GEN A, GEN B, GEN *Anew)
   ltop = avma;
 
   n = 1 + minss(degpol(A), degpol(B)); /* > degree(gcd) */
-  g = gcdii(leading_term(A), leading_term(B)); /* multiple of lead(gcd) */
+  g = gcdii(leading_coeff(A), leading_coeff(B)); /* multiple of lead(gcd) */
   if (is_pm1(g)) {
     g = NULL;
     Ag = A;
@@ -1100,8 +1058,9 @@ ZX_gcd_all(GEN A, GEN B, GEN *Anew)
     Ag = ZX_Z_mul(A,g);
     Bg = ZX_Z_mul(B,g);
   }
-  init_modular(&S);
-  av = avma; avlim = stack_lim(av, 1);
+  small = (ZX_max_lg(A) == 3 && ZX_max_lg(B) == 3);
+  init_modular_big(&S);
+  av = avma;
   R = NULL;/*-Wall*/
   H = NULL;
   while ((p = u_forprime_next(&S)))
@@ -1116,7 +1075,7 @@ ZX_gcd_all(GEN A, GEN B, GEN *Anew)
         if (valA != valX) A = RgX_shift(A, valA - valX);
         *Anew = A;
       }
-      return monomial(gen_1, valX, vA);
+      return pol_xn(valX, vA);
     }
     if (m > n) continue; /* p | Res(A/G, B/G). Discard */
 
@@ -1130,20 +1089,22 @@ ZX_gcd_all(GEN A, GEN B, GEN *Anew)
     if (m < n)
     { /* First time or degree drop [all previous p were as above; restart]. */
       H = ZX_init_CRT(Hp,p,vA);
-      q = utoipos(p); n = m; continue;
+      q = utoipos(p); n = m;
+      if (!small) continue; /* if gcd is small, try our luck */
     }
+    else
+      if (!ZX_incremental_CRT(&H, Hp, &q, p)) continue;
     if (DEBUGLEVEL>5) err_printf("gcd mod %lu (bound 2^%ld)\n", p,expi(q));
-    if (low_stack(avlim, stack_lim(av,1)))
-    {
-      if (DEBUGMEM>1) pari_warn(warnmem,"QX_gcd");
-      gerepileall(av, 3, &H, &q, &Hp);
-    }
-
-    if (!ZX_incremental_CRT(&H, Hp, &q, p)) continue;
     /* H stable: check divisibility */
     if (!ZX_divides(Bg, H)) continue;
     R = ZX_divides(Ag, H);
     if (R) break;
+
+    if (gc_needed(av,1))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"QX_gcd");
+      gerepileall(av, 3, &H, &q, &Hp);
+    }
   }
   if (!p) pari_err_OVERFLOW("ZX_gcd_all [ran out of primes]");
   if (Anew) {
@@ -1185,7 +1146,7 @@ static GEN
 BD_deg1(GEN f)
 {
   GEN a = gel(f,3), b = gel(f,2); /* f = ax + b */
-  if (!absi_equal(a,b)) return NULL;
+  if (!absequalii(a,b)) return NULL;
   return polcyclo((signe(a) == signe(b))? 2: 1, varn(f));
 }
 
@@ -1222,7 +1183,7 @@ BD(GEN f)
   RgX_even_odd(f, &fe, &fo);
   fe1 = ZX_eval1(fe);
   fo1 = ZX_eval1(fo);
-  if (absi_equal(fe1, fo1)) /* f(1) = 0 or f(-1) = 0 */
+  if (absequalii(fe1, fo1)) /* f(1) = 0 or f(-1) = 0 */
   {
     long i, v = varn(f);
     if (!signe(fe1))
@@ -1284,10 +1245,10 @@ GEN
 polcyclofactors(GEN f)
 {
   pari_sp av = avma;
-  if (typ(f) != t_POL) pari_err_TYPE("polcyclofactors",f);
+  if (typ(f) != t_POL || !signe(f)) pari_err_TYPE("polcyclofactors",f);
   (void)RgX_valrem(f, &f);
   f = Q_primpart(f);
-  if (!RgX_is_ZX(f) || !signe(f)) pari_err_TYPE("polcyclofactors",f);
+  RgX_check_ZX(f,"polcyclofactors");
   if (degpol(f))
   {
     (void)ZX_gcd_all(f, ZX_deriv(f), &f);
@@ -1304,19 +1265,19 @@ ZXQ_mul_by_X(GEN t, GEN T)
   GEN lt;
   t = RgX_shift_shallow(t, 1);
   if (degpol(t) < degpol(T)) return t;
-  lt = leading_term(t);
+  lt = leading_coeff(t);
   if (is_pm1(lt)) return signe(lt) > 0 ? ZX_sub(t, T): ZX_add(t, T);
-  return ZX_sub(t, ZX_Z_mul(T, leading_term(t)));
+  return ZX_sub(t, ZX_Z_mul(T, leading_coeff(t)));
 }
 /* f a product of Phi_n, all n odd; deg f > 1. Is it irreducible ? */
 static long
 BD_odd_iscyclo(GEN f)
 {
-  pari_sp av, lim;
+  pari_sp av;
   long d, e, n, bound;
   GEN t;
-  f = RgX_deflate_max(f, &e);
-  av = avma; lim = stack_lim(av,1);
+  f = ZX_deflate_max(f, &e);
+  av = avma;
   /* The original f is cyclotomic (= Phi_{ne}) iff the present one is Phi_n,
    * where all prime dividing e also divide n. If current f is Phi_n,
    * then n is odd and squarefree */
@@ -1335,13 +1296,13 @@ BD_odd_iscyclo(GEN f)
   else
     bound = (long)(2.573 * pow(d,1.01));
   /* IF f = Phi_n, n squarefree odd, then n <= bound */
-  t = monomial(gen_1, d-1, varn(f));
+  t = pol_xn(d-1, varn(f));
   for (n = d; n <= bound; n++)
   {
     t = ZXQ_mul_by_X(t, f);
     /* t = (X mod f(X))^d */
     if (degpol(t) == 0) break;
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"BD_odd_iscyclo");
       t = gerepilecopy(av, t);
@@ -1349,7 +1310,7 @@ BD_odd_iscyclo(GEN f)
   }
   if (n > bound || eulerphiu(n) != (ulong)d) return 0;
 
-  if (e > 1) return (ucoprime_part(e, n) == 1)? e * n : 0;
+  if (e > 1) return (u_ppo(e, n) == 1)? e * n : 0;
   return n;
 }
 
@@ -1372,7 +1333,7 @@ BD_iscyclo(GEN f)
 
   if (issquareall(f1, &f2))
   {
-    GEN lt = leading_term(f2);
+    GEN lt = leading_coeff(f2);
     long c;
     if (signe(lt) < 0) f2 = ZX_neg(f2);
     c = BD_iscyclo(f2);
@@ -1383,14 +1344,13 @@ BD_iscyclo(GEN f)
 long
 poliscyclo(GEN f)
 {
-  pari_sp av = avma;
-  long d = degpol(f);
+  long d;
   if (typ(f) != t_POL) pari_err_TYPE("poliscyclo", f);
+  d = degpol(f);
   if (d <= 0 || !RgX_is_ZX(f)) return 0;
   if (!equali1(gel(f,d+2)) || !is_pm1(gel(f,2))) return 0;
   if (d == 1) return signe(gel(f,2)) > 0? 2: 1;
-  if (degpol(ZX_gcd(f, ZX_deriv(f)))) { avma = av; return 0; }
-  return BD_iscyclo(f);
+  return ZX_is_squarefree(f)? BD_iscyclo(f): 0;
 }
 
 long
@@ -1400,7 +1360,7 @@ poliscycloprod(GEN f)
   long i, d = degpol(f);
   if (typ(f) != t_POL) pari_err_TYPE("poliscycloprod",f);
   if (!RgX_is_ZX(f)) return 0;
-  if (!equali1(leading_term(f)) || !is_pm1(constant_term(f))) return 0;
+  if (!equali1(leading_coeff(f)) || !is_pm1(constant_coeff(f))) return 0;
   if (d < 2) return (d == 1);
   if ( degpol(ZX_gcd_all(f, ZX_deriv(f), &f)) )
   {
