@@ -304,6 +304,7 @@ qfi_1_by_disc(GEN D)
 {
   GEN b,c, y = cgetg(4,t_QFI);
   quadpoly_bc(D, mod2(D), &b,&c);
+  if (b == gen_m1) b = gen_1;
   gel(y,1) = gen_1;
   gel(y,2) = b;
   gel(y,3) = c; return y;
@@ -376,7 +377,7 @@ parteucl(GEN L, GEN *d, GEN *v3, GEN *v, GEN *v2)
 {
   long z;
   *v = gen_0; *v2 = gen_1;
-  for (z=0; absi_cmp(*v3,L) > 0; z++)
+  for (z=0; abscmpii(*v3,L) > 0; z++)
   {
     GEN t3, t2 = subii(*v, mulii(truedvmdii(*d,*v3,&t3),*v2));
     *v = *v2; *d = *v3; *v2 = t2; *v3 = t3;
@@ -397,7 +398,7 @@ nucomp(GEN x, GEN y, GEN L)
   if (typ(x) != t_QFI) pari_err_TYPE("nucomp",x);
   if (typ(y) != t_QFI) pari_err_TYPE("nucomp",y);
 
-  if (absi_cmp(gel(x,1),gel(y,1)) < 0) swap(x, y);
+  if (abscmpii(gel(x,1),gel(y,1)) < 0) swap(x, y);
   s = shifti(addii(gel(x,2),gel(y,2)), -1);
   n = subii(gel(y,2), s);
   a1 = gel(x,1);
@@ -427,7 +428,7 @@ nucomp(GEN x, GEN y, GEN L)
       l = modii(mulii(negi(u1), addii(mulii(u,p1),mulii(v,p2))), d);
       a = subii(mulii(l,diviiexact(a1,d)), mulii(u,diviiexact(n,d)));
     }
-  a = modii(a,a1); p1 = subii(a,a1); if (absi_cmp(a,p1) > 0) a = p1;
+  a = modii(a,a1); p1 = subii(a,a1); if (abscmpii(a,p1) > 0) a = p1;
   d = a1; v3 = a; z = parteucl(L, &d,&v3, &v,&v2);
   Q = cgetg(4,t_QFI);
   if (!z) {
@@ -472,7 +473,7 @@ nudupl(GEN x, GEN L)
     b = diviiexact(b, d1);
   }
   c = modii(negi(mulii(u,gel(x,3))), a);
-  p1 = subii(c,a); if (absi_cmp(c,p1) > 0) c = p1;
+  p1 = subii(c,a); if (abscmpii(c,p1) > 0) c = p1;
   d = a; v3 = c; z = parteucl(L, &d,&v3, &v,&v2);
   a2 = sqri(d);
   c2 = sqri(v3);
@@ -501,7 +502,7 @@ mul_nucomp(void *l, GEN x, GEN y) { return nucomp(x, y, (GEN)l); }
 static GEN
 mul_nudupl(void *l, GEN x) { return nudupl(x, (GEN)l); }
 GEN
-nupow(GEN x, GEN n)
+nupow(GEN x, GEN n, GEN L)
 {
   pari_sp av;
   GEN y, D;
@@ -513,10 +514,11 @@ nupow(GEN x, GEN n)
   D = qfb_disc(x);
   y = qfi_1_by_disc(D);
   if (!signe(n)) return y;
-  y = gen_pow(x, n, (void*)sqrtnint(absi(D), 4), &mul_nudupl, &mul_nucomp);
+  if (!L) L = sqrtnint(absi(D), 4);
+  y = gen_pow(x, n, (void*)L, &mul_nudupl, &mul_nucomp);
   if (signe(n) < 0
-  && !absi_equal(gel(y,1),gel(y,2))
-  && !absi_equal(gel(y,1),gel(y,3))) togglesign(gel(y,2));
+  && !absequalii(gel(y,1),gel(y,2))
+  && !absequalii(gel(y,1),gel(y,3))) togglesign(gel(y,2));
   return gerepileupto(av, y);
 }
 
@@ -528,9 +530,9 @@ dvmdii_round(GEN b, GEN a, GEN *r)
 {
   GEN a2 = shifti(a, 1), q = dvmdii(b, a2, r);
   if (signe(b) >= 0) {
-    if (absi_cmp(*r, a) > 0) { q = addis(q,  1); *r = subii(*r, a2); }
+    if (abscmpii(*r, a) > 0) { q = addis(q,  1); *r = subii(*r, a2); }
   } else { /* r <= 0 */
-    if (absi_cmp(*r, a) >= 0){ q = addis(q, -1); *r = addii(*r, a2); }
+    if (abscmpii(*r, a) >= 0){ q = addis(q, -1); *r = addii(*r, a2); }
   }
   return q;
 }
@@ -613,14 +615,14 @@ GEN
 redimagsl2(GEN q, GEN *U)
 {
   GEN Q = cgetg(4, t_QFI);
-  pari_sp av = avma, av2, lim = stack_lim(av, 1);
+  pari_sp av = avma, av2;
   GEN z, u1,u2,v1,v2, a = gel(q,1), b = gel(q,2), c = gel(q,3);
   long cmp;
   /* upper bound for size of final (a,b,c) */
   (void)new_chunk(2*(lgefint(a) + lgefint(b) + lgefint(c) + 3));
   av2 = avma;
   u1 = gen_1; u2 = gen_0;
-  cmp = absi_cmp(a, b);
+  cmp = abscmpii(a, b);
   if (cmp < 0)
     REDBU(a,&b,&c, u1,&u2);
   else if (cmp == 0 && signe(b) < 0)
@@ -630,11 +632,11 @@ redimagsl2(GEN q, GEN *U)
   }
   for(;;)
   {
-    cmp = absi_cmp(a, c); if (cmp <= 0) break;
+    cmp = abscmpii(a, c); if (cmp <= 0) break;
     swap(a,c); b = negi(b);
     z = u1; u1 = u2; u2 = negi(z);
     REDBU(a,&b,&c, u1,&u2);
-    if (low_stack(lim, stack_lim(av, 1))) {
+    if (gc_needed(av, 1)) {
       if (DEBUGMEM>1) pari_warn(warnmem, "redimagsl2");
       gerepileall(av2, 5, &a,&b,&c, &u1,&u2);
     }
@@ -685,9 +687,9 @@ redimag_1(pari_sp av, GEN a, GEN b, GEN c)
   { /* at most twice */
     long lb = lgefint(b); /* <= 3 after first loop */
     if (lb == 2) return redimag_1_b0(a[2],c[2]);
-    if (lb == 3 && (ulong)b[2] <= (ulong)LONG_MAX) break;
+    if (lb == 3 && uel(b,2) <= (ulong)LONG_MAX) break;
     REDB(a,&b,&c);
-    if ((ulong)a[2] <= (ulong)c[2])
+    if (uel(a,2) <= uel(c,2))
     { /* lg(b) <= 3 but may be too large for itos */
       long s = signe(b);
       avma = av;
@@ -730,14 +732,14 @@ redimag_av(pari_sp av, GEN q)
   long cmp, lc = lgefint(c);
 
   if (lgefint(a) == 3 && lc == 3) return redimag_1(av, a, b, c);
-  cmp = absi_cmp(a, b);
+  cmp = abscmpii(a, b);
   if (cmp < 0)
     REDB(a,&b,&c);
   else if (cmp == 0 && signe(b) < 0)
     b = negi(b);
   for(;;)
   {
-    cmp = absi_cmp(a, c); if (cmp <= 0) break;
+    cmp = abscmpii(a, c); if (cmp <= 0) break;
     lc = lgefint(a); /* lg(future c): we swap a & c next */
     if (lc == 3) return redimag_1(av, a, b, c);
     swap(a,c); b = negi(b); /* apply rho */
@@ -757,9 +759,9 @@ static GEN
 rhoimag(GEN x)
 {
   GEN a = gel(x,1), b = gel(x,2), c = gel(x,3);
-  int fl = absi_cmp(a, c);
+  int fl = abscmpii(a, c);
   if (fl <= 0) {
-    int fg = absi_cmp(a, b);
+    int fg = abscmpii(a, b);
     if (fg >= 0) {
       x = qfi(a,b,c);
       if ((!fl || !fg) && signe(gel(x,2)) < 0) setsigne(gel(x,2), 1);
@@ -817,7 +819,7 @@ rho_get_BC(GEN *B, GEN *C, GEN b, GEN c, struct qfr_data *S)
 {
   GEN t, u;
   u = shifti(c,1);
-  t = (absi_cmp(S->isqrtD,c) >= 0)? S->isqrtD: c;
+  t = (abscmpii(S->isqrtD,c) >= 0)? S->isqrtD: c;
   u = remii(addii_sign(t,1, b,signe(b)), u);
   *B = addii_sign(t, 1, u, -signe(u)); /* |t| - (|t|+b) % |2c| */
   if (*B == gen_0)
@@ -904,10 +906,10 @@ qfr3_to_qfr(GEN x, GEN d)
 static int
 ab_isreduced(GEN a, GEN b, GEN isqrtD)
 {
-  if (signe(b) > 0 && absi_cmp(b, isqrtD) <= 0)
+  if (signe(b) > 0 && abscmpii(b, isqrtD) <= 0)
   {
     GEN t = addii_sign(isqrtD,1, shifti(a,1),-1);
-    long l = absi_cmp(b, t); /* compare |b| and |floor(sqrt(D)) - |2a|| */
+    long l = abscmpii(b, t); /* compare |b| and |floor(sqrt(D)) - |2a|| */
     if (l > 0 || (l == 0 && signe(t) < 0)) return 1;
   }
   return 0;
@@ -952,9 +954,9 @@ static GEN
 qfr5_init(GEN x, struct qfr_data *S)
 {
   GEN d = gel(x,4);
-  long prec = realprec(d), l = nbits2prec(-expo(d));
-  if (l > prec) prec = l;
-  if (prec < LOWDEFAULTPREC) prec = LOWDEFAULTPREC;
+  long prec = realprec(d), l = -expo(d);
+  if (l < BITS_IN_LONG) l = BITS_IN_LONG;
+  prec = maxss(prec, nbits2prec(l));
   x = qfr_to_qfr5(x,prec);
 
   get_disc(x, S);
@@ -1121,7 +1123,7 @@ qfrpow(GEN x, GEN n)
   return gerepilecopy(av, x);
 }
 
-/* Prime forms associated to prime ideals of degree 1 */
+/* Prime forms attached to prime ideals of degree 1 */
 
 /* assume x != 0 a t_INT, p > 0
  * Return a t_QFI, but discriminant sign is not checked: can be used for
@@ -1245,7 +1247,7 @@ static int
 GL2_qfb_equal(GEN a, GEN b)
 {
   return equalii(gel(a,1),gel(b,1))
-   && absi_equal(gel(a,2),gel(b,2))
+   && absequalii(gel(a,2),gel(b,2))
    &&    equalii(gel(a,3),gel(b,3));
 }
 
@@ -1307,12 +1309,12 @@ redrealsl2step(GEN A, GEN d, GEN rd)
   pari_sp ltop = avma;
   GEN N, V = gel(A,1), M = gel(A,2);
   GEN a = gel(V,1), b = gel(V,2), c = gel(V,3);
-  GEN ac = mpabs(c);
-  GEN r = addii(b, gmax(rd, ac));
-  GEN q = truedvmdii(r, shifti(ac, 1), NULL);
-  r = subii(mulii(shifti(q, 1), ac), b);
-  a = c; b = r;
-  c = truedvmdii(subii(sqri(r), d), shifti(c,2), NULL);
+  GEN C = mpabs(c);
+  GEN t = addii(b, gmax(rd, C));
+  GEN r, q = truedvmdii(t, shifti(C,1), &r);
+  b = subii(t, addii(r,b));
+  a = c;
+  c = truedivii(subii(sqri(b), d), shifti(c,2));
   if (signe(a) < 0) togglesign(q);
   N = mkmat2(gel(M,2),
              mkcol2(subii(mulii(q, gcoeff(M, 1, 2)), gcoeff(M, 1, 1)),
@@ -1323,22 +1325,22 @@ redrealsl2step(GEN A, GEN d, GEN rd)
 GEN
 redrealsl2(GEN V, GEN d, GEN rd)
 {
-  pari_sp ltop = avma, st_lim = stack_lim(ltop, 1);
+  pari_sp ltop = avma;
   GEN M, u1, u2, v1, v2;
   GEN a = gel(V,1), b = gel(V,2), c = gel(V,3);
   u1 = v2 = gen_1; v1 = u2 = gen_0;
   while (!ab_isreduced(a,b,rd))
   {
-    GEN ac = mpabs(c);
-    GEN r = addii(b, gmax(rd,ac));
-    GEN q = truedvmdii(r, shifti(ac, 1), NULL);
-    r = subii(mulii(shifti(q, 1), ac), b);
-    a = c; b = r;
-    c = truedvmdii(subii(sqri(r), d), shifti(c, 2), NULL);
+    GEN C = mpabs(c);
+    GEN t = addii(b, gmax(rd,C));
+    GEN r, q = truedvmdii(t, shifti(C,1), &r);
+    b = subii(t, addii(r,b));
+    a = c;
+    c = truedivii(subii(sqri(b), d), shifti(c,2));
     if (signe(a) < 0) togglesign(q);
     r = u1; u1 = v1; v1 = subii(mulii(q, v1), r);
     r = u2; u2 = v2; v2 = subii(mulii(q, v2), r);
-    if (low_stack(st_lim, stack_lim(ltop, 1)))
+    if (gc_needed(ltop, 1))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"redrealsl2");
       gerepileall(ltop, 7, &a,&b,&c,&u1,&u2,&v1,&v2);
@@ -1348,10 +1350,46 @@ redrealsl2(GEN V, GEN d, GEN rd)
   return gerepilecopy(ltop, mkvec2(mkvec3(a,b,c), M));
 }
 
+GEN
+qfbredsl2(GEN q, GEN S)
+{
+  GEN v, D, isD;
+  pari_sp av;
+  switch(typ(q))
+  {
+    case t_QFI:
+      if (S) pari_err_TYPE("qfbredsl2",S);
+      v = cgetg(3,t_VEC);
+      gel(v,1) = redimagsl2(q, &gel(v,2));
+      return v;
+    case t_QFR:
+      av = avma;
+      if (S) {
+        if (typ(S) != t_VEC || lg(S) != 3) pari_err_TYPE("qfbredsl2",S);
+        D = gel(S,1);
+        isD = gel(S,2);
+        if (typ(D) != t_INT || signe(D) <= 0 || typ(isD) != t_INT)
+          pari_err_TYPE("qfbredsl2",S);
+      }
+      else
+      {
+        D = qfb_disc(q);
+        isD = sqrtint(D);
+      }
+      v = redrealsl2(q,D,isD);
+      gel(v,1) = qfr3_to_qfr(gel(v,1), real_0(precision(gel(q,4))));
+      return gerepilecopy(av, v);
+
+    default:
+        pari_err_TYPE("qfbredsl2",q);
+        return NULL;
+  }
+}
+
 GEN
 qfrsolvep(GEN Q, GEN p)
 {
-  pari_sp ltop = avma, btop, st_lim;
+  pari_sp ltop = avma, btop;
   GEN N, P, P1, P2, M, rd, d = qfb_disc(Q);
   if (kronecker(d, p) < 0) { avma = ltop; return gen_0; }
   rd = sqrti(d);
@@ -1360,14 +1398,14 @@ qfrsolvep(GEN Q, GEN p)
   P1 = redrealsl2(P, d,rd);
   togglesign( gel(P,2) );
   P2 = redrealsl2(P, d,rd);
-  btop = avma; st_lim = stack_lim(btop, 1);
+  btop = avma;
   for(;;)
   {
     if (ZV_equal(gel(M,1), gel(P1,1))) { N = gel(P1,2); break; }
     if (ZV_equal(gel(M,1), gel(P2,1))) { N = gel(P2,2); break; }
     M = redrealsl2step(M, d,rd);
     if (ZV_equal(gel(M,1), gel(N,1))) { avma = ltop; return gen_0; }
-    if (low_stack(st_lim, stack_lim(btop, 1))) M = gerepileupto(btop, M);
+    if (gc_needed(btop, 1)) M = gerepileupto(btop, M);
   }
   return gerepilecopy(ltop, SL2_div_mul_e1(gel(M,2),N));
 }
@@ -1390,7 +1428,7 @@ qfbsolve(GEN Q,GEN n)
 long
 cornacchia(GEN d, GEN p, GEN *px, GEN *py)
 {
-  pari_sp av = avma, av2, lim;
+  pari_sp av = avma, av2;
   GEN a, b, c, L, r;
 
   if (typ(d) != t_INT) pari_err_TYPE("cornacchia", d);
@@ -1402,13 +1440,13 @@ cornacchia(GEN d, GEN p, GEN *px, GEN *py)
   if (signe(b) == 0) { avma = av; *py = gen_1; return 1; }
   b = Fp_sqrt(b, p); /* sqrt(-d) */
   if (!b) { avma = av; return 0; }
-  if (absi_cmp(shifti(b,1), p) > 0) b = subii(b,p);
+  if (abscmpii(shifti(b,1), p) > 0) b = subii(b,p);
   a = p; L = sqrti(p);
-  av2 = avma; lim = stack_lim(av2, 1);
-  while (absi_cmp(b, L) > 0)
+  av2 = avma;
+  while (abscmpii(b, L) > 0)
   {
     r = remii(a, b); a = b; b = r;
-    if (low_stack(lim, stack_lim(av2, 1))) {
+    if (gc_needed(av2, 1)) {
       if (DEBUGMEM>1) pari_warn(warnmem,"cornacchia");
       gerepileall(av2, 2, &a,&b);
     }
@@ -1424,7 +1462,7 @@ cornacchia(GEN d, GEN p, GEN *px, GEN *py)
 long
 cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
 {
-  pari_sp av = avma, av2, lim;
+  pari_sp av = avma, av2;
   GEN a, b, c, L, r, px4;
   long k;
 
@@ -1435,8 +1473,8 @@ cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
   k = mod4(d);
   if (k == 1 || k == 2) pari_err_DOMAIN("cornacchia2","-d mod 4", ">",gen_1,d);
   px4 = shifti(p,2);
-  if (absi_cmp(px4, d) < 0) { avma = av; return 0; }
-  if (equaliu(p, 2))
+  if (abscmpii(px4, d) < 0) { avma = av; return 0; }
+  if (absequaliu(p, 2))
   {
     avma = av;
     switch (itou_or_0(d)) {
@@ -1450,17 +1488,17 @@ cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
   if (!b) { avma = av; return 0; }
   if (!signe(b)) { /* d = p,2p,3p,4p */
     avma = av;
-    if (absi_equal(d, px4)){ *py = gen_1; return 1; }
-    if (absi_equal(d, p))  { *py = gen_2; return 1; }
+    if (absequalii(d, px4)){ *py = gen_1; return 1; }
+    if (absequalii(d, p))  { *py = gen_2; return 1; }
     return 0;
   }
   if (mod2(b) != (k & 1)) b = subii(p,b);
   a = shifti(p,1); L = sqrti(px4);
-  av2 = avma; lim = stack_lim(av2, 1);
+  av2 = avma;
   while (cmpii(b, L) > 0)
   {
     r = remii(a, b); a = b; b = r;
-    if (low_stack(lim, stack_lim(av2, 1))) {
+    if (gc_needed(av2, 1)) {
       if (DEBUGMEM>1) pari_warn(warnmem,"cornacchia");
       gerepileall(av2, 2, &a,&b);
     }
