@@ -187,7 +187,7 @@ RgX_get_1(GEN x)
     RgX_type_decode(tx, &i /*junk*/, &tx);
   switch(tx)
   {
-    case t_INTMOD: retmkintmod(gen_1, icopy(p));
+    case t_INTMOD: retmkintmod(is_pm1(p)? gen_0: gen_1, icopy(p));
     case t_PADIC: return cvtop(gen_1, p, lx);
     case t_FFELT: return FF_1(T);
     default: return gen_1;
@@ -379,7 +379,7 @@ RgX_translate(GEN P, GEN c)
   if (!signe(P) || gequal0(c)) return RgX_copy(P);
   Q = leafcopy(P);
   R = (GEN*)(Q+2); n = degpol(P);
-  if (gequal1(c))
+  if (equali1(c))
   {
     for (i=1; i<=n; i++)
     {
@@ -391,7 +391,7 @@ RgX_translate(GEN P, GEN c)
       }
     }
   }
-  else if (gequalm1(c))
+  else if (equalim1(c))
   {
     for (i=1; i<=n; i++)
     {
