@@ -300,6 +300,38 @@ ZX_valrem(GEN x, GEN *Z)
   return vx;
 }
 
+GEN
+ZX_div_by_X_1(GEN a, GEN *r)
+{
+  long l = lg(a), i;
+  GEN a0, z0, z = cgetg(l-1, t_POL);
+  z[1] = a[1];
+  a0 = a + l-1;
+  z0 = z + l-2; *z0 = *a0--;
+  for (i=l-3; i>1; i--) /* z[i] = a[i+1] + z[i+1] */
+  {
+    GEN t = addii(gel(a0--,0), gel(z0--,0));
+    gel(z0,0) = t;
+  }
+  if (r) *r = addii(gel(a0,0), gel(z0,0));
+  return z;
+}
+
+/* Return 2^(n degpol(P))  P(x >> n), not memory clean if P is a ZX */
+GEN
+ZX_rescale2n(GEN P, long n)
+{
+  long i, l = lg(P), ni = n;
+  GEN Q = cgetg(l,t_POL);
+  Q[l-1] = P[l-1];
+  for (i=l-2; i>=2; i--)
+  {
+    gel(Q,i) = shifti(gel(P,i), ni);
+    ni += n;
+  }
+  Q[1] = P[1]; return Q;
+}
+
 /* Return h^deg(P) P(x / h), not memory clean. h integer, P ZX */
 GEN
 ZX_rescale(GEN P, GEN h)
@@ -379,7 +411,7 @@ Z_mod2BIL_ZX(GEN x, long bs, long d, long vx)
   long i, offset, lm = lgefint(x)-2, l = d+vx+3, sx = signe(x);
   GEN s1 = int2n(bs*BITS_IN_LONG), pol = cgetg(l, t_POL);
   int carry = 0;
-
+  pol[1] = evalsigne(1);
   for (i=0; i<vx; i++) gel(pol,i+2) = gen_0;
   for (offset=0; i <= d+vx; i++, offset += bs)
   {
@@ -487,7 +519,7 @@ ZX_sqrspec(GEN x, long nx)
   const long low[]={ 17, 17, 32, 32, 96, 96, 96, 96, 96, 96, 96, 96, 96, 96, 112, 112, 128, 112, 112, 112, 112, 112, 128, 128, 160, 160, 112, 128, 128, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 160, 176, 160, 160, 176, 160, 160, 176, 176, 208, 176, 176, 176, 192, 192, 176, 176, 224, 176, 224, 224, 176, 224, 224, 224, 176, 176, 176, 176, 176, 176, 176, 176, 224, 176, 176, 224, 224, 224, 224, 224, 224, 224, 240, 288, 240, 288, 288, 240, 288, 288, 240, 240, 304, 304};
   const long high[]={ 165657, 85008, 52783, 43622, 32774, 27086, 22385, 15289, 13899, 12635, 11487, 10442, 9493, 7845, 6484, 6484, 5894, 5894, 4871, 4871, 4428, 4026, 3660, 3660, 3660, 3327, 3327, 3024, 2749, 2749, 2272, 2749, 2499, 2499, 2272, 1878, 1878, 1878, 1707, 1552, 1552, 1552, 1552, 1552, 1411, 1411, 1411, 1282, 1282, 1282, 1282, 1282, 1166, 1166, 1166, 1166, 1166, 1166, 1166, 1166, 1060, 1060, 963, 963, 963, 963, 963, 963, 963, 963, 963, 963, 963, 876, 876, 876, 876, 796, 658, 724, 658, 724, 658, 658, 658, 658, 658, 658, 658, 658, 658, 658, 658, 658, 336, 658, 658, 592, 336, 336};
 #endif
-  const long nblow = sizeof(low)/sizeof(*low);
+  const long nblow = numberof(low);
   pari_sp av;
   long ex, vx;
   GEN z;
@@ -720,6 +752,33 @@ ZXX_max_lg(GEN x)
   return prec;
 }
 
+GEN
+ZXX_Z_mul(GEN y, GEN x)
+{
+  long i, l = lg(y);
+  GEN z = cgetg(l,t_POL); z[1] = y[1];
+  for(i=2; i<l; i++)
+    if(typ(gel(y,i))==t_INT)
+      gel(z,i) = mulii(gel(y,i),x);
+    else
+      gel(z,i) = ZX_Z_mul(gel(y,i),x);
+  return z;
+}
+
+GEN
+ZXX_Z_add_shallow(GEN x, GEN y)
+{
+  long i, l = lg(x);
+  GEN z, a;
+  if (signe(x)==0) return scalarpol(y,varn(x));
+  z = cgetg(l,t_POL); z[1] = x[1];
+  a = gel(x,2);
+  gel(z, 2) = typ(a)==t_INT? addii(a,y): ZX_Z_add(a,y);
+  for(i=3; i<l; i++)
+    gel(z,i) = gel(x,i);
+  return z;
+}
+
 GEN
 ZXX_Z_divexact(GEN y, GEN x)
 {
