@@ -872,7 +872,7 @@ tests_OK(GEN a, GEN nfa, GEN b, GEN nfb, long fliso)
 
   if (m <= 0) pari_err_IRREDPOL("nfisincl",a);
   if (n <= 0) pari_err_IRREDPOL("nfisincl",b);
-  q = m / n; /* relative degree */
+  q = n / m; /* relative degree */
   if (fliso) { if (n != m) return 0; } else { if (n % m) return 0; }
   if (m == 1) return 1;
 
@@ -934,24 +934,11 @@ nfiso0(GEN a, GEN b, long fliso)
   if (fliso && nfa && !nfb) { swap(a,b); nfb = nfa; nfa = NULL; }
   if (!tests_OK(a, nfa, b, nfb, fliso)) { avma = av; return gen_0; }
 
-  if (nfb) lb = gen_1; else b = ZX_Q_normalize(b,&lb);
-  if (nfa) la = gen_1; else a = ZX_Q_normalize(a,&la);
+  if (nfb) lb = gen_1; else nfb = b = ZX_Q_normalize(b,&lb);
+  if (nfa) la = gen_1; else nfa = a = ZX_Q_normalize(a,&la);
   vb = varn(b); newvar = (varncmp(vb,varn(a)) <= 0);
   if (newvar) { a = leafcopy(a); setvarn(a, fetch_var_higher()); }
-  if (nfb)
-    y = lift_shallow(nfroots(nfb,a));
-  else
-  {
-    y = gel(polfnf(a,b),1); lx = lg(y);
-    for (i=1; i<lx; i++)
-    {
-      GEN t = gel(y,i);
-      if (degpol(t) != 1) { setlg(y,i); break; }
-      gel(y,i) = gneg_i(lift_shallow(gel(t,2)));
-    }
-    settyp(y, t_VEC);
-    gen_sort_inplace(y, (void*)&cmp_RgX, &cmp_nodata, NULL);
-  }
+  y = lift_shallow(nfroots(nfb,a));
   if (newvar) (void)delete_var();
   lx = lg(y); if (lx==1) { avma=av; return gen_0; }
   for (i=1; i<lx; i++)
@@ -1491,8 +1478,19 @@ set_LLL_basis(nfmaxord_t *S, GEN *pro, double DELTA)
   S->basden = get_bas_den(B);
 }
 
+/* as abscmpii but ensure |x| is picked rather than -|x| when |x| = |y|
+ * (make output canonical) */
 static int
-cmp_abs_ZX(GEN x, GEN y) { return gen_cmp_RgX((void*)&abscmpii, x, y); }
+myabscmpii(GEN x, GEN y)
+{
+  long sx, i = abscmpii(x,y);
+  if (i) return i;
+  sx = signe(x);
+  if (!sx || signe(y) == sx) return 0;
+  return (sx > 0 ? -1: 1);
+}
+static int
+cmp_abs_ZX(GEN x, GEN y) { return gen_cmp_RgX((void*)&myabscmpii, x, y); }
 /* current best: ZX x of discriminant *dx, is ZX y better than x ?
  * (if so update *dx) */
 static int
@@ -1518,7 +1516,8 @@ nfpolred(nfmaxord_t *S, GEN *pro)
 
   if (n == 1) {
     S->T = deg1pol_shallow(gen_1, gen_m1, v);
-    *pro = NULL; return pol_1(v);
+    *pro = NULL;
+    return scalarpol_shallow(negi(gel(x,2)), v);
   }
   polredbest_aux(S, pro, &x, &dx, &b);
   if (x == S->T) return NULL; /* no improvement */
@@ -2472,7 +2471,7 @@ polredabs0(GEN x, long flag)
         GEN w = gel(vw,2);
         for (i = 1; i < l; i++)
           w = ZV_union_shallow(w, gel(Z_factor(gel(v,i)),1));
-        nfinit_basic(&S, mkvec2(x,w));
+        nfinit_basic(&S, mkvec2(S.T0,w));
       }
     }
     v = polredabs_aux(&S, &u);
