@@ -44,14 +44,14 @@ static GEN
 sliding_window_powu(GEN x, ulong n, long e, void *E, GEN (*sqr)(void*,GEN),
                                                      GEN (*mul)(void*,GEN,GEN))
 {
-  pari_sp av, lim;
+  pari_sp av;
   long i, l = expu(n), u = (1UL<<(e-1));
   long w, v;
   GEN tab = cgetg(1+u, t_VEC);
   GEN x2 = sqr(E, x), z = NULL, tw;
   gel(tab, 1) = x;
   for (i=2; i<=u; i++) gel(tab,i) = mul(E, gel(tab,i-1), x2);
-  av = avma; lim = stack_lim(av, 1);
+  av = avma;
   while (l>=0)
   {
     if (e > l+1) e = l+1;
@@ -65,7 +65,7 @@ sliding_window_powu(GEN x, ulong n, long e, void *E, GEN (*sqr)(void*,GEN),
     for (i=1; i<=v; i++) z = sqr(E, z);
     while (l>=0)
     {
-      if (low_stack(lim, stack_lim(av,1)))
+      if (gc_needed(av,1))
       {
         if (DEBUGMEM>1) pari_warn(warnmem,"sliding_window_powu (%ld)", l);
         z = gerepilecopy(av, z);
@@ -83,14 +83,14 @@ static GEN
 sliding_window_pow(GEN x, GEN n, long e, void *E, GEN (*sqr)(void*,GEN),
                                                   GEN (*mul)(void*,GEN,GEN))
 {
-  pari_sp av, lim;
+  pari_sp av;
   long i, l = expi(n), u = (1UL<<(e-1));
   long w, v;
   GEN tab = cgetg(1+u, t_VEC);
   GEN x2 = sqr(E, x), z = NULL, tw;
   gel(tab, 1) = x;
   for (i=2; i<=u; i++) gel(tab,i) = mul(E, gel(tab,i-1), x2);
-  av = avma; lim = stack_lim(av, 1);
+  av = avma;
   while (l>=0)
   {
     if (e > l+1) e = l+1;
@@ -104,7 +104,7 @@ sliding_window_pow(GEN x, GEN n, long e, void *E, GEN (*sqr)(void*,GEN),
     for (i=1; i<=v; i++) z = sqr(E, z);
     while (l>=0)
     {
-      if (low_stack(lim, stack_lim(av,1)))
+      if (gc_needed(av,1))
       {
         if (DEBUGMEM>1) pari_warn(warnmem,"sliding_window_pow (%ld)", l);
         z = gerepilecopy(av, z);
@@ -121,11 +121,11 @@ static GEN
 leftright_binary_powu(GEN x, ulong n, void *E, GEN (*sqr)(void*,GEN),
                                               GEN (*mul)(void*,GEN,GEN))
 {
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma;
   GEN  y;
   int j;
 
-  if (n == 1) return gcopy(x);
+  if (n == 1) return x;
   y = x; j = 1+bfffo(n);
   /* normalize, i.e set highest bit to 1 (we know n != 0) */
   n<<=j; j = BITS_IN_LONG-j;
@@ -134,7 +134,7 @@ leftright_binary_powu(GEN x, ulong n, void *E, GEN (*sqr)(void*,GEN),
   {
     y = sqr(E,y);
     if (n & HIGHBIT) y = mul(E,y,x); /* first bit set: multiply by base */
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"leftright_powu (%d)", j);
       y = gerepilecopy(av, y);
@@ -148,7 +148,7 @@ gen_powu_i(GEN x, ulong n, void *E, GEN (*sqr)(void*,GEN),
                                     GEN (*mul)(void*,GEN,GEN))
 {
   long l;
-  if (n == 1) return gcopy(x);
+  if (n == 1) return x;
   l = expu(n);
   if (l<=8)
     return leftright_binary_powu(x, n, E, sqr, mul);
@@ -170,7 +170,7 @@ gen_pow_i(GEN x, GEN n, void *E, GEN (*sqr)(void*,GEN),
                                  GEN (*mul)(void*,GEN,GEN))
 {
   long l, e;
-  if (lgefint(n)==3) return gen_powu_i(x,(ulong)n[2],E,sqr,mul);
+  if (lgefint(n)==3) return gen_powu_i(x, uel(n,2), E, sqr, mul);
   l = expi(n);
   if      (l<=64)  e = 3;
   else if (l<=160) e = 4;
@@ -193,11 +193,11 @@ GEN
 gen_powu_fold_i(GEN x, ulong n, void *E, GEN  (*sqr)(void*,GEN),
                                          GEN (*msqr)(void*,GEN))
 {
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma;
   GEN y;
   int j;
 
-  if (n == 1) return gcopy(x);
+  if (n == 1) return x;
   y = x; j = 1+bfffo(n);
   /* normalize, i.e set highest bit to 1 (we know n != 0) */
   n<<=j; j = BITS_IN_LONG-j;
@@ -206,7 +206,7 @@ gen_powu_fold_i(GEN x, ulong n, void *E, GEN  (*sqr)(void*,GEN),
   {
     if (n & HIGHBIT) y = msqr(E,y); /* first bit set: multiply by base */
     else y = sqr(E,y);
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"gen_powu_fold (%d)", j);
       y = gerepilecopy(av, y);
@@ -235,11 +235,17 @@ gen_pow_fold_i(GEN x, GEN N, void *E, GEN (*sqr)(void*,GEN),
     GEN nd = int_MSW(N), y = x;
     ulong n = *nd;
     long i;
-    int j = 1+bfffo(n);
-    pari_sp av = avma, lim = stack_lim(av, 1);
+    int j;
+    pari_sp av = avma;
 
-    /* normalize, i.e set highest bit to 1 (we know n != 0) */
-    n<<=j; j = BITS_IN_LONG-j;
+    if (n == 1)
+      j = 0;
+    else
+    {
+      j = 1+bfffo(n); /* < BIL */
+      /* normalize, i.e set highest bit to 1 (we know n != 0) */
+      n <<= j; j = BITS_IN_LONG - j;
+    }
     /* first bit is now implicit */
     for (i=ln-2;;)
     {
@@ -247,7 +253,7 @@ gen_pow_fold_i(GEN x, GEN N, void *E, GEN (*sqr)(void*,GEN),
       {
         if (n & HIGHBIT) y = msqr(E,y); /* first bit set: multiply by base */
         else y = sqr(E,y);
-        if (low_stack(lim, stack_lim(av,1)))
+        if (gc_needed(av,1))
         {
           if (DEBUGMEM>1) pari_warn(warnmem,"gen_pow_fold (%d)", j);
           y = gerepilecopy(av, y);
@@ -286,6 +292,61 @@ gen_powers(GEN x, long l, int use_sqr, void *E, GEN (*sqr)(void*,GEN),
   return V;
 }
 
+GEN
+producttree_scheme(long n)
+{
+  GEN v, w;
+  long i, j, k, u, l;
+  if (n<=2) return mkvecsmall(n);
+  u = expu(n-1);
+  v = cgetg(n+1,t_VECSMALL);
+  w = cgetg(n+1,t_VECSMALL);
+  v[1] = n; l = 1;
+  for (i=1; i<=u; i++)
+  {
+    for(j=1, k=1; j<=l; j++, k+=2)
+    {
+      long vj = v[j], v2 = vj>>1;
+      w[k]    = vj-v2;
+      w[k+1]  = v2;
+    }
+    swap(v,w); l<<=1;
+  }
+  fixlg(v, l+1);
+  avma = (pari_sp) v;
+  return v;
+}
+
+GEN
+gen_product(GEN x, void *data, GEN (*mul)(void *,GEN,GEN))
+{
+  pari_sp ltop;
+  long i,k,lx = lg(x),lv;
+  pari_timer ti;
+  GEN v;
+  if (DEBUGLEVEL>7) timer_start(&ti);
+
+  if (lx == 1) return gen_1;
+  if (lx == 2) return gcopy(gel(x,1));
+  x = leafcopy(x); k = lx;
+  v = producttree_scheme(lx-1); lv = lg(v);
+  ltop = avma;
+  for (k=1, i=1; k<lv; i+=v[k++])
+    gel(x,k) = v[k]==1 ? gel(x,i): mul(data,gel(x,i),gel(x,i+1));
+  while (k > 2)
+  {
+    if (DEBUGLEVEL>7)
+      timer_printf(&ti,"gen_product: remaining objects %ld",k-1);
+    lx = k; k = 1;
+    for (i=1; i<lx-1; i+=2)
+      gel(x,k++) = mul(data,gel(x,i),gel(x,i+1));
+    if (i < lx) gel(x,k++) = gel(x,i);
+    if (gc_needed(ltop,1))
+      gerepilecoeffs(ltop,x+1,k-1);
+  }
+  return gel(x,1);
+}
+
 /***********************************************************************/
 /**                                                                   **/
 /**                    DISCRETE LOGARITHM                             **/
@@ -313,7 +374,7 @@ iter_rho(GEN x, GEN g, GEN q, GEN A, ulong h, void *E, const struct bb_group *gr
 static GEN
 gen_Pollard_log(GEN x, GEN g, GEN q, void *E, const struct bb_group *grp)
 {
-  pari_sp av=avma, lim=stack_lim(av,2);
+  pari_sp av=avma;
   GEN A, B, l, sqrt4q = sqrti(shifti(q,4));
   ulong i, h = 0, imax = itou_or_0(sqrt4q);
   if (!imax) imax = ULONG_MAX;
@@ -332,7 +393,7 @@ gen_Pollard_log(GEN x, GEN g, GEN q, void *E, const struct bb_group *grp)
       A = iter_rho(x, g, q, A, h, E, grp);
       B = iter_rho(x, g, q, B, h, E, grp);
       B = iter_rho(x, g, q, B, h, E, grp);
-      if (low_stack(lim, stack_lim(av,2)))
+      if (gc_needed(av,2))
       {
         if(DEBUGMEM>1) pari_warn(warnmem,"gen_Pollard_log");
         gerepileall(av, 2, &A, &B);
@@ -347,25 +408,82 @@ gen_Pollard_log(GEN x, GEN g, GEN q, void *E, const struct bb_group *grp)
   return gerepileuptoint(av, l);
 }
 
-/*Generic Shanks baby-step/giant-step algorithm*/
-static GEN
-gen_Shanks_log(GEN x, GEN g0,GEN q, void *E, const struct bb_group *grp)
+/* compute a hash of g^(i-1), 1<=i<=n. Return [sorted hash, perm, g^-n] */
+GEN
+gen_Shanks_init(GEN g, long n, void *E, const struct bb_group *grp)
 {
-  pari_sp av=avma,av1,lim;
-  long lbaby,i,k;
-  GEN p1,table,giant,perm,g0inv;
+  GEN p1 = g, G, perm, table = cgetg(n+1,t_VECSMALL);
+  pari_sp av=avma;
+  long i;
+  table[1] = grp->hash(grp->pow(E,g,gen_0));
+  for (i=2; i<=n; i++)
+  {
+    table[i] = grp->hash(p1);
+    p1 = grp->mul(E,p1,g);
+    if (gc_needed(av,2))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_log, baby = %ld", i);
+      p1 = gerepileupto(av, p1);
+    }
+  }
+  G = gerepileupto(av, grp->pow(E,p1,gen_m1)); /* g^-n */
+  perm = vecsmall_indexsort(table);
+  table = vecsmallpermute(table,perm);
+  return mkvec4(table,perm,g,G);
+}
+/* T from gen_Shanks_init(g,n). Return v < n*N such that x = g^v or NULL */
+GEN
+gen_Shanks(GEN T, GEN x, ulong N, void *E, const struct bb_group *grp)
+{
+  pari_sp av=avma;
+  GEN table = gel(T,1), perm = gel(T,2), g = gel(T,3), G = gel(T,4);
+  GEN p1 = x;
+  long n = lg(table)-1;
+  ulong k;
+  for (k=0; k<N; k++)
+  { /* p1 = x G^k, G = g^-n */
+    long h = grp->hash(p1), i = zv_search(table, h);
+    if (i)
+    {
+      do i--; while (i && table[i] == h);
+      for (i++; i <= n && table[i] == h; i++)
+      {
+        GEN v = addiu(muluu(n,k), perm[i]-1);
+        if (grp->equal(grp->pow(E,g,v),x)) return gerepileuptoint(av,v);
+        if (DEBUGLEVEL)
+          err_printf("gen_Shanks_log: false positive %lu, %lu\n", k,h);
+      }
+    }
+    p1 = grp->mul(E,p1,G);
+    if (gc_needed(av,2))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_log, k = %lu", k);
+      p1 = gerepileupto(av, p1);
+    }
+  }
+  return NULL;
+}
+/* Generic Shanks baby-step/giant-step algorithm. Return log_g(x), ord g = q.
+ * One-shot: use gen_Shanks_init/log if many logs are desired; early abort
+ * if log < sqrt(q) */
+static GEN
+gen_Shanks_log(GEN x, GEN g, GEN q, void *E, const struct bb_group *grp)
+{
+  pari_sp av=avma, av1;
+  long lbaby, i, k;
+  GEN p1, table, giant, perm, ginv;
   p1 = sqrti(q);
-  if (cmpiu(p1,LGBITS) >= 0)
-    pari_err_OVERFLOW("gen_Shanks_log() [order too large]");
+  if (abscmpiu(p1,LGBITS) >= 0)
+    pari_err_OVERFLOW("gen_Shanks_log [order too large]");
   lbaby = itos(p1)+1; table = cgetg(lbaby+1,t_VECSMALL);
-  g0inv = grp->pow(E,g0,gen_m1);
-  av1 = avma; lim=stack_lim(av1,2);
+  ginv = grp->pow(E,g,gen_m1);
+  av1 = avma;
   for (p1=x, i=1;;i++)
   {
     if (grp->equal1(p1)) { avma = av; return stoi(i-1); }
     table[i] = grp->hash(p1); if (i==lbaby) break;
-    p1 = grp->mul(E,p1,g0inv);
-    if (low_stack(lim, stack_lim(av1,2)))
+    p1 = grp->mul(E,p1,ginv);
+    if (gc_needed(av1,2))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_log, baby = %ld", i);
       p1 = gerepileupto(av1, p1);
@@ -373,29 +491,26 @@ gen_Shanks_log(GEN x, GEN g0,GEN q, void *E, const struct bb_group *grp)
   }
   p1 = giant = gerepileupto(av1, grp->mul(E,x,grp->pow(E, p1, gen_m1)));
   perm = vecsmall_indexsort(table);
-  table = perm_mul(table,perm);
-  av1 = avma; lim=stack_lim(av1,2);
+  table = vecsmallpermute(table,perm);
+  av1 = avma;
   for (k=1; k<= lbaby; k++)
   {
-    long h = grp->hash(p1);
-    long i = zv_search(table, h);
+    long h = grp->hash(p1), i = zv_search(table, h);
     if (i)
     {
       while (table[i] == h && i) i--;
       for (i++; i <= lbaby && table[i] == h; i++)
       {
-        GEN v=addis(mulss(lbaby-1,k),perm[i]-1);
-        if (grp->equal(grp->pow(E,g0,v),x))
-          return gerepileuptoint(av,v);
-        else if (DEBUGLEVEL)
-          err_printf("gen_Shanks_log: false positive, giant = %ld: %lu: %Ps\n", k,h,p1);
+        GEN v = addiu(mulss(lbaby-1,k),perm[i]-1);
+        if (grp->equal(grp->pow(E,g,v),x)) return gerepileuptoint(av,v);
+        if (DEBUGLEVEL)
+          err_printf("gen_Shanks_log: false positive %ld, %lu\n", k,h);
       }
     }
     p1 = grp->mul(E,p1,giant);
-
-    if (low_stack(lim, stack_lim(av1,2)))
+    if (gc_needed(av1,2))
     {
-      if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_log, giant = %ld", k);
+      if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_log, k = %ld", k);
       p1 = gerepileupto(av1, p1);
     }
   }
@@ -418,7 +533,7 @@ gen_plog(GEN x, GEN g, GEN p, void *E, const struct bb_group *grp)
 }
 
 GEN
-dlog_get_ordfa(GEN o)
+get_arith_ZZM(GEN o)
 {
   if (!o) return NULL;
   switch(typ(o))
@@ -437,7 +552,7 @@ dlog_get_ordfa(GEN o)
   return NULL; /* not reached */
 }
 GEN
-dlog_get_ord(GEN o)
+get_arith_Z(GEN o)
 {
   if (!o) return NULL;
   switch(typ(o))
@@ -477,7 +592,7 @@ gen_PH_log(GEN a, GEN g, GEN ord, void *E, const struct bb_group *grp)
     GEN e = grp->easylog(E, a, g, ord);
     if (e) return e;
   }
-  v = dlog_get_ordfa(ord);
+  v = get_arith_ZZM(ord);
   ord= gel(v,1);
   fa = gel(v,2);
   ex = gel(fa,2);
@@ -538,7 +653,7 @@ gen_order(GEN a, GEN o, void *E, const struct bb_group *grp)
   long i, l;
   GEN m;
 
-  m = dlog_get_ordfa(o);
+  m = get_arith_ZZM(o);
   if (!m) pari_err_TYPE("gen_order [missing order]",a);
   o = gel(m,1);
   m = gel(m,2); l = lgcols(m);
@@ -577,7 +692,7 @@ gen_factored_order(GEN a, GEN o, void *E, const struct bb_group *grp)
   long i, l, ind;
   GEN m, F, P;
 
-  m = dlog_get_ordfa(o);
+  m = get_arith_ZZM(o);
   if (!m) pari_err_TYPE("gen_factored_order [missing order]",a);
   o = gel(m,1);
   m = gel(m,2); l = lgcols(m);
@@ -610,8 +725,8 @@ gen_factored_order(GEN a, GEN o, void *E, const struct bb_group *grp)
       ind++;
     }
   }
-  setlg(P, ind);
-  setlg(F, ind);
+  setlg(P, ind); P = vecreverse(P);
+  setlg(F, ind); F = vecreverse(F);
   return gerepilecopy(av, mkvec2(o, mkmat2(P,F)));
 }
 
@@ -686,12 +801,12 @@ gen_lgener(GEN l, long e, GEN r,GEN *zeta, void *E, const struct bb_group *grp)
 GEN
 gen_gener(GEN o, void *E, const struct bb_group *grp)
 {
-  pari_sp ltop = avma, av, lim;
+  pari_sp ltop = avma, av;
   long i, lpr;
   GEN F, N, pr, z=NULL;
-  F = dlog_get_ordfa(o);
+  F = get_arith_ZZM(o);
   N = gel(F,1); pr = gel(F,2); lpr = lgcols(pr);
-  av = avma; lim = stack_lim(av,2);
+  av = avma;
 
   for (i = 1; i < lpr; i++)
   {
@@ -700,7 +815,7 @@ gen_gener(GEN o, void *E, const struct bb_group *grp)
     GEN r = diviiexact(N,powis(l,e));
     GEN zetan, zl = gen_lgener(l,e,r,&zetan,E,grp);
     z = i==1 ? zl: grp->mul(E,z,zl);
-    if (low_stack(lim, stack_lim(av,2)))
+    if (gc_needed(av,2))
     { /* n can have lots of prime factors*/
       if(DEBUGMEM>1) pari_warn(warnmem,"gen_gener");
       z = gerepileupto(av, z);
@@ -718,7 +833,7 @@ static GEN
 gen_Shanks_sqrtl(GEN a, GEN l, long e, GEN r, GEN y, GEN m,void *E,
                  const struct bb_group *grp)
 {
-  pari_sp av = avma,lim;
+  pari_sp av = avma;
   long k;
   GEN p1, u1, u2, v, w, z, dl;
 
@@ -726,7 +841,6 @@ gen_Shanks_sqrtl(GEN a, GEN l, long e, GEN r, GEN y, GEN m,void *E,
   v = grp->pow(E,a,u2);
   w = grp->pow(E,v,l);
   w = grp->mul(E,w,grp->pow(E,a,gen_m1));
-  lim = stack_lim(av,1);
   while (!grp->equal1(w))
   {
     k = 0;
@@ -746,7 +860,7 @@ gen_Shanks_sqrtl(GEN a, GEN l, long e, GEN r, GEN y, GEN m,void *E,
     v = grp->mul(E,p1,v);
     y = grp->pow(E,p1,l);
     w = grp->mul(E,y,w);
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_sqrtl");
       gerepileall(av,4, &y,&v,&w,&m);
@@ -765,7 +879,7 @@ gen_Shanks_sqrtl(GEN a, GEN l, long e, GEN r, GEN y, GEN m,void *E,
 GEN
 gen_Shanks_sqrtn(GEN a, GEN n, GEN q, GEN *zetan, void *E, const struct bb_group *grp)
 {
-  pari_sp ltop = avma, lim;
+  pari_sp ltop = avma;
   GEN m, u1, u2, z;
   int is_1;
 
@@ -779,7 +893,6 @@ gen_Shanks_sqrtn(GEN a, GEN n, GEN q, GEN *zetan, void *E, const struct bb_group
 
   m = bezout(n,q,&u1,&u2);
   z = grp->pow(E,a,gen_0);
-  lim = stack_lim(ltop,1);
   if (!is_pm1(m))
   {
     GEN F = Z_factor(m);
@@ -800,7 +913,7 @@ gen_Shanks_sqrtn(GEN a, GEN n, GEN q, GEN *zetan, void *E, const struct bb_group
           if (!a) { avma = ltop; return NULL;}
         } while (--j);
       }
-      if (low_stack(lim, stack_lim(ltop,1)))
+      if (gc_needed(ltop,1))
       { /* n can have lots of prime factors*/
         if(DEBUGMEM>1) pari_warn(warnmem,"gen_Shanks_sqrtn");
         gerepileall(av1, zetan? 2: 1, &a, &z);
@@ -889,7 +1002,7 @@ gen_ellgens(GEN D1, GEN d2, GEN m, void *E, const struct bb_group *grp,
   pari_sp ltop = avma, av;
   GEN F, d1, dm;
   GEN P, Q, d, s;
-  F = dlog_get_ordfa(D1);
+  F = get_arith_ZZM(D1);
   d1 = gel(F, 1), dm =  diviiexact(d1,m);
   av = avma;
   do
