@@ -504,24 +504,7 @@ gprec(GEN x, long l)
       gel(y,1) = gprec(gel(x,1),l);
       gel(y,2) = gprec(gel(x,2),l);
       break;
-    case t_PADIC:
-      if (!signe(gel(x,4))) return zeropadic(gel(x,2), l+precp(x));
-      y=cgetg(5,t_PADIC);
-      y[1]=x[1]; setprecp(y,l);
-      gel(y,2) = icopy(gel(x,2));
-      gel(y,3) = powiu(gel(x,2),l);
-      gel(y,4) = modii(gel(x,4), gel(y,3));
-      break;
-
-    case t_SER:
-      if (lg(x) == 2) return zeroser(varn(x), l);
-      y=cgetg(l+2,t_SER); y[1]=x[1]; l++; i=l;
-      lx = lg(x);
-      if (l>=lx)
-        for ( ; i>=lx; i--) gel(y,i) = gen_0;
-      for ( ; i>=2; i--) gel(y,i) = gcopy(gel(x,i));
-      break;
-   case t_POL:
+    case t_POL: case t_SER:
       y = cgetg_copy(x, &lx); y[1] = x[1];
       for (i=2; i<lx; i++) gel(y,i) = gprec(gel(x,i),l);
       break;
@@ -604,23 +587,44 @@ gprec_wtrunc(GEN x, long pr)
 /********************************************************************/
 /**                  LAPLACE TRANSFORM (OF A SERIES)               **/
 /********************************************************************/
-GEN
-laplace(GEN x)
+static GEN
+serlaplace(GEN x)
 {
-  pari_sp av = avma;
   long i, l = lg(x), e = valp(x);
-  GEN y, t;
-
-  if (typ(x) != t_SER) pari_err_TYPE("laplace",x);
+  GEN t, y = cgetg(l,t_SER);
   if (e < 0) pari_err_DOMAIN("laplace","valuation","<",gen_0,stoi(e));
-  y = cgetg(l,t_SER);
   t = mpfact(e); y[1] = x[1];
   for (i=2; i<l; i++)
   {
     gel(y,i) = gmul(t, gel(x,i));
     e++; t = mului(e,t);
   }
-  return gerepilecopy(av,y);
+  return y;
+}
+static GEN
+pollaplace(GEN x)
+{
+  long i, e = 0, l = lg(x);
+  GEN t = gen_1, y = cgetg(l,t_POL);
+  y[1] = x[1];
+  for (i=2; i<l; i++)
+  {
+    gel(y,i) = gmul(t, gel(x,i));
+    e++; t = mului(e,t);
+  }
+  return y;
+}
+GEN
+laplace(GEN x)
+{
+  pari_sp av = avma;
+  switch(typ(x))
+  {
+    case t_POL: x = pollaplace(x); break;
+    case t_SER: x = serlaplace(x); break;
+    default: pari_err_TYPE("laplace",x);
+  }
+  return gerepilecopy(av, x);
 }
 
 /********************************************************************/
@@ -635,8 +639,12 @@ convol(GEN x, GEN y)
   if (typ(x) != t_SER) pari_err_TYPE("convol",x);
   if (typ(y) != t_SER) pari_err_TYPE("convol",y);
   if (varn(y) != vx) pari_err_VAR("convol", x,y);
-  ex = valp(x); lx = lg(x) + ex; x -= ex;
-  ey = valp(y); ly = lg(y) + ey; y -= ey;
+  ex = valp(x);
+  ey = valp(y);
+  if (ser_isexactzero(x))
+    return scalarser(gadd(RgX_get_0(x), RgX_get_0(y)), varn(x), maxss(ex,ey));
+  lx = lg(x) + ex; x -= ex;
+  ly = lg(y) + ey; y -= ey;
   /* inputs shifted: x[i] and y[i] now correspond to monomials of same degree */
   if (ly < lx) lx = ly; /* min length */
   if (ex < ey) ex = ey; /* max valuation */
@@ -663,7 +671,7 @@ dirval(GEN x)
 GEN
 dirmul(GEN x, GEN y)
 {
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma, av2;
   long nx, ny, nz, dx, dy, i, j, k;
   GEN z;
 
@@ -673,21 +681,32 @@ dirmul(GEN x, GEN y)
   dy = dirval(y); ny = lg(y)-1;
   if (ny-dy < nx-dx) { swap(x,y); lswap(nx,ny); lswap(dx,dy); }
   nz = minss(nx*dy,ny*dx);
+  y = RgV_kill0(y);
+  av2 = avma;
   z = zerovec(nz);
   for (j=dx; j<=nx; j++)
   {
     GEN c = gel(x,j);
     if (gequal0(c)) continue;
     if (gequal1(c))
-      for (k=dy,i=j*dy; i<=nz; i+=j,k++) gel(z,i) = gadd(gel(z,i),gel(y,k));
-    else if (gequalm1(c))
-      for (k=dy,i=j*dy; i<=nz; i+=j,k++) gel(z,i) = gsub(gel(z,i),gel(y,k));
-    else
-      for (k=dy,i=j*dy; i<=nz; i+=j,k++) gel(z,i) = gadd(gel(z,i),gmul(c,gel(y,k)));
-    if (low_stack(lim, stack_lim(av,1)))
     {
-      if (DEBUGLEVEL) err_printf("doubling stack in dirmul\n");
-      z = gerepilecopy(av,z);
+      for (k=dy,i=j*dy; i<=nz; i+=j,k++)
+        if (gel(y,k)) gel(z,i) = gadd(gel(z,i),gel(y,k));
+    }
+    else if (gequalm1(c))
+    {
+      for (k=dy,i=j*dy; i<=nz; i+=j,k++)
+        if (gel(y,k)) gel(z,i) = gsub(gel(z,i),gel(y,k));
+    }
+    else
+    {
+      for (k=dy,i=j*dy; i<=nz; i+=j,k++)
+        if (gel(y,k)) gel(z,i) = gadd(gel(z,i),gmul(c,gel(y,k)));
+    }
+    if (gc_needed(av2,3))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"dirmul, %ld/%ld",j,nx);
+      z = gerepilecopy(av2,z);
     }
   }
   return gerepilecopy(av,z);
@@ -696,30 +715,49 @@ dirmul(GEN x, GEN y)
 GEN
 dirdiv(GEN x, GEN y)
 {
-  pari_sp av = avma;
-  long nx,ny,nz,dx,dy,i,j;
-  GEN z,p1;
+  pari_sp av = avma, av2;
+  long nx,ny,nz, dx,dy, i,j,k;
+  GEN p1;
 
   if (typ(x)!=t_VEC) pari_err_TYPE("dirdiv",x);
   if (typ(y)!=t_VEC) pari_err_TYPE("dirdiv",y);
   dx = dirval(x); nx = lg(x)-1;
   dy = dirval(y); ny = lg(y)-1;
   if (dy != 1 || !ny) pari_err_INV("dirdiv",y);
-  nz = minss(nx,ny*dx); p1 = gel(y,1);
-  if (!gequal1(p1)) { y = gdiv(y,p1); x = gdiv(x,p1); } else x = leafcopy(x);
-  z = zerovec(nz);
+  nz = minss(nx,ny*dx);
+  p1 = gel(y,1);
+  if (gequal1(p1)) p1 = NULL; else y = gdiv(y,p1);
+  y = RgV_kill0(y);
+  av2 = avma;
+  x = p1 ? gdiv(x,p1): leafcopy(x);
+  for (j=1; j<dx; j++) gel(x,j) = gen_0;
+  setlg(x,nz+1);
   for (j=dx; j<=nz; j++)
   {
-    GEN c = gel(x,j); gel(z,j) = c;
+    GEN c = gel(x,j);
     if (gequal0(c)) continue;
     if (gequal1(c))
-      for (i=j+j; i<=nz; i+=j) gel(x,i) = gsub(gel(x,i),gel(y,i/j));
+    {
+      for (i=j+j,k=2; i<=nz; i+=j,k++)
+        if (gel(y,k)) gel(x,i) = gsub(gel(x,i),gel(y,k));
+    }
     else if (gequalm1(c))
-      for (i=j+j; i<=nz; i+=j) gel(x,i) = gadd(gel(x,i),gel(y,i/j));
+    {
+      for (i=j+j,k=2; i<=nz; i+=j,k++)
+        if (gel(y,k)) gel(x,i) = gadd(gel(x,i),gel(y,k));
+    }
     else
-      for (i=j+j; i<=nz; i+=j) gel(x,i) = gsub(gel(x,i),gmul(c,gel(y,i/j)));
+    {
+      for (i=j+j,k=2; i<=nz; i+=j,k++)
+        if (gel(y,k)) gel(x,i) = gsub(gel(x,i),gmul(c,gel(y,k)));
+    }
+    if (gc_needed(av2,3))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"dirdiv, %ld/%ld",j,nz);
+      x = gerepilecopy(av2,x);
+    }
   }
-  return gerepilecopy(av,z);
+  return gerepilecopy(av,x);
 }
 
 /*******************************************************************/
@@ -783,7 +821,7 @@ binomial(GEN n, long k)
     {
       y = cgetg(k+1,t_VEC);
       for (i=1; i<=k; i++) gel(y,i) = subis(n,i-1);
-      y = divide_conquer_prod(y,mulii);
+      y = ZV_prod(y);
     }
     y = diviiexact(y, mpfact(k));
     return gerepileuptoint(av, y);
@@ -797,9 +835,7 @@ binomial(GEN n, long k)
 
   y = cgetg(k+1,t_VEC);
   for (i=1; i<=k; i++) gel(y,i) = gsubgs(n,i-1);
-  y = divide_conquer_prod(y,gmul);
-  y = gdiv(y, mpfact(k));
-  return gerepileupto(av, y);
+  return gerepileupto(av, gdiv(RgV_prod(y), mpfact(k)));
 }
 
 /* Assume n >= 0, return bin, bin[k+1] = binomial(n, k) */
@@ -829,7 +865,7 @@ vecbinome(long n)
 GEN
 stirling2(ulong n, ulong m)
 {
-  pari_sp av = avma, lim = stack_lim(av, 2);
+  pari_sp av = avma;
   GEN s, bmk;
   ulong k;
   if (n==0) return (m == 0)? gen_1: gen_0;
@@ -845,7 +881,7 @@ stirling2(ulong n, ulong m)
     c = odd(m)? subii(mkn,kn): addii(mkn,kn);
     c = mulii(bmk, c);
     s = odd(k)? subii(s, c): addii(s, c);
-    if (low_stack(lim, stack_lim(av,2)))
+    if (gc_needed(av,2))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"stirling2");
       gerepileall(av, 2, &s, &bmk);
@@ -909,57 +945,75 @@ stirling(long n, long m, long flag)
 /**                          PERMUTATIONS                             **/
 /***********************************************************************/
 GEN
-numtoperm(long n, GEN x)
+Z_to_perm(long n, GEN x)
 {
-  pari_sp av, lim;
+  pari_sp av;
   ulong i, r;
-  GEN v;
-
-  if (n < 0) pari_err_DOMAIN("numtoperm", "n", "<", gen_0, stoi(n));
-  if (typ(x) != t_INT) pari_err_TYPE("numtoperm",x);
-  v = cgetg(n+1, t_VEC); if (n==0) return v;
-  v[n] = 1; av = avma; lim = stack_lim(av,2);
+  GEN v = cgetg(n+1, t_VECSMALL);
+  if (n==0) return v;
+  uel(v,n) = 1; av = avma;
   if (signe(x) <= 0) x = modii(x, mpfact(n));
   for (r=n-1; r>=1; r--)
   {
     ulong a;
-    x = diviu_rem(x, n+1-r,&a);
+    x = diviu_rem(x, n+1-r, &a);
     for (i=r+1; i<=(ulong)n; i++)
-      if((ulong)v[i]>a) v[i]++;
-    v[r] = a+1;
-    if (low_stack(lim, stack_lim(av,2)))
-      x = gerepileuptoint(av, x);
+      if (uel(v,i) > a) uel(v,i)++;
+    uel(v,r) = a+1;
   }
-  avma = av;
-  for (i=1; i<=(ulong)n; i++) gel(v,i) = utoipos(v[i]);
+  avma = av; return v;
+}
+GEN
+numtoperm(long n, GEN x)
+{
+  long i;
+  GEN v;
+
+  if (n < 0) pari_err_DOMAIN("numtoperm", "n", "<", gen_0, stoi(n));
+  if (typ(x) != t_INT) pari_err_TYPE("numtoperm",x);
+  v = Z_to_perm(n, x); settyp(v, t_VEC);
+  for (i = 1; i <= n; i++) gel(v,i) = utoipos(uel(v,i));
   return v;
 }
 
+/* destroys v */
+static GEN
+perm_to_Z_inplace(GEN v)
+{
+  long l = lg(v), i, r;
+  GEN x = gen_0;
+  for (i = 1; i < l; i++)
+  {
+    long vi = v[i];
+    if (vi <= 0) return NULL;
+    x = i==1 ? utoi(vi-1): addiu(muliu(x,l-i), vi-1);
+    for (r = i+1; r < l; r++)
+      if (v[r] > vi) v[r]--;
+  }
+  return x;
+}
+GEN
+perm_to_Z(GEN v)
+{
+  pari_sp av = avma;
+  GEN x = perm_to_Z_inplace(leafcopy(v));
+  if (!x) pari_err_TYPE("permtonum",v);
+  return gerepileuptoint(av, x);
+}
 GEN
 permtonum(GEN p)
 {
-  long n = lg(p)-1, i, r;
-  pari_sp av = avma, av2, lim;
+  pari_sp av = avma;
   GEN v, x;
-
-  if (!is_vec_t(typ(p))) pari_err_TYPE("permtonum",p);
-  v = cgetg(n+1,t_VECSMALL);
-  for (i=1; i<=n; i++)
+  switch(typ(p))
   {
-    GEN pi = gel(p, i);
-    if (typ(pi) != t_INT) pari_err_TYPE("permtonum",pi);
-    v[i] = itos(pi);
-  }
-  x = gen_0; av2 = avma; lim = stack_lim(av2,2);
-  for (i=1; i<=n; i++)
-  {
-    long vi = v[i];
-    x = i==1 ? stoi(v[1]-1): addiu(mulis(x,n+1-i),vi-1);
-    for (r=i+1; r<=n; r++)
-      if (v[r]>vi) v[r]--;
-    if (low_stack(lim, stack_lim(av,2)))
-      x = gerepileuptoint(av2, x);
+    case t_VECSMALL: return perm_to_Z(p);
+    case t_VEC: case t_COL:
+      if (RgV_is_ZV(p)) { v = ZV_to_zv(p); break; }
+    default: pari_err_TYPE("permtonum",p); return NULL;
   }
+  x = perm_to_Z_inplace(v);
+  if (!x) pari_err_TYPE("permtonum",p);
   return gerepileuptoint(av, x);
 }
 
@@ -985,7 +1039,7 @@ polrecip(GEN x)
 GEN
 RgV_polint(GEN X, GEN Y, long v)
 {
-  pari_sp av0 = avma, av, lim;
+  pari_sp av0 = avma, av;
   GEN Q, P = NULL;
   long i, l = lg(Y);
   if (!X)
@@ -993,7 +1047,7 @@ RgV_polint(GEN X, GEN Y, long v)
     X = cgetg(l, t_VEC);
     for (i=1; i<l; i++) gel(X,i) = utoipos(i);
   }
-  Q = roots_to_pol(X, v); av = avma; lim = stack_lim(av,2);
+  Q = roots_to_pol(X, v); av = avma;
   for (i=1; i<l; i++)
   {
     GEN inv, T, dP;
@@ -1002,7 +1056,7 @@ RgV_polint(GEN X, GEN Y, long v)
     inv = ginv(poleval(T,gel(X,i)));
     dP = RgX_Rg_mul(T, gmul(gel(Y,i),inv));
     P = P? RgX_add(P, dP): dP;
-    if (low_stack(lim, stack_lim(av,2)))
+    if (gc_needed(av,2))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"FpV_polint");
       P = gerepileupto(av, P);
@@ -1011,7 +1065,7 @@ RgV_polint(GEN X, GEN Y, long v)
   if (!P) { avma = av; return zeropol(v); }
   return gerepileupto(av0, P);
 }
-/* X,Y are "spec" GEN vectors with n > 1 components ( at X[0], ... X[n-1] ) */
+/* X,Y are "spec" GEN vectors with n > 0 components ( at X[0], ... X[n-1] ) */
 GEN
 polint_i(GEN X, GEN Y, GEN x, long n, GEN *ptdy)
 {
@@ -1019,6 +1073,11 @@ polint_i(GEN X, GEN Y, GEN x, long n, GEN *ptdy)
   pari_sp av = avma;
   GEN y, c, d, dy = NULL; /* gcc -Wall */
 
+  if (n == 1)
+  {
+    if (ptdy) *ptdy = gen_0;
+    return gmul(gel(Y,0), RgX_get_1(x));
+  }
   if (!X)
   {
     X = cgetg(n+1, t_VEC);
@@ -1078,9 +1137,9 @@ NODY:
 }
 
 GEN
-polint(GEN X, GEN Y, GEN x, GEN *ptdy)
+polint(GEN X, GEN Y, GEN t, GEN *ptdy)
 {
-  long lx = lg(X);
+  long lx = lg(X), vt;
 
   if (! is_vec_t(typ(X))) pari_err_TYPE("polinterpolate",X);
   if (Y)
@@ -1093,18 +1152,27 @@ polint(GEN X, GEN Y, GEN x, GEN *ptdy)
     Y = X;
     X = NULL;
   }
-
-  if (lx <= 2)
-  {
-    if (ptdy) *ptdy = gen_0;
-    if (lx == 1) return zeropol(0);
-    Y = gel(Y,1);
-    if (gvar(Y) == 0) pari_err_PRIORITY("polinterpolate", Y, "=", 0);
-    return scalarpol(Y, 0);
+  if (ptdy) *ptdy = gen_0;
+  vt = t? gvar(t): 0;
+  if (vt != NO_VARIABLE)
+  { /* formal interpolation */
+    pari_sp av;
+    long v0, vY = gvar(Y);
+    GEN P;
+    if (X) vY = varnmax(vY, gvar(X));
+    /* shortcut */
+    if (varncmp(vY, vt) > 0 && (!t || gequalX(t))) return RgV_polint(X, Y, vt);
+    av = avma;
+    /* first interpolate in high priority variable, then substitute t */
+    v0 = fetch_var_higher();
+    P = RgV_polint(X, Y, v0);
+    P = gsubst(P, v0, t? t: pol_x(0));
+    (void)delete_var();
+    return gerepileupto(av, P);
   }
-  if (!x) return RgV_polint(X, Y, 0);
-  if (gequalX(x)) return RgV_polint(X, Y, varn(x));
-  return polint_i(X? X+1: NULL,Y+1,x,lx-1,ptdy);
+  /* numerical interpolation */
+  if (lx == 1) return RgX_get_0(t);
+  return polint_i(X? X+1: NULL,Y+1,t,lx-1,ptdy);
 }
 
 /********************************************************************/
@@ -1312,7 +1380,9 @@ static void
 init_sort(GEN *x, long *tx, long *lx)
 {
   *tx = typ(*x);
-  if (*tx == t_LIST) {
+  if (*tx == t_LIST)
+  {
+    if (list_typ(*x)!=t_LIST_RAW) pari_err_TYPE("sort",*x);
     *x = list_data(*x);
     *lx = *x? lg(*x): 1;
   } else {
@@ -1350,7 +1420,7 @@ gen_sort_uniq(GEN x, void *E, int (*cmp)(void*,GEN,GEN))
   GEN y;
 
   init_sort(&x, &tx, &lx);
-  if (lx==1) return tx == t_LIST? listcreate(): cgetg(1, tx);
+  if (lx==1) return tx == t_LIST? mklist(): cgetg(1, tx);
   y = gen_sortspec_uniq(x,lx-1,E,cmp);
   return sort_extract(x, y, tx, lg(y)); /* lg(y) <= lx */
 }
@@ -1362,7 +1432,7 @@ gen_sort(GEN x, void *E, int (*cmp)(void*,GEN,GEN))
   GEN y;
 
   init_sort(&x, &tx, &lx);
-  if (lx==1) return tx == t_LIST? listcreate(): cgetg(1, tx);
+  if (lx==1) return tx == t_LIST? mklist(): cgetg(1, tx);
   y = gen_sortspec(x,lx-1,E,cmp);
   return sort_extract(x, y, tx, lx);
 }
@@ -1449,7 +1519,7 @@ sort_function(void **E, GEN x, GEN k)
     case t_VEC: case t_COL: k = ZV_to_zv(k); break;
     case t_VECSMALL: break;
     case t_CLOSURE:
-     if (closure_arity(k) != 2)
+     if (closure_arity(k) != 2 || closure_is_variadic(k))
        pari_err_TYPE("sort_function, cmp. fun. needs exactly 2 arguments",k);
      *E = (void*)k;
      return &closurecmp;
@@ -1618,9 +1688,12 @@ gen_cmp_RgX(void *data, GEN x, GEN y)
 static int
 cmp_RgX_Rg(GEN x, GEN y)
 {
-  long lx = lg(x);
-  if (lx > 3) return  1;
-  if (lx < 3) return -1;
+  long lx = lgpol(x), ly;
+  if (lx > 1) return  1;
+  ly = gequal0(y) ? 0:1;
+  if (lx > ly) return  1;
+  if (lx < ly) return -1;
+  if (lx==0) return 0;
   return gcmp(gel(x,2), y);
 }
 int
@@ -1637,11 +1710,21 @@ cmp_RgX(GEN x, GEN y)
   return gen_cmp_RgX((void*)&gcmp,x,y);
 }
 
+int
+cmp_Flx(GEN x, GEN y)
+{
+  long i, lx = lg(x), ly = lg(y);
+  if (lx > ly) return  1;
+  if (lx < ly) return -1;
+  for (i=lx-1; i>1; i--)
+    if (uel(x,i) != uel(y,i)) return uel(x,i)<uel(y,i)? -1: 1;
+  return 0;
+}
 /********************************************************************/
 /**                   MERGE & SORT FACTORIZATIONS                  **/
 /********************************************************************/
 /* merge fx, fy two factorizations, whose 1st column is sorted in strictly
- * increasing order wrt cmp. Keep 0 exponents. */
+ * increasing order wrt cmp */
 GEN
 merge_factor(GEN fx, GEN fy, void *data, int (*cmp)(void *,GEN,GEN))
 {
@@ -1659,7 +1742,11 @@ merge_factor(GEN fx, GEN fy, void *data, int (*cmp)(void *,GEN,GEN))
     if (s < 0)
     { gel(M,m) = gel(x,ix); gel(E,m) = gel(e,ix); ix++; }
     else if (s == 0)
-    { gel(M,m) = gel(x,ix); gel(E,m) = addii(gel(e,ix), gel(f,iy)); iy++; ix++; }
+    {
+      GEN z = gel(x,ix), g = addii(gel(e,ix), gel(f,iy));
+      iy++; ix++; if (!signe(g)) continue;
+      gel(M,m) = z; gel(E,m) = g;
+    }
     else
     { gel(M,m) = gel(y,iy); gel(E,m) = gel(f,iy); iy++; }
     m++;
@@ -1707,8 +1794,8 @@ sort_factor(GEN y, void *data, int (*cmp)(void *,GEN,GEN))
   A = new_chunk(n);
   B = new_chunk(n);
   w = gen_sortspec(a, n-1, data, cmp);
-  for (i=1; i<n; i++) { long k = w[i]; A[i] = a[k]; B[i] = b[k]; }
-  for (i=1; i<n; i++) { a[i] = A[i]; b[i] = B[i]; }
+  for (i=1; i<n; i++) { long k=w[i]; gel(A,i) = gel(a,k); gel(B,i) = gel(b,k); }
+  for (i=1; i<n; i++) { gel(a,i) = gel(A,i); gel(b,i) = gel(B,i); }
   avma = av; return y;
 }
 /* sort polynomial factorization, in place */
@@ -1719,15 +1806,6 @@ sort_factor_pol(GEN y,int (*cmp)(GEN,GEN))
   return y;
 }
 
-/* assume f and g coprime integer factorizations */
-GEN
-merge_factor_i(GEN f, GEN g)
-{
-  if (lg(f) == 1) return g;
-  if (lg(g) == 1) return f;
-  return sort_factor(famat_mul_shallow(f,g), (void*)&cmpii, &cmp_nodata);
-}
-
 /***********************************************************************/
 /*                                                                     */
 /*                          SET OPERATIONS                             */
@@ -1742,7 +1820,9 @@ gtoset(GEN x)
   {
     case t_VEC:
     case t_COL: lx = lg(x); break;
-    case t_LIST: x = list_data(x); lx = x? lg(x): 1; break;
+    case t_LIST:
+      if (list_typ(x)==t_LIST_MAP) return mapdomain(x);
+      x = list_data(x); lx = x? lg(x): 1; break;
     case t_VECSMALL: lx = lg(x); x = zv_to_ZV(x); break;
     default: return mkveccopy(x);
   }
@@ -1771,7 +1851,9 @@ setsearch(GEN T, GEN y, long flag)
   switch(typ(T))
   {
     case t_VEC: lx = lg(T); break;
-    case t_LIST: T = list_data(T); lx = T? lg(T): 1; break;
+    case t_LIST:
+    if (list_typ(T) != t_LIST_RAW) pari_err_TYPE("setsearch",T);
+    T = list_data(T); lx = T? lg(T): 1; break;
     default: pari_err_TYPE("setsearch",T);
       return 0; /*not reached*/
   }
@@ -1880,7 +1962,7 @@ setbinop(GEN f, GEN x, GEN y)
   pari_sp av = avma;
   long i, j, lx, ly, k = 1;
   GEN z;
-  if (typ(f) != t_CLOSURE || closure_arity(f) != 2)
+  if (typ(f) != t_CLOSURE || closure_arity(f) != 2 || closure_is_variadic(f))
     pari_err_TYPE("setbinop [function needs exactly 2 arguments]",f);
   lx = lg(x);
   if (typ(x) != t_VEC) pari_err_TYPE("setbinop", x);
