@@ -22,6 +22,176 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 INLINE GEN
 inegate(GEN z) { return subsi(-1,z); }
 
+GEN
+binary_zv(GEN x)
+{
+  GEN xp, z;
+  long i, k, lx;
+  if (!signe(x)) return cgetg(1,t_VECSMALL);
+  xp = int_LSW(x);
+  lx = lgefint(x);
+  k = expi(x)+2;
+  z = cgetg(k, t_VECSMALL);
+  k--;
+  for(i = 2; i < lx; i++)
+  {
+    ulong u = *xp;
+    long j;
+    for (j=0; j<BITS_IN_LONG && k; j++) z[k--] = (u>>j)&1UL;
+    if (!k) break;
+    xp = int_nextW(xp);
+  }
+  return z;
+}
+static GEN
+F2v_to_ZV_inplace(GEN v)
+{
+  long i, l = lg(v);
+  v[0] = evaltyp(t_VEC) | _evallg(l);
+  for (i = 1; i < l; i++) gel(v,i) = v[i]? gen_1: gen_0;
+  return v;
+}
+/* "vector" of l bits (possibly no code word) to non-negative t_INT */
+GEN
+bits_to_int(GEN x, long l)
+{
+  long i, j, lz;
+  GEN z, zp;
+
+  if (!l) return gen_0;
+  lz = nbits2lg(l);
+  z = cgetg(lz, t_INT);
+  z[1] = evalsigne(1) | evallgefint(lz);
+  zp = int_LSW(z); *zp = 0;
+  for(i=l,j=0; i; i--,j++)
+  {
+    if (j==BITS_IN_LONG) { j=0; zp = int_nextW(zp); *zp = 0; }
+    if (x[i]) *zp |= 1UL<<j;
+  }
+  return int_normalize(z, 0);
+}
+/* "vector" of l < BITS_IN_LONG bits (possibly no code word) to non-negative
+ * ulong */
+ulong
+bits_to_u(GEN v, long l)
+{
+  ulong u = 0;
+  long i;
+  for (i = 1; i <= l; i++) u = (u <<1) | v[i];
+  return u;
+}
+
+/* set BITS_IN_LONG bits starting at word *w plus *r bits,
+ * clearing subsequent bits in the last word touched */
+INLINE void
+int_set_ulong(ulong a, GEN *w, long *r)
+{
+  if (*r) {
+    **w |= (a << *r);
+    *w = int_nextW(*w);
+    **w = (a >> (BITS_IN_LONG - *r));
+  } else {
+    **w = a;
+    *w = int_nextW(*w);
+  }
+}
+
+/* set k bits starting at word *w plus *r bits,
+ * clearing subsequent bits in the last word touched */
+INLINE void
+int_set_bits(ulong a, long k, GEN *w, long *r)
+{
+  if (*r) {
+    **w |= a << *r;
+    a >>= BITS_IN_LONG - *r;
+  } else {
+    **w = a;
+    a = 0;
+  }
+  *r += k;
+  if (*r >= BITS_IN_LONG) {
+    *w = int_nextW(*w);
+    *r -= BITS_IN_LONG;
+    for (; *r >= BITS_IN_LONG; *r -= BITS_IN_LONG) {
+      **w = a;
+      a = 0;
+      *w = int_nextW(*w);
+    }
+    if (*r)
+      **w = a;
+  }
+}
+
+/* set k bits from z (t_INT) starting at word *w plus *r bits,
+ * clearing subsequent bits in the last word touched */
+INLINE void
+int_set_int(GEN z, long k, GEN *w, long *r)
+{
+  long l = lgefint(z) - 2;
+  GEN y;
+  if (!l) {
+    int_set_bits(0, k, w, r);
+    return;
+  }
+  y = int_LSW(z);
+  for (; l > 1; l--) {
+    int_set_ulong((ulong) *y, w, r);
+    y = int_nextW(y);
+    k -= BITS_IN_LONG;
+  }
+  if (k)
+    int_set_bits((ulong) *y, k, w, r);
+}
+
+GEN
+nv_fromdigits_2k(GEN x, long k)
+{
+  long l = lg(x) - 1, r;
+  GEN w, z;
+  if (k == 1) return bits_to_int(x, l);
+  if (!l) return gen_0;
+  z = cgetipos(nbits2lg(k * l));
+  w = int_LSW(z);
+  r = 0;
+  for (; l; l--)
+    int_set_bits(uel(x, l), k, &w, &r);
+  return int_normalize(z, 0);
+}
+
+GEN
+fromdigits_2k(GEN x, long k)
+{
+  long l, m;
+  GEN w, y, z;
+  for (l = lg(x) - 1; l && !signe(gel(x, 1)); x++, l--);
+  if (!l) return gen_0;
+  m = expi(gel(x, 1)) + 1;
+  z = cgetipos(nbits2lg(k * (l - 1) + m));
+  w = int_LSW(z);
+  if (!(k & (BITS_IN_LONG - 1))) {
+    long i, j, t = k >> TWOPOTBITS_IN_LONG;
+    for (; l; l--) {
+      j = lgefint(gel(x, l)) - 2;
+      y = int_LSW(gel(x, l));
+      for (i = 0; i < j; i++) {
+        *w = *y;
+        y = int_nextW(y);
+        w = int_nextW(w);
+      }
+      for (; i < t; i++) {
+        *w = 0;
+        w = int_nextW(w);
+      }
+    }
+  } else {
+    long r = 0;
+    for (; l > 1; l--)
+      int_set_int(gel(x, l), k, &w, &r);
+    int_set_int(gel(x, 1), m, &w, &r);
+  }
+  return int_normalize(z, 0);
+}
+
 GEN
 binaire(GEN x)
 {
@@ -32,24 +202,9 @@ binaire(GEN x)
   switch(tx)
   {
     case t_INT:
-    if (!signe(x))
-      return cgetg(1,t_VEC);
-    else
-    {
-      GEN xp=int_MSW(x);
-      lx=lgefint(x);
-      y = cgetg(2 + expi(x), t_VEC); ly=1;
-      m=HIGHBIT>>bfffo(*xp); u=*xp;
-      do { gel(y,ly) = m & u ? gen_1 : gen_0; ly++; } while (m>>=1);
-      for (i=3; i<lx; i++)
-      {
-        m=HIGHBIT; xp=int_precW(xp); u=*xp;
-        do { gel(y,ly) = m & u ? gen_1 : gen_0; ly++; } while (m>>=1);
-      }
-      break;
-    }
+      return F2v_to_ZV_inplace( binary_zv(x) );
     case t_REAL:
-      ex=expo(x);
+      ex = expo(x);
       if (!signe(x)) return const_vec(maxss(-ex,0), gen_0);
 
       lx=lg(x); y=cgetg(3,t_VEC);
@@ -95,6 +250,103 @@ binaire(GEN x)
   return y;
 }
 
+/* extract k bits (as ulong) starting at word *w plus *r bits */
+INLINE ulong
+int_get_bits(long k, GEN *w, long *r)
+{
+  ulong mask = (1UL << k) - 1;
+  ulong a = (((ulong) **w) >> *r) & mask;
+  *r += k;
+  if (*r >= BITS_IN_LONG) {
+    *r -= BITS_IN_LONG;
+    *w = int_nextW(*w);
+    if (*r)
+      a |= ((ulong)**w << (k - *r)) & mask;
+  }
+  return a;
+}
+
+/* extract BITS_IN_LONG bits starting at word *w plus *r bits */
+INLINE ulong
+int_get_ulong(GEN *w, long *r)
+{
+  ulong a = ((ulong) **w) >> *r;
+  *w = int_nextW(*w);
+  if (*r)
+    a |= ((ulong)**w << (BITS_IN_LONG - *r));
+  return a;
+}
+
+/* extract k bits (as t_INT) starting at word *w plus *r bits */
+INLINE GEN
+int_get_int(long k, GEN *w, long *r)
+{
+  GEN z = cgetipos(nbits2lg(k));
+  GEN y = int_LSW(z);
+  for (; k >= BITS_IN_LONG; k -= BITS_IN_LONG) {
+    *y = int_get_ulong(w, r);
+    y = int_nextW(y);
+  }
+  if (k)
+    *y = int_get_bits(k, w, r);
+  return int_normalize(z, 0);
+}
+
+/* assume k < BITS_IN_LONG */
+GEN
+binary_2k_nv(GEN x, long k)
+{
+  long l, n, r;
+  GEN v, w;
+  if (k == 1) return binary_zv(x);
+  if (!signe(x)) return cgetg(1, t_VECSMALL);
+  n = expi(x) + 1;
+  l = (n + k - 1) / k;
+  v = cgetg(l + 1, t_VECSMALL);
+  w = int_LSW(x);
+  r = 0;
+  for (; l > 1; l--) {
+    uel(v, l) = int_get_bits(k, &w, &r);
+    n -= k;
+  }
+  uel(v, 1) = int_get_bits(n, &w, &r);
+  return v;
+}
+
+GEN
+binary_2k(GEN x, long k)
+{
+  long l, n;
+  GEN v, w, y, z;
+  if (k == 1) return binaire(x);
+  if (!signe(x)) return cgetg(1, t_VEC);
+  n = expi(x) + 1;
+  l = (n + k - 1) / k;
+  v = cgetg(l + 1, t_VEC);
+  w = int_LSW(x);
+  if (!(k & (BITS_IN_LONG - 1))) {
+    long m, t = k >> TWOPOTBITS_IN_LONG, u = lgefint(x) - 2;
+    for (; l; l--) {
+      m = minss(t, u);
+      z = cgetipos(m + 2);
+      y = int_LSW(z);
+      for (; m; m--) {
+        *y = *w;
+        y = int_nextW(y);
+        w = int_nextW(w);
+      }
+      gel(v, l) = int_normalize(z, 0);
+      u -= t;
+    }
+  } else {
+    long r = 0;
+    for (; l > 1; l--, n -= k)
+      gel(v, l) = int_get_int(k, &w, &r);
+    gel(v, 1) = int_get_int(n, &w, &r);
+  }
+  return v;
+}
+
 /* return 1 if bit n of x is set, 0 otherwise */
 long
 bittest(GEN x, long n)
