@@ -56,7 +56,7 @@ qfr3_canon(GEN x, struct qfr_data *S)
 {
   GEN a = gel(x,1), c = gel(x,3);
   if (signe(a) < 0) {
-    if (absi_equal(a,c)) return qfr3_rho(x, S);
+    if (absequalii(a,c)) return qfr3_rho(x, S);
     setsigne(a, 1);
     setsigne(c,-1);
   }
@@ -67,7 +67,7 @@ qfr3_canon_safe(GEN x, struct qfr_data *S)
 {
   GEN a = gel(x,1), c = gel(x,3);
   if (signe(a) < 0) {
-    if (absi_equal(a,c)) return qfr3_rho(x, S);
+    if (absequalii(a,c)) return qfr3_rho(x, S);
     gel(x,1) = negi(a);
     gel(x,3) = negi(c);
   }
@@ -78,7 +78,7 @@ qfr5_canon(GEN x, struct qfr_data *S)
 {
   GEN a = gel(x,1), c = gel(x,3);
   if (signe(a) < 0) {
-    if (absi_equal(a,c)) return qfr5_rho(x, S);
+    if (absequalii(a,c)) return qfr5_rho(x, S);
     setsigne(a, 1);
     setsigne(c,-1);
   }
@@ -96,11 +96,11 @@ static GEN
 qfr5_rho_pow(GEN x, long n, struct qfr_data *S)
 {
   long i;
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma;
   for (i=1; i<=n; i++)
   {
     x = qfr5_rho(x,S);
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"qfr5_rho_pow");
       x = gerepilecopy(av, x);
@@ -174,7 +174,7 @@ qfi_random(struct buch_quad *B,GEN ex) { return random_form(B, ex, &QFI_comp); }
 /*                                                                 */
 /*******************************************************************/
 long
-check_LIMC(long LIMC, long LIMCMAX)
+bnf_increase_LIMC(long LIMC, long LIMCMAX)
 {
   if (LIMC >= LIMCMAX) pari_err_BUG("Buchmann's algorithm");
   if (LIMC <= LIMCMAX/40) /* cbach <= 0.3 */
@@ -191,7 +191,7 @@ check_LIMC(long LIMC, long LIMCMAX)
 static int
 isless_iu(GEN q, ulong p) {
   long l = lgefint(q);
-  return l==2 || (l == 3 && (ulong)q[2] <= p);
+  return l==2 || (l == 3 && uel(q,2) <= p);
 }
 
 static long
@@ -203,7 +203,7 @@ factorquad(struct buch_quad *B, GEN f, long nFB, ulong limp)
 
   for (i=1; lgefint(x) > 3; i++)
   {
-    ulong p = (ulong)FB[i], r;
+    ulong p = uel(FB,i), r;
     GEN q = diviu_rem(x, p, &r);
     if (!r)
     {
@@ -212,16 +212,16 @@ factorquad(struct buch_quad *B, GEN f, long nFB, ulong limp)
       lo++; P[lo] = p; E[lo] = k;
     }
     if (isless_iu(q,p)) {
-      if (lgefint(x) == 3) { X = (ulong)x[2]; goto END; }
+      if (lgefint(x) == 3) { X = uel(x,2); goto END; }
       return 0;
     }
     if (i == nFB) return 0;
   }
-  X = (ulong)x[2];
+  X = uel(x,2);
   if (X == 1) { P[0] = 0; return 1; }
   for (;; i++)
   { /* single precision affair, split for efficiency */
-    ulong p = (ulong)FB[i];
+    ulong p = uel(FB,i);
     ulong q = X / p, r = X % p; /* gcc makes a single div */
     if (!r)
     {
@@ -302,10 +302,10 @@ static void
 cache_prime_quad(GRHcheck_t *S, ulong LIM, GEN D)
 {
   GRHprime_t *pr;
-  double nb;
+  long nb;
 
   if (S->limp >= LIM) return;
-  nb = primepi_upper_bound((double)LIM); /* #{p <= LIM} <= nb */
+  nb = (long)primepi_upper_bound((double)LIM); /* #{p <= LIM} <= nb */
   GRH_ensure(S, nb+1); /* room for one extra prime */
   for (pr = S->primes + S->nprimes;;)
   {
@@ -327,7 +327,7 @@ compute_invresquad(GRHcheck_t *S)
   GEN invres = real_1(DEFAULTPREC);
   GRHprime_t *pr = S->primes;
   long i = S->nprimes, LIMC = GRH_last_prime(S)+diffptr[i]-1; /* nextprime(p+1)-1*/
-  double limp = log(LIMC) / 2;
+  double limp = log((double)LIMC) / 2;
   for (; i > 0; pr++, i--)
   {
     long s = (long)pr->dec;
@@ -369,11 +369,8 @@ nthidealquad(GEN D, long n)
   forprime_t S;
   ulong p;
   (void)u_forprime_init(&S, 2, ULONG_MAX);
-  while (n > 0)
-  {
-    p = u_forprime_next(&S);
+  while ((p = u_forprime_next(&S)) && n > 0)
     if (!is_bad(D, p) && kroiu(D, p) >= 0) n--;
-  }
   avma = av; return p;
 }
 
@@ -405,8 +402,8 @@ quadGRHchk(GEN D, GRHcheck_t *S, ulong LIMC)
     if (M > 1)
     {
       double inv1_q = 1 / (1-q);
-      A *= (1 - pow(q, M)) * inv1_q;
-      B *= (1 - pow(q, M)*(M+1 - M*q)) * inv1_q * inv1_q;
+      A *= (1 - pow(q, (double) M)) * inv1_q;
+      B *= (1 - pow(q, (double) M)*(M+1 - M*q)) * inv1_q * inv1_q;
     }
     if ((long)pr->dec>0) { SA += 2*A;SB += 2*B; } else { SA += A; SB += B; }
   }
@@ -468,7 +465,7 @@ subFBquad(struct buch_quad *B, GEN D, double PROD, long minSFB)
   no    = cgetg(lv, t_VECSMALL);
   for (j = 1; j < lv; j++)
   {
-    ulong p = B->FB[j];
+    ulong p = uel(B->FB,j);
     if (!umodiu(D, p)) no[ino++] = j; /* ramified */
     else
     {
@@ -615,7 +612,7 @@ imag_relations(struct buch_quad *B, long need, long *pc, ulong LIMC, GEN mat)
   GEN col, form, ex = cgetg(lgsub, t_VECSMALL);
 
   if (!current) current = 1;
-  if (DEBUGLEVEL) timer_start(&T);
+  if (DEBUGLEVEL>2) timer_start(&T);
   av = avma;
   for(;;)
   {
@@ -626,7 +623,7 @@ imag_relations(struct buch_quad *B, long need, long *pc, ulong LIMC, GEN mat)
     nbtest++; fpc = factorquad(B,form,B->KC,LIMC);
     if (!fpc)
     {
-      if (DEBUGLEVEL>1) err_printf(".");
+      if (DEBUGLEVEL>3) err_printf(".");
       if ((nbtest & 0xff) == 0 && ++current > B->KC) current = 1;
       continue;
     }
@@ -637,7 +634,7 @@ imag_relations(struct buch_quad *B, long need, long *pc, ulong LIMC, GEN mat)
       GEN form2;
       if (!fpd)
       {
-        if (DEBUGLEVEL>1) err_printf(".");
+        if (DEBUGLEVEL>3) err_printf(".");
         continue;
       }
       form2 = qficomp(qfi_factorback(B,fpd), qfi_pf(B->QFR->D, B->FB[fpd[-2]]));
@@ -659,19 +656,19 @@ imag_relations(struct buch_quad *B, long need, long *pc, ulong LIMC, GEN mat)
         for (i=1; i<lgsub; i++) col[B->subFB[i]] += -fpd[i]-ex[i];
         add_fact(B, col, form2); col[fpd[-2]]--;
       }
-      if (DEBUGLEVEL) err_printf(" %ldP",s);
+      if (DEBUGLEVEL>2) err_printf(" %ldP",s);
     }
     else
     {
       col = gel(mat,++s);
       for (i=1; i<lgsub; i++) col[B->subFB[i]] = -ex[i];
       add_fact(B, col, form);
-      if (DEBUGLEVEL) err_printf(" %ld",s);
+      if (DEBUGLEVEL>2) err_printf(" %ld",s);
     }
     col[current]--;
     if (++current > B->KC) current = 1;
   }
-  if (DEBUGLEVEL) dbg_all(&T, "random", s, nbtest);
+  if (DEBUGLEVEL>2) dbg_all(&T, "random", s, nbtest);
   *pc = current;
 }
 
@@ -684,7 +681,7 @@ imag_be_honest(struct buch_quad *B)
 
   while (s<B->KC2)
   {
-    p = B->FB[s+1]; if (DEBUGLEVEL) err_printf(" %ld",p);
+    p = B->FB[s+1]; if (DEBUGLEVEL>2) err_printf(" %ld",p);
     F = qficomp(qfi_pf(B->QFR->D, p), qfi_random(B, ex));
     fpc = factorquad(B,F,s,p-1);
     if (fpc == 1) { nbtest=0; s++; }
@@ -706,19 +703,19 @@ real_relations(struct buch_quad *B, long need, long *pc, long lim, ulong LIMC, G
   /* in a 2nd phase, don't include FB[current] but run along the cyle
    * ==> get more units */
   int first = (current == 0);
-  pari_sp av, av1, limstack;
+  pari_sp av, av1;
   GEN d, col, form, form0, form1, ex = cgetg(lgsub, t_VECSMALL);
 
-  if (DEBUGLEVEL) timer_start(&T);
+  if (DEBUGLEVEL>2) timer_start(&T);
   if (!current) current = 1;
   if (lim > need) lim = need;
-  av = avma; limstack = stack_lim(av,1);
+  av = avma;
   for(;;)
   {
     if (s >= need) break;
     if (first && s >= lim) {
       first = 0;
-      if (DEBUGLEVEL) dbg_all(&T, "initial", s, nbtest);
+      if (DEBUGLEVEL>2) dbg_all(&T, "initial", s, nbtest);
     }
     avma = av; form = qfr3_random(B, ex);
     if (!first)
@@ -734,7 +731,7 @@ CYCLE:
       if (++current > B->KC) current = 1;
       continue;
     }
-    if (low_stack(limstack, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"real_relations");
       gerepileall(av1, form1? 2: 1, &form, &form1);
@@ -745,13 +742,13 @@ CYCLE:
       form = qfr3_rho(form, B->QFR); rho++;
       rhoacc++;
       if (first)
-        endcycle = (absi_equal(gel(form,1),gel(form0,1))
+        endcycle = (absequalii(gel(form,1),gel(form0,1))
              && equalii(gel(form,2),gel(form0,2)));
       else
       {
-        if (absi_equal(gel(form,1), gel(form,3))) /* a = -c */
+        if (absequalii(gel(form,1), gel(form,3))) /* a = -c */
         {
-          if (absi_equal(gel(form,1),gel(form0,1)) &&
+          if (absequalii(gel(form,1),gel(form0,1)) &&
                   equalii(gel(form,2),gel(form0,2))) continue;
           form = qfr3_rho(form, B->QFR); rho++;
           rhoacc++;
@@ -765,7 +762,7 @@ CYCLE:
     nbtest++; fpc = factorquad(B,form,B->KC,LIMC);
     if (!fpc)
     {
-      if (DEBUGLEVEL>1) err_printf(".");
+      if (DEBUGLEVEL>3) err_printf(".");
       goto CYCLE;
     }
     if (fpc > 1)
@@ -775,7 +772,7 @@ CYCLE:
       GEN form2;
       if (!fpd)
       {
-        if (DEBUGLEVEL>1) err_printf(".");
+        if (DEBUGLEVEL>3) err_printf(".");
         goto CYCLE;
       }
       if (!form1)
@@ -791,7 +788,7 @@ CYCLE:
       if (fpd[-2])
         form2 = QFR5_comp(form2, qfr5_pf(B->QFR, B->FB[fpd[-2]], prec), B->QFR);
       form2 = qfr5_rho_pow(form2, fpd[-3], B->QFR);
-      if (!absi_equal(gel(form2,1),gel(form2,3)))
+      if (!absequalii(gel(form2,1),gel(form2,3)))
       {
         setsigne(form2[1], 1);
         setsigne(form2[3],-1);
@@ -820,7 +817,7 @@ CYCLE:
         d = qfr5_dist(addii(gel(form1,4),gel(form2,4)),
                       mulrr(gel(form1,5),gel(form2,5)), prec);
       }
-      if (DEBUGLEVEL) err_printf(" %ldP",s);
+      if (DEBUGLEVEL>2) err_printf(" %ldP",s);
     }
     else
     { /* standard relation */
@@ -837,7 +834,7 @@ CYCLE:
       for (i=1; i<lgsub; i++) col[B->subFB[i]] = -ex[i];
       add_fact(B, col, form1);
       d = qfr5_dist(gel(form1,4), gel(form1,5), prec);
-      if (DEBUGLEVEL) err_printf(" %ld",s);
+      if (DEBUGLEVEL>2) err_printf(" %ld",s);
     }
     affrr(d, gel(C,s));
     if (first)
@@ -851,7 +848,7 @@ CYCLE:
       if (++current > B->KC) current = 1;
     }
   }
-  if (DEBUGLEVEL) dbg_all(&T, "random", s, nbtest);
+  if (DEBUGLEVEL>2) dbg_all(&T, "random", s, nbtest);
   *pc = current;
 }
 
@@ -864,7 +861,7 @@ real_be_honest(struct buch_quad *B)
 
   while (s<B->KC2)
   {
-    p = B->FB[s+1]; if (DEBUGLEVEL) err_printf(" %ld",p);
+    p = B->FB[s+1]; if (DEBUGLEVEL>2) err_printf(" %ld",p);
     F = QFR3_comp(qfr3_random(B, ex), qfr3_pf(B->QFR, p), B->QFR);
     for (F0 = F;;)
     {
@@ -922,10 +919,10 @@ get_R(struct buch_quad *B, GEN C, long sreg, GEN z, GEN *ptR)
     }
     if (gexpo(R) <= -3)
     {
-      if (DEBUGLEVEL) err_printf("regulator is zero.\n");
+      if (DEBUGLEVEL>2) err_printf("regulator is zero.\n");
       return fupb_RELAT;
     }
-    if (DEBUGLEVEL) err_printf("#### Tentative regulator: %Ps\n",R);
+    if (DEBUGLEVEL>2) err_printf("#### Tentative regulator: %Ps\n",R);
   }
   c = gtodouble(gmul(z, R));
   if (c < 0.8 || c > 1.3) return fupb_RELAT;
@@ -937,10 +934,10 @@ quad_be_honest(struct buch_quad *B)
 {
   int r;
   if (B->KC2 <= B->KC) return 1;
-  if (DEBUGLEVEL)
+  if (DEBUGLEVEL>2)
     err_printf("be honest for primes from %ld to %ld\n", B->FB[B->KC+1],B->FB[B->KC2]);
   r = B->PRECREG? real_be_honest(B): imag_be_honest(B);
-  if (DEBUGLEVEL) err_printf("\n");
+  if (DEBUGLEVEL>2) err_printf("\n");
   return r;
 }
 
@@ -967,17 +964,17 @@ Buchquad(GEN D, double cbach, double cbach2, long prec)
   QFR.D = D;
   if (s < 0)
   {
-    if (cmpiu(QFR.D,4) <= 0)
+    if (abscmpiu(QFR.D,4) <= 0)
     {
       GEN z = cgetg(5,t_VEC);
       gel(z,1) = gel(z,4) = gen_1; gel(z,2) = gel(z,3) = cgetg(1,t_VEC);
       return z;
     }
-    BQ.PRECREG = 0;
+    prec = BQ.PRECREG = 0;
   } else {
     BQ.PRECREG = maxss(prec+EXTRAPRECWORD, nbits2prec(2*expi(QFR.D) + 128));
   }
-  if (DEBUGLEVEL) timer_start(&T);
+  if (DEBUGLEVEL>2) timer_start(&T);
   BQ.primfact   = new_chunk(100);
   BQ.exprimfact = new_chunk(100);
   BQ.hashtab = (long**) new_chunk(HASHT);
@@ -1031,8 +1028,8 @@ Buchquad(GEN D, double cbach, double cbach2, long prec)
 START:
   do
   {
-    if (!FIRST) LIMC = check_LIMC(LIMC,LIMCMAX);
-    if (DEBUGLEVEL && LIMC > LIMC0)
+    if (!FIRST) LIMC = bnf_increase_LIMC(LIMC,LIMCMAX);
+    if (DEBUGLEVEL>2 && LIMC > LIMC0)
       err_printf("%s*** Bach constant: %f\n", FIRST?"":"\n", LIMC/LOGD2);
     FIRST = 0; avma = av;
     if (BQ.subFB) gunclone(BQ.subFB);
@@ -1043,17 +1040,17 @@ START:
     if (BQ.PRECREG) qfr_data_init(QFR.D, BQ.PRECREG, &QFR);
 
     FBquad(&BQ, LIMC2, LIMC, &GRHcheck);
-    if (DEBUGLEVEL) timer_printf(&T, "factor base");
+    if (DEBUGLEVEL>2) timer_printf(&T, "factor base");
     BQ.subFB = subFBquad(&BQ, QFR.D, lim + 0.5, minSFB);
-    if (DEBUGLEVEL) timer_printf(&T, "subFBquad = %Ps",
+    if (DEBUGLEVEL>2) timer_printf(&T, "subFBquad = %Ps",
                                  vecpermute(BQ.FB, BQ.subFB));
     nsubFB = lg(BQ.subFB) - 1;
   }
   while (nsubFB < (expi(D) > 15 ? 3 : 2));
   /* invhr = 2^r1 (2pi)^r2 / sqrt(D) w ~ L(chi,1) / hR */
-  invhr = gmul(dbltor((BQ.PRECREG?2.:PI)/sdrc), compute_invresquad(&GRHcheck));
+  invhr = gmul(dbltor((BQ.PRECREG?2.:M_PI)/sdrc), compute_invresquad(&GRHcheck));
   BQ.powsubFB = powsubFBquad(&BQ,CBUCH+1);
-  if (DEBUGLEVEL) timer_printf(&T, "powsubFBquad");
+  if (DEBUGLEVEL>2) timer_printf(&T, "powsubFBquad");
   BQ.limhash = (LIMC & HIGHMASK)? (HIGHBIT>>1): LIMC*LIMC;
 
   need = BQ.KC + RELSUP - 2;
@@ -1066,12 +1063,12 @@ START:
   do
   {
     if ((nreldep & 3) == 1 || (nrelsup & 7) == 1) {
-      if (DEBUGLEVEL) err_printf("*** Changing sub factor base\n");
+      if (DEBUGLEVEL>2) err_printf("*** Changing sub factor base\n");
       gunclone(BQ.subFB);
       gunclone(BQ.powsubFB);
       BQ.subFB = gclone(vecslice(BQ.vperm, 1, nsubFB));
       BQ.powsubFB = powsubFBquad(&BQ,CBUCH+1);
-      if (DEBUGLEVEL) timer_printf(&T, "powsubFBquad");
+      if (DEBUGLEVEL>2) timer_printf(&T, "powsubFBquad");
       clearhash(BQ.hashtab);
     }
     need += 2;
@@ -1080,10 +1077,10 @@ START:
     if (!W) { /* first time */
       C = extraC;
       triv = trivial_relations(&BQ, mat, C);
-      if (DEBUGLEVEL) err_printf("KC = %ld, need %ld relations\n", BQ.KC, need);
+      if (DEBUGLEVEL>2) err_printf("KC = %ld, need %ld relations\n", BQ.KC, need);
     } else {
       triv = 0;
-      if (DEBUGLEVEL) err_printf("...need %ld more relations\n", need);
+      if (DEBUGLEVEL>2) err_printf("...need %ld more relations\n", need);
     }
     if (BQ.PRECREG) {
       for (i = triv+1; i<=need; i++) {
@@ -1112,7 +1109,7 @@ START:
     }
 
     h = ZM_det_triangular(W);
-    if (DEBUGLEVEL) err_printf("\n#### Tentative class number: %Ps\n", h);
+    if (DEBUGLEVEL>2) err_printf("\n#### Tentative class number: %Ps\n", h);
 
     switch(get_R(&BQ, C, (lg(C)-1) - (lg(B)-1) - (lg(W)-1), mulir(h,invhr), &R))
     {
@@ -1132,11 +1129,11 @@ START:
   while (need);
   /* DONE */
   if (!quad_be_honest(&BQ)) goto START;
-  if (DEBUGLEVEL) timer_printf(&T, "be honest");
+  if (DEBUGLEVEL>2) timer_printf(&T, "be honest");
   clearhash(BQ.hashtab);
   free_GRHcheck(&GRHcheck);
 
-  gen = get_clgp(&BQ,W,&cyc,BQ.PRECREG);
+  gen = get_clgp(&BQ,W,&cyc,prec);
   gunclone(BQ.subFB);
   gunclone(BQ.powsubFB);
   res = cgetg(5,t_VEC);
