@@ -81,7 +81,7 @@ zpsol(GEN T, GEN p, long nu, GEN pnu, GEN x0)
   pari_sp av = avma;
   GEN x, pnup;
 
-  res = equaliu(p,2)? lemma7(T,nu,x0): lemma6(T,p,nu,x0);
+  res = absequaliu(p,2)? lemma7(T,nu,x0): lemma6(T,p,nu,x0);
   if (res== 1) return 1;
   if (res==-1) return 0;
   x = x0; pnup = mulii(pnu,p);
@@ -214,7 +214,7 @@ lemma7nf(GEN nf, GEN T, GEN pr, long nu, GEN x, GEN zinit)
 
   gpx = nfpoleval(nf, RgX_deriv(T), x);
   /* gx /= pi^la, pi a pr-uniformizer */
-  la = ZC_nfvalrem(nf, gx, pr, &gx);
+  la = nfvalrem(nf, gx, pr, &gx);
   mu = gequal0(gpx)? la+nu+1: nfval(nf,gpx,pr);
 
   if (la > (mu<<1)) return 1;
@@ -234,7 +234,7 @@ lemma7nf(GEN nf, GEN T, GEN pr, long nu, GEN x, GEN zinit)
   if (q == 1) return res;
 
   /* is gx a square mod pi^q ? FIXME : highly inefficient */
-  zinit = zidealstarinit(nf, idealpows(nf,pr,q));
+  zinit = Idealstarprk(nf, pr, q, nf_INIT);
   if (!check2(nf, gx, zinit)) res = -1;
   return res;
 }
@@ -265,7 +265,7 @@ zpsolnf(GEN nf,GEN T,GEN pr,long nu,GEN pnu,GEN x0,GEN repr,GEN zinit)
 /* Let y = copy(x); y[k] := j; return y */
 static GEN
 ZC_add_coeff(GEN x, long k, long j)
-{ GEN y = shallowcopy(x); gel(y, k) = utoi(j); return y; }
+{ GEN y = shallowcopy(x); gel(y, k) = utoipos(j); return y; }
 
 /* system of representatives for Zk/pr */
 static GEN
@@ -273,18 +273,8 @@ repres(GEN nf, GEN pr)
 {
   long f = pr_get_f(pr), N = nf_get_degree(nf), p = itos(pr_get_p(pr));
   long i, j, k, pi, pf = upowuu(p, f);
-  GEN rep, perm = cgetg(f+1, t_VECSMALL);
+  GEN perm = pr_basis_perm(nf, pr), rep = cgetg(pf+1,t_VEC);
 
-  perm[1] = 1;
-  if (f > 1) {
-    GEN H = idealhnf_two(nf,pr);
-    for (i = k = 2; k <= f; i++)
-    {
-      if (is_pm1(gcoeff(H,i,i))) continue;
-      perm[k++] = i;
-    }
-  }
-  rep = cgetg(pf+1,t_VEC);
   gel(rep,1) = zerocol(N);
   for (pi=i=1; i<=f; i++,pi*=p)
   {
@@ -307,17 +297,17 @@ nf_hyperell_locally_soluble(GEN nf,GEN T,GEN pr)
   if (typ(T)!=t_POL) pari_err_TYPE("nf_hyperell_locally_soluble",T);
   if (gequal0(T)) return 1;
   checkprid(pr); nf = checknf(nf);
-  if (equaliu(pr_get_p(pr), 2))
+  if (absequaliu(pr_get_p(pr), 2))
   { /* tough case */
-    zinit = Idealstar(nf, idealpows(nf,pr,1+2*pr_get_e(pr)), nf_INIT);
-    if (psquare2nf(nf,constant_term(T),pr,zinit)) return 1;
-    if (psquare2nf(nf, leading_term(T),pr,zinit)) return 1;
+    zinit = Idealstarprk(nf, pr, 1+2*pr_get_e(pr), nf_INIT);
+    if (psquare2nf(nf,constant_coeff(T),pr,zinit)) return 1;
+    if (psquare2nf(nf, leading_coeff(T),pr,zinit)) return 1;
   }
   else
   {
     zinit = zkmodprinit(nf, pr);
-    if (psquarenf(nf,constant_term(T),pr,zinit)) return 1;
-    if (psquarenf(nf, leading_term(T),pr,zinit)) return 1;
+    if (psquarenf(nf,constant_coeff(T),pr,zinit)) return 1;
+    if (psquarenf(nf, leading_coeff(T),pr,zinit)) return 1;
   }
   repr = repres(nf,pr);
   if (zpsolnf(nf,T,pr,0,gen_1,gen_0,repr,zinit)) { avma=av; return 1; }
@@ -372,7 +362,7 @@ nfhilbertp(GEN nf, GEN a, GEN b, GEN pr)
   long va, vb, rep;
   pari_sp av = avma;
 
-  if (equaliu(pr_get_p(pr), 2)) return hilb2nf(nf,a,b,pr);
+  if (absequaliu(pr_get_p(pr), 2)) return hilb2nf(nf,a,b,pr);
 
   /* pr not above 2, compute t = tame symbol */
   va = nfval(nf,a,pr);
@@ -465,7 +455,7 @@ nfhilbert0(GEN nf,GEN a,GEN b,GEN p)
  * res[5] = S class group
  * res[6] = S */
 GEN
-bnfsunit(GEN bnf,GEN S,long prec)
+bnfsunit0(GEN bnf, GEN S, long flag, long prec)
 {
   pari_sp av = avma;
   long i,j,ls;
@@ -503,9 +493,11 @@ bnfsunit(GEN bnf,GEN S,long prec)
   if (lg(H) > 1)
   { /* non trivial (rare!) */
     GEN A, u, D = ZM_snfall_i(H, &u, NULL, 1);
-    card = detcyc(D, &i); setlg(D,i);
-    A = cgetg(i,t_VEC); pow = ZM_inv(u,gen_1);
-    for(i--; i; i--) gel(A,i) = idealfactorback(nf, gen, gel(pow,i), 1);
+    long l;
+    ZV_snf_trunc(D); l = lg(D);
+    card = ZV_prod(D);
+    A = cgetg(l, t_VEC); pow = ZM_inv(u,gen_1);
+    for(i = 1; i < l; i++) gel(A,i) = idealfactorback(nf, gen, gel(pow,i), 1);
     gel(res,5) = mkvec3(card, D, A);
   }
 
@@ -513,7 +505,7 @@ bnfsunit(GEN bnf,GEN S,long prec)
   if (ls>1)
   {
     GEN den, Sperm, perm, dep, B, A, U1 = U;
-    long lH, lB;
+    long lH, lB, FLAG = flag|nf_FORCE;
 
    /* U1 = upper left corner of U, invertible. S * U1 = principal ideals
     * whose generators generate the S-units */
@@ -533,13 +525,15 @@ bnfsunit(GEN bnf,GEN S,long prec)
     setlg(Sperm, lH);
     for (i=1; i<lH; i++)
     {
-      GEN v = isprincipalfact(bnf, NULL,Sperm,gel(H,i), nf_GEN|nf_FORCE);
-      gel(sunit,i) = coltoliftalg(nf, gel(v,2));
+      GEN v = isprincipalfact(bnf, NULL,Sperm,gel(H,i), FLAG);
+      v = gel(v,2); if (flag == nf_GEN) v = nf_to_scalar_or_alg(nf, v);
+      gel(sunit,i) = v;
     }
     for (j=1; j<lB; j++,i++)
     {
-      GEN v = isprincipalfact(bnf, gel(Sperm,i),Sperm,gel(B,j),nf_GEN|nf_FORCE);
-      gel(sunit,i) = coltoliftalg(nf, gel(v,2));
+      GEN v = isprincipalfact(bnf, gel(Sperm,i),Sperm,gel(B,j),FLAG);
+      v = gel(v,2); if (flag == nf_GEN) v = nf_to_scalar_or_alg(nf, v);
+      gel(sunit,i) = v;
    }
     den = ZM_det_triangular(H); H = ZM_inv(H,den);
     A = shallowconcat(H, ZM_neg(ZM_mul(H,B))); /* top part of inverse * den */
@@ -558,6 +552,8 @@ bnfsunit(GEN bnf,GEN S,long prec)
   gel(res,4) = sreg;
   return gerepilecopy(av,res);
 }
+GEN
+bnfsunit(GEN bnf,GEN S,long prec) { return bnfsunit0(bnf,S,nf_GEN,prec); }
 
 static GEN
 make_unit(GEN nf, GEN bnfS, GEN *px)
@@ -597,7 +593,7 @@ make_unit(GEN nf, GEN bnfS, GEN *px)
   }
   if (!is_pm1(N0)) return NULL;
   /* here, x = S v */
-  p1 = vecpermute(v, perm);
+  p1 = vecsmallpermute(v, perm);
   v = ZM_zc_mul(HB, p1);
   for (i=1; i<=cH; i++)
   {
@@ -614,9 +610,9 @@ make_unit(GEN nf, GEN bnfS, GEN *px)
   {
     GEN e = gel(v,i);
     if (!signe(e)) continue;
-    xp = famat_mul(xp, to_famat_shallow(gel(gen,i), negi(e)));
+    xp = famat_mulpow_shallow(xp, gel(gen,i), negi(e));
   }
-  if (lg(xp) > 1) *px = famat_mul(xp, to_famat_shallow(xb, gen_1));
+  if (lg(xp) > 1) *px = famat_mulpow_shallow(xp, xb, gen_1);
   return v;
 }
 
@@ -638,40 +634,49 @@ bnfissunit(GEN bnf,GEN bnfS,GEN x)
   v = NULL;
   if ( (w = make_unit(nf, bnfS, &x)) ) v = bnfisunit(bnf, x);
   if (!v || lg(v) == 1) { avma = av; return cgetg(1,t_COL); }
-  return gerepileupto(av, concat(v, w));
+  return gerepileupto(av, gconcat(v, w));
 }
 
 static void
-pr_append(GEN nf, GEN rel, GEN p, GEN *prod, GEN *S1, GEN *S2)
+p_append(GEN p, hashtable *H, hashtable *H2)
 {
-  if (dvdii(*prod, p)) return;
-  *prod = mulii(*prod, p);
-  *S1 = shallowconcat(*S1, idealprimedec(nf,p));
-  *S2 = shallowconcat(*S2, idealprimedec(rel,p));
+  ulong h = H->hash(p);
+  hashentry *e = hash_search2(H, (void*)p, h);
+  if (!e)
+  {
+    hash_insert2(H, (void*)p, NULL, h);
+    if (H2) hash_insert2(H2, (void*)p, NULL, h);
+  }
 }
 
 /* N a t_INT */
 static void
-Zfa_pr_append(GEN nf,GEN rel,GEN N,GEN *prod,GEN *S1,GEN *S2)
+Zfa_append(GEN N, hashtable *H, hashtable *H2)
 {
   if (!is_pm1(N))
   {
-    GEN v = gel(Z_factor(N),1);
+    GEN v = gel(absZ_factor(N),1);
     long i, l = lg(v);
-    for (i=1; i<l; i++) pr_append(nf,rel,gel(v,i),prod,S1,S2);
+    for (i=1; i<l; i++) p_append(gel(v,i), H, H2);
   }
 }
-/* N a t_INT or t_FRAC */
+/* N a t_INT or t_FRAC or ideal in HNF*/
 static void
-fa_pr_append(GEN nf,GEN rel,GEN N,GEN *prod,GEN *S1,GEN *S2)
+fa_append(GEN N, hashtable *H, hashtable *H2)
 {
-  if (typ(N) == t_FRAC)
+  switch(typ(N))
   {
-    Zfa_pr_append(nf,rel,gel(N,1),prod,S1,S2);
-    Zfa_pr_append(nf,rel,gel(N,2),prod,S1,S2);
+    case t_INT:
+      Zfa_append(N,H,H2);
+      break;
+    case t_FRAC:
+      Zfa_append(gel(N,1),H,H2);
+      Zfa_append(gel(N,2),H,H2);
+      break;
+    default: /*t_MAT*/
+      Zfa_append(gcoeff(N,1,1),H,H2);
+      break;
   }
-  else
-    Zfa_pr_append(nf,rel,N,prod,S1,S2);
 }
 
 /* apply lift(rnfeltup) to all coeffs, without rnf structure */
@@ -684,25 +689,19 @@ nfX_eltup(GEN nf, GEN rnfeq, GEN x)
   for (i=2; i<l; i++) gel(y,i) = nfeltup(nf, gel(x,i), zknf, czknf);
   return y;
 }
-/* FIXME: remove this */
-static void
-nfX_fix_var(GEN P, long v)
-{
-  long i, l = lg(P);
-  for (i=2; i<l; i++)
-  {
-    GEN c = gel(P,i);
-    if (typ(c) == t_POL) setvarn(c, v);
-  }
-}
 
+static hashtable *
+hash_create_INT(ulong s)
+{ return hash_create(s, (ulong(*)(void*))&hash_GEN,
+                        (int(*)(void*,void*))&equalii, 1); }
 GEN
 rnfisnorminit(GEN T, GEN relpol, int galois)
 {
   pari_sp av = avma;
   long i, l, drel;
-  GEN prod, S1, S2, gen, cyc, bnf, nf, nfabs, rnfeq, bnfabs, k, polabs;
+  GEN S, gen, cyc, bnf, nf, nfabs, rnfeq, bnfabs, k, polabs;
   GEN y = cgetg(9, t_VEC);
+  hashtable *H = hash_create_INT(100UL);
 
   if (galois < 0 || galois > 2) pari_err_FLAG("rnfisnorminit");
   T = get_bnfpol(T, &bnf, &nf);
@@ -710,7 +709,7 @@ rnfisnorminit(GEN T, GEN relpol, int galois)
   if (!nf) nf = bnf_get_nf(bnf);
 
   relpol = get_bnfpol(relpol, &bnfabs, &nfabs);
-  if (!gequal1(leading_term(relpol))) pari_err_IMPL("non monic relative equation");
+  if (!gequal1(leading_coeff(relpol))) pari_err_IMPL("non monic relative equation");
   drel = degpol(relpol);
   if (drel <= 2) galois = 1;
 
@@ -729,41 +728,33 @@ rnfisnorminit(GEN T, GEN relpol, int galois)
 
   if (galois == 2)
   {
-    GEN P;
-    long v = varn(T);
-    if (polabs == relpol)
-      P = relpol;
-    else
-    { /* FIXME: don't mess with variables, use proper priorities in nfabs. */
-      P = nfX_eltup(nf, rnfeq, relpol);
-      nfX_fix_var(P, v);
-    }
-    /* FIXME */
-    galois = nfissplit(gsubst(nfabs, nf_get_varn(nfabs), pol_x(v)), P);
+    GEN P = polabs==relpol? leafcopy(relpol): nfX_eltup(nf, rnfeq, relpol);
+    setvarn(P, fetch_var_higher());
+    galois = !!nfroots_if_split(&nfabs, P);
+    (void)delete_var();
   }
 
-  prod = gen_1; S1 = S2 = cgetg(1, t_VEC);
   cyc = bnf_get_cyc(bnfabs);
   gen = bnf_get_gen(bnfabs); l = lg(cyc);
   for(i=1; i<l; i++)
   {
     GEN g = gel(gen,i);
     if (ugcd(umodiu(gel(cyc,i), drel), drel) == 1) break;
-    Zfa_pr_append(nf,bnfabs,gcoeff(g,1,1),&prod,&S1,&S2);
+    Zfa_append(gcoeff(g,1,1), H, NULL);
   }
   if (!galois)
   {
     GEN Ndiscrel = diviiexact(nf_get_disc(nfabs), powiu(nf_get_disc(nf), drel));
-    Zfa_pr_append(nf,bnfabs,absi(Ndiscrel), &prod,&S1,&S2);
+    Zfa_append(Ndiscrel, H, NULL);
   }
-
+  S = hash_keys(H); settyp(S,t_VEC);
   gel(y,1) = bnf;
   gel(y,2) = bnfabs;
   gel(y,3) = relpol;
   gel(y,4) = rnfeq;
-  gel(y,5) = prod;
-  gel(y,6) = S1;
-  gel(y,7) = S2;
+  gel(y,5) = S;
+  gel(y,6) = nf_pV_to_prV(nf, S);
+  gel(y,7) = nf_pV_to_prV(nfabs, S);
   gel(y,8) = stoi(galois); return gerepilecopy(av, y);
 }
 
@@ -779,29 +770,33 @@ rnfisnorm(GEN T, GEN x, long flag)
 {
   pari_sp av = avma;
   GEN bnf, rel, relpol, rnfeq, nfpol;
-  GEN nf, aux, H, U, Y, M, A, bnfS, sunitrel, futu, prod, S1, S2;
+  GEN nf, aux, H, U, Y, M, A, bnfS, sunitrel, futu, S, S1, S2, Sx;
   long L, i, drel, itu;
-
+  hashtable *H0, *H2;
   if (typ(T) != t_VEC || lg(T) != 9)
     pari_err_TYPE("rnfisnorm [please apply rnfisnorminit()]", T);
   bnf = gel(T,1);
   rel = gel(T,2);
-  relpol = gel(T,3);
-  rnfeq = gel(T,4);
-  drel = degpol(relpol);
   bnf = checkbnf(bnf);
   rel = checkbnf(rel);
   nf = bnf_get_nf(bnf);
   x = nf_to_scalar_or_alg(nf,x);
   if (gequal0(x)) { avma = av; return mkvec2(gen_0, gen_1); }
   if (gequal1(x)) { avma = av; return mkvec2(gen_1, gen_1); }
+  relpol = gel(T,3);
+  rnfeq = gel(T,4);
+  drel = degpol(relpol);
   if (gequalm1(x) && odd(drel)) { avma = av; return mkvec2(gen_m1, gen_1); }
 
   /* build set T of ideals involved in the solutions */
   nfpol = nf_get_pol(nf);
-  prod = gel(T,5);
-  S1   = gel(T,6);
-  S2   = gel(T,7);
+  S = gel(T,5);
+  H0 = hash_create_INT(100UL);
+  H2 = hash_create_INT(100UL);
+  L = lg(S);
+  for (i = 1; i < L; i++) p_append(gel(S,i),H0,NULL);
+  S1 = gel(T,6);
+  S2 = gel(T,7);
   if (flag && !gequal0(gel(T,8)))
     pari_warn(warner,"useless flag in rnfisnorm: the extension is Galois");
   if (flag > 0)
@@ -809,20 +804,28 @@ rnfisnorm(GEN T, GEN x, long flag)
     forprime_t T;
     ulong p;
     u_forprime_init(&T, 2, flag);
-    while ( (p = u_forprime_next(&T)) )
-      pr_append(nf,rel, utoipos(p),&prod,&S1,&S2);
+    while ((p = u_forprime_next(&T))) p_append(utoipos(p), H0,H2);
   }
   else if (flag < 0)
-    Zfa_pr_append(nf,rel, utoipos(-flag),&prod,&S1,&S2);
+    Zfa_append(utoipos(-flag),H0,H2);
   /* overkill: prime ideals dividing x would be enough */
-  fa_pr_append(nf,rel,idealnorm(nf,x), &prod,&S1,&S2);
+  A = idealnumden(nf, x);
+  fa_append(gel(A,1), H0,H2);
+  fa_append(gel(A,2), H0,H2);
+  Sx = hash_keys(H2); L = lg(Sx);
+  if (L > 1)
+  { /* new primes */
+    settyp(Sx, t_VEC);
+    S1 = shallowconcat(S1, nf_pV_to_prV(nf, Sx));
+    S2 = shallowconcat(S2, nf_pV_to_prV(rel, Sx));
+  }
 
   /* computation on T-units */
   futu = shallowconcat(bnf_get_fu(rel), bnf_get_tuU(rel));
   bnfS = bnfsunit(bnf,S1,LOWDEFAULTPREC);
   sunitrel = shallowconcat(futu, gel(bnfsunit(rel,S2,LOWDEFAULTPREC), 1));
 
-  A = lift_intern(bnfissunit(bnf,bnfS,x));
+  A = lift_shallow(bnfissunit(bnf,bnfS,x));
   L = lg(sunitrel);
   itu = lg(nf_get_roots(nf))-1; /* index of torsion unit in bnfsunit(nf) output */
   M = cgetg(L+1,t_MAT);
@@ -832,7 +835,7 @@ rnfisnorm(GEN T, GEN x, long flag)
     gel(sunitrel,i) = u;
     u = bnfissunit(bnf,bnfS, gnorm(u));
     if (lg(u) == 1) pari_err_BUG("rnfisnorm");
-    gel(u,itu) = lift_intern(gel(u,itu)); /* lift root of 1 part */
+    gel(u,itu) = lift_shallow(gel(u,itu)); /* lift root of 1 part */
     gel(M,i) = u;
   }
   aux = zerocol(lg(A)-1); gel(aux,itu) = utoipos( bnf_get_tuN(rel) );
@@ -854,6 +857,8 @@ GEN
 bnfisnorm(GEN bnf, GEN x, long flag)
 {
   pari_sp av = avma;
-  GEN T = rnfisnorminit(pol_x(MAXVARN), bnf, flag == 0? 1: 2);
-  return gerepileupto(av, rnfisnorm(T, x, flag == 1? 0: flag));
+  GEN T = rnfisnorminit(pol_x(fetch_var()), bnf, flag == 0? 1: 2);
+  GEN r = rnfisnorm(T, x, flag == 1? 0: flag);
+  (void)delete_var();
+  return gerepileupto(av,r);
 }
