@@ -27,30 +27,34 @@ listconcat(GEN A, GEN B)
   GEN L, z, L1, L2;
 
   if (typ(A) != t_LIST) {
+    if (list_typ(B)!=t_LIST_RAW) pari_err_TYPE("listconcat",B);
     L2 = list_data(B);
     if (!L2) return mklistcopy(A);
     lx = lg(L2) + 1;
-    z = listcreate();
+    z = mklist();
     list_data(z) = L = cgetg(lx, t_VEC);
     for (i = 2; i < lx; i++) gel(L,i) = gcopy(gel(L2,i-1));
     gel(L,1) = gcopy(A); return z;
   } else if (typ(B) != t_LIST) {
+    if (list_typ(A)!=t_LIST_RAW) pari_err_TYPE("listconcat",A);
     L1 = list_data(A);
     if (!L1) return mklistcopy(B);
     lx = lg(L1) + 1;
-    z = listcreate();
+    z = mklist();
     list_data(z) = L = cgetg(lx, t_VEC);
     for (i = 1; i < lx-1; i++) gel(L,i) = gcopy(gel(L1,i));
     gel(L,i) = gcopy(B); return z;
   }
   /* A, B both t_LISTs */
+  if (list_typ(A)!=t_LIST_RAW) pari_err_TYPE("listconcat",A);
+  if (list_typ(B)!=t_LIST_RAW) pari_err_TYPE("listconcat",B);
   L1 = list_data(A); if (!L1) return listcopy(B);
   L2 = list_data(B); if (!L2) return listcopy(A);
 
   l1 = lg(L1);
   lx = l1-1 + lg(L2);
   z = cgetg(3, t_LIST);
-  list_nmax(z) = 0;
+  z[1] = 0UL;
   list_data(z) = L = cgetg(lx, t_VEC);
   L2 -= l1-1;
   for (i=1; i<l1; i++) gel(L,i) = gclone(gel(L1,i));
@@ -283,7 +287,7 @@ catmany(GEN y1, GEN y2, long t)
 GEN
 shallowconcat1(GEN x)
 {
-  pari_sp av = avma, lim = stack_lim(av, 3);
+  pari_sp av = avma;
   long lx, t, i;
   GEN z;
   switch(typ(x))
@@ -293,6 +297,7 @@ shallowconcat1(GEN x)
       if (lx==1) pari_err_DOMAIN("concat","vector","=",x,x);
       break;
     case t_LIST:
+      if (list_typ(x)!=t_LIST_RAW) pari_err_TYPE("concat",x);
       if (!list_data(x)) pari_err_DOMAIN("concat","vector","=",x,x);
       x = list_data(x); lx = lg(x);
       break;
@@ -310,7 +315,7 @@ shallowconcat1(GEN x)
   }
   for (; i<lx; i++) {
     z = shallowconcat(z, gel(x,i));
-    if (low_stack(lim, stack_lim(av,3)))
+    if (gc_needed(av,3))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"concat: i = %ld", i);
       z = gerepilecopy(av, z);
@@ -320,7 +325,7 @@ shallowconcat1(GEN x)
 }
 
 GEN
-concat1(GEN x)
+gconcat1(GEN x)
 {
   pari_sp av = avma;
   return gerepilecopy(av, shallowconcat1(x));
@@ -453,12 +458,12 @@ matconcat(GEN v)
 }
 
 GEN
-concat(GEN x, GEN y)
+gconcat(GEN x, GEN y)
 {
   long tx, lx,ty,ly,i;
   GEN z,p1;
 
-  if (!y) return concat1(x);
+  if (!y) return gconcat1(x);
   tx = typ(x);
   ty = typ(y);
   if (tx==t_STR  || ty==t_STR)
@@ -531,12 +536,12 @@ concat(GEN x, GEN y)
       switch(ty)
       {
         case t_COL:
-          if (lx<=2) return (lx==1)? gcopy(y): concat(gel(x,1),y);
+          if (lx<=2) return (lx==1)? gcopy(y): gconcat(gel(x,1),y);
           if (ly>=3) break;
-          return (ly==1)? gcopy(x): concat(x,gel(y,1));
+          return (ly==1)? gcopy(x): gconcat(x,gel(y,1));
         case t_MAT:
           z=cgetg(ly,t_MAT); if (lx != ly) break;
-          for (i=1; i<ly; i++) gel(z,i) = concat(gel(x,i),gel(y,i));
+          for (i=1; i<ly; i++) gel(z,i) = gconcat(gel(x,i),gel(y,i));
           return z;
       }
       break;
@@ -545,9 +550,9 @@ concat(GEN x, GEN y)
       switch(ty)
       {
         case t_VEC:
-          if (lx<=2) return (lx==1)? gcopy(y): concat(gel(x,1),y);
+          if (lx<=2) return (lx==1)? gcopy(y): gconcat(gel(x,1),y);
           if (ly>=3) break;
-          return (ly==1)? gcopy(x): concat(x,gel(y,1));
+          return (ly==1)? gcopy(x): gconcat(x,gel(y,1));
         case t_MAT:
           if (lx != lgcols(y)) break;
           z=cgetg(ly+1,t_MAT); gel(z,1) = gcopy(x);
@@ -561,7 +566,7 @@ concat(GEN x, GEN y)
       {
         case t_VEC:
           z=cgetg(lx,t_MAT); if (ly != lx) break;
-          for (i=1; i<lx; i++) gel(z,i) = concat(gel(x,i),gel(y,i));
+          for (i=1; i<lx; i++) gel(z,i) = gconcat(gel(x,i),gel(y,i));
           return z;
         case t_COL:
           if (ly != lgcols(x)) break;
