@@ -19,30 +19,25 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 /**                                                                     **/
 /*************************************************************************/
 
-static int is2sparse(GEN x)
+static int
+is2sparse(GEN x)
 {
   long i, l = lg(x);
   if (odd(l-3)) return 0;
   for(i=3; i<l; i+=2)
-    if (signe(gel(x,i)))
-      return 0;
+    if (signe(gel(x,i))) return 0;
   return 1;
 }
 
 static GEN
 galoisconj1(GEN nf)
 {
-  GEN x = get_nfpol(nf, &nf), y, z;
+  GEN x = get_nfpol(nf, &nf), f = nf? nf : x, y, z;
   long i, lz, v = varn(x), nbmax;
   pari_sp av = avma;
   RgX_check_ZX(x, "nfgaloisconj");
   nbmax = numberofconjugates(x, 2);
-  if (nbmax==1)
-  {
-    GEN res = cgetg(2,t_COL);
-    gel(res,1) = pol_x(v);
-    return res;
-  }
+  if (nbmax==1) retmkcol(pol_x(v));
   if (nbmax==2 && is2sparse(x))
   {
     GEN res = cgetg(3,t_COL);
@@ -50,18 +45,8 @@ galoisconj1(GEN nf)
     gel(res,2) = pol_x(v);
     return res;
   }
-  if (v == 0)
-  {
-    long w = fetch_var();
-    if (nf) y = gsubst(nf, 0, pol_x(w));
-    else { y = leafcopy(x); setvarn(y, w); }
-  }
-  else
-  {
-    y = x;
-    x = leafcopy(x); setvarn(x, 0);
-  }
-  z = nfroots(y, x); lz = lg(z);
+  x = leafcopy(x); setvarn(x, fetch_var_higher());
+  z = nfroots(f, x); lz = lg(z);
   y = cgetg(lz, t_COL);
   for (i = 1; i < lz; i++)
   {
@@ -69,93 +54,10 @@ galoisconj1(GEN nf)
     if (typ(t) == t_POL) setvarn(t, v);
     gel(y,i) = t;
   }
-  if (v == 0) delete_var();
+  (void)delete_var();
   return gerepileupto(av, y);
 }
 
-/* nbmax: bound for the number of conjugates */
-static GEN
-galoisconj2pol(GEN x, long prec)
-{
-  long i, n, v, nbauto, nbmax;
-  pari_sp av = avma;
-  GEN y, w, polr, p1, p2;
-  n = degpol(x);
-  if (n <= 0) return cgetg(1, t_VEC);
-  RgX_check_ZX(x, "galoisconj2pol");
-  nbmax = numberofconjugates(x, 2);
-  polr = QX_complex_roots(x, prec);
-  p1 = gel(polr,1);
-  /* accuracy in decimal digits */
-  prec = (long)prec2nbits_mul(prec, LOG10_2 * 0.75);
-  nbauto = 1;
-  w = cgetg(n + 2, t_VEC);
-  gel(w,1) = gen_1;
-  for (i = 2; i <= n; i++) gel(w,i) = gmul(p1, gel(w,i-1));
-  v = varn(x);
-  y = cgetg(nbmax + 1, t_COL);
-  gel(y,1) = pol_x(v);
-  for (i = 2; i <= n && nbauto < nbmax; i++)
-  {
-    gel(w,n+1) = gel(polr,i);
-    p1 = lindep2(w, prec);
-    if (signe(gel(p1,n+1)))
-    {
-      p1[0] = evallg(n+1) | evaltyp(t_COL);
-      p2 = gdiv(RgV_to_RgX(p1, v), negi(gel(p1,n+1)));
-      if (gdvd(poleval(x, p2), x))
-      {
-        gel(y,++nbauto) = p2;
-        if (DEBUGLEVEL > 1) err_printf("conjugate %ld: %Ps\n", i, y[nbauto]);
-      }
-    }
-  }
-  if (nbauto < nbmax)
-    pari_warn(warner, "conjugates list may be incomplete in nfgaloisconj");
-  setlg(y, 1 + nbauto);
-  return gerepileupto(av, gen_sort(y, (void*)&gcmp, &gen_cmp_RgX));
-}
-
-static GEN
-galoisconj2(GEN nf, long prec)
-{
-  long i, n, nbauto, nbmax;
-  pari_sp av = avma;
-  GEN T = get_nfpol(nf,&nf), y, w, polr, p1, p2;
-
-  if (!nf) return galoisconj2pol(T, prec);
-  n = degpol(T);
-  if (n <= 0) return cgetg(1, t_VEC);
-  nbmax = numberofconjugates(T, 2);
-  y = cgetg(nbmax + 1, t_COL);
-  gel(y,1) = pol_x(varn(T));
-  /* accuracy in decimal digits */
-  prec = (long)prec2nbits_mul(nf_get_prec(nf), LOG10_2 * 0.75);
-  nbauto = 1;
-  polr = nf_get_allroots(nf);
-  p2 = nf_get_M(nf);
-  w = cgetg(n + 2, t_VEC);
-  for (i = 1; i <= n; i++) gel(w,i) = gcoeff(p2, 1, i);
-  for (i = 2; i <= n && nbauto < nbmax; i++)
-  {
-    gel(w,n+1) = gel(polr,i);
-    p1 = lindep2(w, prec);
-    if (signe(gel(p1,n+1)))
-    {
-      p1[0] = evallg(n+1) | evaltyp(t_COL);
-      p2 = gdiv(coltoliftalg(nf, p1), negi(gel(p1, n+1)));
-      if (gdvd(poleval(T, p2), T))
-      {
-        gel(y,++nbauto) = p2;
-        if (DEBUGLEVEL > 1) err_printf("conjugate %ld: %Ps\n", i, y[nbauto]);
-      }
-    }
-  }
-  if (nbauto < nbmax)
-    pari_warn(warner, "conjugates list may be incomplete in nfgaloisconj");
-  setlg(y, 1 + nbauto);
-  return gerepileupto(av, gen_sort(y, (void*)&gcmp, &gen_cmp_RgX));
-}
 /*************************************************************************/
 /**                                                                     **/
 /**                           GALOISCONJ4                               **/
@@ -211,6 +113,7 @@ enum ga_code {ga_all_normal=1,ga_ext_2=2,ga_non_wss=4};
 struct galois_analysis {
   long p; /* prime to be lifted */
   long deg; /* degree of the lift */
+  long mindeg; /* minimal acceptable degree */
   long ord;
   long l; /* l: prime number such that T is totally split mod l */
   long p4;
@@ -224,7 +127,9 @@ struct galois_frobenius {
   GEN psi;
 };
 
-GEN
+/* given complex roots L[i], i <= n of some monic T in C[X], return
+ * the T'(L[i]), computed stably via products of differences */
+static GEN
 vandermondeinverseprep(GEN L)
 {
   long i, j, n = lg(L);
@@ -237,7 +142,7 @@ vandermondeinverseprep(GEN L)
     long k = 1;
     for (j = 1; j < n; j++)
       if (i != j) gel(W, k++) = gsub(gel(L,i),gel(L,j));
-    gel(V,i) = gerepileupto(ltop,divide_conquer_prod(W,&gmul));
+    gel(V,i) = gerepileupto(ltop, RgV_prod(W));
   }
   return V;
 }
@@ -250,13 +155,14 @@ vandermondeinverse(GEN L, GEN T, GEN den, GEN prep)
   long i, n = lg(L)-1;
   GEN M, P;
   if (!prep) prep = vandermondeinverseprep(L);
+  if (den && !equali1(den)) T = RgX_Rg_mul(T,den);
   M = cgetg(n+1, t_MAT);
   for (i = 1; i <= n; i++)
   {
     P = RgX_Rg_div(RgX_div_by_X_x(T, gel(L,i), NULL), gel(prep,i));
-    gel(M,i) = RgX_to_RgV(P,n);
+    gel(M,i) = RgX_to_RgC(P,n);
   }
-  return den? gerepileupto(ltop, gmul(den, M)): gerepilecopy(ltop, M);
+  return gerepilecopy(ltop, M);
 }
 
 /* #r = r1 + r2 */
@@ -341,9 +247,9 @@ initgaloisborne(GEN T, GEN dn, long prec, GEN *ptL, GEN *ptprep, GEN *ptdis)
   prep = vandermondeinverseprep(L);
   if (!dn)
   {
-    GEN dis, res = divide_conquer_prod(gabs(prep,prec), mpmul);
-    /*Add +2 to cater for accuracy error in res */
-    dis = ZX_disc_all(T, 2+expi(ceil_safe(res)));
+    GEN dis, res = RgV_prod(gabs(prep,prec));
+    /*Add +1 to cater for accuracy error in res */
+    dis = ZX_disc_all(T, 1+expi(ceil_safe(res)));
     den = indexpartial(T,dis);
     if (ptdis) *ptdis = dis;
   }
@@ -357,17 +263,19 @@ initgaloisborne(GEN T, GEN dn, long prec, GEN *ptL, GEN *ptprep, GEN *ptdis)
   *ptL = L; return den;
 }
 
-/* ||| M ||| with respect to || x ||_oo. Assume M square t_MAT */
+/* ||| M ||| with respect to || x ||_oo, M t_MAT */
 GEN
 matrixnorm(GEN M, long prec)
 {
-  long i,j, n = lg(M);
+  long i,j,m, l = lg(M);
   GEN B = real_0(prec);
 
-  for (i = 1; i < n; i++)
+  if (l == 1) return B;
+  m = lgcols(M);
+  for (i = 1; i < m; i++)
   {
     GEN z = gabs(gcoeff(M,i,1), prec);
-    for (j = 2; j < n; j++) z = gadd(z, gabs(gcoeff(M,i,j), prec));
+    for (j = 2; j < l; j++) z = gadd(z, gabs(gcoeff(M,i,j), prec));
     if (gcmp(z, B) > 0) B = z;
   }
   return B;
@@ -376,7 +284,7 @@ matrixnorm(GEN M, long prec)
 static GEN
 galoisborne(GEN T, GEN dn, struct galois_borne *gb, long d)
 {
-  pari_sp ltop = avma, av2;
+  pari_sp ltop, av2;
   GEN borne, borneroots, borneabs;
   long prec;
   GEN L, M, prep, den;
@@ -384,7 +292,8 @@ galoisborne(GEN T, GEN dn, struct galois_borne *gb, long d)
 
   prec = nbits2prec(bit_accuracy(ZX_max_lg(T)));
   den = initgaloisborne(T,dn,prec, &L,&prep,NULL);
-  if (!dn) den = gclone(den);
+  if (!dn) dn = den;
+  ltop = avma;
   if (DEBUGLEVEL>=4) timer_start(&ti);
   M = vandermondeinverse(L, RgX_gtofp(T, prec), den, prep);
   if (DEBUGLEVEL>=4) timer_printf(&ti,"vandermondeinverse");
@@ -394,18 +303,17 @@ galoisborne(GEN T, GEN dn, struct galois_borne *gb, long d)
   borneroots = ceil_safe(gmul(borne, borneroots));
   av2 = avma;
   /*We use d-1 test, so we must overlift to 2^BITS_IN_LONG*/
-  gb->valsol = logint(shifti(borneroots,2+BITS_IN_LONG), gb->l,NULL);
-  gb->valabs = logint(shifti(borneabs,2), gb->l, NULL);
+  gb->valsol = logint(shifti(borneroots,2+BITS_IN_LONG), gb->l) + 1;
+  gb->valabs = logint(shifti(borneabs,2), gb->l) + 1;
   gb->valabs = maxss(gb->valsol, gb->valabs);
   if (DEBUGLEVEL >= 4)
-    err_printf("GaloisConj:val1=%ld val2=%ld\n", gb->valsol, gb->valabs);
+    err_printf("GaloisConj: val1=%ld val2=%ld\n", gb->valsol, gb->valabs);
   avma = av2;
   gb->bornesol = gerepileuptoint(ltop, shifti(borneroots,1));
   if (DEBUGLEVEL >= 9)
     err_printf("GaloisConj: Bound %Ps\n",borneroots);
   gb->ladicsol = powiu(gb->l, gb->valsol);
   gb->ladicabs = powiu(gb->l, gb->valabs);
-  if (!dn) { dn = icopy(den); gunclone(den); }
   return dn;
 }
 
@@ -425,7 +333,7 @@ initlift(GEN T, GEN den, GEN p, GEN L, GEN Lden, struct galois_borne *gb, struct
   gl->p = p;
   gl->L = L;
   gl->Lden = Lden;
-  e = logint(shifti(gb->bornesol, 2+BITS_IN_LONG),p,NULL);
+  e = logint(shifti(gb->bornesol, 2+BITS_IN_LONG),p) + 1;
   avma = av;
   if (e < 2) e = 2;
   gl->e = e;
@@ -442,7 +350,7 @@ poltopermtest(GEN f, struct galois_lift *gl, GEN pf)
   GEN fx, fp, B = gl->gb->bornesol;
   long i, j, ll;
   for (i = 2; i < lg(f); i++)
-    if (absi_cmp(gel(f,i),B) > 0)
+    if (abscmpii(gel(f,i),B) > 0)
     {
       if (DEBUGLEVEL>=4) err_printf("GaloisConj: Solution too large.\n");
       if (DEBUGLEVEL>=8) err_printf("f=%Ps\n borne=%Ps\n",f,B);
@@ -461,9 +369,19 @@ poltopermtest(GEN f, struct galois_lift *gl, GEN pf)
   return 1;
 }
 
-static long
-monoratlift(GEN S, GEN q, GEN qm1old,struct galois_lift *gl, GEN frob)
+struct monoratlift
 {
+  struct galois_lift *gl;
+  GEN frob;
+  long early;
+};
+
+static int
+monoratlift(void *E, GEN S, GEN q)
+{
+  struct monoratlift *d = (struct monoratlift *) E;
+  struct galois_lift *gl = d->gl;
+  GEN qm1old = sqrti(shifti(q,-2));
   GEN tlift = FpX_ratlift(S,q,qm1old,qm1old,gl->den);
   if (tlift)
   {
@@ -474,9 +392,10 @@ monoratlift(GEN S, GEN q, GEN qm1old,struct galois_lift *gl, GEN frob)
       GEN N = gl->gb->ladicsol, N2 = shifti(N,-1);
       tlift = FpX_center(FpX_red(Q_muli_to_int(tlift, gl->den), N), N,N2);
     }
-    if (poltopermtest(tlift, gl, frob))
+    if (poltopermtest(tlift, gl, d->frob))
     {
       if(DEBUGLEVEL>=4) err_printf("MonomorphismLift: true early solution.\n");
+      d->early = 1;
       avma = ltop; return 1;
     }
     avma = ltop;
@@ -488,51 +407,18 @@ monoratlift(GEN S, GEN q, GEN qm1old,struct galois_lift *gl, GEN frob)
 static GEN
 monomorphismratlift(GEN P, GEN S, struct galois_lift *gl, GEN frob)
 {
-  pari_sp ltop, lbot;
-  GEN Q = gl->T, p = gl->p, qold = NULL, q = p;
-  GEN Sr, Spow, Wr = NULL, W, Prold = NULL, Pr, Qrold = NULL, Qr;
-  long e = gl->e, level = 1, rt;
-  ulong mask;
-  GEN *gptr[2];
   pari_timer ti;
-
-  if (DEBUGLEVEL == 1) timer_start(&ti);
-  rt = brent_kung_optpow(degpol(P), 4, 3);
-  mask = quadratic_prec_mask(e);
-  Pr = FpX_red(P,q);
-  Qr = (P==Q)? Pr: FpX_red(Q, q);/*A little speed up for automorphismlift*/
-  W = FpXQ_inv(FpX_FpXQ_eval(ZX_deriv(Pr),S, Qr,q), Qr,q);
-  gptr[0] = &S;
-  gptr[1] = &Wr;
-  for (;;)
+  if (DEBUGLEVEL >= 1) timer_start(&ti);
+  if (frob)
   {
-    if (DEBUGLEVEL>=2) { level = (level<<1) - (mask & 1); timer_start(&ti); }
-    q = sqri(q);
-    if (mask & 1) q = diviiexact(q, p);
-    mask >>= 1;
-    Pr = FpX_red(P, q);
-    Qr = (P==Q)? Pr: FpX_red(Q, q);/*A little speed up for automorphismlift*/
-    ltop = avma;
-    Sr = S;
-    Spow = FpXQ_powers(Sr, rt, Qr, q);
-    if (Wr)
-    {
-      W = FpXQ_mul(Wr, FpX_FpXQV_eval(ZX_deriv(Prold),FpXV_red(Spow,qold),Qrold,qold),
-                   Qrold,qold);
-      W = FpXQ_mul(Wr, Fp_FpX_sub(gen_2, W, qold), Qrold,qold);
-    }
-    Wr = W;
-    S = FpXQ_mul(Wr, FpX_FpXQV_eval(Pr, Spow, Qr, q),Qr,q);
-    lbot = avma;
-    S = FpX_sub(Sr, S, q);
-    if (DEBUGLEVEL >= 4) timer_printf(&ti, "MonomorphismLift: lift to prec %d",level);
-    if (mask == 1) break;
-    Wr = ZX_copy(Wr);
-    gerepilemanysp(ltop, lbot, gptr, 2);
-    if (qold && frob && monoratlift(S,q,diviiexact(qold,p),gl,frob)) return NULL;
-    qold = q; Prold = Pr; Qrold = Qr;
+    struct monoratlift d;
+    d.gl = gl; d.frob = frob; d.early = 0;
+    S = ZpX_ZpXQ_liftroot_ea(P,S,gl->T,gl->p, gl->e, (void*)&d, monoratlift);
+    S = d.early ? NULL: S;
   }
-  if (DEBUGLEVEL == 1) timer_printf(&ti, "monomorphismlift()");
+  else
+    S = ZpX_ZpXQ_liftroot(P,S,gl->T,gl->p, gl->e);
+  if (DEBUGLEVEL >= 1) timer_printf(&ti, "monomorphismlift()");
   return S;
 }
 
@@ -545,26 +431,12 @@ automorphismlift(GEN S, struct galois_lift *gl, GEN frob)
   return monomorphismratlift(gl->T, S, gl, frob);
 }
 
-/* Let P be a polynomial in Z[X] , p a prime number, S in Fp[X]/(Q) so
- * that T(S)=0 [p,T]. Lift S in S_0 so that T(S_0)=0 [Q,p^e]
- * Unclean stack */
-
-GEN
-monomorphismlift(GEN P, GEN S, GEN Q, GEN p, long e)
-{
-  struct galois_lift gl;
-  gl.T = Q;
-  gl.p = p;
-  gl.e = e;
-  return monomorphismratlift(P,S,&gl,NULL);
-}
-
 static GEN
 galoisdolift(struct galois_lift *gl, GEN frob)
 {
   pari_sp av = avma;
   GEN Tp = FpX_red(gl->T, gl->p);
-  GEN S = FpXQ_pow(pol_x(varn(Tp)), gl->p, Tp, gl->p);
+  GEN S = FpX_Frobenius(Tp, gl->p);
   return gerepileupto(av, automorphismlift(S, gl, frob));
 }
 
@@ -593,7 +465,7 @@ inittestlift(GEN plift, GEN Tmod, struct galois_lift *gl,
  * So we compute z = - sum b_i [mod 2^B] and check if 0 <= z <= n. */
 
 /* Assume 0 <= x < mod. */
-long
+static ulong
 intheadlong(GEN x, GEN mod)
 {
   pari_sp av = avma;
@@ -608,7 +480,7 @@ vecheadlong(GEN W, GEN mod)
   for(i=1; i<l; i++) V[i] = intheadlong(gel(W,i), mod);
   return V;
 }
-GEN
+static GEN
 matheadlong(GEN W, GEN mod)
 {
   long i, l = lg(W);
@@ -616,7 +488,7 @@ matheadlong(GEN W, GEN mod)
   for(i=1; i<l; i++) gel(V,i) = vecheadlong(gel(W,i), mod);
   return V;
 }
-long
+static ulong
 polheadlong(GEN P, long n, GEN mod)
 {
   return (lg(P)>n+2)? intheadlong(gel(P,n+2),mod): 0;
@@ -631,7 +503,8 @@ frobeniusliftall(GEN sg, long el, GEN *psi, struct galois_lift *gl,
   pari_sp av, ltop2, ltop = avma;
   long i,j,k, c = lg(sg)-1, n = lg(gl->L)-1, m = gt->g, d = m / c;
   GEN pf, u, v, C, Cd, SG, cache;
-  long N1, N2, R1, Ni, Z, ord = gt->f, c_idx = gt->g-1;
+  long N1, N2, R1, Ni, ord = gt->f, c_idx = gt->g-1;
+  ulong headcache;
   long hop = 0;
   GEN NN, NQ;
   pari_timer ti;
@@ -640,10 +513,10 @@ frobeniusliftall(GEN sg, long el, GEN *psi, struct galois_lift *gl,
   ltop2 = avma;
   NN = diviiexact(mpfact(m), mului(c, powiu(mpfact(d), c)));
   if (DEBUGLEVEL >= 4)
-    err_printf("GaloisConj:I will try %Ps permutations\n", NN);
+    err_printf("GaloisConj: I will try %Ps permutations\n", NN);
   N1=10000000;
   NQ=divis_rem(NN,N1,&R1);
-  if (cmpiu(NQ,1000000000)>0)
+  if (abscmpiu(NQ,1000000000)>0)
   {
     pari_warn(warner,"Combinatorics too hard : would need %Ps tests!\n"
         "I will skip it, but it may induce an infinite loop",NN);
@@ -659,7 +532,7 @@ frobeniusliftall(GEN sg, long el, GEN *psi, struct galois_lift *gl,
   SG = cgetg(lg(sg),t_VECSMALL);
   for(i=1; i<lg(SG); i++) SG[i] = (el*sg[i])%ord + 1;
   cache = cgetg(m+1,t_VECSMALL); cache[m] = polheadlong(v,1,gl->Q);
-  Z = polheadlong(v,2,gl->Q);
+  headcache = polheadlong(v,2,gl->Q);
   for (i = 1; i < m; i++) pf[i] = 1 + i/d;
   av = avma;
   for (Ni = 0, i = 0; ;i++)
@@ -680,9 +553,9 @@ frobeniusliftall(GEN sg, long el, GEN *psi, struct galois_lift *gl,
     }
     if (headlongisint(uel(cache,1),n))
     {
-      long ZZ = Z;
-      for (j = 1; j < m; j++) ZZ += polheadlong(gmael(C,SG[pf[j]],j),2,gl->Q);
-      if (headlongisint(ZZ,n))
+      ulong head = headcache;
+      for (j = 1; j < m; j++) head += polheadlong(gmael(C,SG[pf[j]],j),2,gl->Q);
+      if (headlongisint(head,n))
       {
         u = v;
         for (j = 1; j < m; j++) u = ZX_add(u, gmael(C,SG[pf[j]],j));
@@ -736,7 +609,7 @@ inittest(GEN L, GEN M, GEN borne, GEN ladic, struct galois_test *td)
 {
   long i, n = lg(L)-1;
   GEN p = cgetg(n+1, t_VECSMALL);
-  if (DEBUGLEVEL >= 8) err_printf("GaloisConj:Init Test\n");
+  if (DEBUGLEVEL >= 8) err_printf("GaloisConj: Init Test\n");
   td->order = p;
   for (i = 1; i <= n-2; i++) p[i] = i+2;
   p[n-1] = 1; p[n] = 2;
@@ -784,9 +657,9 @@ galois_test_perm(struct galois_test *td, GEN pf)
     GEN PW = gel(td->PV, ord);
     if (PW)
     {
-      ulong Z = umael(PW,1,pf[1]);
-      for (j = 2; j <= n; j++) Z += umael(PW,j,pf[j]);
-      if (!headlongisint(Z,n)) break;
+      ulong head = umael(PW,1,pf[1]);
+      for (j = 2; j <= n; j++) head += umael(PW,j,pf[j]);
+      if (!headlongisint(head,n)) break;
     } else
     {
       if (!P) P = vecpermute(td->L, pf);
@@ -848,13 +721,14 @@ testpermutation(GEN F, GEN B, GEN x, long s, long e, long cut,
     if (i == d) { i = 0; j++; }
   }
   NN = divis(powuu(b, c * (d - d/e)), cut);
-  if (DEBUGLEVEL>=4) err_printf("GaloisConj:I will try %Ps permutations\n", NN);
+  if (DEBUGLEVEL>=4) err_printf("GaloisConj: I will try %Ps permutations\n", NN);
   N1 = 1000000;
   NQ = divis_rem(NN,N1,&R1);
-  if (cmpiu(NQ,100000000)>0)
+  if (abscmpiu(NQ,100000000)>0)
   {
     avma = avm;
-    pari_warn(warner,"Combinatorics too hard : would need %Ps tests!\n I'll skip it but you will get a partial result...",NN);
+    pari_warn(warner,"Combinatorics too hard: would need %Ps tests!\n"
+                     "I'll skip it but you will get a partial result...",NN);
     return identity_perm(n);
   }
   N2 = itos(NQ);
@@ -920,7 +794,7 @@ testpermutation(GEN F, GEN B, GEN x, long s, long e, long cut,
           GEN nb = addis(mulss(l2,N1),l1);
           timer_printf(&ti, "testpermutation(%Ps)", nb);
           if (DEBUGLEVEL >= 2 && hop)
-            err_printf("GaloisConj:%d hop over %Ps iterations\n", hop, nb);
+            err_printf("GaloisConj: %d hop over %Ps iterations\n", hop, nb);
         }
         avma = av; return pf;
       }
@@ -931,25 +805,20 @@ testpermutation(GEN F, GEN B, GEN x, long s, long e, long cut,
   {
     timer_printf(&ti, "testpermutation(%Ps)", NN);
     if (DEBUGLEVEL >= 2 && hop)
-      err_printf("GaloisConj:%d hop over %Ps iterations\n", hop, NN);
+      err_printf("GaloisConj: %d hop over %Ps iterations\n", hop, NN);
   }
   avma = avm; return NULL;
 }
 
 /* List of subgroups of (Z/mZ)^* whose order divide o, and return the list
  * of their elements, sorted by increasing order */
-GEN
+static GEN
 listznstarelts(long m, long o)
 {
   pari_sp av = avma;
-  GEN L, zn, zns, res;
+  GEN L, zn, zns;
   long i, phi, ind, l;
-  if (m == 2)
-  {
-    res = cgetg(2, t_VEC);
-    gel(res,1) = mkvecsmall(1);
-    return res;
-  }
+  if (m == 2) retmkvec(mkvecsmall(1));
   zn = znstar(stoi(m));
   phi = itos(gel(zn,1));
   o = ugcd(o, phi); /* do we impose this on input ? */
@@ -1124,30 +993,7 @@ fixedfieldinclusion(GEN O, GEN PL)
   return S;
 }
 
-/*Usually mod > den so there is no need to reduce it.*/
-GEN
-vandermondeinversemod(GEN L, GEN T, GEN den, GEN mod)
-{
-  pari_sp av;
-  long i, n = lg(L);
-  GEN P, Tp, M = cgetg(n, t_MAT);
-  av = avma;
-  Tp = gclone(FpX_deriv(T,mod)); /*clone*/
-  avma = av;
-  for (i = 1; i < n; i++)
-  {
-    GEN z;
-    av = avma;
-    z = Fp_inv(FpX_eval(Tp, gel(L,i),mod),mod);
-    z = Fp_mul(den,z,mod);
-    P = FpX_Fp_mul(FpX_div_by_X_x(T, gel(L,i), mod, NULL), z, mod);
-    gel(M,i) = gerepilecopy(av, RgX_to_RgV(P, n-1));
-  }
-  gunclone(Tp); /*unclone*/
-  return M;
-}
-
-/* Polynomial associated to a vector of conjugates. Not stack clean */
+/* Polynomial attached to a vector of conjugates. Not stack clean */
 static GEN
 vectopol(GEN v, GEN M, GEN den , GEN mod, GEN mod2, long x)
 {
@@ -1188,19 +1034,16 @@ notgalois(long p, struct galois_analysis *ga)
   ga->deg = 0;
 }
 
-#define numberof(x) (long)(sizeof(x) / sizeof((x)[0]))
-
 /*Gather information about the group*/
 static long
 init_group(long n, long np, GEN Fp, GEN Fe, long *porder)
 {
-  /*TODO: complete the table to at least 200*/
-  const long prim_nonss_orders[] = { 36,48,56,60,72,75,80,96,108,120,132 };
+  const long prim_nonwss_orders[] = { 36,48,56,60,75,80,196,200 };
   long i, phi_order = 1, order = 1, group = 0;
 
  /* non-WSS groups of this order? */
-  for (i=0; i < numberof(prim_nonss_orders); i++)
-    if (n % prim_nonss_orders[i] == 0) { group |= ga_non_wss; break; }
+  for (i=0; i < (long)numberof(prim_nonwss_orders); i++)
+    if (n % prim_nonwss_orders[i] == 0) { group |= ga_non_wss; break; }
   if (np == 2 && Fp[2] == 3 && Fe[2] == 1 && Fe[1] > 2) group |= ga_ext_2;
 
   for (i = np; i > 0; i--)
@@ -1334,6 +1177,7 @@ galoisanalysis(GEN T, struct galois_analysis *ga, long calcul_l)
   }
   ga->group = (enum ga_code)group;
   ga->deg = deg;
+  ga->mindeg =  n == 135 ? 15: 0; /* otherwise the second phase is too slow */
   ga->ord = order;
   ga->l  = O[1];
   ga->p4 = n >= 4 ? O[4] : 0;
@@ -1374,7 +1218,7 @@ a4galoisgen(struct galois_test *td)
   /* N = itos(gdiv(mpfact(n), mpfact(n >> 1))) >> (n >> 1); */
   /* n = 2k = 12; N = (2k)! / (k! * 2^k) = 10395 */
   N = 10395;
-  if (DEBUGLEVEL>=4) err_printf("A4GaloisConj:will test %ld permutations\n", N);
+  if (DEBUGLEVEL>=4) err_printf("A4GaloisConj: will test %ld permutations\n", N);
   uel(ar,4) = umael(MT,11,12);
   uel(ar,3) = uel(ar,4) + umael(MT,9,10);
   uel(ar,2) = uel(ar,3) + umael(MT,7,8);
@@ -1438,7 +1282,7 @@ a4galoisgen(struct galois_test *td)
   if (i == N) { avma = ltop; return gen_0; }
   /* N = itos(gdiv(mpfact(n >> 1), mpfact(n >> 2))) >> 1; */
   N = 60;
-  if (DEBUGLEVEL >= 4) err_printf("A4GaloisConj:sigma=%Ps \n", pft);
+  if (DEBUGLEVEL >= 4) err_printf("A4GaloisConj: sigma=%Ps \n", pft);
   for (k = 0; k < n; k += 4)
   {
     u[k+3] = t[k+3];
@@ -1498,13 +1342,13 @@ a4galoisgen(struct galois_test *td)
   if (i == N) { avma = ltop; return gen_0; }
   if (DEBUGLEVEL >= 1 && hop)
     err_printf("A4GaloisConj: %ld hop over %ld iterations\n", hop, N);
-  if (DEBUGLEVEL >= 4) err_printf("A4GaloisConj:tau=%Ps \n", pfu);
+  if (DEBUGLEVEL >= 4) err_printf("A4GaloisConj: tau=%Ps \n", pfu);
   avma = av2;
   orb = mkvec2(pft,pfu);
   O = vecperm_orbits(orb, 12);
   if (DEBUGLEVEL >= 4) {
-    err_printf("A4GaloisConj:orb=%Ps\n", orb);
-    err_printf("A4GaloisConj:O=%Ps \n", O);
+    err_printf("A4GaloisConj: orb=%Ps\n", orb);
+    err_printf("A4GaloisConj: O=%Ps \n", O);
   }
   av2 = avma;
   O1 = gel(O,1); O2 = gel(O,2); O3 = gel(O,3);
@@ -1537,7 +1381,7 @@ a4galoisgen(struct galois_test *td)
       {
         avma = av;
         if (DEBUGLEVEL >= 1)
-          err_printf("A4GaloisConj:%ld hop over %d iterations max\n",
+          err_printf("A4GaloisConj: %ld hop over %d iterations max\n",
                      hop, 10395 + 68);
         return res;
       }
@@ -1574,7 +1418,7 @@ s4test(GEN u, GEN liftpow, struct galois_lift *gl, GEN phi)
   res = remii(res,Q);
   if (gl->den != gen_1) res = mulii(res, gl->den);
   res = centermodii(res, Q,Q2);
-  if (absi_cmp(res, gl->gb->bornesol) > 0) { avma = av; return 0; }
+  if (abscmpii(res, gl->gb->bornesol) > 0) { avma = av; return 0; }
   res = scalar_ZX_shallow(gel(u,2),varn(u));
   for (i = 1; i < d ; i++)
     if (lg(gel(liftpow,i))>2)
@@ -1654,7 +1498,7 @@ s4galoisgen(struct galois_lift *gl)
     gel(misom,i) = cgetg(lg(Tmod), t_COL);
     gel(isom,i) = FpX_ffisom(gel(Tmod,1), gel(Tmod,i), p);
     if (DEBUGLEVEL >= 6)
-      err_printf("S4GaloisConj:Computing isomorphisms %d:%Ps\n", i,
+      err_printf("S4GaloisConj: Computing isomorphisms %d:%Ps\n", i,
                  gel(isom,i));
     gel(isominv,i) = FpXQ_ffisom_inv(gel(isom,i), gel(Tmod,i),p);
   }
@@ -1690,7 +1534,7 @@ s4galoisgen(struct galois_lift *gl)
           u3 = lincomb(gel(Bcoeff,sg[1]),gel(Bcoeff,sg[2]), pauto,j3);
           u3 = FpX_rem(FpX_add(u2, u3, Q), TQ,Q);
           if (DEBUGLEVEL >= 4)
-            err_printf("S4GaloisConj:Testing %d/3:%d/4:%d/4:%d/4:%Ps\n",
+            err_printf("S4GaloisConj: Testing %d/3:%d/4:%d/4:%d/4:%Ps\n",
                        i,j1,j2,j3, sg);
           if (s4test(u3, liftpow, gl, sigma))
           {
@@ -1704,8 +1548,8 @@ s4galoisgen(struct galois_lift *gl)
   }
   avma = ltop; return gen_0;
 suites4:
-  if (DEBUGLEVEL >= 4) err_printf("S4GaloisConj:sigma=%Ps\n", sigma);
-  if (DEBUGLEVEL >= 4) err_printf("S4GaloisConj:pj=%Ps\n", pj);
+  if (DEBUGLEVEL >= 4) err_printf("S4GaloisConj: sigma=%Ps\n", sigma);
+  if (DEBUGLEVEL >= 4) err_printf("S4GaloisConj: pj=%Ps\n", pj);
   avma = av;
   for (j = 1; j <= 3; j++)
   {
@@ -1733,7 +1577,7 @@ suites4:
           pj[4] = i;
           pj[5] = (i + pj[2] - pj[1]) & 3;
           if (DEBUGLEVEL >= 4)
-            err_printf("S4GaloisConj:Testing %d/3:%d/2:%d/2:%d/4:%Ps:%Ps\n",
+            err_printf("S4GaloisConj: Testing %d/3:%d/2:%d/2:%d/4:%Ps:%Ps\n",
                        j-1, w >> 1, l, i, sg, pj);
           u = ZX_add(lincomb(gel(Bcoeff,sg[1]),gel(Bcoeff,sg[3]), pauto,pj[4]),
                      lincomb(gel(Bcoeff,sg[2]),gel(Bcoeff,sg[4]), pauto,pj[5]));
@@ -1767,7 +1611,7 @@ suites4_2:
       u = FpX_add(u, lincomb(gel(Bcoeff,sg[3]), gel(Bcoeff,sg[6]), pauto, i),Q);
       u = FpX_rem(u, TQ, Q);
       if (DEBUGLEVEL >= 4)
-        err_printf("S4GaloisConj:Testing %d/8 %d:%d:%d\n", j,g,h,i);
+        err_printf("S4GaloisConj: Testing %d/8 %d:%d:%d\n", j,g,h,i);
       if (s4test(u, liftpow, gl, phi)) break;
       avma = av2;
     }
@@ -1831,11 +1675,12 @@ galoisfrobeniuslift(GEN T, GEN den, GEN L,  GEN Lden,
   struct galois_lift gl;
   long i, j, k, n = lg(L)-1, deg = 1, g = lg(gf->Tmod)-1;
   GEN F,Fp,Fe, aut, frob, ip = utoipos(gf->p), res = cgetg(lg(L), t_VECSMALL);
-  if (DEBUGLEVEL >= 4)
-    err_printf("GaloisConj:p=%ld deg=%ld fp=%ld\n", gf->p, deg, gf->fp);
   gf->psi = const_vecsmall(g,1);
   av2 = avma;
   initlift(T, den, ip, L, Lden, gb, &gl);
+  if (DEBUGLEVEL >= 4)
+    err_printf("GaloisConj: p=%ld e=%ld deg=%ld fp=%ld\n",
+                            gf->p, gl.e, deg, gf->fp);
   aut = galoisdolift(&gl, res);
   if (!aut || galoisfrobeniustest(aut,&gl,res))
   {
@@ -1945,6 +1790,13 @@ galoisfindfrobenius(GEN T, GEN L, GEN den, struct galois_frobenius *gf,
     if (frob)
     {
       GEN *gptr[3];
+      if (gf->deg < ga->mindeg)
+      {
+        if (DEBUGLEVEL >= 4)
+          err_printf("GaloisConj: lift degree too small %ld < %ld\n",
+                     gf->deg, ga->mindeg);
+        continue;
+      }
       gf->Tmod = gcopy(Ti);
       gptr[0]=&gf->Tmod; gptr[1]=&gf->psi; gptr[2]=&frob;
       gerepilemanysp(ltop,lbot,gptr,3); return frob;
@@ -2016,13 +1868,13 @@ galoisgenfixedfield(GEN Tp, GEN Pmod, GEN V, GEN ip, struct galois_borne *gb)
     if (Pgb.valabs > gb->valabs)
     {
       if (DEBUGLEVEL>=4)
-        err_printf("GaloisConj:increase prec of p-adic roots of %ld.\n"
+        err_printf("GaloisConj: increase prec of p-adic roots of %ld.\n"
             ,Pgb.valabs-gb->valabs);
       PL = ZpX_liftroots(P,PL,gb->l,Pgb.valabs);
     }
     else if (Pgb.valabs < gb->valabs)
       PL = FpC_red(PL, Pgb.ladicabs);
-    PM = vandermondeinversemod(PL, P, Pden, Pgb.ladicabs);
+    PM = FpV_invVandermonde(PL, Pden, Pgb.ladicabs);
     PG = galoisgen(P, PL, PM, Pden, &Pgb, &Pga);
     if (PG == gen_0) return NULL;
     lP = lg(gel(PG,1));
@@ -2149,11 +2001,11 @@ galoisgen(GEN T, GEN L, GEN M, GEN den, struct galois_borne *gb,
 
   if (!ga->deg) return gen_0;
   x = varn(T);
-  if (DEBUGLEVEL >= 9) err_printf("GaloisConj:denominator:%Ps\n", den);
+  if (DEBUGLEVEL >= 9) err_printf("GaloisConj: denominator:%Ps\n", den);
   if (n == 12 && ga->ord==3 && !ga->p4)
   { /* A4 is very probable: test it first */
     pari_sp av = avma;
-    if (DEBUGLEVEL >= 4) err_printf("GaloisConj:Testing A4 first\n");
+    if (DEBUGLEVEL >= 4) err_printf("GaloisConj: Testing A4 first\n");
     inittest(L, M, gb->bornesol, gb->ladicsol, &td);
     PG = a4galoisgen(&td);
     freetest(&td);
@@ -2164,7 +2016,7 @@ galoisgen(GEN T, GEN L, GEN M, GEN den, struct galois_borne *gb,
   { /* S4 is very probable: test it first */
     pari_sp av = avma;
     struct galois_lift gl;
-    if (DEBUGLEVEL >= 4) err_printf("GaloisConj:Testing S4 first\n");
+    if (DEBUGLEVEL >= 4) err_printf("GaloisConj: Testing S4 first\n");
     initlift(T, den, stoi(ga->p4), L, makeLden(L,den,gb), gb, &gl);
     PG = s4galoisgen(&gl);
     if (PG != gen_0) return gerepileupto(ltop, PG);
@@ -2176,11 +2028,11 @@ galoisgen(GEN T, GEN L, GEN M, GEN den, struct galois_borne *gb,
   Tmod = gf.Tmod;
   O = perm_cycles(frob);
   deg = lg(gel(O,1))-1;
-  if (DEBUGLEVEL >= 9) err_printf("GaloisConj:Orbite:%Ps\n", O);
+  if (DEBUGLEVEL >= 9) err_printf("GaloisConj: Orbit:%Ps\n", O);
   if (deg == n)        /* cyclic */
     return gerepilecopy(ltop, mkvec2(mkvec(frob), mkvecsmall(deg)));
   sigma = permtopol(frob, L, M, den, gb->ladicabs, shifti(gb->ladicabs,-1), x);
-  if (DEBUGLEVEL >= 9) err_printf("GaloisConj:Frobenius:%Ps\n", sigma);
+  if (DEBUGLEVEL >= 9) err_printf("GaloisConj: Frobenius:%Ps\n", sigma);
   {
     pari_sp btop=avma;
     GEN V, Tp, Sp, Pmod;
@@ -2191,7 +2043,7 @@ galoisgen(GEN T, GEN L, GEN M, GEN den, struct galois_borne *gb,
     Pmod = fixedfieldfactmod(Sp,ip,Tmod);
     PG = galoisgenfixedfield(Tp, Pmod, V, ip, gb);
     if (PG == NULL) { avma = ltop; return gen_0; }
-    if (DEBUGLEVEL >= 4) err_printf("GaloisConj:Back to Earth:%Ps\n", PG);
+    if (DEBUGLEVEL >= 4) err_printf("GaloisConj: Back to Earth:%Ps\n", PG);
     PG=gerepilecopy(btop, PG);
   }
   inittest(L, M, gb->bornesol, gb->ladicsol, &td);
@@ -2208,7 +2060,7 @@ galoisgen(GEN T, GEN L, GEN M, GEN den, struct galois_borne *gb,
     if (!pf) { freetest(&td); avma = ltop; return gen_0; }
     gel(res1, j+1) = pf;
   }
-  if (DEBUGLEVEL >= 4) err_printf("GaloisConj:Fini!\n");
+  if (DEBUGLEVEL >= 4) err_printf("GaloisConj: Fini!\n");
   freetest(&td);
   return gerepileupto(ltop, res);
 }
@@ -2223,7 +2075,7 @@ conjcyclo(GEN T, long N)
   for (i=1; i<=N; i++)
     if (ugcd(i, N)==1)
     {
-      GEN s = monomial(gen_1, i, v);
+      GEN s = pol_xn(i, v);
       if (i >= d) s = ZX_rem(s, T);
       gel(L,k++) = s;
     }
@@ -2274,10 +2126,10 @@ galoisconj4_main(GEN T, GEN den, long flag)
   if (DEBUGLEVEL >= 1) timer_start(&ti);
   den = galoisborne(T, den, &gb, degpol(T));
   if (DEBUGLEVEL >= 1) timer_printf(&ti, "galoisborne()");
-  L = rootpadicfast(T, gb.l, gb.valabs);
-  if (DEBUGLEVEL >= 1) timer_printf(&ti, "rootpadicfast()");
-  M = vandermondeinversemod(L, T, den, gb.ladicabs);
-  if (DEBUGLEVEL >= 1) timer_printf(&ti, "vandermondeinversemod()");
+  L = ZpX_roots(T, gb.l, gb.valabs);
+  if (DEBUGLEVEL >= 1) timer_printf(&ti, "ZpX_roots");
+  M = FpV_invVandermonde(L, den, gb.ladicabs);
+  if (DEBUGLEVEL >= 1) timer_printf(&ti, "FpV_invVandermonde()");
   if (n == 1)
   {
     G = cgetg(3, t_VEC);
@@ -2286,7 +2138,7 @@ galoisconj4_main(GEN T, GEN den, long flag)
   }
   else
     G = galoisgen(T, L, M, den, &gb, &ga);
-  if (DEBUGLEVEL >= 6) err_printf("GaloisConj:%Ps\n", G);
+  if (DEBUGLEVEL >= 6) err_printf("GaloisConj: %Ps\n", G);
   if (G == gen_0) { avma = ltop; return gen_0; }
   if (DEBUGLEVEL >= 1) timer_start(&ti);
   if (flag)
@@ -2319,7 +2171,7 @@ numberofconjugates(GEN T, long pinit)
   nbmax = (n < 10)? 20: (n<<1) + 1;
   nbtest = 0;
 #if 0
-  c = sturm(T); c = ugcd(c, n - c); /* too costly: finite primes are cheaper */
+  c = ZX_sturm(T); c = ugcd(c, n-c); /* too costly: finite primes are cheaper */
 #else
   c = n;
 #endif
@@ -2362,23 +2214,37 @@ galoisconj4(GEN nf, GEN d)
 
 }
 
-/* d multipllicative bound for the automorphism's denominators */
+/* d multiplicative bound for the automorphism's denominators */
 GEN
 galoisconj(GEN nf, GEN d)
 {
   pari_sp av = avma;
-  GEN G = galoisconj4_main(nf, d, 0);
+  GEN G, NF, T = get_nfpol(nf,&NF);
+  if (degpol(T) == 2)
+  { /* fast shortcut */
+    GEN a = gel(T,4), b = gel(T,3);
+    long v = varn(T);
+    RgX_check_ZX(T, "galoisconj");
+    if (!gequal1(a)) pari_err_IMPL("galoisconj(non-monic)");
+    b = negi(b);
+    G = cgetg(3, t_COL);
+    gel(G,1) = pol_x(v);
+    gel(G,2) = deg1pol(gen_m1, b, v); return G;
+  }
+  G = galoisconj4_main(nf, d, 0);
   if (typ(G) != t_INT) return G; /* Success */
   avma = av; return galoisconj1(nf);
 }
 
+/* FIXME: obsolete, use galoisconj(nf, d) directly */
 GEN
 galoisconj0(GEN nf, long flag, GEN d, long prec)
 {
+  (void)prec;
   switch(flag) {
+    case 2:
     case 0: return galoisconj(nf, d);
     case 1: return galoisconj1(nf);
-    case 2: return galoisconj2(nf, prec);
     case 4: return galoisconj4(nf, d);
   }
   pari_err_FLAG("nfgaloisconj");
@@ -2406,26 +2272,25 @@ galoisinit(GEN nf, GEN den)
 static GEN
 galoispermtopol_i(GEN gal, GEN perm, GEN mod, GEN mod2)
 {
-  long t = typ(perm), i;
-  GEN v;
-  switch (t)
+  switch (typ(perm))
   {
-  case t_VECSMALL:
-  {
-    long v = varn(gel(gal,1));
-    return permtopol(perm, gal_get_roots(gal), gal_get_invvdm(gal),
-                           gal_get_den(gal), mod, mod2, v);
-  }
-  case t_VEC: case t_COL: case t_MAT:
-    v = cgetg(lg(perm), t);
-    if (DEBUGLEVEL>=4) err_printf("GaloisPermToPol:");
-    for (i = 1; i < lg(v); i++)
+    case t_VECSMALL:
+      return permtopol(perm, gal_get_roots(gal), gal_get_invvdm(gal),
+                             gal_get_den(gal), mod, mod2,
+                             varn(gal_get_pol(gal)));
+    case t_VEC: case t_COL: case t_MAT:
     {
-      gel(v,i) = galoispermtopol_i(gal, gel(perm,i), mod, mod2);
-      if (DEBUGLEVEL>=4) err_printf("%ld ",i);
+      long i, lv;
+      GEN v = cgetg_copy(perm, &lv);
+      if (DEBUGLEVEL>=4) err_printf("GaloisPermToPol:");
+      for (i = 1; i < lv; i++)
+      {
+        gel(v,i) = galoispermtopol_i(gal, gel(perm,i), mod, mod2);
+        if (DEBUGLEVEL>=4) err_printf("%ld ",i);
+      }
+      if (DEBUGLEVEL>=4) err_printf("\n");
+      return v;
     }
-    if (DEBUGLEVEL>=4) err_printf("\n");
-    return v;
   }
   pari_err_TYPE("galoispermtopol", perm);
   return NULL; /* not reached */
@@ -2546,13 +2411,13 @@ galoisfixedfield(GEN gal, GEN perm, long flag, long y)
     if (Pgb.valabs > val)
     {
       if (DEBUGLEVEL>=4)
-        err_printf("GaloisConj:increase p-adic prec by %ld.\n", Pgb.valabs-val);
+        err_printf("GaloisConj: increase p-adic prec by %ld.\n", Pgb.valabs-val);
       PL = ZpX_liftroots(P, PL, Pgb.l, Pgb.valabs);
       L  = ZpX_liftroots(T, L, Pgb.l, Pgb.valabs);
       mod = Pgb.ladicabs; mod2 = shifti(mod,-1);
     }
-    PM = vandermondeinversemod(PL, P, Pden, mod);
-    if (y < 0) y = fetch_user_var("y");
+    PM = FpV_invVandermonde(PL, Pden, mod);
+    if (y < 0) y = 1;
     if (varncmp(y, vT) <= 0)
       pari_err_PRIORITY("galoisfixedfield", T, "<=", y);
     res = cgetg(4, t_VEC);
