@@ -1894,7 +1894,7 @@ gmul(GEN x, GEN y)
       lx = lg(x);
       ly = lg(y); if (lx > ly) { lx = ly; swap(x, y); }
       if (lx == 2) return zeroser(vx, valp(x)+valp(y));
-      z = cgetg(lx,t_SER);
+      av = avma; z = cgetg(lx,t_SER);
       z[1] = evalvalp(valp(x)+valp(y)) | evalvarn(vx) | evalsigne(1);
       x = ser2pol_i(x, lx);
       y = ser2pol_i(y, lx);
@@ -1911,7 +1911,7 @@ gmul(GEN x, GEN y)
       else
         y = RgXn_mul(x, y, lx-2);
       z = fill_ser(z, y);
-      return gerepilecopy((pari_sp)(z + lx), z);
+      return gerepilecopy(av, fill_ser(z,y));
     }
     case t_QFI: return qficomp(x,y);
     case t_QFR: return qfrcomp(x,y);
