@@ -26,7 +26,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 /**                                                                 **/
 /*********************************************************************/
 GEN
-map_proto_G(GEN f(GEN), GEN x)
+map_proto_G(GEN (*f)(GEN), GEN x)
 {
   if (is_matvec_t(typ(x)))
   {
@@ -39,7 +39,7 @@ map_proto_G(GEN f(GEN), GEN x)
 }
 
 GEN
-map_proto_lG(long f(GEN), GEN x)
+map_proto_lG(long (*f)(GEN), GEN x)
 {
   if (is_matvec_t(typ(x)))
   {
@@ -52,7 +52,7 @@ map_proto_lG(long f(GEN), GEN x)
 }
 
 GEN
-map_proto_lGL(long f(GEN,long), GEN x, long y)
+map_proto_lGL(long (*f)(GEN,long), GEN x, long y)
 {
   if (is_matvec_t(typ(x)))
   {
@@ -64,8 +64,15 @@ map_proto_lGL(long f(GEN,long), GEN x, long y)
   return stoi(f(x,y));
 }
 
+static GEN
+_domul(void *data, GEN x, GEN y)
+{
+  GEN (*mul)(GEN,GEN)=(GEN (*)(GEN,GEN)) data;
+  return mul(x,y);
+}
+
 GEN
-gassoc_proto(GEN f(GEN,GEN), GEN x, GEN y)
+gassoc_proto(GEN (*f)(GEN,GEN), GEN x, GEN y)
 {
   if (!y)
   {
@@ -78,7 +85,8 @@ gassoc_proto(GEN f(GEN,GEN), GEN x, GEN y)
       case t_COL: break;
       default: pari_err_TYPE("association",x);
     }
-    return gerepileupto(av, divide_conquer_prod(x,f));
+    return gerepileupto(av, gen_product(x, (void *)f, _domul));
+
   }
   return f(x,y);
 }
@@ -160,11 +168,8 @@ greffe_aux(GEN x, long l, long lx, long v)
 GEN
 RgX_to_ser(GEN x, long l)
 {
-  long i, lx = lg(x);
-  if (lx == 2) return zeroser(varn(x), l-2);
-  /* analogous to RgX_valrem + normalize */
-  i = 2; while (i<lx && isrationalzero(gel(x,i))) i++;
-  return greffe_aux(x, l, lx, i - 2);
+  if (lg(x) == 2) return zeroser(varn(x), l-2);
+  return greffe_aux(x, l, lg(x), RgX_val(x));
 }
 GEN
 RgX_to_ser_inexact(GEN x, long l)
@@ -253,6 +258,48 @@ isexactzero(GEN g)
   }
   return 0;
 }
+GEN
+gisexactzero(GEN g)
+{
+  long i, lx;
+  GEN a, b;
+  switch (typ(g))
+  {
+    case t_INT:
+      return !signe(g)? g: NULL;
+    case t_INTMOD:
+      return !signe(gel(g,2))? g: NULL;
+    case t_COMPLEX:
+      a = gisexactzero(gel(g,1)); if (!a) return NULL;
+      b = gisexactzero(gel(g,2)); if (!b) return NULL;
+      return ggcd(a,b);
+    case t_FFELT:
+      return FF_equal0(g)? g: NULL;
+    case t_QUAD:
+      a = gisexactzero(gel(g,2)); if (!a) return NULL;
+      b = gisexactzero(gel(g,3)); if (!b) return NULL;
+      return ggcd(a,b);
+    case t_POLMOD:
+      return gisexactzero(gel(g,2));
+    case t_POL:
+      lx = lg(g); /* cater for Mod(0,2)*x^0 */
+      if (lx == 2) return gen_0;
+      if (lx == 3) return gisexactzero(gel(g,2));
+      return NULL;
+    case t_RFRAC:
+      return gisexactzero(gel(g,1)); /* may occur: Mod(0,2)/x */
+    case t_VEC: case t_COL: case t_MAT:
+      a = gen_0;
+      for (i=lg(g)-1; i; i--)
+      {
+        b = gisexactzero(gel(g,i));
+        if (!b) return NULL;
+        a = ggcd(a, b);
+      }
+      return a;
+  }
+  return NULL;
+}
 
 int
 isrationalzero(GEN g)
@@ -338,7 +385,7 @@ static int
 is_monomial_test(GEN x, long v, int(*test)(GEN))
 {
   long d, i, l;
-  if (!signe(x)) return 0;
+  if (!signe(x)) return (typ(x) == t_SER && v <= 0);
   if (v > 0) return 0;
   l = lg(x); d = 2-v;
   if (l <= d) return 0;
@@ -354,8 +401,7 @@ static int
 col_test(GEN x, int(*test)(GEN))
 {
   long i, l = lg(x);
-  if (l == 1) return 0;
-  if (!test(gel(x,1))) return 0;
+  if (l == 1 || !test(gel(x,1))) return 0;
   for (i = 2; i < l; i++)
     if (!gequal0(gel(x,i))) return 0;
   return 1;
@@ -386,10 +432,15 @@ gequal1(GEN x)
       return equali1(x);
 
     case t_REAL:
-      return signe(x) > 0 ? absrnz_equal1(x): 0;
-
-    case t_INTMOD: case t_POLMOD:
-      return gequal1(gel(x,2));
+    {
+      long s = signe(x);
+      if (!s) return expo(x) >= 0;
+      return s > 0 ? absrnz_equal1(x): 0;
+    }
+    case t_INTMOD:
+      return is_pm1(gel(x,2)) || is_pm1(gel(x,1));
+    case t_POLMOD:
+      return gequal1(gel(x,2)) || gequal1(gel(x,1));
 
     case t_FFELT:
       return FF_equal1(x);
@@ -430,8 +481,11 @@ gequalm1(GEN x)
       return equalim1(x);
 
     case t_REAL:
-      return signe(x) < 0 ? absrnz_equal1(x): 0;
-
+    {
+      long s = signe(x);
+      if (!s) return expo(x) >= 0;
+      return s < 0 ? absrnz_equal1(x): 0;
+    }
     case t_INTMOD:
       av=avma; y=equalii(addsi(1,gel(x,2)), gel(x,1)); avma=av; return y;
 
@@ -452,7 +506,7 @@ gequalm1(GEN x)
 
     case t_POLMOD:
       av=avma; p1 = gaddgs(gel(x,2), 1);
-      y = signe(p1) && !gequal(p1,gel(x,1)); avma=av; return !y;
+      y = gequal0(p1) || gequal(p1,gel(x,1)); avma=av; return y;
 
     case t_POL: return is_monomial_test(x, 0, &gequalm1);
     case t_SER: return is_monomial_test(x, valp(x), &gequalm1);
@@ -525,12 +579,27 @@ cmp_universal(GEN x, GEN y)
       return cmp_universal_rec(x, y, 2);
 
     case t_LIST:
-      x = list_data(x);
-      y = list_data(y);
-      if (!x) return y? -1: 0;
-      if (!y) return 1;
-      return cmp_universal_rec(x, y, 1);
-
+      {
+        long tx = list_typ(x), ty = list_typ(y);
+        GEN vx, vy;
+        if (tx < ty) return -1;
+        if (tx > ty) return 1;
+        vx = list_data(x);
+        vy = list_data(y);
+        if (!vx) return vy? -1: 0;
+        if (!vy) return 1;
+        switch (tx)
+        {
+        case t_LIST_MAP:
+          {
+            pari_sp av = avma;
+            int ret = cmp_universal_rec(maptomat_shallow(x), maptomat_shallow(y),1);
+            avma = av; return ret;
+          }
+        default:
+          return cmp_universal_rec(vx, vy, 1);
+        }
+      }
     default:
       return cmp_universal_rec(x, y, lontyp[tx]);
   }
@@ -561,6 +630,13 @@ cmprfrac(GEN a, GEN y)
   int r = cmpri(mulri(a, d), c);
   avma = av; return r;
 }
+static int
+cmpgen(GEN x, GEN y)
+{
+  pari_sp av = avma;
+  int s = gsigne(gsub(x,y));
+  avma = av; return s;
+}
 
 /* returns the sign of x - y when it makes sense. 0 otherwise */
 int
@@ -574,8 +650,17 @@ gcmp(GEN x, GEN y)
       case t_INT:  return cmpii(x, y);
       case t_REAL: return cmprr(x, y);
       case t_FRAC: return cmpfrac(x, y);
+      case t_QUAD: return cmpgen(x, y);
       case t_STR:  return cmp_str(GSTR(x), GSTR(y));
+      case t_INFINITY:
+      {
+        long sx = inf_get_sign(x), sy = inf_get_sign(y);
+        if (sx < sy) return -1;
+        if (sx > sy) return 1;
+        return 0;
+      }
     }
+  if (ty == t_INFINITY) return -inf_get_sign(y);
   switch(tx)
   {
     case t_INT:
@@ -583,7 +668,7 @@ gcmp(GEN x, GEN y)
       {
         case t_REAL: return cmpir(x, y);
         case t_FRAC: return cmpifrac(x, y);
-        case t_STR:  return -1;
+        case t_QUAD: return cmpgen(x, y);
       }
       break;
     case t_REAL:
@@ -591,7 +676,7 @@ gcmp(GEN x, GEN y)
       {
         case t_INT:  return cmpri(x, y);
         case t_FRAC: return cmprfrac(x, y);
-        case t_STR:  return -1;
+        case t_QUAD: return cmpgen(x, y);
       }
       break;
     case t_FRAC:
@@ -599,10 +684,12 @@ gcmp(GEN x, GEN y)
       {
         case t_INT:  return -cmpifrac(y, x);
         case t_REAL: return -cmprfrac(y, x);
-        case t_STR:  return -1;
+        case t_QUAD: return cmpgen(x, y);
       }
       break;
-    case t_STR: return 1;
+    case t_QUAD:
+      return cmpgen(x, y);
+    case t_INFINITY: return inf_get_sign(x);
   }
   pari_err_TYPE2("comparison",x,y);
   return 0;/*not reached*/
@@ -824,6 +911,31 @@ closure_identical(GEN x, GEN y)
   return gidentical(gel(x,7),gel(y,7));
 }
 
+static int
+list_cmp(GEN x, GEN y, int cmp(GEN x, GEN y))
+{
+  int t = list_typ(x);
+  GEN vx, vy;
+  if (list_typ(y)!=t) return 0;
+  vx = list_data(x);
+  vy = list_data(y);
+  if (!vx) return vy? 0: 1;
+  if (!vy) return 0;
+  if (lg(vx) != lg(vy)) return 0;
+  switch (t)
+  {
+  case t_LIST_MAP:
+    {
+      pari_sp av = avma;
+      GEN mx  = maptomat_shallow(x), my = maptomat_shallow(y);
+      int ret = gidentical(gel(mx, 1), gel(my, 1)) && cmp(gel(mx, 2), gel(my, 2));
+      avma = av; return ret;
+    }
+  default:
+    return cmp(vx, vy);
+  }
+}
+
 int
 gidentical(GEN x, GEN y)
 {
@@ -882,11 +994,8 @@ gidentical(GEN x, GEN y)
     case t_CLOSURE:
       return closure_identical(x,y);
     case t_LIST:
-      x = list_data(x);
-      y = list_data(y);
-      if (!x) return y? 0: 1;
-      if (!y) return 0;
-      return vecidentical(x, y);
+      return list_cmp(x, y, gidentical);
+    case t_INFINITY: return gidentical(gel(x,1),gel(y,1));
   }
   return 0;
 }
@@ -980,6 +1089,7 @@ gequal(GEN x, GEN y)
         av = avma; i = gequal0(gsub(x,y)); avma = av;
         return i;
       case t_POLMOD:
+        if (varn(gel(x,1)) != varn(gel(y,1))) break;
         return gequal(gel(x,2),gel(y,2)) && RgX_equal_var(gel(x,1),gel(y,1));
       case t_POL:
         return polequal(x,y);
@@ -1016,13 +1126,11 @@ gequal(GEN x, GEN y)
       case t_VECSMALL:
         return zv_equal(x,y);
       case t_LIST:
-        x = list_data(x);
-        y = list_data(y);
-        if (!x) return y? 0: 1;
-        if (!y) return 0;
-        return gequal(x, y);
+        return list_cmp(x, y, gequal);
       case t_CLOSURE:
         return closure_identical(x,y);
+      case t_INFINITY:
+        return gequal(gel(x,1),gel(y,1));
     }
   (void)&av; /* emulate volatile */
   av = avma; i = gequal_try(x, y);
@@ -1090,7 +1198,7 @@ long
 gvaluation(GEN x, GEN p)
 {
   long tx = typ(x), tp = typ(p);
-  pari_sp av, limit;
+  pari_sp av;
 
   switch(tp)
   {
@@ -1161,12 +1269,12 @@ gvaluation(GEN x, GEN p)
         {
           long val;
           if (RgX_is_monomial(p)) return RgX_val(x) / degpol(p);
-          av = avma; limit=stack_lim(av,1);
+          av = avma;
           for (val=0; ; val++)
           {
             x = RgX_divrem(x,p,ONLY_DIVIDES);
             if (!x) { avma = av; return val; }
-            if (low_stack(limit, stack_lim(av,1)))
+            if (gc_needed(av,1))
             {
               if(DEBUGMEM>1) pari_warn(warnmem,"gvaluation");
               x = gerepilecopy(av, x);
@@ -1201,6 +1309,12 @@ gvaluation(GEN x, GEN p)
   pari_err_OP("valuation", x,p);
   return 0; /* not reached */
 }
+GEN
+gpvaluation(GEN x, GEN p)
+{
+  long v = gvaluation(x,p);
+  return v == LONG_MAX? mkoo(): stoi(v);
+}
 
 /* x is non-zero */
 long
@@ -1253,14 +1367,14 @@ long
 z_pval(long s, GEN p)
 {
   if (lgefint(p) > 3) return 0;
-  return z_lval(s, (ulong)p[2]);
+  return z_lval(s, uel(p,2));
 }
 /* assume |p| > 1 */
 long
 z_pvalrem(long s, GEN p, long *py)
 {
   if (lgefint(p) > 3) { *py = s; return 0; }
-  return z_lvalrem(s, (ulong)p[2], py);
+  return z_lvalrem(s, uel(p,2), py);
 }
 
 /* return v_q(x) and set *py = x / q^v_q(x), using divide & conquer */
@@ -1287,7 +1401,7 @@ Z_lval(GEN x, ulong p)
   long vx;
   pari_sp av;
   if (p == 2) return vali(x);
-  if (lgefint(x) == 3) return u_lval((ulong)x[2], p);
+  if (lgefint(x) == 3) return u_lval(uel(x,2), p);
   av = avma;
   for(vx = 0;;)
   {
@@ -1312,7 +1426,7 @@ Z_lvalrem(GEN x, ulong p, GEN *py)
   if (p == 2) { vx = vali(x); *py = shifti(x, -vx); return vx; }
   if (lgefint(x) == 3) {
     ulong u;
-    vx = u_lvalrem((ulong)x[2], p, &u);
+    vx = u_lvalrem(uel(x,2), p, &u);
     *py = signe(x) < 0? utoineg(u): utoipos(u);
     return vx;
   }
@@ -1338,7 +1452,7 @@ Z_lvalrem(GEN x, ulong p, GEN *py)
 static int
 isless_iu(GEN q, ulong p) {
   long l = lgefint(q);
-  return l==2 || (l == 3 && (ulong)q[2] <= p);
+  return l==2 || (l == 3 && uel(q,2) <= p);
 }
 
 long
@@ -1396,7 +1510,7 @@ Z_pvalrem(GEN x, GEN p, GEN *py)
   long vx;
   pari_sp av;
 
-  if (lgefint(p) == 3) return Z_lvalrem(x, (ulong)p[2], py);
+  if (lgefint(p) == 3) return Z_lvalrem(x, uel(p,2), py);
   if (lgefint(x) == 3) { *py = icopy(x); return 0; }
   av = avma; vx = 0; (void)new_chunk(lgefint(x));
   for(;;)
@@ -1409,13 +1523,13 @@ Z_pvalrem(GEN x, GEN p, GEN *py)
 long
 u_pvalrem(ulong x, GEN p, ulong *py)
 {
-  if (lgefint(p) == 3) return u_lvalrem(x, (ulong)p[2], py);
+  if (lgefint(p) == 3) return u_lvalrem(x, uel(p,2), py);
   *py = x; return 0;
 }
 long
 u_pval(ulong x, GEN p)
 {
-  if (lgefint(p) == 3) return u_lval(x, (ulong)p[2]);
+  if (lgefint(p) == 3) return u_lval(x, uel(p,2));
   return 0;
 }
 long
@@ -1423,7 +1537,7 @@ Z_pval(GEN x, GEN p) {
   long vx;
   pari_sp av;
 
-  if (lgefint(p) == 3) return Z_lval(x, (ulong)p[2]);
+  if (lgefint(p) == 3) return Z_lval(x, uel(p,2));
   if (lgefint(x) == 3) return 0;
   av = avma; vx = 0;
   for(;;)
@@ -1739,6 +1853,7 @@ gneg(GEN x)
     case t_VEC: return RgV_neg(x);
     case t_COL: return RgC_neg(x);
     case t_MAT: return RgM_neg(x);
+    case t_INFINITY: return inf_get_sign(x) == 1? mkmoo(): mkoo();
     default:
       pari_err_TYPE("gneg",x);
       return NULL; /* not reached */
@@ -2273,7 +2388,7 @@ qtop(GEN x, GEN p, long d)
   P = gel(x,1);
   b = gel(P,3);
   av = avma; D = quad_disc(x);
-  if (equaliu(p,2)) d += 2;
+  if (absequaliu(p,2)) d += 2;
   z = Qp_sqrt(cvtop(D,p,d));
   if (!z) pari_err_SQRTN("Qp_sqrt",D);
   z = gmul2n(gsub(z, b), -1);
@@ -2290,6 +2405,7 @@ ctop(GEN x, GEN p, long d)
   GEN z, u = gel(x,1), v = gel(x,2);
   if (isrationalzero(v)) return cvtop(u, p, d);
   z = Qp_sqrt(cvtop(gen_m1, p, d - gvaluation(v, p))); /* = I */
+  if (!z) pari_err_SQRTN("Qp_sqrt",gen_m1);
 
   z = gadd(u, gmul(v, z));
   if (typ(z) != t_PADIC) /* t_INTMOD for t_COMPLEX of t_INTMODs... */
@@ -2313,7 +2429,7 @@ cvstop2(long s, GEN y)
   gel(z,4) = modsi(s, gel(y,3)); return z;
 }
 
-/* cvtop(x, gel(y,2), precp(y)), internal, not memory-clean */
+/* cvtop(x, gel(y,2), precp(y)), shallow */
 GEN
 cvtop2(GEN x, GEN y)
 {
@@ -2323,8 +2439,8 @@ cvtop2(GEN x, GEN y)
   {
     case t_INT:
       if (!signe(x)) return zeropadic(p, d);
+      if (d <= 0) return zeropadic(p, Z_pval(x,p));
       v = Z_pvalrem(x, p, &x);
-      if (d <= 0) return zeropadic(p, v);
       z = cgetg(5, t_PADIC);
       z[1] = evalprecp(d) | evalvalp(v);
       gel(z,2) = p;
@@ -2335,11 +2451,12 @@ cvtop2(GEN x, GEN y)
       v = Z_pval(gel(x,1),p); if (v > d) v = d;
       return cvtop(gel(x,2), p, v);
 
-    case t_FRAC: { GEN num = gel(x,1), den = gel(x,2);
-      if (!signe(num)) return zeropadic(p, d);
-      v = Z_pvalrem(num, p, &num);
-      if (!v) v = -Z_pvalrem(den, p, &den); /* assume (num,den) = 1 */
-      if (d <= 0) return zeropadic(p, v);
+    case t_FRAC:
+    {
+      GEN num, den;
+      if (d <= 0) return zeropadic(p, Q_pval(x,p));
+      num = gel(x,1); v = Z_pvalrem(num, p, &num);
+      den = gel(x,2); if (!v) v = -Z_pvalrem(den, p, &den);
       z = cgetg(5, t_PADIC);
       z[1] = evalprecp(d) | evalvalp(v);
       gel(z,2) = p;
@@ -2366,8 +2483,8 @@ cvtop(GEN x, GEN p, long d)
   {
     case t_INT:
       if (!signe(x)) return zeropadic(p, d);
+      if (d <= 0) return zeropadic(p, Z_pval(x,p));
       v = Z_pvalrem(x, p, &x);
-      if (d <= 0) return zeropadic(p, v);
       z = cgetg(5, t_PADIC);
       z[1] = evalprecp(d) | evalvalp(v);
       gel(z,2) = icopy(p);
@@ -2378,11 +2495,12 @@ cvtop(GEN x, GEN p, long d)
       v = Z_pval(gel(x,1),p); if (v > d) v = d;
       return cvtop(gel(x,2), p, v);
 
-    case t_FRAC: { GEN num = gel(x,1), den = gel(x,2);
-      if (!signe(num)) return zeropadic(p, d);
-      v = Z_pvalrem(num, p, &num);
-      if (!v) v = -Z_pvalrem(den, p, &den); /* assume (num,den) = 1 */
-      if (d <= 0) return zeropadic(p, v);
+    case t_FRAC:
+    {
+      GEN num, den;
+      if (d <= 0) return zeropadic(p, Q_pval(x,p));
+      num = gel(x,1); v = Z_pvalrem(num, p, &num);
+      den = gel(x,2); if (!v) v = -Z_pvalrem(den, p, &den);
       z = cgetg(5, t_PADIC);
       z[1] = evalprecp(d) | evalvalp(v);
       gel(z,2) = icopy(p);
@@ -2391,7 +2509,15 @@ cvtop(GEN x, GEN p, long d)
       gel(z,4) = modii(num, gel(z,3)); return z; /* not memory-clean */
     }
     case t_COMPLEX: return ctop(x, p, d);
-    case t_PADIC: return gprec(x,d);
+    case t_PADIC:
+      p = gel(x,2); /* override */
+      if (!signe(gel(x,4))) return zeropadic(p, d);
+      z = cgetg(5,t_PADIC);
+      z[1] = x[1]; setprecp(z,d);
+      gel(z,2) = icopy(p);
+      gel(z,3) = powiu(p, d);
+      gel(z,4) = modii(gel(x,4), gel(z,3)); return z;
+
     case t_QUAD: return qtop(x, p, d);
   }
   pari_err_TYPE("cvtop",x);
@@ -2410,8 +2536,7 @@ gcvtop(GEN x, GEN p, long r)
       y = cgetg_copy(x, &lx); y[1] = x[1];
       for (i=2; i<lx; i++) gel(y,i) = gcvtop(gel(x,i),p,r);
       return y;
-    case t_POLMOD: case t_RFRAC:
-    case t_VEC: case t_COL: case t_MAT:
+    case t_POLMOD: case t_RFRAC: case t_VEC: case t_COL: case t_MAT:
       y = cgetg_copy(x, &lx);
       for (i=1; i<lx; i++) gel(y,i) = gcvtop(gel(x,i),p,r);
       return y;
@@ -2472,7 +2597,18 @@ normalize(GEN x)
   GEN y, z;
 
   if (typ(x) != t_SER) pari_err_TYPE("normalize",x);
-  if (lx==2) { setsigne(x,0); return x; }
+  if (lx == 2) { setsigne(x,0); return x; }
+  if (lx == 3) {
+    z = gel(x,2);
+    if (!gequal0(z)) { setsigne(x,1); return x; }
+    if (isrationalzero(z)) return zeroser(vx,vp+1);
+    if (isexactzero(z)) {
+      /* dangerous case: already normalized ? */
+      if (!signe(x)) return x;
+      setvalp(x,vp+1); /* no: normalize */
+    }
+    setsigne(x,0); return x;
+  }
   for (i=2; i<lx; i++)
     if (! isrationalzero(gel(x,i))) break;
   if (i == lx) return zeroser(vx,lx-2+vp);
@@ -2553,6 +2689,24 @@ gsigne(GEN x)
   {
     case t_INT: case t_REAL: return signe(x);
     case t_FRAC: return signe(gel(x,1));
+    case t_QUAD:
+    {
+      pari_sp av = avma;
+      GEN T = gel(x,1), a = gel(x,2), b = gel(x,3);
+      long sa, sb;
+      if (signe(gel(T,2)) > 0) break;
+      a = gmul2n(a,1);
+      if (signe(gel(T,3))) a = gadd(a,b);
+      /* a + b sqrt(D) > 0 ? */
+      sa = gsigne(a);
+      sb = gsigne(b); if (sa == sb) { avma = av; return sa; }
+      if (sa == 0) { avma = av; return sb; }
+      if (sb == 0) { avma = av; return sa; }
+      /* different signs, take conjugate expression */
+      sb = gsigne(gsub(gsqr(a), gmul(quad_disc(x), gsqr(b))));
+      avma = av; return sb * sa;
+    }
+    case t_INFINITY: return inf_get_sign(x);
   }
   pari_err_TYPE("gsigne",x);
   return 0; /* not reached */
@@ -2568,25 +2722,29 @@ gsigne(GEN x)
 static void
 ensure_nb(GEN L, long l)
 {
-  long nmax = list_nmax(L);
-  GEN v;
+  long nmax = list_nmax(L), i, lw;
+  GEN v, w;
   if (l <= nmax) return;
   if (nmax)
   {
     nmax <<= 1;
     if (l > nmax) nmax = l;
-    v = (GEN)pari_realloc(list_data(L), (nmax+1) * sizeof(long));
+    w = list_data(L); lw = lg(w);
+    v = newblock(nmax+1);
+    v[0] = w[0];
+    for (i=1; i < lw; i++) gel(v,i) = gel(w, i);
+    killblock(w);
   }
   else /* unallocated */
   {
     nmax = 32;
     if (list_data(L))
       pari_err(e_MISC, "store list in variable before appending elements");
-    v = (GEN)pari_malloc((nmax+1) * sizeof(long));
+    v = newblock(nmax+1);
     v[0] = evaltyp(t_VEC) | _evallg(1);
   }
   list_data(L) = v;
-  list_nmax(L) = nmax;
+  L[1] = evaltyp(list_typ(L))|evallg(nmax);
 }
 
 void
@@ -2598,31 +2756,58 @@ listkill(GEN L)
     GEN v = list_data(L);
     long i, l = lg(v);
     for (i=1; i<l; i++) gunclone_deep(gel(v,i));
-    pari_free(v);
-    list_nmax(L) = 0;
+    killblock(v);
+    L[1] = evaltyp(list_typ(L));
     list_data(L) = NULL;
   }
 }
 
 GEN
-listcreate(void)
+mklist_typ(long t)
 {
   GEN L = cgetg(3,t_LIST);
-  list_nmax(L) = 0;
+  L[1] = evaltyp(t);
   list_data(L) = NULL; return L;
 }
 
+GEN
+mklist(void)
+{
+  return mklist_typ(t_LIST_RAW);
+}
+
+GEN
+mkmap(void)
+{
+  return mklist_typ(t_LIST_MAP);
+}
+
+/* return a list with single element x, allocated on stack */
+GEN
+mklistcopy(GEN x)
+{
+  GEN y = mklist();
+  list_data(y) = mkveccopy(x);
+  return y;
+}
+
+GEN
+listcreate_gp(long n)
+{
+  (void) n; return mklist();
+}
+
 GEN
 listput(GEN L, GEN x, long index)
 {
   long l;
   GEN z;
 
-  if (typ(L) != t_LIST) pari_err_TYPE("listput",L);
   if (index < 0) pari_err_COMPONENT("listput", "<", gen_0, stoi(index));
   z = list_data(L);
   l = z? lg(z): 1;
 
+  x = gclone(x);
   if (!index || index >= l)
   {
     ensure_nb(L, l);
@@ -2631,26 +2816,38 @@ listput(GEN L, GEN x, long index)
     l++;
   } else
     gunclone_deep( gel(z, index) );
-  gel(z,index) = gclone(x);
+  gel(z,index) = x;
   z[0] = evaltyp(t_VEC) | evallg(l); /*must be after gel(z,index) is set*/
   return gel(z,index);
 }
 
+GEN
+listput0(GEN L, GEN x, long index)
+{
+  if (typ(L) != t_LIST || list_typ(L) != t_LIST_RAW)
+    pari_err_TYPE("listput",L);
+  return listput(L, x, index);
+}
+
 GEN
 listinsert(GEN L, GEN x, long index)
 {
   long l, i;
   GEN z;
 
-  if (typ(L) != t_LIST) pari_err_TYPE("listinsert",L);
+  if (typ(L) != t_LIST || list_typ(L) != t_LIST_RAW)
+    pari_err_TYPE("listinsert",L);
   z = list_data(L); l = z? lg(z): 1;
   if (index <= 0) pari_err_COMPONENT("listinsert", "<=", gen_0, stoi(index));
   if (index > l) pari_err_COMPONENT("listinsert", ">", stoi(l), stoi(index));
   ensure_nb(L, l);
+  BLOCK_SIGINT_START
   z = list_data(L);
   for (i=l; i > index; i--) gel(z,i) = gel(z,i-1);
   z[0] = evaltyp(t_VEC) | evallg(l+1);
-  return gel(z,index) = gclone(x);
+  gel(z,index) = gclone(x);
+  BLOCK_SIGINT_END
+  return gel(z,index);
 }
 
 void
@@ -2665,18 +2862,19 @@ listpop(GEN L, long index)
   if (!z || (l = lg(z)-1) == 0) return;
 
   if (!index || index > l) index = l;
+  BLOCK_SIGINT_START
   gunclone_deep( gel(z, index) );
   z[0] = evaltyp(t_VEC) | evallg(l);
   for (i=index; i < l; i++) z[i] = z[i+1];
+  BLOCK_SIGINT_END
 }
 
-/* return a list with single element x, allocated on stack */
-GEN
-mklistcopy(GEN x)
+void
+listpop0(GEN L, long index)
 {
-  GEN y = listcreate();
-  list_data(y) = mkveccopy(x);
-  return y;
+  if (typ(L) != t_LIST || list_typ(L) != t_LIST_RAW)
+    pari_err_TYPE("listpop",L);
+  listpop(L, index);
 }
 
 /* return a copy fully allocated on stack. gclone from changevalue is
@@ -2686,17 +2884,17 @@ gtolist(GEN x)
 {
   GEN y;
 
-  if (!x) return listcreate();
+  if (!x) return mklist();
   switch(typ(x))
   {
     case t_VEC: case t_COL:
-      y = listcreate();
+      y = mklist();
       if (lg(x) == 1) return y;
       list_data(y) = gcopy(x);
       settyp(list_data(y), t_VEC);
       return y;
     case t_LIST:
-      y = listcreate();
+      y = mklist();
       list_data(y) = list_data(x)? gcopy(list_data(x)): NULL;
       return y;
     default:
