@@ -100,19 +100,24 @@ static long
 Babai(pari_sp av, long kappa, GEN *pG, GEN *pB, GEN *pU, GEN mu, GEN r, GEN s,
       long a, long zeros, long maxG, long n, GEN eta, GEN halfplus1, long prec)
 {
-  const pari_sp lim = stack_lim(av,2);
+  pari_sp av0 = avma;
   GEN B = *pB, G = *pG, U = *pU, tmp, rtmp, ztmp;
   long k, aa = (a > zeros)? a : zeros+1;
   GEN maxmu = gen_0, max2mu = gen_0;
   /* N.B: we set d = 0 (resp. n = 0) to avoid updating U (resp. B) */
   const long d = U ? lg(U)-1: 0;
 
+  if (gc_needed(av,2))
+  {
+    if(DEBUGMEM>1) pari_warn(warnmem,"Babai[0], a=%ld", aa);
+    gerepileall(av,U?3:2,&B,&G,&U);
+  }
   for (;;) {
     int go_on = 0;
     GEN max3mu;
     long i, j;
 
-    if (low_stack(lim, stack_lim(av,2)))
+    if (gc_needed(av0,2))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"Babai[1], a=%ld", aa);
       gerepileall(av,U?5:4,&B,&G,&maxmu,&max2mu,&U);
@@ -139,12 +144,12 @@ Babai(pari_sp av, long kappa, GEN *pG, GEN *pB, GEN *pU, GEN mu, GEN r, GEN s,
       else
         affir(gmael(G,kappa,j), gmael(r,kappa,j));
       affrr(divrr(gmael(r,kappa,j), gmael(r,j,j)), gmael(mu,kappa,j));
-      if (absr_cmp(maxmu, gmael(mu,kappa,j))<0)
+      if (abscmprr(maxmu, gmael(mu,kappa,j))<0)
         maxmu = gmael(mu,kappa,j);
       avma = btop;
     }
     maxmu = absr(maxmu);
-    if (typ(max3mu)==t_REAL && absr_cmp(max3mu, shiftr(max2mu, 5))<=0)
+    if (typ(max3mu)==t_REAL && abscmprr(max3mu, shiftr(max2mu, 5))<=0)
     {
       *pB = B; *pG = G; *pU = U;
       if (DEBUGLEVEL>5) err_printf("prec too low\n");
@@ -155,16 +160,16 @@ Babai(pari_sp av, long kappa, GEN *pG, GEN *pB, GEN *pU, GEN mu, GEN r, GEN s,
     for (j=kappa-1; j>zeros; j--)
     {
       tmp = gmael(mu,kappa,j);
-      if (absr_cmp(tmp, eta) <= 0) continue; /* (essentially) size-reduced */
+      if (abscmprr(tmp, eta) <= 0) continue; /* (essentially) size-reduced */
 
-      if (low_stack(lim, stack_lim(av,2)))
+      if (gc_needed(av0,2))
       {
         if(DEBUGMEM>1) pari_warn(warnmem,"Babai[2], a=%ld, j=%ld", aa,j);
         gerepileall(av,U?5:4,&B,&G,&maxmu,&max2mu,&U);
       }
       go_on = 1;
       /* we consider separately the case |X| = 1 */
-      if (absr_cmp(tmp, halfplus1) <= 0)
+      if (abscmprr(tmp, halfplus1) <= 0)
       {
         if (signe(tmp) > 0) { /* in this case, X = 1 */
           pari_sp btop = avma;
@@ -339,6 +344,7 @@ fplll(GEN *ptrB, GEN *ptrU, GEN *ptrr, double DELTA, double ETA, long flag, long
   const long triangular = 0;
   pari_timer T;
   GEN B = *ptrB, U;
+  long cnt = 0;
 
   d = lg(B)-1;
   if (gram)
@@ -421,12 +427,8 @@ fplll(GEN *ptrB, GEN *ptrU, GEN *ptrr, double DELTA, double ETA, long flag, long
     }
     else
     { /* Step5: Find the right insertion index kappa, kappa2 = initial kappa */
-      if (DEBUGLEVEL>=10 && kappa==kappamax && signe(gel(s,kappa-1)))
-      {
-        long e = expo(divrr(mulrr(gmael(r,kappa-1,kappa-1), delta),
-                            gel(s,kappa-1)));
-        err_printf("(%ld) ", e);
-      }
+      if (DEBUGLEVEL>=4 && kappa==kappamax && signe(gel(s,kappa-1)))
+        if (++cnt > 20) { cnt = 0; err_printf("(%ld) ", expo(gel(s,1))); }
       kappa2 = kappa;
       do {
         kappa--;
@@ -504,19 +506,23 @@ GEN
 ZM_lll_norms(GEN x, double DELTA, long flag, GEN *B)
 {
   pari_sp ltop = avma;
+  const long compat = flag & LLL_COMPATIBLE;
   const double ETA = 0.51;
   long p, n = lg(x)-1;
   GEN U;
   if (n <= 1) return lll_trivial(x, flag);
   x = RgM_shallowcopy(x);
   U = (flag & LLL_INPLACE)? NULL: matid(n);
-  for (p = LOWDEFAULTPREC; ; incrprec(p))
+  for (p = compat? DEFAULTPREC: LOWDEFAULTPREC;;)
   {
     GEN m = fplll(&x, &U, B, DELTA, ETA, flag, p);
     if (m) return m;
+    if (compat)
+      p += DEFAULTPREC-2;
+    else
+      incrprec(p);
     gerepileall(ltop, U? 2: 1, &x, &U);
   }
-  return NULL; /* NOT REACHED */
 }
 
 /********************************************************************/
@@ -632,7 +638,7 @@ static GEN
 lllgramallgen(GEN x, long flag)
 {
   long lx = lg(x), i, j, k, l, n;
-  pari_sp av, lim;
+  pari_sp av;
   GEN B, L, h, fl;
   int flc;
 
@@ -641,7 +647,7 @@ lllgramallgen(GEN x, long flag)
 
   fl = cgetg(lx, t_VECSMALL);
 
-  av = avma; lim = stack_lim(av,1);
+  av = avma;
   B = scalarcol_shallow(gen_1, lx);
   L = cgetg(lx,t_MAT);
   for (j=1; j<lx; j++) { gel(L,j) = zerocol(n); fl[j] = 0; }
@@ -660,7 +666,7 @@ lllgramallgen(GEN x, long flag)
         if (REDgen(k, l, h, L, gel(B,l+1))) flc = 1;
       if (++k > n) break;
     }
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"lllgramallgen");
       gerepileall(av,3,&B,&L,&h);
@@ -794,32 +800,32 @@ qflllgram0(GEN x, long flag)
 /**                   INTEGRAL KERNEL (LLL REDUCED)                **/
 /**                                                                **/
 /********************************************************************/
-/* Horribly slow (coeff explosion in small dimension): never use this */
 static GEN
-kerint1(GEN x)
+kerint0(GEN M)
+{
+  GEN U, H = ZM_hnfall(M,&U,1);
+  long d = lg(M)-lg(H);
+  if (!d) return cgetg(1, t_MAT);
+  return ZM_lll(vecslice(U,1,d), LLLDFT, LLL_INPLACE);
+}
+GEN
+kerint(GEN M)
 {
   pari_sp av = avma;
-  return gerepilecopy(av, ZM_lll(QM_ImQ_hnf(ker(x)), LLLDFT, LLL_INPLACE));
+  return gerepilecopy(av, kerint0(M));
 }
-/* Mostly useless: use ZM_lll(x, 0.99, LLL_KER) directly */
+/* OBSOLETE: use kerint */
 GEN
-kerint(GEN x)
+matkerint0(GEN M, long flag)
 {
   pari_sp av = avma;
-  GEN h = ZM_lll(x, LLLDFT, LLL_KER);
-  if (lg(h)==1) { avma = av; return cgetg(1, t_MAT); }
-  return gerepilecopy(av, ZM_lll(h, LLLDFT, LLL_INPLACE));
-}
-
-GEN
-matkerint0(GEN x, long flag)
-{
-  if (typ(x) != t_MAT) pari_err_TYPE("matkerint",x);
-  RgM_check_ZM(x, "kerint");
+  if (typ(M) != t_MAT) pari_err_TYPE("matkerint",M);
+  M = Q_primpart(M);
+  RgM_check_ZM(M, "kerint");
   switch(flag)
   {
-    case 0: return kerint(x);
-    case 1: return kerint1(x);
+    case 0:
+    case 1: return gerepilecopy(av, kerint0(M));
     default: pari_err_FLAG("matkerint");
   }
   return NULL; /* not reached */
