@@ -464,7 +464,7 @@ perm_generate(GEN S, GEN H, long o)
   return L;
 }
 
-/*Return the order (cardinal) of a group */
+/*Return the order (cardinality) of a group */
 long
 group_order(GEN G)
 {
@@ -724,7 +724,7 @@ liftlistsubgroups(GEN L, GEN C, long r)
     {
       GEN p = gel(C,j);
       if (perm_relorder(p, Selt) == r && group_perm_normalize(S, p))
-        gel(R,k++) = mkvec2(vecsmall_append(gen, (long)p),
+        gel(R,k++) = mkvec2(vec_append(gen, p),
                             vecsmall_append(ord, r));
     }
   }
@@ -844,10 +844,10 @@ group_subgroups(GEN G)
   sg2 = cgetg(lM, t_VEC);
   /* Loop over all subgroups of G/H */
   for (j = 1; j < lM; ++j) gel(sg2,j) = liftsubgroup(C, H, gel(M,j));
-  p1 = concat(sg1, shallowconcat1(sg2));
+  p1 = gconcat(sg1, shallowconcat1(sg2));
   if (sg3)
   {
-    p1 = concat(p1, sg3);
+    p1 = gconcat(p1, sg3);
     if (n==5) /*ensure that the D4 subgroups of S4 are in supersolvable format*/
       for(j = 3; j <= 5; j++)
       {
