@@ -1436,8 +1436,10 @@ FpX_FpXY_resultant(GEN a, GEN b, GEN p)
 GEN
 FpX_direct_compositum(GEN a, GEN b, GEN p)
 {
-  GEN x = deg1pol_shallow(gen_1, pol_x(varn(a)), fetch_var_higher()); /* x+y */
+  long v = varn(a), w = fetch_var_higher();
+  GEN x = deg1pol_shallow(gen_1, pol_x(v), w); /* x+y */
   x = FpX_FpXY_resultant(a, poleval(b,x),p);
+  setvarn(x, v);
   (void)delete_var(); return x;
 }
 
@@ -1517,6 +1519,7 @@ INIT:
     else
       H = resultant(A,B);
     if (checksqfree && !ZX_is_squarefree(H)) goto INIT;
+    if (dB) H = ZX_Z_divexact(H, powiu(dB, degA));
     goto END;
   }
 
