@@ -1314,10 +1314,10 @@ setup_norm_eqn(norm_eqn_t ne, long D, long u, GEN norm_eqn)
 INLINE ulong
 Flv_powsum_pre(GEN v, ulong n, ulong p, ulong pi)
 {
-  long i;
+  long i, l = lg(v);
   ulong psum = 0;
-  for (i = 1; i < lg(v); ++i)
-    psum += Fl_powu_pre(v[i], n, p, pi);
+  for (i = 1; i < l; ++i)
+    psum = Fl_add(psum, Fl_powu_pre(uel(v,i), n, p, pi), p);
   return psum;
 }
 
@@ -1361,7 +1361,7 @@ adjust_signs(GEN js, ulong p, ulong pi, long inv, GEN T, long e)
   } else {
     ulong tp, t;
     tp = umodiu(T, p);
-    t = Fl_div(Flv_powsum_pre(js, e, p, pi), e % p, p);
+    t = Flv_powsum_pre(js, e, p, pi);
     if (t != tp) {
       if (Fl_neg(t, p) != tp)
         pari_err_BUG("adjust_signs: incorrect trace");
@@ -1607,31 +1607,42 @@ polclass_psum(
   e = -1;
   do {
     e += 2;
-    psum_sqr = Z_init_CRT(0, 1);
-    P = gen_1;
-    for (i = 1, stabcnt = 0; stabcnt < MIN_STAB_CNT && i <= nprimes; ++i) {
+    for (i = 1; i <= nprimes; ++i) {
       GEN roots_modp;
       ulong ps, p, pi;
-      long stab;
 
       roots_modp = gel(roots, i);
       p = uel(primes, i);
       pi = uel(pilist, i);
-      ps = Fl_div(Flv_powsum_pre(roots_modp, e, p, pi), e % p, p);
-      ps = Fl_sqr_pre(ps, p, pi);
-      stab = Z_incremental_CRT(&psum_sqr, ps, &P, p);
-
-      /* stabcnt = stab * (stabcnt + 1) */
-      if (stab)
-        ++stabcnt;
-      else
-        stabcnt = 0;
-      if (gc_needed(av, 2))
-        gerepileall(av, 2, &psum_sqr, &P);
+      ps = Flv_powsum_pre(roots_modp, e, p, pi);
+      if (ps == 0) break;
     }
-    if (stabcnt < MIN_STAB_CNT && nprimes >= MIN_STAB_CNT)
-      pari_err_BUG("polclass_pum");
-  } while (gequal0(psum_sqr));
+    if (i > nprimes) break;
+  } while (1);
+  psum_sqr = Z_init_CRT(0, 1);
+  P = gen_1;
+  for (i = 1, stabcnt = 0; stabcnt < MIN_STAB_CNT && i <= nprimes; ++i) {
+    GEN roots_modp;
+    ulong ps, p, pi;
+    long stab;
+
+    roots_modp = gel(roots, i);
+    p = uel(primes, i);
+    pi = uel(pilist, i);
+    ps = Flv_powsum_pre(roots_modp, e, p, pi);
+    ps = Fl_sqr_pre(ps, p, pi);
+    stab = Z_incremental_CRT(&psum_sqr, ps, &P, p);
+
+    /* stabcnt = stab * (stabcnt + 1) */
+    if (stab)
+      ++stabcnt;
+    else
+      stabcnt = 0;
+    if (gc_needed(av, 2))
+      gerepileall(av, 2, &psum_sqr, &P);
+  }
+  if (stabcnt < MIN_STAB_CNT && nprimes >= MIN_STAB_CNT)
+    pari_err_BUG("polclass_psum");
 
   if ( ! Z_issquareall(psum_sqr, psum)) pari_err_BUG("polclass_psum");
 
