@@ -91,18 +91,18 @@ Fl_MR_Jaeschke_ok(Fl_MR_Jaeschke_t *S, ulong c)
 static int
 bad_for_base(MR_Jaeschke_t *S, GEN a)
 {
-  long r, lim, av = avma;
+  pari_sp av = avma;
+  long r;
   GEN c2, c = Fp_pow(a, S->t1, S->n);
 
   if (is_pm1(c) || equalii(S->t, c)) return 0;
 
-  lim = stack_lim(av,1);
   /* go fishing for -1, not for 1 (saves one squaring) */
   for (r = S->r1 - 1; r; r--) /* r1 - 1 squarings */
   {
     c2 = c; c = remii(sqri(c), S->n);
     if (equalii(S->t, c)) return MR_Jaeschke_ok(S, c2);
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"Rabin-Miller");
       c = gerepileuptoint(av, c);
@@ -139,7 +139,7 @@ millerrabin(GEN n, long k)
   if (typ(n) != t_INT) pari_err_TYPE("millerrabin",n);
   if (signe(n)<=0) return 0;
   /* If |n| <= 3, check if n = +- 1 */
-  if (lgefint(n)==3 && (ulong)(n[2])<=3) return (n[2] != 1);
+  if (lgefint(n)==3 && uel(n,2)<=3) return uel(n,2) != 1;
 
   if (!mod2(n)) return 0;
   init_MR_Jaeschke(&S, n); av2 = avma;
@@ -231,7 +231,7 @@ MR_Jaeschke(GEN n, long k)
   long i;
   MR_Jaeschke_t S;
 
-  if (lgefint(n) == 3) return Fl_MR_Jaeschke((ulong)n[2], k);
+  if (lgefint(n) == 3) return Fl_MR_Jaeschke(uel(n,2), k);
 
   if (!mod2(n)) return 0;
   if      (k == 16) { p = pr+13; k = 4; } /* 2,13,23,1662803 */
@@ -257,29 +257,36 @@ MR_Jaeschke(GEN n, long k)
 static GEN
 LucasMod(GEN n, ulong P, GEN N)
 {
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma;
   GEN nd = int_MSW(n);
-  long i, m = *nd, j = 1+bfffo((ulong)m);
+  ulong m = *nd;
+  long i, j;
   GEN v = utoipos(P), v1 = utoipos(P*P - 2);
 
-  m <<= j; j = BITS_IN_LONG - j;
+  if (m == 1)
+    j = 0;
+  else
+  {
+    j = 1+bfffo(m); /* < BIL */
+    m <<= j; j = BITS_IN_LONG - j;
+  }
   for (i=lgefint(n)-2;;) /* cf. leftright_pow */
   {
     for (; j; m<<=1,j--)
     { /* v = v_k, v1 = v_{k+1} */
-      if (m < 0)
+      if (m&HIGHBIT)
       { /* set v = v_{2k+1}, v1 = v_{2k+2} */
-        v = subis(mulii(v,v1), (long)P);
-        v1= subis(sqri(v1), 2);
+        v = subiu(mulii(v,v1), P);
+        v1= subiu(sqri(v1), 2);
       }
       else
       {/* set v = v_{2k}, v1 = v_{2k+1} */
-        v1= subis(mulii(v,v1), (long)P);
-        v = subis(sqri(v), 2);
+        v1= subiu(mulii(v,v1), P);
+        v = subiu(sqri(v), 2);
       }
       v = modii(v, N);
       v1= modii(v1,N);
-      if (low_stack(lim,stack_lim(av,1)))
+      if (gc_needed(av,1))
       {
         if(DEBUGMEM>1) pari_warn(warnmem,"LucasMod");
         gerepileall(av, 2, &v,&v1);
@@ -297,21 +304,24 @@ LucasMod(GEN n, ulong P, GEN N)
 static ulong
 u_LucasMod(ulong n, ulong P, ulong N)
 {
-  long j = 1 + bfffo(n);
-  ulong v = P, v1 = P*P - 2, mP = N - P, m2 = N - 2, m = n << j;
+  ulong v, v1, m;
+  long j;
 
-  j = BITS_IN_LONG - j;
+  if (n == 1) return P;
+  j = 1 + bfffo(n); /* < BIL */
+  v = P; v1 = P*P - 2;
+  m = n<<j; j = BITS_IN_LONG - j;
   for (; j; m<<=1,j--)
   { /* v = v_k, v1 = v_{k+1} */
-    if (((long)m) < 0)
+    if (m & HIGHBIT)
     { /* set v = v_{2k+1}, v1 = v_{2k+2} */
-      v = Fl_add(Fl_mul(v,v1,N), mP, N);
-      v1= Fl_add(Fl_mul(v1,v1,N),m2, N);
+      v = Fl_sub(Fl_mul(v,v1,N), P, N);
+      v1= Fl_sub(Fl_sqr(v1,N), 2UL, N);
     }
     else
     {/* set v = v_{2k}, v1 = v_{2k+1} */
-      v1= Fl_add(Fl_mul(v,v1,N),mP, N);
-      v = Fl_add(Fl_mul(v,v,N), m2, N);
+      v1= Fl_sub(Fl_mul(v,v1,N),P, N);
+      v = Fl_sub(Fl_sqr(v,N), 2UL, N);
     }
   }
   return v;
@@ -321,7 +331,7 @@ int
 uislucaspsp(ulong n)
 {
   long i, v;
-  ulong b, z, m2, m = n + 1;
+  ulong b, z, m = n + 1;
 
   for (b=3, i=0;; b+=2, i++)
   {
@@ -332,44 +342,41 @@ uislucaspsp(ulong n)
   if (!m) return 0; /* neither 2^32-1 nor 2^64-1 are Lucas-pp */
   v = vals(m); m >>= v;
   z = u_LucasMod(m, b, n);
-  if (z == 2) return 1;
-  m2 = n - 2;
-  if (z == m2) return 1;
+  if (z == 2 || z == n-2) return 1;
   for (i=1; i<v; i++)
   {
     if (!z) return 1;
-    z = Fl_add(Fl_mul(z,z, n), m2, n);
+    z = Fl_sub(Fl_sqr(z,n), 2UL, n);
     if (z == 2) return 0;
   }
   return 0;
 }
-/* check that N not a square first (taken care of here, but inefficient)
- * assume N > 3 */
+/* N > 3. Caller should check that N is not a square first (taken care of here,
+ * but inefficient) */
 static int
 IsLucasPsP(GEN N)
 {
-  pari_sp av = avma, lim = stack_lim(av, 1);
-  GEN N_2, m, z;
+  pari_sp av = avma;
+  GEN m, z;
   long i, v;
   ulong b;
 
-  for (b=3, i=0;; b+=2, i++)
+  for (b=3;; b+=2)
   {
     ulong c = b*b - 4; /* = 1 mod 4 */
-    if (i == 64 && Z_issquare(N)) return 0; /* avoid oo loop if N = m^2 */
+    if (b == 129 && Z_issquare(N)) return 0; /* avoid oo loop if N = m^2 */
     if (krouu(umodiu(N,c), c) < 0) break;
   }
   m = addis(N,1); v = vali(m); m = shifti(m,-v);
   z = LucasMod(m, b, N);
-  if (equaliu(z, 2)) return 1;
-  N_2 = subis(N,2);
-  if (equalii(z, N_2)) return 1;
+  if (absequaliu(z, 2)) return 1;
+  if (equalii(z, subiu(N,2))) return 1;
   for (i=1; i<v; i++)
   {
     if (!signe(z)) return 1;
-    z = modii(subis(sqri(z), 2), N);
-    if (equaliu(z, 2)) return 0;
-    if (low_stack(lim,stack_lim(av,1)))
+    z = modii(subiu(sqri(z), 2), N);
+    if (absequaliu(z, 2)) return 0;
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"IsLucasPsP");
       z = gerepileupto(av, z);
@@ -513,7 +520,7 @@ BPSW_psp(GEN N)
 
   if (typ(N) != t_INT) pari_err_TYPE("BPSW_psp",N);
   if (signe(N) <= 0) return 0;
-  if (lgefint(N) == 3) return uisprime((ulong)N[2]);
+  if (lgefint(N) == 3) return uisprime(uel(N,2));
   if (!mod2(N)) return 0;
 #ifdef LONG_IS_64BIT
   /* 16294579238595022365 = 3*5*7*11*13*17*19*23*29*31*37*41*43*47*53
@@ -563,7 +570,7 @@ BPSW_psp_nosmalldiv(GEN N)
   long l = lgefint(N);
   int k;
 
-  if (l == 3) return uisprime_661((ulong)N[2]);
+  if (l == 3) return uisprime_661(uel(N,2));
   av = avma;
   /* N large: test for pure power, rarely succeeds, but requires < 1% of
    * compositeness test times */
@@ -596,6 +603,26 @@ BPSW_isprime_small(GEN x)
 #endif
 }
 
+/* Assume N > 1, p^e || N-1, p prime. Find a witness a(p) such that
+ *   a^(N-1) = 1 (mod N)
+ *   a^(N-1)/p - 1 invertible mod N.
+ * Proves that any divisor of N is 1 mod p^e. Return 0 if N is composite */
+static ulong
+pl831(GEN N, GEN p)
+{
+  GEN b, c, g, Nmunp = diviiexact(addis(N,-1), p);
+  pari_sp av = avma;
+  ulong a;
+  for(a = 2;; a++, avma = av)
+  {
+    b = Fp_pow(utoipos(a), Nmunp, N);
+    if (!equali1(b)) break;
+  }
+  c = Fp_pow(b,p,N);
+  g = gcdii(addis(b,-1), N); /* 0 < g < N */
+  return (equali1(c) && equali1(g))? a: 0;
+}
+
 /* Brillhart, Lehmer, Selfridge test (Crandall & Pomerance, Th 4.1.5)
  * N^(1/3) <= f fully factored, f | N-1. If pl831(p) is true for
  * any prime divisor p of f, then any divisor of N is 1 mod f.
@@ -608,162 +635,139 @@ BLS_test(GEN N, GEN f)
   if (!is_pm1(r)) return 0;
   c2 = dvmdii(q, f, &c1);
   /* N = 1 + f c1 + f^2 c2, 0 <= c_i < f; check whether it is of the form
-   * (1 + f a)(1 + fb) */
+   * (1 + fa)(1 + fb) */
   return ! Z_issquare(subii(sqri(c1), shifti(c2,2)));
 }
 
-/*assume N>1, p^e || N-1. Find a witness a(p) such that
- * a^(N-1) = 1 (mod N)
- * a^(N-1)/p - 1 invertible mod N.
- * Proves that any divisor of N is 1 mod p^e */
-static ulong
-pl831(GEN N, GEN p)
-{
-  pari_sp ltop = avma, av;
-  ulong a;
-  GEN Nmunp = diviiexact(addis(N,-1), p);
-  av = avma;
-  for(a = 2;; a++, avma = av)
-  {
-    GEN b = Fp_pow(utoipos(a), Nmunp, N);
-    GEN c = Fp_pow(b,p,N), g = gcdii(addis(b,-1), N);
-    if (!is_pm1(c)) return 0;
-    if (is_pm1(g)) { avma=ltop; return a; }
-    if (!equalii(g,N)) return 0;
-  }
-}
-/* Assume x BPSW pseudoprime. Return gen_0 if not prime, and a primality
- * certificate otherwise */
-static GEN isprimePL(GEN N);
+/* BPSW_psp(N) && !BPSW_isprime_small(N). Decide between Pocklington-Lehmer
+ * and APRCL. Return a vector of (small) primes such that PL-witnesses
+ * guarantee the primality of N. Return NULL if PL is likely too expensive.
+ * Return gen_0 if BLS test finds N to be composite */
 static GEN
-check_prime(GEN p)
+BPSW_try_PL(GEN N)
 {
-  if (BPSW_isprime_small(p)) return gen_1;
-  if (expi(p) <= 250) return isprimePL(p);
-  return isprimeAPRCL(p)? gen_2: gen_0;
-}
-/* initialize Selfridge / Pocklington-Lehmer test, return 0
- * if proven composite */
-static int
-selfridge_init(GEN N, GEN *pF, GEN *pf)
-{
-  GEN cbrtN = sqrtnint(N, 3);
-  GEN N_1 = addis(N,-1);
-  GEN F = Z_factor_until(N_1, sqri(cbrtN));
-  GEN f = factorback(F); /* factored part of N-1, f^3 > N */
-  *pF = gel(F,1);
-  *pf = f;
-  /* smooth or N^(1/2)-smooth (Pocklington) or N^(1/3)-smooth (BLS) */
-  return equalii(f, N_1) || cmpii(sqri(f), N) > 0 || BLS_test(N,f);
-}
-/* Assume N is a strong BPSW pseudoprime, Pocklington-Lehmer primality proof.
- *
- * return gen_0 (non-prime), gen_1 (small prime), matrix (large prime)
- *
- * The matrix has 3 columns, [a,b,c] with
- * a[i] prime factor of N-1,
- * b[i] witness for a[i] as in pl831
- * c[i] isprimePL(a[i]) */
-static GEN
-isprimePL(GEN N)
-{
-  pari_sp ltop = avma;
-  long i, l;
-  int eps;
-  GEN C, P, W, R, F, f;
+  ulong B = minuu(1UL<<19, maxprime());
+  GEN E, p, U, F, N_1 = subiu(N,1);
+  GEN fa = Z_factor_limit(N_1, B), P = gel(fa,1);
+  long n = lg(P)-1;
 
-  if (typ(N) != t_INT) pari_err_TYPE("isprimePL",N);
-  eps = cmpis(N,2);
-  if (eps <= 0) return eps? gen_0: gen_1;
-  /* N > 2 */
-  if (!selfridge_init(N, &F, &f)) { avma = ltop; return gen_0; }
-  if (DEBUGLEVEL>3)
-  {
-    err_printf("Pocklington-Lehmer: proving primality of N = %Ps\n", N);
-    err_printf("Pocklington-Lehmer: N-1 factored up to %Ps! (%.3Ps%%)\n", f, divri(itor(f,LOWDEFAULTPREC), N));
-    err_printf("Pocklington-Lehmer: N-1 smooth enough! Computing certificate\n");
-  }
-  C = cgetg(4,t_MAT); l = lg(F);
-  gel(C,1) = P = cgetg(l,t_COL);
-  gel(C,2) = W = cgetg(l,t_COL);
-  gel(C,3) = R = cgetg(l,t_COL);
+  p = gel(P,n);
+  /* if p prime, then N-1 is fully factored */
+  if (cmpii(p,sqru(B)) <= 0 || (BPSW_psp_nosmalldiv(p) && BPSW_isprime(p)))
+    return P;
+
+  E = gel(fa,2);
+  U = powii(p, gel(E,n)); /* unfactored part of N-1 */
+  /* factored part of N-1; n >= 2 since 2p | N-1 */
+  F = (n == 2)? powii(gel(P,1), gel(E,1)): diviiexact(N_1,  U);
+  setlg(P, n); /* remove last (composite) entry */
+
+  /* N-1 = F U, F factored, U possibly composite, (U,F) = 1 */
+  if (cmpii(F, U) > 0) return P; /* 1/2-smooth */
+  if (cmpii(sqri(F), U) > 0) return BLS_test(N,F)? P: gen_0; /* 1/3-smooth */
+  return NULL; /* not smooth enough */
+}
+
+static GEN isprimePL(GEN N);
+static GEN PL_certificate(GEN N, GEN F);
+/* Assume N a BPSW pseudoprime. Return 0 if not prime, and a primality label
+ * otherwise: 1 (small), 2 (APRCL), or PL-certificate  */
+static GEN
+check_prime(GEN N)
+{
+  GEN P;
+  if (BPSW_isprime_small(N)) return gen_1;
+  /* PL for small N: APRCL is faster but we prefer a certificate */
+  if (expi(N) <= 250) return isprimePL(N);
+  P = BPSW_try_PL(N);
+  /* if PL likely too expensive: give up certificate and use APRCL */
+  if (!P) return isprimeAPRCL(N)? gen_2: gen_0;
+  return typ(P) != t_INT? PL_certificate(N,P): gen_0;
+}
+
+/* F is a vector whose entries are primes. For each of them, find a PL
+ * witness. Return 0 if caller lied and F contains a composite */
+static long
+PL_certify(GEN N, GEN F)
+{
+  long i, l = lg(F);
+  for(i = 1; i < l; i++)
+    if (! pl831(N, gel(F,i))) return 0;
+  return 1;
+}
+/* F is a vector whose entries are *believed* to be primes. For each of them,
+ * recording a witness and recursive primality certificate */
+static GEN
+PL_certificate(GEN N, GEN F)
+{
+  long i, l = lg(F);
+  GEN W = cgetg(l,t_COL);
+  GEN R = cgetg(l,t_COL);
   for(i=1; i<l; i++)
   {
     GEN p = gel(F,i);
     ulong witness = pl831(N,p);
-
-    if (!witness) { avma = ltop; return gen_0; }
-    gel(P,i) = icopy(p);
+    if (!witness) return gen_0;
     gel(W,i) = utoipos(witness);
     gel(R,i) = check_prime(p);
-    if (gel(R,i) == gen_0)
+    if (isintzero(gel(R,i)))
     { /* composite in prime factorisation ! */
       err_printf("Not a prime: %Ps", p);
-      pari_err_BUG("isprimePL [false prime number]");
+      pari_err_BUG("PL_certificate [false prime number]");
     }
   }
-  return gerepileupto(ltop,C);
+  return mkmat3(F, W, R);
 }
-
-/* F is a vector of BPSW pseudoprimes of N. For each of then, find a PL witness,
- * then prove primality. If (avoid_last) do not apply test to last entry */
-static long
-isprimeSelfridge(GEN N, GEN F, int avoid_last)
+/* Assume N is a strong BPSW pseudoprime, Pocklington-Lehmer primality proof.
+ * Return gen_0 (non-prime), gen_1 (small prime), matrix (large prime)
+ *
+ * The matrix has 3 columns, [a,b,c] with
+ * a[i] prime factor of N-1,
+ * b[i] witness for a[i] as in pl831
+ * c[i] check_prime(a[i]) */
+static GEN
+isprimePL(GEN N)
 {
-  long i, l = lg(F);
-  if (avoid_last) l--;
-  for(i = 1; i < l; i++)
+  pari_sp ltop = avma;
+  GEN cbrtN, N_1, F, f;
+
+  if (typ(N) != t_INT) pari_err_TYPE("isprimePL",N);
+  switch(cmpis(N,2))
   {
-    GEN p = gel(F,i);
-    if (! pl831(N, p)) return 0;
-    if (DEBUGLEVEL>3)
-      err_printf("Pocklington-Lehmer: recursively proving primality of p = %Ps\n", p);
-    if (!BPSW_isprime(p)) return 0;
+    case -1:return gen_0;
+    case 0: return gen_1;
   }
-  return 1;
+  /* N > 2 */
+  cbrtN = sqrtnint(N, 3);
+  N_1 = addis(N,-1);
+  F = Z_factor_until(N_1, sqri(cbrtN));
+  f = factorback(F); /* factored part of N-1, f^3 > N */
+  if (DEBUGLEVEL>3)
+  {
+    GEN r = divri(itor(f,LOWDEFAULTPREC), N);
+    err_printf("Pocklington-Lehmer: proving primality of N = %Ps\n", N);
+    err_printf("Pocklington-Lehmer: N-1 factored up to %Ps! (%.3Ps%%)\n", f, r);
+  }
+  /* if N-1 is only N^(1/3)-smooth, BLS test */
+  if (!equalii(f,N_1) && cmpii(sqri(f),N) <= 0 && !BLS_test(N,f))
+  { avma = ltop; return gen_0; } /* Failed, N is composite */
+  return gerepilecopy(ltop, PL_certificate(N, gel(F,1)));
 }
 
-/* assume N a BPSW pseudoprime, in particular, it is odd > 2 */
+/* assume N a BPSW pseudoprime, in particular, it is odd > 2. Prove N prime */
 long
 BPSW_isprime(GEN N)
 {
-  pari_sp av = avma;
-  long l, res;
-  ulong B;
-  GEN fa, P, E, p, U, F, N_1;
-
+  pari_sp av;
+  long t;
+  GEN P;
   if (BPSW_isprime_small(N)) return 1;
-  N_1 = subis(N,1);
-  B = minuu(1UL<<19, maxprime());
-  fa = Z_factor_limit(N_1, B);
-  P = gel(fa,1);
-  E = gel(fa,2);
-  l = lg(P)-1;
-  p = gel(P,l);
-  U = powii(p, gel(E,l)); /* (possibly) unfactored part of N-1 */
-  if (cmpiu(U, B) <= 0) { U = gen_1; F = N_1; }
+  av = avma; P = BPSW_try_PL(N);
+  if (!P)
+    t = isprimeAPRCL(N); /* not smooth enough */
   else
-  { /* l >= 2, since 2 and U divide  N-1 */
-    if (l == 2)
-      F = powii(gel(P,1), gel(E,1));
-    else
-      F = diviiexact(N_1,  U);
-  }
-  /* N-1 = F U, F factored, U possibly composite */
-  if (U == gen_1) /* smooth */
-    res = isprimeSelfridge(N, P, 0);
-  else if (cmpii(F, U) >= 0) /* 1/2-smooth */
-    res = isprimeSelfridge(N, P, 1);
-  else if (cmpii(sqri(F), U) >= 0) /* 1/3-smooth */
-  {
-    res = BLS_test(N, F);
-    if (res) res = isprimeSelfridge(N, P, 1);
-  }
-  else if (BPSW_psp_nosmalldiv(p)) /* smooth after all */
-    res = isprimeSelfridge(N, P, 0);
-  else
-    res = isprimeAPRCL(N);
-  avma = av; return res;
+    t = (typ(P) == t_INT)? 0: PL_certify(N,P);
+  avma = av; return t;
 }
 
 GEN
@@ -824,14 +828,14 @@ static struct {
   {  4000000483UL,189961831},
   {  4222234741UL,200000000},
 #if BITS_IN_LONG == 64
-  { 11037271757,  500000000},
-  { 22801763489, 1000000000},
-  { 47055833459, 2000000000},
-  {122430513841, 5000000000},
-  {200000000507, 8007105083},
+  { 11037271757UL,  500000000L},
+  { 22801763489UL, 1000000000L},
+  { 47055833459UL, 2000000000L},
+  {122430513841UL, 5000000000L},
+  {200000000507UL, 8007105083L},
 #endif
 };
-static const int prime_table_len = sizeof(prime_table)/sizeof(prime_table[0]);
+static const int prime_table_len = numberof(prime_table);
 
 /* find prime closest to n in prime_table. */
 static long
@@ -947,7 +951,7 @@ randomprime(GEN N)
       b = subiu(N,1); /* between 2 and N-1 */
       d = subiu(N,2);
       if (signe(d) <= 0)
-        pari_err_DOMAIN("randomprime","N", "<", gen_2, N);
+        pari_err_DOMAIN("randomprime","N", "<=", gen_2, N);
       break;
     case t_VEC:
       if (lg(N) != 3) pari_err_TYPE("randomprime",N);
@@ -1125,7 +1129,7 @@ gprimepi_lower_bound(GEN x)
   pari_sp av = avma;
   double L;
   if (typ(x) != t_INT) x = gfloor(x);
-  if (cmpiu(x, 55) <= 0) return gen_0;
+  if (abscmpiu(x, 55) <= 0) return gen_0;
   if (expi(x) <= 1022)
   {
     avma = av;
@@ -1206,7 +1210,7 @@ primes_interval(GEN a, GEN b)
   }
   /* at most d+1 primes in [a,b]. If d large, try better bound to lower
    * memory use */
-  if (cmpiu(d,100000) > 0)
+  if (abscmpiu(d,100000) > 0)
   {
     GEN D = gsub(gprimepi_upper_bound(b), gprimepi_lower_bound(a));
     D = ceil_safe(D);
@@ -1251,7 +1255,7 @@ primes_upto_zv(ulong b)
 {
   ulong d;
   if (b < 2) return cgetg(1, t_VECSMALL);
-  d = (b > 100000UL)? primepi_upper_bound(b): b;
+  d = (b > 100000UL)? (ulong)primepi_upper_bound(b): b;
   return primes_interval_i(2, b, d);
 }
 
@@ -1322,7 +1326,7 @@ addp(GEN *T, GEN p)
   RgV_check_ZV(p, "addprimes");
   v = gen_indexsort_uniq(p, (void*)&cmpii, &cmp_nodata);
   p = vecpermute(p, v);
-  if (cmpiu(gel(p,1), 2) < 0) pari_err_DOMAIN("addprimes", "p", "<", gen_2,p);
+  if (abscmpiu(gel(p,1), 2) < 0) pari_err_DOMAIN("addprimes", "p", "<", gen_2,p);
   p = addp_union(*T, p);
   l = lg(p);
   if (l != lg(*T))
