@@ -70,7 +70,6 @@ zm_divmod(GEN A, GEN B, ulong p)
   return gerepileupto(av, C);
 }
 
-
 static int
 zv_canon(GEN V)
 {
@@ -85,6 +84,31 @@ zv_canon(GEN V)
   return 1;
 }
 
+static GEN
+ZM_to_zm_canon(GEN V)
+{
+  GEN W = ZM_to_zm(V);
+  long i, l = lg(W);
+  for(i=1; i<l; i++)
+    (void)zv_canon(gel(W,i));
+  return W;
+}
+
+static GEN
+zm_apply_zm(GEN M, GEN U)
+{
+  return zm_mul(zm_transpose(U),zm_mul(M, U));
+}
+
+static GEN
+zmV_apply_zm(GEN v, GEN U)
+{
+  long i, l;
+  GEN V = cgetg_copy(v, &l);
+  for(i=1; i<l; i++) gel(V,i) = zm_apply_zm(gel(v,i), U);
+  return V;
+}
+
 /********************************************************************/
 /**                                                                **/
 /**      QFAUTO/QFISOM ported from B. Souvignier ISOM program      **/
@@ -119,7 +143,7 @@ struct fingerprint
 struct qfauto
 {
   long dim;
-  GEN F, V, W, v;
+  GEN F, U, V, W, v;
   ulong p;
 };
 
@@ -302,7 +326,7 @@ bacher(long I, long S, struct qfauto *qf)
   for (i = 1; i <= nlist; ++i)
     coef[1+counts[i] - mind] += 1;
   if (DEBUGLEVEL)
-    err_printf("mind=%ld maxd=%ld sum=%ld\n",mind,maxd,sum);
+    err_printf("QFIsom: mind=%ld maxd=%ld sum=%ld\n",mind,maxd,sum);
   /* the Bacher-polynomial is now: sum from i=mind to maxd over
      coef[i - mind] * X^i */
   return gerepilecopy(av, mkvec2(mkvecsmall3(sum, mind, maxd),coef));
@@ -323,16 +347,15 @@ init_bacher(long bachdep, struct fingerprint *fp, struct qfauto *qf)
 
 /* checks, whether the vector v[I] has the Bacher-polynomial pol  */
 static long
-bachcomp(GEN pol, long I, long S, GEN V, GEN W, GEN Fv)
+bachcomp(GEN pol, long I, GEN V, GEN W, GEN Fv)
 {
   pari_sp av = avma;
   GEN co, list, listxy, vI;
   long i, j, k;
   long nlist, nxy, count;
-  long n = lg(V)-1;
+  const long n = lg(V)-1, S = mael(W,I,1) / 2;
   long sum = mael(pol,1,1), mind = mael(pol,1,2), maxd = mael(pol,1,3);
   GEN coef = gel(pol,2);
-  I = labs(I);
   vI = gel(V,I);
   list = zero_Flv(sum);
   /* nlist should be equal to pol.sum */
@@ -451,11 +474,11 @@ operate(long nr, GEN A, GEN V)
 static GEN
 orbit(GEN pt, long ipt, long npt, GEN H, GEN V)
 {
-  long i, cnd, im;
-  long n = lg(V)-1, nH = lg(H)-1, norb = npt+16, no = npt;
-  GEN flag = zero_Flv(2*n+1)+n+1; /*We need negative indices*/
   pari_sp av = avma;
-  GEN orb = cgetg(norb+1,t_VECSMALL);
+  long i, cnd, im;
+  long n = lg(V)-1, nH = lg(H)-1, no = npt;
+  GEN flag = zero_Flv(2*n+1)+n+1; /*We need negative indices*/
+  GEN orb = cgetg(2*n+1,t_VECSMALL);
   for (i = 1; i <= npt; ++i)
   {
     orb[i] = pt[ipt+i];
@@ -468,16 +491,11 @@ orbit(GEN pt, long ipt, long npt, GEN H, GEN V)
       if (flag[im] == 0)
         /* the image is a new point in the orbit */
       {
-        if (no==norb)
-        {
-          norb<<=1;
-          orb = gerepileuptoleaf(av, vecsmall_lengthen(orb, norb));
-        }
         orb[++no] = im;
         flag[im] = 1;
       }
     }
-  setlg(orb,no+1); return orb;
+  setlg(orb,no+1); return gerepileuptoleaf(av, orb);
 }
 
 /* return the length of the orbit of pt under the first nG matrices in G */
@@ -654,8 +672,8 @@ stab(long I, struct group *G, struct fingerprint *fp, GEN V, ulong p)
            generator is applied to a random point of the orbit to get Rest more
            stabilizer elements */
       {
-        cnd = 1+(long)(pari_rand() % len);
-        i = 1+(long)(pari_rand() % nH);
+        cnd = 1+(long)random_Fl(len);
+        i = 1+(long)random_Fl(nH);
       }
       im = operate(orb[cnd], gel(H,i), V);
       if (flag[im+n+1] == 0)
@@ -822,10 +840,9 @@ qfisom_candidates(GEN CI, long I, GEN x, struct qfauto *qf,
   long DEP = qfcand->cdep, len = f * DEP;
   if (I >= 2  &&  I <= lg(qfcand->bacher_pol))
   {
-    long BACHSCP = mael(W,labs(x[I-1]),1) / 2;
+    long t = labs(x[I-1]);
     GEN bpolI = gel(qfcand->bacher_pol,I-1);
-    if (bachcomp(bpolI, x[I-1], BACHSCP, V, W, gel(v,1)) == 0)
-      return 0;
+    if (bachcomp(bpolI, t, V, W, gel(v,1)) == 0) return 0;
   }
   if (I==1 || DEP ==0)
     return qfisom_candidates_novec(CI,I,x,qf,qff,fp);
@@ -1140,19 +1157,16 @@ zm_maxdiag(GEN A)
 }
 
 static GEN
-init_qfauto(GEN F, long max, struct qfauto *qf, GEN norm)
+init_qfauto(GEN F, GEN U, long max, struct qfauto *qf, GEN norm, GEN minvec)
 {
   long i, j, k;
   GEN W, v;
-  GEN M = minim(zm_to_ZM(gel(F,1)), stoi(max), NULL);
-  GEN V = ZM_to_zm(gel(M, 3));
+  GEN M = minvec? minvec: gel(minim(zm_to_ZM(gel(F,1)), stoi(max), NULL), 3);
+  GEN V = ZM_to_zm_canon(M);
   long n = lg(V)-1, f = lg(F)-1, dim = lg(gel(F,1))-1;
   for (i = 1; i <= n; ++i)
   {
     GEN Vi = gel(V,i);
-    if (typ(Vi)!=t_VECSMALL)
-      pari_err_TYPE("init_qfauto",Vi);
-    zv_canon(Vi);
     for (k = 1; k <= dim; ++k)
     {
       long l = labs(Vi[k]);
@@ -1197,7 +1211,7 @@ init_qfauto(GEN F, long max, struct qfauto *qf, GEN norm)
       }
     }
   }
-  qf->dim = dim; qf->F = F; qf->V = V; qf->W = W; qf->v = v;
+  qf->dim = dim; qf->F = F; qf->V = V; qf->W = W; qf->v = v; qf->U = U;
   return norm;
 }
 
@@ -1401,7 +1415,7 @@ init_flags(struct qfcand *cand, GEN A, struct fingerprint *fp,
 }
 
 static GEN
-gen_group(struct group *G)
+gen_group(struct group *G, GEN U)
 {
   GEN V;
   long i, j, n=1, dim = lg(G->ord)-1;
@@ -1413,7 +1427,8 @@ gen_group(struct group *G)
   V = cgetg(n, t_VEC); n = 1;
   for (i = 1; i <= dim; ++i)
     for (j=G->nsg[i]+1; j<=G->ng[i]; j++)
-      gel(V,n++) = gmael(G->g,i,j);
+      gel(V,n++) = U ? zm_mul(gel(U,1), zm_mul(gmael(G->g,i,j), gel(U,2)))
+                     : gmael(G->g,i,j);
   return mkvec2(o, V);
 }
 
@@ -1434,6 +1449,7 @@ unpack_qfisominit(GEN F, GEN *norm, struct qfauto *qf,
   qf->W = gel(QF,3);
   qf->v = gel(QF,4);
   qf->p = itou(gel(QF,5));
+  qf->U = lg(QF)>6 ? gel(QF,6):NULL;
   QF = gel(F,4);
   fp->diag = gel(QF,1);
   fp->per  = gel(QF,2);
@@ -1444,14 +1460,33 @@ unpack_qfisominit(GEN F, GEN *norm, struct qfauto *qf,
   cand->bacher_pol = gel(QF,3);
   *norm = gel(F,2);
   qf->dim = lg(gmael(F,1,1))-1;
-  return gel(F,1);
+  return qf->F;
+}
+
+static GEN
+qfisom_bestmat(GEN A, long *pt_max)
+{
+  long max = zm_maxdiag(A), max2;
+  GEN A1 = zm_to_ZM(A), A2;
+  GEN U = lllgramint(A1);
+  if (lg(U) != lg(A1))
+    pari_err_DOMAIN("qfisom","form","is not", strtoGENstr("positive definite"), A1);
+  A2 = ZM_to_zm(qf_apply_ZM(A1, U));
+  max2 = zm_maxdiag(A2);
+  if (max2 < max)
+  {
+    *pt_max = max2; return mkvec2(ZM_to_zm(U),ZM_to_zm(ZM_inv(U, gen_1)));
+  } else
+  {
+    *pt_max = max; return NULL;
+  }
 }
 
 static GEN
 init_qfisom(GEN F, struct fingerprint *fp, struct qfcand *cand,
-                   struct qfauto *qf, GEN flags, long *max)
+                   struct qfauto *qf, GEN flags, long *max, GEN minvec)
 {
-  GEN A, norm;
+  GEN U, A, norm;
   if (is_qfisom(F))
   {
     F = unpack_qfisominit(F, &norm, qf, fp, cand);
@@ -1465,11 +1500,27 @@ init_qfisom(GEN F, struct fingerprint *fp, struct qfcand *cand,
     if (lg(F)<2) pari_err_TYPE("qfisom",F);
     A = gel(F,1);
     if (lg(A)<2) pari_err_TYPE("qfisom",A);
-    *max = zm_maxdiag(A);
-    if (DEBUGLEVEL) err_printf("max=%ld\n",*max);
-    norm=init_qfauto(F, *max, qf, NULL);
+    if (!minvec)
+    {
+      U = qfisom_bestmat(A, max);
+      if (DEBUGLEVEL) err_printf("QFIsom: max=%ld\n",*max);
+      if (U) F = zmV_apply_zm(F, gel(U,1));
+    } else
+    {
+      *max = zm_maxdiag(A); U = NULL;
+      if (typ(minvec)==t_VEC && lg(minvec)==4 && typ(gel(minvec,2))==t_INT)
+      {
+        long n = itos(gel(minvec,2));
+        if (n != *max)
+          pari_err_DOMAIN("qfisominit","m[2]","!=",stoi(*max),stoi(n));
+        minvec = gel(minvec, 3);
+      }
+      if (typ(minvec)!=t_MAT || lg(gel(minvec,1))!=lg(A))
+        pari_err_TYPE("qfisominit",minvec);
+    }
+    norm = init_qfauto(F, U, *max, qf, NULL, minvec);
     fingerprint(fp, qf);
-    if (DEBUGLEVEL) err_printf("fp=%Ps\n",fp->diag);
+    if (DEBUGLEVEL) err_printf("QFIsom: fp=%Ps\n",fp->diag);
     init_flags(cand, A, fp, qf, flags);
   }
   return norm;
@@ -1484,10 +1535,10 @@ qfauto(GEN F, GEN flags)
   struct qfcand cand;
   struct qfauto qf;
   long max;
-  (void)init_qfisom(F, &fp, &cand, &qf, flags, &max);
+  (void)init_qfisom(F, &fp, &cand, &qf, flags, &max, NULL);
   init_qfgroup(&G, &fp, &qf);
   autom(&G, &qf, &fp, &cand);
-  return gerepilecopy(av, gen_group(&G));
+  return gerepilecopy(av, gen_group(&G, qf.U));
 }
 
 static GEN
@@ -1574,13 +1625,13 @@ isostab(long pt, GEN G, GEN V, long Maxfail, ulong p)
         if (!zvV_equal(B, gel(w,im+n+1)))
         {
           gel(H,nH+1) = zm_divmod(gel(w,im+n+1),B,p);
-          rpt = 1+(long)(pari_rand() % n);
+          rpt = 1+(long)random_Fl(n);
           tmplen = orbitlen(rpt, 2*n, H, nH+1, V);
           while (tmplen < orblen)
             /* the orbit of this vector is shorter than a previous one, hence
                choose a new random vector */
           {
-            rpt = 1+(long)(pari_rand() % n);
+            rpt = 1+(long)random_Fl(n);
             tmplen = orbitlen(rpt, 2*n, H, nH+1, V);
           }
           if (tmplen > orblen)
@@ -1679,27 +1730,28 @@ isometry(struct qfauto *qf, struct qfauto *qff, struct fingerprint *fp, GEN G,
 }
 
 GEN
-qfisominit(GEN F, GEN flags)
+qfisominit(GEN F, GEN flags, GEN minvec)
 {
   pari_sp av = avma;
   struct fingerprint fp;
   struct qfauto qf;
   struct qfcand cand;
   long max;
-  GEN norm = init_qfisom(F, &fp, &cand, &qf, flags, &max);
-  return gerepilecopy(av, mkvec5(F, norm, mkvec5(qf.F, qf.V, qf.W, qf.v, utoi(qf.p)),
+  GEN norm = init_qfisom(F, &fp, &cand, &qf, flags, &max, minvec);
+  return gerepilecopy(av, mkvec5(F, norm,
+                          mkvecn(qf.U?6:5, qf.F, qf.V, qf.W, qf.v, utoi(qf.p), qf.U),
                           mkvec3(fp.diag, fp.per, fp.e),
                           mkvec3(stoi(cand.cdep),cand.comb?cand.comb:cgetg(1,t_VEC),
                                  cand.bacher_pol)));
 }
 
 GEN
-qfisominit0(GEN x, GEN flags)
+qfisominit0(GEN x, GEN flags, GEN minvec)
 {
   pari_sp av = avma;
   GEN F = qf_to_zmV(x);
   if (!F) pari_err_TYPE("qfisom",x);
-  return gerepileupto(av, qfisominit(F, flags));
+  return gerepileupto(av, qfisominit(F, flags, minvec));
 }
 
 GEN
@@ -1711,15 +1763,15 @@ qfisom(GEN F, GEN FF, GEN flags)
   struct qfauto qf, qff;
   struct qfcand cand;
   long max;
-  GEN norm = init_qfisom(F, &fp, &cand, &qf, flags, &max);
-  init_qfauto(FF, max, &qff, norm);
+  GEN norm = init_qfisom(F, &fp, &cand, &qf, flags, &max, NULL);
+  init_qfauto(FF, NULL, max, &qff, norm, NULL);
   if (lg(qf.W)!=lg(qff.W)
       || !zvV_equal(vecvecsmall_sort(qf.W), vecvecsmall_sort(qff.W)))
     { avma=av; return gen_0; }
   G = mkvec(scalar_Flm(-1, qff.dim));
   res = isometry(&qf, &qff, &fp, G, &cand);
   if (!res) { avma=av; return gen_0; }
-  return gerepilecopy(av, zm_to_ZM(res));
+  return gerepilecopy(av, zm_to_ZM(qf.U? zm_mul(res,gel(qf.U, 2)):res));
 }
 
 GEN
@@ -1794,3 +1846,55 @@ qfautoexport(GEN G, long flag)
   gel(str,c++) = strtoGENstr(flag ? ">":")");
   return gerepilecopy(av, shallowconcat1(str));
 }
+
+GEN
+qforbits(GEN G, GEN V)
+{
+  pari_sp av = avma;
+  GEN gen, w, W, p, v, orb, o;
+  long i, j, n, ng;
+  long nborbits = 0;
+  if (typ(G)==t_VEC && lg(G)==3 && typ(gel(G,1))==t_INT)
+    G = gel(G,2);
+  gen = qf_to_zmV(G);
+  if (!gen) pari_err_TYPE("qforbits", G);
+  if (typ(V)==t_VEC && lg(V)==4
+   && typ(gel(V,1))==t_INT && typ(gel(V,2))==t_INT)
+    V = gel(V,3);
+  if (typ(V)!=t_MAT || !RgM_is_ZM(V)) pari_err_TYPE("qforbits", V);
+  n = lg(V)-1; ng = lg(gen)-1;
+  W = ZM_to_zm_canon(V);
+  p = vecvecsmall_indexsort(W);
+  v = vecpermute(W, p);
+  w = zero_zv(n);
+  orb = cgetg(n+1, t_VEC);
+  o = cgetg(n+1, t_VECSMALL);
+  if (lg(v) != lg(V)) return gen_0;
+  for (i=1; i<=n; i++)
+  {
+    long cnd, no = 1;
+    GEN T;
+    if (w[i]) continue;
+    w[i] = ++nborbits;
+    o[1] = i;
+    for (cnd=1; cnd <= no; ++cnd)
+      for(j=1; j <= ng; j++)
+      {
+        long k;
+        GEN Vij = zm_zc_mul(gel(gen, j), gel(v, o[cnd]));
+        (void) zv_canon(Vij);
+        k = vecvecsmall_search(v, Vij, 0);
+        if (k == 0) { avma = av; return gen_0; }
+        if (w[k] == 0)
+        {
+          o[++no] = k;
+          w[k] = nborbits;
+        }
+      }
+    T = cgetg(no+1, t_VEC);
+    for (j=1; j<=no; j++) gel(T,j) = gel(V,p[o[j]]);
+    gel(orb, nborbits) = T;
+  }
+  setlg(orb, nborbits+1);
+  return gerepilecopy(av, orb);
+}
