@@ -450,7 +450,8 @@ gpowg0(GEN x)
     case t_INTMOD:
       y = cgetg(3,t_INTMOD);
       gel(y,1) = icopy(gel(x,1));
-      gel(y,2) = gen_1; return y;
+      gel(y,2) = is_pm1(gel(x,1))? gen_0: gen_1;
+      return y;
 
     case t_FFELT: return FF_1(x);
 
@@ -1365,7 +1366,7 @@ sqrt_ser(GEN b, long prec)
   while (mask > 1)
   {
     GEN y, x2 = gmul2n(x,1);
-    long l = lold << 1;
+    long l = lold << 1, lx;
 
     if (mask & 1) l--;
     mask >>= 1;
@@ -1376,7 +1377,8 @@ sqrt_ser(GEN b, long prec)
     y += lold; setvalp(y, lold);
     y = normalize(y);
     y = gsub(x, gdiv(y, x2)); /* = gmul2n(gsub(x, gdiv(a,x)), -1); */
-    for (j = lold+2; j < l+2; j++) gel(x,j) = gel(y,j);
+    lx = minss(l+2, lg(y));
+    for (j = lold+2; j < lx; j++) gel(x,j) = gel(y,j);
     lold = l;
   }
   x[1] = evalsigne(1) | evalvarn(vx) | _evalvalp(e >> 1);
