@@ -23,9 +23,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 static void SetForeground(void *data, long col)
 {
   int r,g,b;
-  color_to_rgb(gel(GP_DATA->colormap,col), &r, &g, &b);
+  HPEN hOldPen;
+  color_to_rgb(gel(GP_DATA->colormap,col+1), &r, &g, &b);
 
-  HPEN hOldPen = SelectObject((HDC)data, CreatePen(PS_SOLID, 2, RGB(r,g,b)));
+  hOldPen = SelectObject((HDC)data, CreatePen(PS_SOLID, 1, RGB(r,g,b)));
   if( hOldPen ) DeleteObject(hOldPen);
 }
 
@@ -42,10 +43,10 @@ static void DrawLine(void *data, long x1, long y1, long x2, long y2)
 
 static void DrawRectangle(void *data, long x, long y, long w, long h)
 {
-  RECT rc;
-  rc.left = x; rc.right  = x+w;
-  rc.top  = y; rc.bottom = y+h;
-  FrameRect((HDC)data, &rc, GetStockObject(HOLLOW_BRUSH));
+  DrawLine(data, x,y,x+w,y);
+  DrawLine(data, x,y,x,y+h);
+  DrawLine(data, x+w,y,x+w,y+h);
+  DrawLine(data, x,y+h,x+w,y+h);
 }
 
 static void DrawPoints(void *data, long nb, struct plot_points *p)
@@ -109,8 +110,8 @@ PARI_get_plot(void)
   if (pari_plot.init) return;      /* pari_plot is already set */
 
   pari_plot.init    = 1;
-  pari_plot.width   = GetSystemMetrics(SM_CXSCREEN)/2;
-  pari_plot.height  = GetSystemMetrics(SM_CYSCREEN)/2;
+  pari_plot.width   = GetSystemMetrics(SM_CXSCREEN)*4/5;
+  pari_plot.height  = GetSystemMetrics(SM_CYSCREEN)*4/5;
   pari_plot.hunit   = pari_plot.width/100;
   pari_plot.vunit   = pari_plot.height/100;
 
