@@ -1235,7 +1235,7 @@ rectplothin(GEN a, GEN b, GEN code, long prec, ulong flags, long testpoints)
       nl = lg(t)-1;
       nc = nl/2;
       if (odd(nl))
-        pari_err_TYPE("ploth [parametric ploc with odd # of components]",t);
+        pari_err_TYPE("ploth [parametric plot with odd # of components]",t);
     }
   }
   else
@@ -1783,6 +1783,7 @@ static void
 ps_rect(void *data, long x, long y, long w, long h)
 {
   fprintf((FILE*)data,"%ld %ld m %ld %ld l %ld %ld l %ld %ld l closepath\n",y,x, y,x+w, y+h,x+w, y+h,x);
+  fprintf((FILE*)data,"stroke\n");
 }
 
 static void
