@@ -238,7 +238,6 @@ void  initrect(long ne, long x, long y);
 void  initrect_gen(long ne, GEN x, GEN y, long flag);
 void  killrect(long ne);
 void  plot_count(long *w, long lw, col_counter rcolcnt);
-void  plot(GEN a, GEN b, GEN code, GEN ysmlu, GEN ybigu, long prec);
 GEN   ploth(GEN a, GEN b, GEN code, long prec, long flag, long numpoints);
 GEN   ploth2(GEN a, GEN b, GEN code, long prec);
 GEN   plothmult(GEN a, GEN b, GEN code, long prec);
@@ -281,7 +280,7 @@ void  rectclip(long rect);
 void gen_rectdraw0(struct plot_eng *eng, long *w, long *x, long *y, long lw, double xs, double ys);
 
 /* architecture-dependent plot file (plotX.c ...) */
-void  PARI_get_plot();
+void  PARI_get_plot(void);
 void  rectdraw0(long *w, long *x, long *y, long lw);
 
 ENDEXTERN
