@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -13,8 +11,63 @@ Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
-/* For compatibility with 1.x.x */
+/* For compatibility with older PARI versions */
 /*functions renamed*/
+#define cmpui abscmpui
+#define equalui absequalui
+#define equaliu absequaliu
+#define cmpiu abscmpiu
+#define absi_factor absZ_factor
+#define absi_factor_limit absZ_factor_limit
+#define absi_cmp abscmpii
+#define absr_cmp abscmprr
+#define absi_equal absequalii
+#define anell ellan
+#define anellsmall ellanQ_zv
+#define leading_term leading_coeff
+#define constant_term constant_coeff
+#define concat gconcat
+#define concat1 gconcat1
+#define mathell ellheightmatrix
+#define ghell ellheight
+#define mpexp1 mpexpm1
+#define ggamd ggammah
+#define gach gacosh
+#define gash gasinh
+#define gath gatanh
+#define gch gcosh
+#define gsh gsinh
+#define gth gtanh
+#define recip serreverse
+#define gcmpX gequalX
+#define ZM_hnfremdiv ZM_hnfdivrem
+#define rnfidealhermite rnfidealhnf
+#define rnfelementabstorel rnfeltabstorel
+#define rnfelementreltoabs rnfeltreltoabs
+#define rnfelementdown rnfeltdown
+#define rnfelementup rnfeltup
+#define exp_Ir expIr
+#define vecbezout gcdext0
+#define vecbezoutres polresultantext
+#define init_primepointer init_primepointer_geq
+#define ellap0(e,p,flag) ellap(e,p)
+#define apell2(e,p) ellap(e,p)
+#define geulerphi eulerphi
+#define numbdiv numdiv
+#define gnumbdiv numdiv
+#define gsumdivk sumdivk
+#define gnextprime nextprime
+#define gprecprime precprime
+#define leftright_pow_fold gen_pow_fold
+#define leftright_pow_u_fold gen_powu_fold
+#define subell ellsub
+#define addell elladd
+#define powell ellmul
+#define ggval gvaluation
+#define stackmalloc stack_malloc
+#define listcreate mklist
+
+/* Following obsoleted in 2.5.* (2011) */
 #define fprintferr err_printf
 #define msgTIMER timer_printf
 #define TIMER timer_delay
@@ -53,7 +106,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define nfdiscf0(x) nfdisc0(x)
 #define discf(x) nfdisc(x)
 #define discsr(x) poldisc0((x),-1)
-#define isfundamental Z_isfundamental
 #define factorpadic4 factorpadic
 #define smith2 smithall
 #define gsmith2 gsmithall
@@ -65,8 +117,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define pariputc pari_putc
 #define pariputs pari_puts
 #define pariflush pari_flush
-#define ellap0(e,p,flag) apell(e,p)
-#define apell2(e,p) apell(e,p)
+
+/* Following deprecated for a long time now, obsoleted in 2.3.* (2007) */
+#ifdef PARI_OLD_NAMES
 #define apell ellap
 #define gscalsmat scalarmat_s
 #define sqred qfgaussred
@@ -194,9 +247,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define absrz(x,z)  mpabsz((x),(z))
 #define negrz(x,z)  mpnegz((x),(z))
 
-/* Following deprecated for a long time now. Or really, really bad, e.g
- * un, init, er. */
-#ifdef PARI_OLD_NAMES
 #define err pari_err
 #define init pari_init
 
@@ -214,7 +264,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 /*casts*/
 #define labsi   (long)absi
 #define labsr   (long)absr
-#define lach    (long)gach
+#define lach    (long)gacosh
 #define lacos   (long)gacos
 #define ladd    (long)gadd
 #define laddgs  (long)gaddgs
@@ -228,20 +278,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define laddsr  (long)addsr
 #define ladj    (long)adj
 #define larg    (long)garg
-#define lash    (long)gash
+#define lash    (long)gasinh
 #define lasin   (long)gasin
 #define lassmat (long)matcompanion
 #define latan   (long)gatan
-#define lath    (long)gath
+#define lath    (long)gatanh
 #define lbezout (long)bezout
 #define lbinome (long)binomial
 #define lcaract (long)caract
 #define lcaradj (long)caradj
 #define lceil   (long)gceil
-#define lch     (long)gch
+#define lch     (long)gcosh
 #define lchangevar (long)changevar
 #define lclone  (long)gclone
-#define lconcat (long)concat
+#define lconcat (long)gconcat
 #define lconj   (long)gconj
 #define lcontent (long)content
 #define lcopy   (long)gcopy
@@ -272,7 +322,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define lfibo   (long)fibo
 #define lfloor  (long)gfloor
 #define lfrac   (long)gfrac
-#define lgamd   (long)ggamd
+#define lgamd   (long)ggammah
 #define lgamma  (long)ggamma
 #define lgauss  (long)gauss
 #define lgcd    (long)ggcd
@@ -309,7 +359,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define lmpdiv  (long)mpdiv
 #define lmpent  (long)mpent
 #define lmpeuler (long)mpeuler
-#define lmpexp1 (long)mpexp1
+#define lmpexp1 (long)mpexpm1
 #define lmpexp  (long)mpexp
 #define lmpfact (long)mpfact
 #define lmplog  (long)mplog
@@ -364,7 +414,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define lroots  (long)roots
 #define lscalmat (long)scalarmat
 #define lscalsmat (long)scalarmat_s
-#define lsh     (long)gsh
+#define lsh     (long)gsinh
 #define lshifti (long)shifti
 #define lshift  (long)gshift
 #define lshiftr (long)shiftr
@@ -388,7 +438,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #define lsubsr  (long)subsr
 #define ltan    (long)gtan
 #define ltchebi (long)polchebyshev
-#define lth     (long)gth
+#define lth     (long)gtanh
 #define ltrace  (long)gtrace
 #define ltrans  (long)gtrans
 #define ltrunc  (long)gtrunc
