@@ -157,7 +157,7 @@ incpos(GEN a)
 {
   long i, l = lgefint(a);
   for (i=2; i<l; i++)
-    if (++a[i]) return a;
+    if (++uel(a,i)) return a;
   a[l] = 1; l++;
   a[0]=evaltyp(t_INT) | _evallg(l);
   a[1]=evalsigne(1) | evallgefint(l);
@@ -169,7 +169,7 @@ static GEN
 incneg(GEN a)
 {
   long i, l = lgefint(a)-1;
-  if (a[2]--)
+  if (uel(a,2)--)
   {
     if (!a[l]) /* implies l = 2 */
     {
@@ -179,7 +179,7 @@ incneg(GEN a)
     return a;
   }
   for (i=3; i<=l; i++)
-    if (a[i]--) break;
+    if (uel(a,i)--) break;
   if (!a[l])
   {
     a[0] = evaltyp(t_INT) | _evallg(l);
@@ -209,7 +209,7 @@ adduispec(ulong s, GEN x, long nx)
   GEN  zd;
   long lz;
 
-  if (nx == 1) return adduu((ulong)x[0], s);
+  if (nx == 1) return adduu(uel(x,0), s);
   lz = nx+3; zd = cgeti(lz);
   if (mpn_add_1(LIMBS(zd),(mp_limb_t *)x,nx,s))
     zd[lz-1]=1;
@@ -313,10 +313,10 @@ affir(GEN x, GEN y)
       return;
     }
     mpn_lshift(LIMBS(y),LIMBS(x)+lx-ly,ly-2,sh);
-    y[2]|=((ulong) x[lx-ly+1])>>(BITS_IN_LONG-sh);
+    uel(y,2) |= uel(x,lx-ly+1) >> (BITS_IN_LONG-sh);
     xmpn_mirror(LIMBS(y),ly-2);
     /* lx > ly: round properly */
-    if ((x[lx-ly+1]<<sh) & HIGHBIT) roundr_up_ip(y, ly);
+    if ((uel(x,lx-ly+1)<<sh) & HIGHBIT) roundr_up_ip(y, ly);
   }
   else {
     GEN xd=int_MSW(x);
@@ -328,7 +328,7 @@ affir(GEN x, GEN y)
     }
     for (i=2; i<ly; i++,xd=int_precW(xd)) y[i]=*xd;
     /* lx > ly: round properly */
-    if (x[lx-ly+1] & HIGHBIT) roundr_up_ip(y, ly);
+    if (uel(x,lx-ly+1) & HIGHBIT) roundr_up_ip(y, ly);
   }
 }
 
@@ -405,7 +405,7 @@ mantissa2nr(GEN x, long n)
     {
       register const ulong sh = BITS_IN_LONG - m;
       shift_left(y,x, 2,lx-1, 0,m);
-      i = ((ulong)x[2]) >> sh;
+      i = uel(x,2) >> sh;
       /* Extend y on the left? */
       if (i) { ly++; y = new_chunk(1); y[2] = i; }
     }
@@ -475,7 +475,7 @@ floorr(GEN x)
     GEN z=cgeti(d);
     for (i=2; i<d; i++) z[d-i+1]=x[i];
     mpn_rshift(LIMBS(y),LIMBS(z),d-2,BITS_IN_LONG-m);
-    if (x[d-1]<<m == 0)
+    if (uel(x,d-1)<<m == 0)
     {
       i=d; while (i<lx && !x[i]) i++;
       if (i==lx) goto END;
@@ -532,7 +532,7 @@ addumului(ulong a, ulong b, GEN y)
   GEN z;
   long i, lz;
   ulong hi;
-  if (!signe(y)) return utoi(a);
+  if (!b || !signe(y)) return utoi(a);
   lz = lgefint(y)+1;
   z = cgeti(lz);
   z[2]=a;
@@ -572,7 +572,7 @@ diviu_rem(GEN y, ulong x, ulong *rem)
   if (!signe(y)) { *rem = 0; return gen_0; }
 
   ly = lgefint(y);
-  if (ly == 3 && (ulong)x > (ulong)y[2]) { *rem = (ulong)y[2]; return gen_0; }
+  if (ly == 3 && (ulong)x > uel(y,2)) { *rem = uel(y,2); return gen_0; }
 
   z = cgeti(ly);
   *rem = mpn_divrem_1(LIMBS(z), 0, LIMBS(y), NLIMBS(y), x);
@@ -592,7 +592,7 @@ divis_rem(GEN y, long x, long *rem)
   if (x<0) { s = -sy; x = -x; } else s = sy;
 
   ly = lgefint(y);
-  if (ly == 3 && (ulong)x > (ulong)y[2]) { *rem = itos(y); return gen_0; }
+  if (ly == 3 && (ulong)x > uel(y,2)) { *rem = itos(y); return gen_0; }
 
   z = cgeti(ly);
   *rem = mpn_divrem_1(LIMBS(z), 0, LIMBS(y), NLIMBS(y), x);
@@ -613,7 +613,7 @@ divis(GEN y, long x)
   if (x<0) { s = -sy; x = -x; } else s=sy;
 
   ly = lgefint(y);
-  if (ly == 3 && (ulong)x > (ulong)y[2]) return gen_0;
+  if (ly == 3 && (ulong)x > uel(y,2)) return gen_0;
 
   z = cgeti(ly);
   (void)mpn_divrem_1(LIMBS(z), 0, LIMBS(y), NLIMBS(y), x);
@@ -646,7 +646,7 @@ divrr_with_gmp(GEN x, GEN y)
   mpn_tdiv_qr(q,r,0,u,lu,z,lly);
 
   /*Round up: This is not exactly correct we should test 2*r>z*/
-  if ((ulong)r[lly-1] > ((ulong)z[lly-1]>>1))
+  if (uel(r,lly-1) > (uel(z,lly-1)>>1))
     mpn_add_1(q,q,lw+1,1);
 
   xmpn_mirrorcopy(RLIMBS(w),q,lw);
@@ -684,7 +684,7 @@ divri_with_gmp(GEN x, GEN y)
   mpn_tdiv_qr(q,r,0,u,lu,z,lly);
 
   /*Round up: This is not exactly correct we should test 2*r>z*/
-  if ((ulong)r[lly-1] > ((ulong)z[lly-1]>>1))
+  if (uel(r,lly-1) > (uel(z,lly-1)>>1))
     mpn_add_1(q,q,llx+1,1);
 
   xmpn_mirrorcopy(RLIMBS(w),q,llx);
@@ -730,7 +730,7 @@ divrr(GEN x, GEN y)
   {
     ulong k = x[2], l = (lx>3)? x[3]: 0;
     LOCAL_HIREMAINDER;
-    if (k < (ulong)y[2]) e--;
+    if (k < uel(y,2)) e--;
     else
     {
       l >>= 1; if (k&1) l |= HIGHBIT;
@@ -761,18 +761,18 @@ divrr(GEN x, GEN y)
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
 
-    if ((ulong)r1[1] == y0)
+    if (uel(r1,1) == y0)
     {
       qp = ULONG_MAX; k = addll(y0,r1[2]);
     }
     else
     {
-      if ((ulong)r1[1] > y0) /* can't happen if i=0 */
+      if (uel(r1,1) > y0) /* can't happen if i=0 */
       {
         GEN y1 = y+1;
         j = lr-i; r1[j] = subll(r1[j],y1[j]);
         for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
-        j=i; do ((ulong*)r)[--j]++; while (j && !r[j]);
+        j=i; do uel(r,--j)++; while (j && !r[j]);
       }
       hiremainder = r1[1]; overflow = 0;
       qp = divll(r1[2],y0); k = hiremainder;
@@ -797,9 +797,9 @@ divrr(GEN x, GEN y)
       r1[j] = subll(r1[j], addmul(qp,y[j]));
       hiremainder += overflow;
     }
-    if ((ulong)r1[1] != hiremainder)
+    if (uel(r1,1) != hiremainder)
     {
-      if ((ulong)r1[1] < hiremainder)
+      if (uel(r1,1) < hiremainder)
       {
         qp--;
         j = lr-i-(lr-i>=ly); r1[j] = addll(r1[j], y[j]);
@@ -807,13 +807,13 @@ divrr(GEN x, GEN y)
       }
       else
       {
-        r1[1] -= hiremainder;
+        uel(r1,1) -= hiremainder;
         while (r1[1])
         {
-          qp++; if (!qp) { j=i; do ((ulong*)r)[--j]++; while (j && !r[j]); }
+          qp++; if (!qp) { j=i; do uel(r,--j)++; while (j && !r[j]); }
           j = lr-i-(lr-i>=ly); r1[j] = subll(r1[j],y[j]);
           for (j--; j>1; j--) r1[j] = subllx(r1[j],y[j]);
-          r1[1] -= overflow;
+          uel(r1,1) -= overflow;
         }
       }
     }
@@ -821,9 +821,9 @@ divrr(GEN x, GEN y)
   }
   /* i = lr-1 */
   /* round correctly */
-  if ((ulong)r1[1] > (y0>>1))
+  if (uel(r1,1) > (y0>>1))
   {
-    j=i; do r[--j]++; while (j && !r[j]);
+    j=i; do uel(r,--j)++; while (j && !r[j]);
   }
   r1 = r-1; for (j=i; j>=2; j--) r[j]=r1[j];
   if (r[0] == 0) e--;
@@ -852,11 +852,7 @@ dvmdii(GEN x, GEN y, GEN *z)
   pari_sp av;
   GEN r,q;
 
-  if (!sy)
-  {
-    if (z == ONLY_REM && !sx) return gen_0;
-    pari_err_INV("dvmdii",y);
-  }
+  if (!sy) pari_err_INV("dvmdii",y);
   if (!sx)
   {
     if (!z || z == ONLY_REM) return gen_0;
@@ -962,17 +958,17 @@ red_montgomery(GEN T, GEN N, ulong inv)
 #endif
   if (k == 1)
   { /* as below, special cased for efficiency */
-    ulong n = (ulong)N[2];
+    ulong n = uel(N,2);
     if (d == 1) {
-      hiremainder = (ulong)T[2];
+      hiremainder = uel(T,2);
       m = hiremainder * inv;
       (void)addmul(m, n); /* t + m*n = 0 */
       return utoi(hiremainder);
     } else { /* d = 2 */
-      hiremainder = (ulong)T[2];
+      hiremainder = uel(T,2);
       m = hiremainder * inv;
       (void)addmul(m, n); /* t + m*n = 0 */
-      t = addll(hiremainder, (ulong)T[3]);
+      t = addll(hiremainder, uel(T,3));
       if (overflow) t -= n; /* t > n doesn't fit in 1 word */
       return utoi(t);
     }
@@ -1117,6 +1113,7 @@ GEN
 diviiexact(GEN x, GEN y)
 {
   /*TODO: use mpn_bdivmod instead*/
+  if (!signe(y)) pari_err_INV("diviiexact",y);
   return divii(x,y);
 }
 #endif
@@ -1334,7 +1331,7 @@ sqrtr_abs(GEN a)
   {
     ulong u;
     b = (mp_limb_t *) mantissa2nr(a,-1);
-    b[1] = ((ulong)a[l+1])<<(BITS_IN_LONG-1);
+    b[1] = uel(a,l+1)<<(BITS_IN_LONG-1);
     b = (mp_limb_t *) new_chunk(l);
     xmpn_zero(b,l+1); /* overwrites the former b[0] */
     c = (mp_limb_t *) new_chunk(l + 1);
