@@ -31,13 +31,13 @@ bezout(GEN a, GEN b, GEN *pu, GEN *pv)
 {
   GEN t,u,u1,v,v1,d,d1,q,r;
   GEN *pt;
-  pari_sp av, av1, lim;
+  pari_sp av, av1;
   long s, sa, sb;
   ulong g;
   ulong xu,xu1,xv,xv1;                /* Lehmer stage recurrence matrix */
   int lhmres;                        /* Lehmer stage return value */
 
-  s = absi_cmp(a,b);
+  s = abscmpii(a,b);
   if (s < 0)
   {
     t=b; b=a; a=t;
@@ -68,7 +68,7 @@ bezout(GEN a, GEN b, GEN *pu, GEN *pv)
 
   if (lgefint(a) == 3)                /* single-word affair */
   {
-    g = xxgcduu((ulong)a[2], (ulong)b[2], 0, &xu, &xu1, &xv, &xv1, &s);
+    g = xxgcduu(uel(a,2), uel(b,2), 0, &xu, &xu1, &xv, &xv1, &s);
     sa = s > 0 ? sa : -sa;
     sb = s > 0 ? -sb : sb;
     if (pu)
@@ -128,7 +128,7 @@ bezout(GEN a, GEN b, GEN *pu, GEN *pv)
     d = absi(a); d1 = absi(b);
     u = v1 = gen_1; u1 = v = gen_0;
   }
-  av1 = avma; lim = stack_lim(av, 1);
+  av1 = avma;
 
   /* main loop is almost identical to that of invmod() */
   while (lgefint(d) > 3 && signe(d1))
@@ -166,17 +166,13 @@ bezout(GEN a, GEN b, GEN *pu, GEN *pv)
     if (lhmres <= 0 && signe(d1))
     {
       q = dvmdii(d,d1,&r);
-#ifdef DEBUG_LEHMER
-      err_printf("Full division:\n");
-      printf("  q = "); output(q); sleep (1);
-#endif
       a = subii(u,mulii(q,u1));
       u=u1; u1=a;
       a = subii(v,mulii(q,v1));
       v=v1; v1=a;
       d=d1; d1=r;
     }
-    if (low_stack(lim, stack_lim(av,1)))
+    if (gc_needed(av,1))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"bezout");
       gerepileall(av1,6, &d,&d1,&u,&u1,&v,&v1);
@@ -189,7 +185,7 @@ bezout(GEN a, GEN b, GEN *pu, GEN *pv)
     /* Assertions: lgefint(d)==lgefint(d1)==3, and
      * gcd(d,d1) is nonzero and fits into one word
      */
-    g = xxgcduu((ulong)(d[2]), (ulong)(d1[2]), 0, &xu, &xu1, &xv, &xv1, &s);
+    g = xxgcduu(uel(d,2), uel(d1,2), 0, &xu, &xu1, &xv, &xv1, &s);
     u = subii(muliu(u,xu), muliu(u1, xv));
     v = subii(muliu(v,xu), muliu(v1, xv));
     if (s < 0) { sa = -sa; sb = -sb; }
