@@ -93,7 +93,7 @@ static GEN
 incneg(GEN a)
 {
   long i, l = lgefint(a)-1;
-  if (a[l]--)
+  if (uel(a,l)--)
   {
     if (l == 2 && !a[2])
     {
@@ -104,7 +104,7 @@ incneg(GEN a)
     return a;
   }
   for (i = l-1;; i--) /* finishes since a[2] != 0 */
-    if (a[i]--) break;
+    if (uel(a,i)--) break;
   if (!a[2])
   {
     a++; /* save one cell */
@@ -135,7 +135,7 @@ adduispec(ulong s, GEN x, long nx)
   GEN xd, zd = (GEN)avma;
   long lz;
 
-  if (nx == 1) return adduu(s, (ulong)x[0]);
+  if (nx == 1) return adduu(s, uel(x,0));
   lz = nx+3; (void)new_chunk(lz);
   xd = x + nx;
   *--zd = (ulong)*--xd + s;
@@ -185,7 +185,7 @@ addiispec(GEN x, GEN y, long nx, long ny)
     for(;;)
     {
       if (i < -nx) { zd[i] = 1; i--; break; } /* enlarge z */
-      zd[i] = (ulong)xd[i] + 1;
+      zd[i] = uel(xd,i) + 1;
       if (zd[i]) { i--; lz--; break; }
       i--;
     }
@@ -247,7 +247,7 @@ subiispec(GEN x, GEN y, long nx, long ny)
   if (overflow)
     for(;;)
     {
-      zd[i] = ((ulong)xd[i]) - 1;
+      zd[i] = uel(xd,i) - 1;
       if (xd[i--]) break;
     }
   if (i>=-nx)
@@ -266,7 +266,7 @@ roundr_up_ip(GEN x, long l)
   long i = l;
   for(;;)
   {
-    if (++((ulong*)x)[--i]) break;
+    if (++uel(x,--i)) break;
     if (i == 2) { x[2] = (long)HIGHBIT; shiftr_inplace(x, 1); break; }
   }
 }
@@ -279,7 +279,7 @@ affir(GEN x, GEN y)
 
   if (!s)
   {
-    y[1] = evalexpo(-bit_accuracy(ly));
+    y[1] = evalexpo(-prec2nbits(ly));
     return;
   }
 
@@ -294,7 +294,7 @@ affir(GEN x, GEN y)
     }
     shift_left(y,x,2,ly-1, x[ly],sh);
     /* lx > ly: round properly */
-    if ((x[ly]<<sh) & HIGHBIT) roundr_up_ip(y, ly);
+    if ((uel(x,ly)<<sh) & HIGHBIT) roundr_up_ip(y, ly);
   }
   else {
     if (lx <= ly)
@@ -305,7 +305,7 @@ affir(GEN x, GEN y)
     }
     for (i=2; i<ly; i++) y[i]=x[i];
     /* lx > ly: round properly */
-    if (x[ly] & HIGHBIT) roundr_up_ip(y, ly);
+    if (uel(x,ly) & HIGHBIT) roundr_up_ip(y, ly);
   }
 }
 
@@ -328,7 +328,7 @@ shiftispec(GEN x, long nx, long n)
     {
       register const ulong sh = BITS_IN_LONG - m;
       shift_left(yd,x, 0,nx-1, 0,m);
-      i = ((ulong)x[0]) >> sh;
+      i = uel(x,0) >> sh;
       /* Extend y on the left? */
       if (i) { ny++; y = new_chunk(1); y[2] = i; }
     }
@@ -404,14 +404,14 @@ floorr(GEN x)
   else
   {
     shift_right(y,x, 2,d,0, BITS_IN_LONG - m);
-    if (x[d-1]<<m == 0)
+    if (uel(x,d-1)<<m == 0)
     {
       i=d; while (i<lx && !x[i]) i++;
       if (i==lx) goto END;
     }
   }
   /* set y:=y+1 */
-  for (i=d-1; i>=2; i--) { y[i]++; if (y[i]) goto END; }
+  for (i=d-1; i>=2; i--) { uel(y,i)++; if (y[i]) goto END; }
   y=new_chunk(1); y[2]=1; d++;
 END:
   y[1] = evalsigne(-1) | evallgefint(d);
@@ -426,7 +426,7 @@ cmpiispec(GEN x, GEN y, long lx, long ly)
   if (lx > ly) return  1;
   i = 0; while (i<lx && x[i]==y[i]) i++;
   if (i==lx) return 0;
-  return ((ulong)x[i] > (ulong)y[i])? 1: -1;
+  return (uel(x,i) > uel(y,i))? 1: -1;
 }
 
 INLINE int
@@ -469,7 +469,7 @@ addumului(ulong a, ulong b, GEN Y)
   LOCAL_HIREMAINDER;
   LOCAL_OVERFLOW;
 
-  if (!signe(Y)) return utoi(a);
+  if (!b || !signe(Y)) return utoi(a);
 
   y = LIMBS(Y); z = (GEN)avma;
   ny = NLIMBS(Y);
@@ -495,18 +495,29 @@ ulong
 umodiu(GEN y, ulong x)
 {
   long sy=signe(y),ly,i;
+  ulong xi;
   LOCAL_HIREMAINDER;
 
   if (!x) pari_err_INV("umodiu",gen_0);
   if (!sy) return 0;
   ly = lgefint(y);
-  if (x <= (ulong)y[2]) hiremainder=0;
+  if (x <= uel(y,2))
+  {
+    hiremainder=0;
+    if (ly==3)
+    {
+      hiremainder=uel(y,2)%x;
+      if (!hiremainder) return 0;
+      return (sy > 0)? hiremainder: x - hiremainder;
+    }
+  }
   else
   {
-    if (ly==3) return (sy > 0)? (ulong)y[2]: x - (ulong)y[2];
-    hiremainder=y[2]; ly--; y++;
+    if (ly==3) return (sy > 0)? uel(y,2): x - uel(y,2);
+    hiremainder=uel(y,2); ly--; y++;
   }
-  for (i=2; i<ly; i++) (void)divll(y[i],x);
+  xi = get_Fl_red(x);
+  for (i=2; i<ly; i++) (void)divll_pre(y[i],x,xi);
   if (!hiremainder) return 0;
   return (sy > 0)? hiremainder: x - hiremainder;
 }
@@ -517,20 +528,31 @@ diviu_rem(GEN y, ulong x, ulong *rem)
 {
   long ly,i;
   GEN z;
+  ulong xi;
   LOCAL_HIREMAINDER;
 
   if (!x) pari_err_INV("diviu_rem",gen_0);
   if (!signe(y)) { *rem = 0; return gen_0; }
 
   ly = lgefint(y);
-  if (x <= (ulong)y[2]) hiremainder=0;
+  if (x <= uel(y,2))
+  {
+    hiremainder=0;
+    if (ly==3)
+    {
+      z = cgetipos(3);
+      z[2] = divll(uel(y,2),x);
+      *rem = hiremainder; return z;
+    }
+  }
   else
   {
-    if (ly==3) { *rem = (ulong)y[2]; return gen_0; }
-    hiremainder=y[2]; ly--; y++;
+    if (ly==3) { *rem = uel(y,2); return gen_0; }
+    hiremainder = uel(y,2); ly--; y++;
   }
+  xi = get_Fl_red(x);
   z = cgetipos(ly);
-  for (i=2; i<ly; i++) z[i]=divll(y[i],x);
+  for (i=2; i<ly; i++) z[i]=divll_pre(y[i],x,xi);
   *rem = hiremainder; return z;
 }
 
@@ -539,6 +561,7 @@ divis_rem(GEN y, long x, long *rem)
 {
   long sy=signe(y),ly,s,i;
   GEN z;
+  ulong xi;
   LOCAL_HIREMAINDER;
 
   if (!x) pari_err_INV("divis_rem",gen_0);
@@ -546,14 +569,25 @@ divis_rem(GEN y, long x, long *rem)
   if (x<0) { s = -sy; x = -x; } else s = sy;
 
   ly = lgefint(y);
-  if ((ulong)x <= (ulong)y[2]) hiremainder=0;
+  if ((ulong)x <= uel(y,2))
+  {
+    hiremainder=0;
+    if (ly==3)
+    {
+      z = cgeti(3); z[1] = evallgefint(3) | evalsigne(s);
+      z[2] = divll(uel(y,2),x);
+      if (sy<0) hiremainder = - ((long)hiremainder);
+      *rem = (long)hiremainder; return z;
+    }
+  }
   else
   {
     if (ly==3) { *rem = itos(y); return gen_0; }
-    hiremainder=y[2]; ly--; y++;
+    hiremainder = uel(y,2); ly--; y++;
   }
+  xi = get_Fl_red(x);
   z = cgeti(ly); z[1] = evallgefint(ly) | evalsigne(s);
-  for (i=2; i<ly; i++) z[i]=divll(y[i],x);
+  for (i=2; i<ly; i++) z[i]=divll_pre(y[i],x,xi);
   if (sy<0) hiremainder = - ((long)hiremainder);
   *rem = (long)hiremainder; return z;
 }
@@ -562,6 +596,7 @@ GEN
 divis(GEN y, long x)
 {
   long sy=signe(y),ly,s,i;
+  ulong xi;
   GEN z;
   LOCAL_HIREMAINDER;
 
@@ -570,14 +605,24 @@ divis(GEN y, long x)
   if (x<0) { s = -sy; x = -x; } else s = sy;
 
   ly = lgefint(y);
-  if ((ulong)x <= (ulong)y[2]) hiremainder=0;
+  if ((ulong)x <= uel(y,2))
+  {
+    hiremainder=0;
+    if (ly==3)
+    {
+      z = cgeti(3); z[1] = evallgefint(3) | evalsigne(s);
+      z[2] = divll(y[2],x);
+      return z;
+    }
+  }
   else
   {
     if (ly==3) return gen_0;
     hiremainder=y[2]; ly--; y++;
   }
+  xi = get_Fl_red(x);
   z = cgeti(ly); z[1] = evallgefint(ly) | evalsigne(s);
-  for (i=2; i<ly; i++) z[i]=divll(y[i],x);
+  for (i=2; i<ly; i++) z[i]=divll_pre(y[i],x, xi);
   return z;
 }
 
@@ -598,7 +643,7 @@ divrr(GEN x, GEN y)
   {
     ulong k = x[2], l = (lx>3)? x[3]: 0;
     LOCAL_HIREMAINDER;
-    if (k < (ulong)y[2]) e--;
+    if (k < uel(y,2)) e--;
     else
     {
       l >>= 1; if (k&1) l |= HIGHBIT;
@@ -626,18 +671,18 @@ divrr(GEN x, GEN y)
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
 
-    if ((ulong)r1[1] == y0)
+    if (uel(r1,1) == y0)
     {
       qp = ULONG_MAX; k = addll(y0,r1[2]);
     }
     else
     {
-      if ((ulong)r1[1] > y0) /* can't happen if i=0 */
+      if (uel(r1,1) > y0) /* can't happen if i=0 */
       {
         GEN y1 = y+1;
         j = lr-i; r1[j] = subll(r1[j],y1[j]);
         for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
-        j=i; do ((ulong*)r)[--j]++; while (j && !r[j]);
+        j=i; do uel(r,--j)++; while (j && !uel(r,j));
       }
       hiremainder = r1[1]; overflow = 0;
       qp = divll(r1[2],y0); k = hiremainder;
@@ -662,9 +707,9 @@ divrr(GEN x, GEN y)
       r1[j] = subll(r1[j], addmul(qp,y[j]));
       hiremainder += overflow;
     }
-    if ((ulong)r1[1] != hiremainder)
+    if (uel(r1,1) != hiremainder)
     {
-      if ((ulong)r1[1] < hiremainder)
+      if (uel(r1,1) < hiremainder)
       {
         qp--;
         j = lr-i-(lr-i>=ly); r1[j] = addll(r1[j], y[j]);
@@ -675,7 +720,7 @@ divrr(GEN x, GEN y)
         r1[1] -= hiremainder;
         while (r1[1])
         {
-          qp++; if (!qp) { j=i; do ((ulong*)r)[--j]++; while (j && !r[j]); }
+          qp++; if (!qp) { j=i; do uel(r,--j)++; while (j && !r[j]); }
           j = lr-i-(lr-i>=ly); r1[j] = subll(r1[j],y[j]);
           for (j--; j>1; j--) r1[j] = subllx(r1[j],y[j]);
           r1[1] -= overflow;
@@ -686,9 +731,9 @@ divrr(GEN x, GEN y)
   }
   /* i = lr-1 */
   /* round correctly */
-  if ((ulong)r1[1] > (y0>>1))
+  if (uel(r1,1) > (y0>>1))
   {
-    j=i; do ((ulong*)r)[--j]++; while (j && !r[j]);
+    j=i; do uel(r,--j)++; while (j && !r[j]);
   }
   r1 = r-1; for (j=i; j>=2; j--) r[j]=r1[j];
   if (r[0] == 0) e--;
@@ -730,24 +775,37 @@ divri(GEN x, GEN y)
 GEN
 dvmdii(GEN x, GEN y, GEN *z)
 {
-  long sx=signe(x),sy=signe(y);
-  long lx, ly, lz, i, j, sh, lq, lr;
+  long sx = signe(x), sy = signe(y);
+  long lx, ly = lgefint(y), lz, i, j, sh, lq, lr;
   pari_sp av;
-  ulong y0,y1, *xd,*rd,*qd;
+  ulong y0,y0i,y1, *xd,*rd,*qd;
   GEN q, r, r1;
 
-  if (!sy)
-  {
-    if (z == ONLY_REM && !sx) return gen_0;
-    if (!sy) pari_err_INV("dvmdii",gen_0);
-  }
   if (!sx)
   {
+    if (ly < 3) pari_err_INV("dvmdii",gen_0);
     if (!z || z == ONLY_REM) return gen_0;
     *z=gen_0; return gen_0;
   }
+  if (ly <= 3)
+  {
+    ulong rem;
+    if (ly < 3) pari_err_INV("dvmdii",gen_0);
+    if (z == ONLY_REM)
+    {
+      rem = umodiu(x,uel(y,2));
+      if (!rem) return gen_0;
+      return (sx < 0)? utoineg(uel(y,2) - rem): utoipos(rem);
+    }
+    q = diviu_rem(x, uel(y,2), &rem);
+    if (sx != sy) togglesign(q);
+    if (!z) return q;
+    if (!rem) *z = gen_0;
+    else *z = sx < 0? utoineg(rem): utoipos(rem);
+    return q;
+  }
   lx=lgefint(x);
-  ly=lgefint(y); lz=lx-ly;
+  lz=lx-ly;
   if (lz <= 0)
   {
     if (lz == 0)
@@ -755,7 +813,7 @@ dvmdii(GEN x, GEN y, GEN *z)
       for (i=2; i<lx; i++)
         if (x[i] != y[i])
         {
-          if ((ulong)x[i] > (ulong)y[i]) goto DIVIDE;
+          if (uel(x,i) > uel(y,i)) goto DIVIDE;
           goto TRIVIAL;
         }
       if (z == ONLY_REM) return gen_0;
@@ -770,32 +828,6 @@ TRIVIAL:
   }
 DIVIDE: /* quotient is non-zero */
   av=avma; if (sx<0) sy = -sy;
-  if (ly==3)
-  {
-    LOCAL_HIREMAINDER;
-    y0 = y[2];
-    if (y0 <= (ulong)x[2]) hiremainder=0;
-    else
-    {
-      hiremainder = x[2]; lx--; x++;
-    }
-    q = new_chunk(lx); for (i=2; i<lx; i++) q[i]=divll(x[i],y0);
-    if (z == ONLY_REM)
-    {
-      avma=av; if (!hiremainder) return gen_0;
-      r=cgeti(3);
-      r[1] = evalsigne(sx) | evallgefint(3);
-      r[2]=hiremainder; return r;
-    }
-    q[1] = evalsigne(sy) | evallgefint(lx);
-    q[0] = evaltyp(t_INT) | evallg(lx);
-    if (!z) return q;
-    if (!hiremainder) { *z=gen_0; return q; }
-    r=cgeti(3);
-    r[1] = evalsigne(sx) | evallgefint(3);
-    r[2] = hiremainder; *z=r; return q;
-  }
-
   r1 = new_chunk(lx); sh = bfffo(y[2]);
   if (sh)
   { /* normalize so that highbit(y) = 1 (shift left x and y by sh bits)*/
@@ -803,28 +835,29 @@ DIVIDE: /* quotient is non-zero */
     r = new_chunk(ly);
     shift_left(r, y,2,ly-1, 0,sh); y = r;
     shift_left(r1,x,2,lx-1, 0,sh);
-    r1[1] = ((ulong)x[2]) >> m;
+    r1[1] = uel(x,2) >> m;
   }
   else
   {
     r1[1] = 0; for (j=2; j<lx; j++) r1[j] = x[j];
   }
   x = r1;
-  y0 = y[2]; y1 = y[3];
+  y0 = y[2]; y0i = get_Fl_red(y0);
+  y1 = y[3];
   for (i=0; i<=lz; i++)
   { /* r1 = x + i */
     ulong k, qp;
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
 
-    if ((ulong)r1[1] == y0)
+    if (uel(r1,1) == y0)
     {
       qp = ULONG_MAX; k = addll(y0,r1[2]);
     }
     else
     {
       hiremainder = r1[1]; overflow = 0;
-      qp = divll(r1[2],y0); k = hiremainder;
+      qp = divll_pre(r1[2],y0,y0i); k = hiremainder;
     }
     if (!overflow)
     {
@@ -838,7 +871,7 @@ DIVIDE: /* quotient is non-zero */
       r1[j] = subll(r1[j], addmul(qp,y[j]));
       hiremainder += overflow;
     }
-    if ((ulong)r1[1] < hiremainder)
+    if (uel(r1,1) < hiremainder)
     {
       qp--;
       j = ly-1; r1[j] = addll(r1[j],y[j]);
@@ -951,17 +984,17 @@ red_montgomery(GEN T, GEN N, ulong inv)
 #endif
   if (k == 1)
   { /* as below, special cased for efficiency */
-    ulong n = (ulong)N[2];
+    ulong n = uel(N,2);
     if (d == 1) {
-      hiremainder = (ulong)T[2];
+      hiremainder = uel(T,2);
       m = hiremainder * inv;
       (void)addmul(m, n); /* t + m*n = 0 */
       return utoi(hiremainder);
     } else { /* d = 2 */
-      hiremainder = (ulong)T[3];
+      hiremainder = uel(T,3);
       m = hiremainder * inv;
       (void)addmul(m, n); /* t + m*n = 0 */
-      t = addll(hiremainder, (ulong)T[2]);
+      t = addll(hiremainder, uel(T,2));
       if (overflow) t -= n; /* t > n doesn't fit in 1 word */
       return utoi(t);
     }
@@ -1039,16 +1072,16 @@ diviuexact_i(GEN x, ulong y)
 
   if (y == 1) return icopy(x);
   lx = lgefint(x);
-  if (lx == 3) return utoipos((ulong)x[2] / y);
+  if (lx == 3) return utoipos(uel(x,2) / y);
   yinv = invmod2BIL(y);
-  lz = (y <= (ulong)x[2]) ? lx : lx-1;
+  lz = (y <= uel(x,2)) ? lx : lx-1;
   z = new_chunk(lz);
   z0 = z + lz;
   x0 = x + lx; x0min = x + lx-lz+2;
 
   while (x0 > x0min)
   {
-    *--z0 = q = yinv*((ulong)*--x0); /* i-th quotient */
+    *--z0 = q = yinv*uel(--x0,0); /* i-th quotient */
     if (!q) continue;
     /* x := x - q * y */
     { /* update neither lowest word (could set it to 0) nor highest ones */
@@ -1057,13 +1090,13 @@ diviuexact_i(GEN x, ulong y)
       (void)mulll(q,y);
       if (hiremainder)
       {
-        if ((ulong)*x1 < hiremainder)
+        if (uel(x1,0) < hiremainder)
         {
-          *x1 -= hiremainder;
-          do (*--x1)--; while ((ulong)*x1 == ULONG_MAX);
+          uel(x1,0) -= hiremainder;
+          do uel(--x1,0)--; while (uel(x1,0) == ULONG_MAX);
         }
         else
-          *x1 -= hiremainder;
+          uel(x1,0) -= hiremainder;
       }
     }
   }
@@ -1086,7 +1119,7 @@ diviuexact(GEN x, ulong y)
   if (y == 1) return icopy(x);
   lx = lgefint(x);
   if (lx == 3) {
-    ulong q = (ulong)x[2] / y;
+    ulong q = uel(x,2) / y;
     return (s > 0)? utoipos(q): utoineg(q);
   }
   av = avma; (void)new_chunk(lx); vy = vals(y);
@@ -1095,7 +1128,7 @@ diviuexact(GEN x, ulong y)
     if (y == 1) { avma = av; return shifti(x, -vy); }
     x = shifti(x, -vy);
     if (lx == 3) {
-      ulong q = (ulong)x[2] / y;
+      ulong q = uel(x,2) / y;
       avma = av;
       return (s > 0)? utoipos(q): utoineg(q);
     }
@@ -1120,7 +1153,7 @@ diviiexact(GEN x, GEN y)
   if (!sx) return gen_0;
   lx = lgefint(x);
   if (lx == 3) {
-    q = (ulong)x[2] / (ulong)y[2];
+    q = uel(x,2) / uel(y,2);
     return (sx+sy) ? utoipos(q): utoineg(q);
   }
   vy = vali(y); av = avma;
@@ -1136,12 +1169,12 @@ diviiexact(GEN x, GEN y)
   ly = lgefint(y);
   if (ly == 3)
   {
-    x = diviuexact_i(x,(ulong)y[2]); /* x != 0 */
+    x = diviuexact_i(x,uel(y,2)); /* x != 0 */
     setsigne(x, (sx+sy)? 1: -1); return x;
   }
   y0inv = invmod2BIL(y[ly-1]);
   i=2; while (i<ly && y[i]==x[i]) i++;
-  lz = (i==ly || (ulong)y[i] < (ulong)x[i]) ? lx-ly+3 : lx-ly+2;
+  lz = (i==ly || uel(y,i) < uel(x,i)) ? lx-ly+3 : lx-ly+2;
   z = new_chunk(lz);
 
   y += ly - 1; /* now y[-i] = i-th word of y */
@@ -1151,7 +1184,7 @@ diviiexact(GEN x, GEN y)
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
 
-    z[i] = q = y0inv*((ulong)x[ii]); /* i-th quotient */
+    z[i] = q = y0inv*uel(x,ii); /* i-th quotient */
     if (!q) continue;
 
     /* x := x - q * y */
@@ -1891,7 +1924,7 @@ static GEN
 sqrtispec1_sh(GEN n, GEN *pr)
 {
   GEN S;
-  ulong r, s, u0 = (ulong)n[0];
+  ulong r, s, u0 = uel(n,0);
   int sh = bfffo(u0) & ~1UL;
   if (sh) u0 <<= sh;
   p_sqrtu1(&u0, &s, &r);
@@ -1915,7 +1948,7 @@ static GEN
 sqrtispec2_sh(GEN n, GEN *pr)
 {
   GEN S;
-  ulong U[2], r, s, u0 = (ulong)n[0], u1 = (ulong)n[1];
+  ulong U[2], r, s, u0 = uel(n,0), u1 = uel(n,1);
   int hi, sh = bfffo(u0) & ~1UL;
   if (sh) {
     u0 = (u0 << sh) | (u1 >> (BITS_IN_LONG-sh));
@@ -2022,7 +2055,7 @@ sqrtremi(GEN N, GEN *r)
 GEN
 sqrtr_abs(GEN x)
 {
-  long l = lg(x) - 2, e = expo(x), er = e>>1;
+  long l = realprec(x) - 2, e = expo(x), er = e>>1;
   GEN b, c, res = cgetr(2 + l);
   res[1] = evalsigne(1) | evalexpo(er);
   if (e&1) {
@@ -2036,11 +2069,11 @@ sqrtr_abs(GEN x)
     ulong u;
     b = new_chunk(2 + (l << 1));
     shift_left(b+1, x+2, 0,l-1, 0, BITS_IN_LONG-1);
-    b[0] = ((ulong)x[2])>>1;
+    b[0] = uel(x,2)>>1;
     xmpn_zero(b + l+1,l+1);
     b = sqrtispec(b, l+1, &c);
     xmpn_copy(res+2, b+2, l);
-    u = (ulong)b[l+2];
+    u = uel(b,l+2);
     if ( u&HIGHBIT || (u == ~HIGHBIT && cmpii(c,b) > 0))
       roundr_up_ip(res, l+2);
   }
@@ -2141,7 +2174,7 @@ convi(GEN x, long *l)
 {
   long lz, lx = lgefint(x);
   ulong *z;
-  if (lx == 3 && (ulong)x[2] < 1000000000UL) {
+  if (lx == 3 && uel(x,2) < 1000000000UL) {
     z = (ulong*)new_chunk(1);
     *z = x[2];
     *l = 1; return z+1;
