@@ -154,14 +154,14 @@ INLINE GEN
 mul0r(GEN x)
 {
   long l = lg(x), e = expo(x);
-  e = (l > 2)? -bit_accuracy(l) + e: (e < 0? (e<<1): 0);
+  e = (l > 2)? -prec2nbits(l) + e: (e < 0? 2*e: 0);
   return real_0_bit(e);
 }
 /* lg(x) > 2 */
 INLINE GEN
 div0r(GEN x) {
   long l = lg(x), e = expo(x);
-  return real_0_bit(-bit_accuracy(l) - e);
+  return real_0_bit(-prec2nbits(l) - e);
 }
 
 GEN
@@ -229,11 +229,11 @@ mulrrz_3end(GEN z, long sz, long ez, ulong garde)
   }
   else
   {
-    z[2] = (z[2]<<1) | (garde>>(BITS_IN_LONG-1));
+    uel(z,2) = (uel(z,2)<<1) | (garde>>(BITS_IN_LONG-1));
     if (garde & (1UL<<(BITS_IN_LONG-2)))
     {
-      z[2]++; /* round properly, z2+1 can overflow */
-      if (!z[2]) { z[2] = (long)HIGHBIT; ez++; }
+      uel(z,2)++; /* round properly, z2+1 can overflow */
+      if (!uel(z,2)) { uel(z,2) = HIGHBIT; ez++; }
     }
   }
   z[1] = evalsigne(sz)|evalexpo(ez);
@@ -244,7 +244,7 @@ mulrrz_3end(GEN z, long sz, long ez, ulong garde)
 INLINE void
 sqrz_i(GEN z, GEN x, long lz)
 {
-  long ez = expo(x) << 1;
+  long ez = 2*expo(x);
   long i, j, lzz, p1;
   ulong garde;
   GEN x1;
@@ -420,7 +420,7 @@ mulir(GEN x, GEN y)
   long sx = signe(x), sy;
   if (!sx) return mul0r(y);
   if (lgefint(x) == 3) {
-    GEN z = mulur((ulong)x[2], y);
+    GEN z = mulur(uel(x,2), y);
     if (sx < 0) togglesign(z);
     return z;
   }
@@ -677,57 +677,71 @@ modiiz(GEN x, GEN y, GEN z)
 GEN
 divrs(GEN x, long y)
 {
-  long i, lx, garde, sh, s = signe(x);
   GEN z;
-  LOCAL_HIREMAINDER;
-
-  if (!y) pari_err_INV("divrs",gen_0);
-  if (y<0) { s = -s; y = -y; }
-  if (!s) return real_0_bit(expo(x) - expu(y));
-  if (y==1) { z = rcopy(x); setsigne(z,s); return z; }
-  if (y==2) { z = shiftr(x, -1); setsigne(z,s); return z; }
-
-  z=cgetr(lx=lg(x)); hiremainder=0;
-  for (i=2; i<lx; i++) z[i] = divll(x[i],y);
-
-  /* we may have hiremainder != 0 ==> garde */
-  garde=divll(0,y); sh=bfffo(z[2]);
-  if (sh) shift_left(z,z, 2,lx-1, garde,sh);
-  z[1] = evalsigne(s) | evalexpo(expo(x)-sh);
-  if ((garde << sh) & HIGHBIT) roundr_up_ip(z, lx);
+  if (y < 0)
+  {
+    z = divru(x, (ulong)-y);
+    togglesign(z);
+  }
+  else
+    z = divru(x, (ulong)y);
   return z;
 }
 
 GEN
 divru(GEN x, ulong y)
 {
-  long i, lx, garde, sh, e, s = signe(x);
+  long i, lx, sh, e, s = signe(x);
+  ulong garde;
   GEN z;
   LOCAL_HIREMAINDER;
 
   if (!y) pari_err_INV("divru",gen_0);
   if (!s) return real_0_bit(expo(x) - expu(y));
-  if (y==1) return rcopy(x);
-  if (y==2) return shiftr(x, -1);
-
+  if (!(y & (y-1))) /* power of 2 */
+  {
+    if (y == 1) return rcopy(x);
+    return shiftr(x, -expu(y));
+  }
   e = expo(x);
   lx = lg(x);
   z = cgetr(lx);
-  if (y <= (ulong)x[2])
+  if (lx == 3)
   {
-    hiremainder = 0;
-    for (i=2; i<lx; i++) z[i] = divll(x[i],y);
-    /* we may have hiremainder != 0 ==> garde */
-    garde = divll(0,y);
+    if (y <= uel(x,2))
+    {
+      hiremainder = 0;
+      z[2] = divll(x[2],y);
+      /* we may have hiremainder != 0 ==> garde */
+      garde = divll(0,y);
+    }
+    else
+    {
+      hiremainder = x[2];
+      z[2] = divll(0,y);
+      garde = hiremainder;
+      e -= BITS_IN_LONG;
+    }
   }
   else
   {
-    long l = lx-1;
-    hiremainder = x[2];
-    for (i=2; i<l; i++) z[i] = divll(x[i+1],y);
-    z[i] = divll(0,y);
-    garde = hiremainder;
-    e -= BITS_IN_LONG;
+    ulong yp = get_Fl_red(y);
+    if (y <= uel(x,2))
+    {
+      hiremainder = 0;
+      for (i=2; i<lx; i++) z[i] = divll_pre(x[i],y,yp);
+      /* we may have hiremainder != 0 ==> garde */
+      garde = divll_pre(0,y,yp);
+    }
+    else
+    {
+      long l = lx-1;
+      hiremainder = x[2];
+      for (i=2; i<l; i++) z[i] = divll_pre(x[i+1],y,yp);
+      z[i] = divll_pre(0,y,yp);
+      garde = hiremainder;
+      e -= BITS_IN_LONG;
+    }
   }
   sh=bfffo(z[2]); /* z[2] != 0 */
   if (sh) shift_left(z,z, 2,lx-1, garde,sh);
@@ -810,7 +824,7 @@ int2n(long n) {
   l = dvmdsBIL(n, &m) + 3;
   z = cgetipos(l);
   for (i = 2; i < l; i++) z[i] = 0;
-  *int_MSW(z) = 1L << m; return z;
+  *int_MSW(z) = 1UL << m; return z;
 }
 /* To avoid problems when 2^(BIL-1) < n. Overflow cleanly, where int2n
  * returns gen_0 */
@@ -823,7 +837,7 @@ int2u(ulong n) {
   l = dvmduBIL(n, &m) + 3;
   z = cgetipos(l);
   for (i = 2; i < l; i++) z[i] = 0;
-  *int_MSW(z) = 1L << m; return z;
+  *int_MSW(z) = 1UL << m; return z;
 }
 
 GEN
