@@ -417,11 +417,11 @@ localvars_find(GEN pack, entree *ep)
 enum FLflag {FLreturn=1, FLsurvive=2, FLnocopy=4, FLnocopylex=8};
 
 static void
-copyifclone(long n, long mode, long flag, long mask)
+addcopy(long n, long mode, long flag, long mask)
 {
   if (mode==Ggen && !(flag&mask))
   {
-    op_push(OCcopyifclone,0,n);
+    op_push(OCcopy,0,n);
     if (!(flag&FLsurvive) && DEBUGLEVEL)
       pari_warn(warner,"compiler generates copy for `%.*s'",
                        tree[n].len,tree[n].str);
@@ -2054,7 +2054,7 @@ compilenode(long n, int mode, long flag)
       if (vn)
       {
         op_push(OCpushlex,(long)vn,n);
-        copyifclone(n,mode,flag,FLnocopy|FLnocopylex);
+        addcopy(n,mode,flag,FLnocopy|FLnocopylex);
         compilecast(n,Ggen,mode);
       }
       else if (ep->valence==EpVAR || ep->valence==EpNEW)
@@ -2062,7 +2062,7 @@ compilenode(long n, int mode, long flag)
         if (DEBUGLEVEL && mode==Gvoid)
           pari_warn(warner,"statement with no effect: `%s'",ep->name);
         op_push(OCpushdyn,(long)ep,n);
-        copyifclone(n,mode,flag,FLnocopy);
+        addcopy(n,mode,flag,FLnocopy);
         compilecast(n,Ggen,mode);
       }
       else
@@ -2286,10 +2286,13 @@ optimizefunc(entree *ep, long n)
   Gtype t;
   PPproto mod;
   long fl=COsafelex|COsafedyn;
-  const char *p=ep->code;
+  const char *p;
   char c;
   GEN arg = listtogen(y,Flistarg);
   long nb=lg(arg)-1, ret_flag;
+  if (is_func_named(ep,"if") && nb>=4)
+    ep=is_entry("_multi_if");
+  p = ep->code;
   if (!p)
     fl=0;
   else
@@ -2405,6 +2408,8 @@ optimizefunc(entree *ep, long n)
         pari_err_BUG("optimizefun [unknown PPproto]");
       }
     }
+    if (j<=nb)
+      compile_err("too many arguments",tree[arg[j]].str);
   }
   else (void)vec_optimize(arg);
   avma=av; tree[n].flags=fl;
