@@ -679,11 +679,18 @@ closure_castlong(long z, long mode)
   case Gsmall:
     st[sp++]=z;
     break;
+  case Gusmall:
+    if (z < 0)
+      pari_err_TYPE("stou [integer >=0 expected]", stoi(z));
+    st[sp++]=(ulong) z;
+    break;
   case Ggen:
     gel(st,sp++)=stoi(z);
     break;
   case Gvar:
     err_var(stoi(z));
+  case Gvoid:
+    break;
   default:
     pari_err_BUG("closure_castlong, type unknown");
   }
@@ -834,6 +841,11 @@ closure_eval(GEN C)
   long saved_rp=rp, saved_prec=s_prec.n;
   long j, nbmvar=0, nblvar=0;
   long pc, t;
+#ifdef STACK_CHECK
+  GEN stackelt;
+  if (PARI_stack_limit && (void*) &stackelt <= PARI_stack_limit)
+    pari_err(e_MISC, "deep recursion");
+#endif
   clone_lock(C);
   t = trace_push(0, C);
   if (lg(C)==8)
@@ -1351,10 +1363,6 @@ closure_eval(GEN C)
             gel(v,j) = gel(st,sp+j-1)? gcopy(gel(st,sp+j-1)): gen_0;
           gel(st,sp++)=v;
         }
-#ifdef STACK_CHECK
-        if (PARI_stack_limit && (void*) &z <= PARI_stack_limit)
-          pari_err(e_MISC, "deep recursion");
-#endif
         z = closure_return(fun);
         if (br_status) goto endeval;
         gel(st, sp-1) = z;
