@@ -670,9 +670,13 @@ forprime_init(forprime_t *T, GEN a, GEN b)
       return 0;
     }
     lb = lgefint(b);
+    T->bb = b;
   }
   else if (!b || inf_get_sign(b) > 0)
+  {
     lb = lgefint(a) + 4;
+    T->bb = NULL;
+  }
   else /* b == -oo */
   {
     T->strategy = PRST_nextprime; /* paranoia */
@@ -680,7 +684,6 @@ forprime_init(forprime_t *T, GEN a, GEN b)
     T->pp = gen_0;
     return 0;
   }
-  T->bb = b;
   T->pp = cgeti(lb);
   /* a, b are positive integers, a <= b */
   if (lgefint(a) == 3) /* lb == 3 implies b != NULL */
@@ -692,7 +695,8 @@ forprime_init(forprime_t *T, GEN a, GEN b)
 
 /* assume a <= b <= maxprime()^2, a,b odd, sieve[n] corresponds to
  *   a+16*n, a+16*n+2, ..., a+16*n+14 (bits 0 to 7)
- * maxpos = index of last sieve cell. */
+ * maxpos = index of last sieve cell.
+ * b-a+2 must be divisible by 16 for use by u_forprime_next */
 static void
 sieve_block(ulong a, ulong b, ulong maxpos, unsigned char* sieve)
 {
@@ -743,7 +747,7 @@ static struct pari_sieve pari_sieve_modular;
 void
 pari_init_primes(ulong maxprime)
 {
-  ulong a = PARI_MODULAR_BASE, b = a + (1UL<<20);
+  ulong a = PARI_MODULAR_BASE, b = a + (1UL<<20)-2;
   initprimetable(maxprime);
   pari_sieve_init(&pari_sieve_modular, a, b);
 }
