@@ -906,6 +906,7 @@ pari_init_opts(size_t parisize, ulong maxprime, ulong init_opts)
     pari_init_defaults();
     GP_DATA = default_gp_data();
     gp_expand_path(GP_DATA->path);
+    gp_expand_path(GP_DATA->sopath);
   }
 
   pari_mainstack = (struct pari_mainstack *) malloc(sizeof(*pari_mainstack));
@@ -955,7 +956,7 @@ pari_close_opts(ulong init_opts)
   pari_thread_close();
   pari_close_files();
   pari_close_homedir();
-  pari_kernel_close();
+  if (!(init_opts&INIT_noINTGMPm)) pari_kernel_close();
 
   free((void*)functions_hash);
   free((void*)defaults_hash);
