@@ -1581,6 +1581,18 @@ wrapmonw2(void* E, GEN x)
   return gdiv(wnx, gpow(x, gadd(gmulgs(W->a, W->j), W->b), W->prec));
 }
 
+/* add 'a' to all components of v */
+static GEN
+RgV_Rg_addall(GEN v, GEN a)
+{
+  long i, l;
+  GEN w;
+  if (!signe(a)) return v;
+  w = cgetg_copy(v,&l);
+  for (i = 1; i < l; i++) gel(w,i) = gadd(gel(v,i), a);
+  return w;
+}
+
 static GEN
 sumnummonieninit_w(GEN w, GEN wfast, GEN a, GEN b, GEN n0, long prec)
 {
@@ -1638,7 +1650,11 @@ sumnummonieninit_w(GEN w, GEN wfast, GEN a, GEN b, GEN n0, long prec)
   R = gneg(gdiv(P, RgX_deriv(Q)));
   vwt = cgetg(n+1, t_VEC);
   for (j = 1; j <= n; j++)
-    gel(vwt,j) = poleval(R, gel(vr,j));
+  {
+    GEN r = gel(vr,j), t = poleval(R,r);
+    if (c) t = gmul(t, gpow(r, c, prec));
+    gel(vwt,j) = t;
+  }
   return mkvec3(vabs, vwt, n0);
 }
 
@@ -1676,7 +1692,9 @@ sumnummonieninit_i(GEN asymp, GEN w, GEN n0, long prec)
       if (abscmpiu(n0, 2) <= 0)
       {
         GEN tab = sumnummonieninit0(a, b, itos(w), prec);
-        return shallowconcat(tab,n0);
+        GEN A = gel(tab,1), B = gel(tab,2);
+        A = RgV_Rg_addall(A, subis(n0,1));
+        return mkvec3(A, B, n0);
       }
       w = strtofunction("log");
       break;
@@ -1698,16 +1716,6 @@ sumnummonieninit(GEN asymp, GEN w, GEN n0, long prec)
   return gerepilecopy(av, sumnummonieninit_i(asymp,w,n0,prec));
 }
 
-/* add 'a' to all components of v */
-static GEN
-RgV_Rg_addall(GEN v, GEN a)
-{
-  long i, l;
-  GEN w = cgetg_copy(v,&l);
-  for (i = 1; i < l; i++) gel(w,i) = gadd(gel(v,i), a);
-  return w;
-}
-
 GEN
 sumnummonien(void *E, GEN (*eval)(void*,GEN), GEN n0, GEN tab, long prec)
 {
@@ -1716,7 +1724,10 @@ sumnummonien(void *E, GEN (*eval)(void*,GEN), GEN n0, GEN tab, long prec)
   long l, i;
   if (typ(n0) != t_INT) pari_err_TYPE("sumnummonien", n0);
   if (!tab)
+  {
     tab = sumnummonieninit0(gen_1,gen_1,0,prec);
+    gel(tab,1) = RgV_Rg_addall(gel(tab,1), subis(n0,1));
+  }
   else switch(lg(tab))
   {
     case 4:
@@ -1730,7 +1741,6 @@ sumnummonien(void *E, GEN (*eval)(void*,GEN), GEN n0, GEN tab, long prec)
   vwt = gel(tab,2);
   if (typ(vabs) != t_VEC || typ(vwt) != t_VEC || lg(vwt) != l)
     pari_err_TYPE("sumnummonien", tab);
-  if (!isint1(n0)) vabs = RgV_Rg_addall(vabs, subis(n0,1));
   S = gen_0;
   for (i = 1; i < l; i++) S = gadd(S, gmul(gel(vwt,i), eval(E, gel(vabs,i))));
   return gerepileupto(av, gprec_w(S, prec));
