@@ -95,7 +95,14 @@ GEN
 member_bnf(GEN x)
 {
   long t; GEN y = get_bnf(x,&t);
-  if (!y) member_err("bnf",x);
+  if (!y) {
+    if (t == typ_ELL && ell_get_type(x) == t_ELL_NF)
+    {
+      y = ellnf_get_bnf(x);
+      if (y) return y;
+    }
+    member_err("bnf",x);
+  }
   return y;
 }
 
@@ -105,6 +112,7 @@ member_nf(GEN x)
   long t; GEN y = get_nf(x,&t);
   if (!y) {
     if (t == typ_RNF) return gel(x,10);
+    if (t == typ_ELL && ell_get_type(x) == t_ELL_NF) return ellnf_get_nf(x);
     member_err("nf",x);
   }
   return y;
@@ -183,7 +191,8 @@ member_mod(GEN x) /* modulus */
   switch(t) {
     case typ_GAL: return gal_get_mod(x);
     case typ_BNR: return bnr_get_mod(x);
-    case typ_BID: return gel(x,1);
+    case typ_BID: return typ(gel(x,3)) == t_MAT? bid_get_mod(x)
+                                               : bid_get_ideal(x);
   }
   switch(typ(x))
   {
@@ -357,7 +366,7 @@ member_tu(GEN x)
     GEN y;
     if (t != typ_Q) member_err("tu",x);
     y = quad_disc(x);
-    if (signe(y) > 0 || cmpiu(y,4) > 0) return mkvec2(gen_m1, gen_2);
+    if (signe(y) > 0 || abscmpiu(y,4) > 0) return mkvec2(gen_m1, gen_2);
 
     gel(res,1) = utoipos((itos(y) == -4)? 4: 6);
     gel(res,2) = gcopy(x);
@@ -391,10 +400,10 @@ member_zkst(GEN x)
   long t; (void)get_nf(x,&t);
   switch(t)
   {
-    case typ_BID: return gel(x,2);
+    case typ_BID: return bid_get_grp(x);
     case typ_BNR: {
       GEN bid = bnr_get_bid(x);
-      if (typ(bid) == t_VEC && lg(bid) > 2) return gel(bid,2);
+      if (typ(bid) == t_VEC && lg(bid) > 2) return bid_get_grp(bid);
     }
   }
   member_err("zkst",x);
