@@ -21,7 +21,7 @@ ENDEXTERN
 
 static THREAD int pari_once;
 static THREAD long pari_discarded;
-static THREAD const char *pari_lex_start, *pari_unused_chars;
+static THREAD const char *pari_lex_start;
 static THREAD GEN pari_lasterror;
 
 static void pari_error(YYLTYPE *yylloc, char **lex, const char *s)
@@ -56,17 +56,6 @@ pari_init_parser(void)
 void
 pari_close_parser(void) { pari_stack_delete(&s_node); }
 
-static void
-unused_chars(const char *lex, int strict)
-{
-  long n = 2 * term_width() - (17+19+1); /* Warning + unused... + . */
-  if (strict) compile_err("unused characters", lex);
-  if ((long)strlen(lex) > n) /* at most 2 lines */
-    pari_warn(warner, "unused characters: %.*s[+++]", n-5, lex);
-  else
-    pari_warn(warner, "unused characters: %s", lex);
-}
-
 void
 compile_err(const char *msg, const char *str)
 {
@@ -84,7 +73,6 @@ parsestate_reset(void)
 {
   s_node.n = OPnboperator;
   pari_lex_start = NULL;
-  pari_unused_chars=NULL;
   pari_once=1;
   pari_discarded=0;
   pari_lasterror=NULL;
@@ -94,7 +82,6 @@ parsestate_save(struct pari_parsestate *state)
 {
   state->node = s_node.n;
   state->lex_start = pari_lex_start;
-  state->unused_chars = pari_unused_chars;
   state->once = pari_once;
   state->discarded = pari_discarded;
   state->lasterror = pari_lasterror;
@@ -104,29 +91,25 @@ parsestate_restore(struct pari_parsestate *state)
 {
   s_node.n = state->node;
   pari_lex_start = state->lex_start;
-  pari_unused_chars = state->unused_chars;
   pari_once = state->once;
   pari_discarded = state->discarded;
   pari_lasterror = state->lasterror;
 }
 
 GEN
-pari_compile_str(char *lex, int strict)
+pari_compile_str(const char *lex)
 {
   pari_sp ltop=avma;
   GEN code;
   struct pari_parsestate state;
   parsestate_save(&state);
   pari_lex_start = lex;
-  pari_unused_chars=NULL;
   pari_once=1;
   pari_discarded=0;
   pari_lasterror=NULL;
-  if (pari_parse(&lex) || pari_discarded)
+  if (pari_parse((char**)&lex) || pari_discarded)
   {
-    if (pari_unused_chars && !pari_discarded)
-      unused_chars(pari_unused_chars,strict);
-    else if (pari_lasterror)
+    if (pari_lasterror)
       compile_err(GSTR(pari_lasterror),lex-1);
     else /* should not happen */
       compile_err("syntax error",lex-1);
