@@ -11,14 +11,13 @@ Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
-/**************************************************************/
-/*                                                            */
-/*    Galois group for degree between 8 and 11 (included)     */
-/*                                                            */
-/**************************************************************/
 #include "pari.h"
 #include "paripriv.h"
 
+/**************************************************************/
+/*              Galois group for degree in [8, 11]            */
+/**************************************************************/
+
 #define NMAX 11 /* maximum degree */
 
 typedef GEN PERM;
@@ -385,8 +384,8 @@ Monomial(GEN r, PERM bb, long nbv)
     if (typ(c) == t_COMPLEX && i < nbv)
     { /* detect conjugates */
       GEN n = gel(R,++i);
-      if (!absr_cmp(gel(n,1), gel(c,1))
-       && !absr_cmp(gel(n,2), gel(c,2))
+      if (!abscmprr(gel(n,1), gel(c,1))
+       && !abscmprr(gel(n,2), gel(c,2))
        && signe(gel(c,2)) != signe(gel(n,2)))
         c = addrr(sqrr(gel(c,1)), sqrr(gel(c,2)));
       else
@@ -416,8 +415,8 @@ gpolynomial(GEN r, resolv *R)
     }
   }
   setlg(IM, k);
-  RE = vecpermute(RE, gen_indexsort(RE, (void*)&absr_cmp, cmp_nodata));
-  IM = vecpermute(IM, gen_indexsort(IM, (void*)&absr_cmp, cmp_nodata));
+  RE = vecpermute(RE, gen_indexsort(RE, (void*)&abscmprr, cmp_nodata));
+  IM = vecpermute(IM, gen_indexsort(IM, (void*)&abscmprr, cmp_nodata));
   re = gel(RE,1);
   for (i = 2; i <= R->nm; i++) re = addrr(re, gel(RE,i));
   if (k == 1) return re;
@@ -2192,9 +2191,10 @@ closure11(long EVEN, buildroot *BR)
     GEN h = BR->p, r = compositum(h, h);
     r = gel(r,lg(r)-1);
     if (degpol(r) == 22) return 2; /* D11 */
-    h = leafcopy(h); setvarn(h, MAXVARN);
+    h = leafcopy(h); setvarn(h, fetch_var());
     setvarn(r, 0); r = nffactor(h, r);
     /* S11 (P10*P10*P90) or F_110[11] (11 factors of degree 10) */
+    (void)delete_var();
     return (lgcols(r)-1 == 11)? 4: 8;
   }
 }
@@ -2357,7 +2357,7 @@ polgaloisnamesbig(long n, long k)
 }
 
 /* pol a monic ZX */
-GEN
+static GEN
 galoisbig(GEN pol, long prec)
 {
   pari_sp av = avma;
@@ -2403,7 +2403,7 @@ galoisbig(GEN pol, long prec)
     }
     BR.coef = z;
     BR.p = pol;
-    BR.pr = prec + nbits2extraprec((long)(cauchy_bound(pol) / LOG2));
+    BR.pr = prec + nbits2extraprec((long)fujiwara_bound(pol));
     BR.prmax = BR.pr + BIGDEFAULTPREC-2;
     BR.N = N;
     BR.r = vectrunc_init(N+1);
@@ -2426,3 +2426,311 @@ galoisbig(GEN pol, long prec)
   gel(res,4) = polgaloisnamesbig(N,t);
   return res;
 }
+
+/**************************************************************/
+/*               Galois group for degree <= 7                 */
+/**************************************************************/
+
+/* exchange elements i and j in vector x */
+static GEN
+transroot(GEN x, int i, int j)
+{ x = leafcopy(x); swap(gel(x,i), gel(x,j)); return x; }
+
+/* x1*x2^2 + x2*x3^2 + x3*x4^2 + x4*x1^2 */
+static GEN
+F4(GEN x)
+{
+  return gadd(
+    gmul(gel(x,1), gadd(gsqr(gel(x,2)), gmul(gel(x,4),gel(x,1)))),
+    gmul(gel(x,3), gadd(gsqr(gel(x,4)), gmul(gel(x,2),gel(x,3)))));
+}
+
+static GEN
+roots_to_ZX(GEN z, long *e)
+{
+  GEN a = roots_to_pol(z,0);
+  GEN b = grndtoi(real_i(a),e);
+  long e1 = gexpo(imag_i(a));
+  if (e1 > *e) *e = e1;
+  return b;
+}
+
+static GEN
+polgaloisnames(long a, long b)
+{
+  const char * const t[]={"S1", "S2", "A3", "S3",
+       "C(4) = 4", "E(4) = 2[x]2", "D(4)", "A4", "S4",
+       "C(5) = 5", "D(5) = 5:2", "F(5) = 5:4", "A5", "S5",
+       "C(6) = 6 = 3[x]2", "D_6(6) = [3]2", "D(6) = S(3)[x]2",
+             "A_4(6) = [2^2]3", "F_18(6) = [3^2]2 = 3 wr 2",
+             "2A_4(6) = [2^3]3 = 2 wr 3", "S_4(6d) = [2^2]S(3)",
+             "S_4(6c) = 1/2[2^3]S(3)", "F_18(6):2 = [1/2.S(3)^2]2",
+             "F_36(6) = 1/2[S(3)^2]2", "2S_4(6) = [2^3]S(3) = 2 wr S(3)",
+             "L(6) = PSL(2,5) = A_5(6)", "F_36(6):2 = [S(3)^2]2 = S(3) wr 2",
+             "L(6):2 = PGL(2,5) = S_5(6)", "A6", "S6",
+       "C(7) = 7", "D(7) = 7:2", "F_21(7) = 7:3", "F_42(7) = 7:6",
+             "L(7) = L(3,2)", "A7", "S7"};
+
+   const long idx[]={0,1,2,4,9,14,30};
+   return strtoGENstr(t[idx[a-1]+b-1]);
+}
+
+static GEN
+galois_res(long d, long n, long s, long k)
+{
+  GEN z = cgetg(5,t_VEC);
+  long kk;
+  if (new_galois_format)
+    kk = k;
+  else
+    kk = (d == 6 && (k==6 || k==2))? 2: 1;
+  gel(z,1) = stoi(n);
+  gel(z,2) = stoi(s);
+  gel(z,3) = stoi(kk);
+  gel(z,4) = polgaloisnames(d,k);
+  return z;
+}
+
+GEN
+polgalois(GEN x, long prec)
+{
+  pari_sp av = avma, av1;
+  long i,j,k,n,f,l,l2,e,e1,pr,ind;
+  GEN x1,p1,p2,p3,p4,p5,w,z,ee;
+  const int ind5[20]={2,5,3,4, 1,3,4,5, 1,5,2,4, 1,2,3,5, 1,4,2,3};
+  const int ind6[60]={3,5,4,6, 2,6,4,5, 2,3,5,6, 2,4,3,6, 2,5,3,4,
+                      1,4,5,6, 1,5,3,6, 1,6,3,4, 1,3,4,5, 1,6,2,5,
+                      1,2,4,6, 1,5,2,4, 1,3,2,6, 1,2,3,5, 1,4,2,3};
+  if (typ(x)!=t_POL) pari_err_TYPE("galois",x);
+  n=degpol(x);
+  if (n>11) pari_err_IMPL("galois of degree higher than 11");
+  x = Q_primpart(x);
+  RgX_check_ZX(x, "galois");
+  if (!ZX_is_irred(x)) pari_err_IRREDPOL("galois",x);
+
+  if (n<4)
+  {
+    if (n == 1) { avma = av; return galois_res(n,1, 1,1); }
+    if (n == 2) { avma = av; return galois_res(n,2,-1,1); }
+    /* n = 3 */
+    f = Z_issquare(ZX_disc(x));
+    avma = av;
+    return f? galois_res(n,3,1,1):
+              galois_res(n,6,-1,2);
+  }
+  x1 = x = ZX_Q_normalize(x,NULL); av1=avma;
+  if (n > 7) return galoisbig(x, prec);
+  for(;;)
+  {
+    double fb = fujiwara_bound(x);
+    switch(n)
+    {
+      case 4: z = cgetg(7,t_VEC);
+        prec = nbits2prec((long)(fb*18.) + 64);
+        for(;;)
+        {
+          p1=QX_complex_roots(x,prec);
+          gel(z,1) = F4(p1);
+          gel(z,2) = F4(transroot(p1,1,2));
+          gel(z,3) = F4(transroot(p1,1,3));
+          gel(z,4) = F4(transroot(p1,1,4));
+          gel(z,5) = F4(transroot(p1,2,3));
+          gel(z,6) = F4(transroot(p1,3,4));
+          p5 = roots_to_ZX(z, &e); if (e <= -10) break;
+          prec = precdbl(prec);
+        }
+        if (!ZX_is_squarefree(p5)) goto tchi;
+        p2 = gel(ZX_factor(p5),1);
+        switch(lg(p2)-1)
+        {
+          case 1: f = Z_issquare(ZX_disc(x)); avma = av;
+            return f? galois_res(n,12,1,4): galois_res(n,24,-1,5);
+
+          case 2: avma = av; return galois_res(n,8,-1,3);
+
+          case 3: avma = av;
+            return (degpol(gel(p2,1))==2)? galois_res(n,4,1,2)
+                                         : galois_res(n,4,-1,1);
+
+          default: pari_err_BUG("galois (bug1)");
+        }
+
+      case 5: z = cgetg(7,t_VEC);
+        ee= cgetg(7,t_VECSMALL);
+        w = cgetg(7,t_VECSMALL);
+        prec = nbits2prec((long)(fb*21.) + 64);
+        for(;;)
+        {
+          for(;;)
+          {
+            p1=QX_complex_roots(x,prec);
+            for (l=1; l<=6; l++)
+            {
+              p2=(l==1)?p1: ((l<6)?transroot(p1,1,l): transroot(p1,2,5));
+              p3=gen_0;
+              for (k=0,i=1; i<=5; i++,k+=4)
+              {
+                p5 = gadd(gmul(gel(p2,ind5[k]),gel(p2,ind5[k+1])),
+                          gmul(gel(p2,ind5[k+2]),gel(p2,ind5[k+3])));
+                p3 = gadd(p3, gmul(gsqr(gel(p2,i)),p5));
+              }
+              gel(w,l) = grndtoi(real_i(p3),&e);
+              e1 = gexpo(imag_i(p3)); if (e1>e) e=e1;
+              ee[l]=e; gel(z,l) = p3;
+            }
+            p5 = roots_to_ZX(z, &e); if (e <= -10) break;
+            prec = precdbl(prec);
+          }
+          if (!ZX_is_squarefree(p5)) goto tchi;
+          p3=gel(ZX_factor(p5),1);
+          f=Z_issquare(ZX_disc(x));
+          if (lg(p3)-1==1)
+          {
+            avma = av;
+            return f? galois_res(n,60,1,4): galois_res(n,120,-1,5);
+          }
+          if (!f) { avma = av; return galois_res(n,20,-1,3); }
+
+          pr = - (prec2nbits(prec) >> 1);
+          for (l=1; l<=6; l++)
+            if (ee[l] <= pr && gequal0(poleval(p5,gel(w,l)))) break;
+          if (l>6) pari_err_BUG("galois (bug4)");
+          p2=(l==6)? transroot(p1,2,5):transroot(p1,1,l);
+          p3=gen_0;
+          for (i=1; i<=5; i++)
+          {
+            j = (i == 5)? 1: i+1;
+            p3 = gadd(p3,gmul(gmul(gel(p2,i),gel(p2,j)),
+                              gsub(gel(p2,j),gel(p2,i))));
+          }
+          p5=gsqr(p3); p4=grndtoi(real_i(p5),&e);
+          e1 = gexpo(imag_i(p5)); if (e1>e) e=e1;
+          if (e <= -10)
+          {
+            if (gequal0(p4)) goto tchi;
+            f = Z_issquare(p4); avma = av;
+            return f? galois_res(n,5,1,1): galois_res(n,10,1,2);
+          }
+          prec = precdbl(prec);
+        }
+
+      case 6: z = cgetg(7, t_VEC);
+        prec = nbits2prec((long) (fb * 42) + 64);
+        for(;;)
+        {
+          for(;;)
+          {
+            p1=QX_complex_roots(x,prec);
+            for (l=1; l<=6; l++)
+            {
+              p2=(l==1)?p1:transroot(p1,1,l);
+              p3=gen_0; k=0;
+              for (i=1; i<=5; i++) for (j=i+1; j<=6; j++)
+              {
+                p5=gadd(gmul(gel(p2,ind6[k]),gel(p2,ind6[k+1])),
+                        gmul(gel(p2,ind6[k+2]),gel(p2,ind6[k+3])));
+                p3=gadd(p3,gmul(gsqr(gmul(gel(p2,i),gel(p2,j))),p5));
+                k += 4;
+              }
+              gel(z,l) = p3;
+            }
+            p5 = roots_to_ZX(z, &e); if (e <= -10) break;
+            prec = precdbl(prec);
+          }
+          if (!ZX_is_squarefree(p5)) goto tchi;
+          p2=gel(ZX_factor(p5),1);
+          switch(lg(p2)-1)
+          {
+            case 1:
+              z = cgetg(11,t_VEC); ind=0;
+              p3=gadd(gmul(gmul(gel(p1,1),gel(p1,2)),gel(p1,3)),
+                      gmul(gmul(gel(p1,4),gel(p1,5)),gel(p1,6)));
+              gel(z,++ind) = p3;
+              for (i=1; i<=3; i++)
+                for (j=4; j<=6; j++)
+                {
+                  p2=transroot(p1,i,j);
+                  p3=gadd(gmul(gmul(gel(p2,1),gel(p2,2)),gel(p2,3)),
+                          gmul(gmul(gel(p2,4),gel(p2,5)),gel(p2,6)));
+                  gel(z,++ind) = p3;
+                }
+              p5 = roots_to_ZX(z, &e);
+              if (e <= -10)
+              {
+                if (!ZX_is_squarefree(p5)) goto tchi;
+                p2 = gel(ZX_factor(p5),1);
+                f = Z_issquare(ZX_disc(x));
+                avma = av;
+                if (lg(p2)-1==1)
+                  return f? galois_res(n,360,1,15): galois_res(n,720,-1,16);
+                else
+                  return f? galois_res(n,36,1,10): galois_res(n,72,-1,13);
+              }
+              prec = precdbl(prec); break;
+
+            case 2: l2=degpol(gel(p2,1)); if (l2>3) l2=6-l2;
+              switch(l2)
+              {
+                case 1: f = Z_issquare(ZX_disc(x));
+                  avma = av;
+                  return f? galois_res(n,60,1,12): galois_res(n,120,-1,14);
+                case 2: f = Z_issquare(ZX_disc(x));
+                  if (f) { avma = av; return galois_res(n,24,1,7); }
+                  p3 = (degpol(gel(p2,1))==2)? gel(p2,2): gel(p2,1);
+                  f = Z_issquare(ZX_disc(p3));
+                  avma = av;
+                  return f? galois_res(n,24,-1,6): galois_res(n,48,-1,11);
+                case 3: f = Z_issquare(ZX_disc(gel(p2,1)))
+                         || Z_issquare(ZX_disc(gel(p2,2)));
+                  avma = av;
+                  return f? galois_res(n,18,-1,5): galois_res(n,36,-1,9);
+              }
+            case 3:
+              for (l2=1; l2<=3; l2++)
+                if (degpol(gel(p2,l2)) >= 3) p3 = gel(p2,l2);
+              if (degpol(p3) == 3)
+              {
+                f = Z_issquare(ZX_disc(p3)); avma = av;
+                return f? galois_res(n,6,-1,1): galois_res(n,12,-1,3);
+              }
+              else
+              {
+                f = Z_issquare(ZX_disc(x)); avma = av;
+                return f? galois_res(n,12,1,4): galois_res(n,24,-1,8);
+              }
+            case 4: avma = av; return galois_res(n,6,-1,2);
+            default: pari_err_BUG("galois (bug3)");
+          }
+        }
+
+      case 7: z = cgetg(36,t_VEC);
+        prec = nbits2prec((long)(fb*7.) + 64);
+        for(;;)
+        {
+          ind = 0; p1=QX_complex_roots(x,prec);
+          for (i=1; i<=5; i++)
+            for (j=i+1; j<=6; j++)
+            {
+              GEN t = gadd(gel(p1,i),gel(p1,j));
+              for (k=j+1; k<=7; k++) gel(z,++ind) = gadd(t, gel(p1,k));
+            }
+          p5 = roots_to_ZX(z, &e); if (e <= -10) break;
+          prec = precdbl(prec);
+        }
+        if (!ZX_is_squarefree(p5)) goto tchi;
+        p2=gel(ZX_factor(p5),1);
+        switch(lg(p2)-1)
+        {
+          case 1: f = Z_issquare(ZX_disc(x)); avma = av;
+            return f? galois_res(n,2520,1,6): galois_res(n,5040,-1,7);
+          case 2: f = degpol(gel(p2,1)); avma = av;
+            return (f==7 || f==28)? galois_res(n,168,1,5): galois_res(n,42,-1,4);
+          case 3: avma = av; return galois_res(n,21,1,3);
+          case 4: avma = av; return galois_res(n,14,-1,2);
+          case 5: avma = av; return galois_res(n,7,1,1);
+          default: pari_err_BUG("galois (bug2)");
+        }
+    }
+    tchi: avma = av1; x = tschirnhaus(x1);
+  }
+}
