@@ -174,15 +174,17 @@ group_ident_i(GEN G, GEN S)
     switch(e[1]+e[2]+e[3])
     {
     case 3: /*pqr*/
-      if (p[1]==2 && p[2]==3)  /*6p*/
+      if (p[1]==2)  /*2qr*/
       {
-        long q = p[3],q2 = q*q, pmq = (q%3==1)? 2: 0;
-        if (s==13-13*q+13*q2) return 1+pmq; /* S3xp */
-        if (s==7+7*q+7*q2) return 2+pmq; /* D2px3 */
-        if (s==7-q+7*q2) return 3+pmq; /* 3:2+p:2 */
-        if (s==21-21*q+21*q2) return 4+pmq; /* 6p */
-        if (s==1+19*q+q2) return 1; /* p:6 */
-        return 2;       /* p:3x2 */
+        long q = p[2],q2 = q*q, qc = 1-q+q2, qd = 1+q+q2;
+        long r = p[3],r2 = r*r, rc = 1-r+r2, rd = 1+r+r2;
+        long pmq = (r%q==1)? 2: 0;
+        if (pmq && s==3*r*(q2-q)+rd) return 1; /* r:2q */
+        if (pmq && s==3*(r2+(q2-q-1)*r+1)) return 2; /* r:qx2 */
+        if (s==qd*rc) return 1+pmq; /* D2qxr */
+        if (s==rd*qc) return 2+pmq; /* D2rxq */
+        if (s==3*qc*rc) return 4+pmq; /* 2qr */
+        return 3+pmq; /* q:2+r:2 */
       }
       break;
     }
@@ -196,7 +198,6 @@ group_ident_i(GEN G, GEN S)
        -1,
     56, 697, 1849, 585, 557, 529, 413, 385, 989, 817, 1161, 351, 357, 645, -1,
     60, 945, 721, 561, 1617, 211, 497, 337, 373, 651, 581, 693, 501, 1029, -1,
-    70, 1333, 1197, 973, 2709, -1,
     75, 3647, 271, 847, -1,
     84, 647, 935, 1935, 1295, 1071, 3311, 451, 699, 595, 1333, 469, 1099, 1419,
         987, 2107, -1,
@@ -205,7 +206,6 @@ group_ident_i(GEN G, GEN S)
     104, 2143, 6751, 991, 1935, 1883, 1831, 1307, 1255, 3611, 2983, 4239, 731,
          1203, 2355, -1,
     105, 1785, 6321, -1,
-    110, 793, 993, 3441, 2793, 2441, 6993,-1,
     126, 1533, 2037, 3397, 3477, 2749, 7869, 777, 937, 721, 1281, 1425, 2881,
          1369, 1849, 1201, 3225, -1,
       -1};
