@@ -38,7 +38,7 @@ typedef struct _primedata {
   GEN firstroot; /* *[i] = index of first root of fk[i] */
   GEN interp;    /* *[i] = interpolation polynomial for fk[i]
                   * [= 1 on the first root firstroot[i], 0 on the others] */
-  GEN bezoutC; /* Bezout coefficients associated to the ff[i] */
+  GEN bezoutC; /* Bezout coefficients attached to the ff[i] */
   GEN Trk;     /* used to compute traces (cf poltrace) */
 } primedata;
 typedef struct _blockdata {
@@ -56,7 +56,7 @@ static GEN test_block(blockdata *B, GEN L, GEN D);
 /* COMBINATORIAL PART: generate potential block systems */
 
 #define BIL 32 /* for 64bit machines also */
-/* Computation of potential block systems of given size d associated to a
+/* Computation of potential block systems of given size d attached to a
  * rational prime p: give a row vector of row vectors containing the
  * potential block systems of imprimitivity; a potential block system is a
  * vector of row vectors (enumeration of the roots). */
@@ -193,7 +193,7 @@ im_block_by_perm(GEN D,GEN perm)
 {
   long i, lb = lg(D);
   GEN Dn = cgetg(lb,t_VEC);
-  for (i=1; i<lb; i++) gel(Dn,i) = vecpermute(perm, gel(D,i));
+  for (i=1; i<lb; i++) gel(Dn,i) = vecsmallpermute(perm, gel(D,i));
   return Dn;
 }
 
@@ -308,7 +308,7 @@ polsimplify(GEN x)
 {
   long i,lx = lg(x);
   for (i=2; i<lx; i++)
-    if (typ(gel(x,i)) == t_POL) gel(x,i) = constant_term(gel(x,i));
+    if (typ(gel(x,i)) == t_POL) gel(x,i) = constant_coeff(gel(x,i));
   return x;
 }
 
@@ -318,7 +318,7 @@ ok_coeffs(GEN g,GEN M)
 {
   long i, lg = lg(g)-1; /* g is monic, and cst term is ok */
   for (i=3; i<lg; i++)
-    if (absi_cmp(gel(g,i), gel(M,i)) > 0) return 0;
+    if (abscmpii(gel(g,i), gel(M,i)) > 0) return 0;
   return 1;
 }
 
@@ -447,7 +447,7 @@ get_bezout(GEN pol, GEN fk, GEN p)
     B = FpX_div(pol, A, p);
     d = FpX_extgcd(A,B,p, &u, &v);
     if (degpol(d) > 0) pari_err_COPRIME("get_bezout",A,B);
-    d = constant_term(d);
+    d = constant_coeff(d);
     if (!gequal1(d)) v = FpX_Fp_mul(v, Fp_inv(d, p), p);
     gel(U,i) = FpX_mul(B,v, p);
   }
@@ -458,7 +458,7 @@ static GEN
 init_traces(GEN ff, GEN T, GEN p)
 {
   long N = degpol(T),i,j,k, r = lg(ff);
-  GEN Frob = FpXQ_matrix_pow(FpXQ_pow(pol_x(varn(T)),p, T,p), N,N, T,p);
+  GEN Frob = FpX_matFrobenius(T,p);
   GEN y,p1,p2,Trk,pow,pow1;
 
   k = degpol(gel(ff,r-1)); /* largest degree in modular factorization */
@@ -507,7 +507,7 @@ init_primedata(primedata *S)
   for (l=1,j=1; j<lff; j++)
   { /* compute roots and fix ordering (Frobenius cycles) */
     GEN F = gel(S->ff, j), deg1 = FpX_factorff_irred(F, T,p);
-    GEN H = gel(deg1,1), a = Fq_neg(constant_term(H), T,p);
+    GEN H = gel(deg1,1), a = Fq_neg(constant_coeff(H), T,p);
     GEN Q = FqX_div(F, H, T,p);
     GEN q = Fq_inv(FqX_eval(Q, a, T,p), T,p);
     gel(S->interp,j) = FqX_Fq_mul(Q, q, T,p); /* = 1 at a, 0 at other roots */
@@ -543,7 +543,7 @@ choose_prime(primedata *S, GEN pol, GEN dpol)
     for (j=2; j<=r; j++) { n[j] = degpol(gel(ff,j)); lcm = clcm(lcm, n[j]); }
     if (lcm <= oldlcm) continue; /* false when oldlcm = 0 */
 
-    if (DEBUGLEVEL) err_printf("p = %ld,\tlcm = %ld,\torbits: %Ps\n",p,lcm,n);
+    if (DEBUGLEVEL) err_printf("p = %lu,\tlcm = %ld,\torbits: %Ps\n",p,lcm,n);
     pp = p;
     oldn = n;
     oldff = ff;
@@ -600,7 +600,7 @@ bound_for_coeff(long m, GEN rr, GEN *maxroot)
  * 3: Hensel lift to precision p^e of DATA[4]
  * 4: roots of pol in F_(p^S->lcm),
  * 5: number of polynomial changes (translations)
- * 6: Bezout coefficients associated to the S->ff[i]
+ * 6: Bezout coefficients attached to the S->ff[i]
  * 7: Hadamard bound for coefficients of h(x) such that g o h = 0 mod pol.
  * 8: bound M for polynomials defining subfields x PD->den
  * 9: *[i] = interpolation polynomial for S->ff[i] [= 1 on the first root
@@ -663,13 +663,13 @@ compute_data(blockdata *B)
   gel(DATA,1) = pol;
   MM = gmul2n(bound_for_coeff(B->d, roo, &maxroot), 1);
   gel(DATA,8) = MM;
-  e = logint(shifti(vecmax(MM),20), p, &pe); /* overlift 2^20 [for d-1 test] */
+  e = logintall(shifti(vecmax(MM),20), p, &pe); /* overlift 2^20 [d-1 test] */
   gel(DATA,2) = pe;
   gel(DATA,4) = roots_from_deg1(fk);
 
   /* compute fhk = ZpX_liftfact(pol,fk,T,p,e,pe) in 2 steps
    * 1) lift in Zp to precision p^e */
-  ffL = ZpX_liftfact(pol, ff, NULL, p, e, pe);
+  ffL = ZpX_liftfact(pol, ff, pe, p, e);
   fhk = NULL;
   for (l=i=1; i<lff; i++)
   { /* 2) lift factorization of ff[i] in Qp[X] / T */
@@ -677,7 +677,7 @@ compute_data(blockdata *B)
     long di = degpol(L);
     F = cgetg(di+1, t_VEC);
     for (j=1; j<=di; j++) F[j] = fk[l++];
-    L = ZpX_liftfact(L, F, T, p, e, pe);
+    L = ZqX_liftfact(L, F, T, pe, p, e);
     fhk = fhk? shallowconcat(fhk, L): L;
   }
   gel(DATA,3) = roots_from_deg1(fhk);
@@ -731,11 +731,11 @@ subfield(GEN A, blockdata *B)
      * if g o h = 0 (pol), we'll have h(Ai[j]) = delta[i] for all j */
     /* fk[k] belongs to block number whichdelta[k] */
     for (j=1; j<=d; j++) whichdelta[Ai[j]] = i;
-    if (typ(p1) == t_POL) p1 = constant_term(p1);
+    if (typ(p1) == t_POL) p1 = constant_coeff(p1);
     d_1_term = addii(d_1_term, p1);
   }
   d_1_term = centermod(d_1_term, pe); /* Tr(g) */
-  if (absi_cmp(d_1_term, gel(M,3)) > 0) {
+  if (abscmpii(d_1_term, gel(M,3)) > 0) {
     if (DEBUGLEVEL>1) err_printf("d-1 test failed\n");
     return NULL;
   }
@@ -841,7 +841,7 @@ subfieldsall(GEN nf)
     long l;
 
     pol = get_nfpol(nf, &nf);
-    L = lift_intern( galoissubfields(G, 0, varn(pol)) );
+    L = lift_shallow( galoissubfields(G, 0, varn(pol)) );
     l = lg(L);
     S = cgetg(l, t_VECSMALL);
     for (i=1; i<l; i++) S[i] = lg(gmael(L,i,1));
@@ -868,7 +868,7 @@ subfieldsall(GEN nf)
       B.size  = dg[i];
       B.d = N / B.size;
       NLSB = subfields_of_given_degree(&B);
-      if (NLSB) { LSB = concat(LSB, NLSB); gunclone(NLSB); }
+      if (NLSB) { LSB = gconcat(LSB, NLSB); gunclone(NLSB); }
     }
     (void)delete_var(); /* from choose_prime */
   }
@@ -908,7 +908,7 @@ nfsubfields(GEN nf, long d)
     {
       GEN H = gel(L,i);
       if (group_order(H) == o)
-        gel(F,k++) = lift_intern(galoisfixedfield(G, gel(H,1), 0, v0));
+        gel(F,k++) = lift_shallow(galoisfixedfield(G, gel(H,1), 0, v0));
     }
     setlg(F, k);
     return gerepilecopy(av, F);
