@@ -18,8 +18,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 static THREAD int pari_MPI_size, pari_MPI_rank;
 static THREAD long nbreq = 0;
 
-enum PMPI_cmd { PMPI_close, PMPI_worker, PMPI_work, PMPI_parisize,
-                PMPI_precreal, PMPI_eval };
+enum PMPI_cmd { PMPI_close, PMPI_worker, PMPI_work, PMPI_parisizemax,
+                PMPI_parisize, PMPI_precreal, PMPI_eval };
 
 struct mt_mstate
 {
@@ -110,8 +110,10 @@ recvstatus_buf(int source, MPI_Status *status)
 static GEN
 recvstatus_GEN(int source, MPI_Status *status)
 {
+  GEN res;
   GENbin *buf = recvstatus_buf(source, status);
-  GEN res = bin_copy(buf);
+  buf->rebase = &shiftaddress;
+  res = bin_copy(buf);
   bincopy_relink(gel(res,1),gel(res,2));
   return gel(res,1);
 }
@@ -161,10 +163,9 @@ static void
 pari_MPI_child(void)
 {
   pari_sp av = avma;
-  long size;
+  ulong rsize = 0, vsize = 0;
   GEN worker = NULL, work, done;
   struct gp_context rec;
-  if (!diffptr) initprimetable(500000);
   gp_context_save(&rec);
   if (setjmp(child_env))
   {
@@ -175,7 +176,8 @@ pari_MPI_child(void)
     switch (recvfrom_request(0))
     {
     case PMPI_worker:
-      avma = top;
+      paristack_setsize(rsize, vsize);
+      gp_context_save(&rec);
       worker = recvfrom_GEN(0);
       av = avma;
       break;
@@ -185,10 +187,11 @@ pari_MPI_child(void)
       send_GEN(done, 0);
       avma = av;
       break;
+    case PMPI_parisizemax:
+      vsize = recvfrom_long(0);
+      break;
     case PMPI_parisize:
-      size = recvfrom_long(0);
-      pari_init_stack(size,top-bot);
-      gp_context_save(&rec);
+      rsize = recvfrom_long(0);
       break;
     case PMPI_precreal:
       precreal = recvfrom_long(0);
@@ -244,7 +247,7 @@ pari_mt_init(void)
     MPI_Comm_size(MPI_COMM_WORLD, &pari_MPI_size);
     MPI_Comm_rank(MPI_COMM_WORLD, &pari_MPI_rank);
     if (pari_MPI_rank) pari_MPI_child();
-    pari_mt_nbthreads = pari_MPI_size-1;
+    if (!pari_mt_nbthreads) pari_mt_nbthreads = pari_MPI_size-1;
   }
   else
   {
@@ -271,12 +274,18 @@ mtmpi_queue_get(struct mt_state *junk, long *workid, long *pending)
 {
   struct mt_mstate *mt = pari_mt;
   GEN done;
+  (void) junk;
   if (mt->nbint<=mt->n) { mt->source=mt->nbint; *pending = nbreq; return NULL; }
   done = recvany_GEN(&mt->source);
   nbreq--; *pending = nbreq;
   if (workid) *workid = mt->workid[mt->source];
   if (typ(done) == t_ERROR)
-    pari_err(0,done);
+  {
+    if (err_get_num(done)==e_STACK)
+      pari_err(e_STACKTHREAD);
+    else
+      pari_err(0,done);
+  }
   return done;
 }
 
@@ -284,6 +293,7 @@ static void
 mtmpi_queue_submit(struct mt_state *junk, long workid, GEN work)
 {
   struct mt_mstate *mt = pari_mt;
+  (void) junk;
   if (!work) { mt->nbint=mt->n+1; return; }
   if (mt->nbint<=mt->n) mt->nbint++;
   nbreq++;
@@ -311,13 +321,15 @@ mt_queue_start(struct pari_mt *pt, GEN worker)
   {
     struct mt_mstate *mt = &pari_mt_data;
     long i, n = minss(pari_mt_nbthreads, pari_MPI_size-1);
-    long mtparisize = GP_DATA->threadsize? GP_DATA->threadsize: top-bot;
+    long mtparisize = GP_DATA->threadsize? GP_DATA->threadsize: pari_mainstack->rsize;
+    long mtparisizemax = GP_DATA->threadsizemax;
     pari_mt = mt;
     mt->workid = (long*) pari_malloc(sizeof(long)*(n+1));
     for (i=1; i <= n; i++)
     {
       send_request_long(PMPI_parisize, mtparisize, i);
-      send_request_long(PMPI_precreal, precreal, i);
+      send_request_long(PMPI_parisizemax, mtparisizemax, i);
+      send_request_long(PMPI_precreal, get_localbitprec(), i);
       send_request_GEN(PMPI_worker, worker, i);
     }
     mt->n = n;
