@@ -19,11 +19,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include <pari.h>
 #include <paripriv.h>
 
-#define numberof(x) (sizeof(x) / sizeof((x)[0]))
-
 int option_trace = 0;
 double Step_Factor = .01; /* small steps by default */
-ulong DFLT_mod1, DFLT_hmod, DFLT_mod2;
+ulong DFLT_mod1, DFLT_hmod, DFLT_qmod, DFLT_mod2;
 GEN LARGE_mod;
 
 #ifdef LONG_IS_64BIT
@@ -38,10 +36,10 @@ GEN LARGE_mod;
 
 typedef struct {
   ulong reps, type;
-  long *var, *var_disable, size, enabled;
+  long *var, *var_disable, *var_enable, var_enable_min,  size, enabled;
   GEN x, y;
   ulong l;
-  GEN p;
+  GEN T, p;
 } speed_param;
 
 typedef double (*speed_function_t)(speed_param *s);
@@ -57,6 +55,7 @@ typedef struct {
   double            step_factor; /* how much to step sizes (rounded down) */
   double            stop_factor;
   long              *var_disable;
+  long              *var_enable;
 } tune_param;
 
 /* ========================================================== */
@@ -100,6 +99,14 @@ rand_FpX(long n)
 }
 /* Flx, degree n */
 static GEN
+rand_F2x(long n)
+{
+  GEN x;
+  do x = random_F2x(BITS_IN_LONG*n, 0); while (lgpol(x) < n);
+  return x;
+}
+/* Flx, degree n */
+static GEN
 rand_Flx(long n, ulong l)
 {
   GEN x;
@@ -107,12 +114,27 @@ rand_Flx(long n, ulong l)
   return x;
 }
 
+static GEN
+rand_FlxqX(long n, GEN T, ulong l)
+{
+  GEN x;
+  do x = random_FlxqX(n+1, 0, T, l); while (degpol(x) < n);
+  return x;
+}
+
+static GEN
+rand_FpXQX(long n, GEN T)
+{
+  GEN x;
+  do x = random_FpXQX(n+1, 0, T, LARGE_mod); while (degpol(x) < n);
+  return x;
+}
 /* normalized Fpx, degree n */
 static GEN
 rand_NFpX(long n)
 {
   pari_sp av = avma;
-  GEN x = gadd(monomial(gen_1,n,0), random_FpX(n, 0, LARGE_mod));
+  GEN x = gadd(pol_xn(n,0), random_FpX(n, 0, LARGE_mod));
   return gerepileupto(av, x);
 }
 
@@ -125,42 +147,107 @@ rand_NFlx(long n, ulong l)
   return gerepileuptoleaf(av, x);
 }
 
-#define t_Fhx   99
-#define t_Flx  100
-#define t_Fl1x 101
-#define t_Fl2x 102
-#define t_NFlx 103
-#define t_FpX  104
-#define t_NFpX 105
+static GEN
+rand_NFlxqX(long n, GEN T, long l)
+{
+  pari_sp av = avma;
+  GEN x = FlxX_add(monomial(pol1_Flx(0),n,0), random_FlxqX(n, 0, T, l), l);
+  return gerepileupto(av, x);
+}
 
 static GEN
-rand_g(long n, long type)
+rand_NFpXQX(long n, GEN T)
 {
-  switch (type) {
+  pari_sp av = avma;
+  GEN x = gadd(pol_xn(n,0), random_FpXQX(n, 0, T, LARGE_mod));
+  return gerepileupto(av, x);
+}
+
+#define t_F2x     100
+#define t_Fqx     200
+#define t_Fhx     201
+#define t_Flx     202
+#define t_Fl1x    203
+#define t_Fl2x    204
+#define t_NFqx    210
+#define t_NFhx    211
+#define t_NFlx    212
+#define t_NFl1x   213
+#define t_NFl2x   214
+#define t_FpX     300
+#define t_NFpX    310
+#define t_FlxqX   400
+#define t_NFlxqX  410
+#define t_FpXQX   500
+#define t_NFpXQX  510
+
+static GEN
+rand_g(speed_param *s)
+{
+  long n = s->size;
+  switch (s->type) {
     case t_INT:  return rand_INT(n);
     case t_REAL: return rand_REAL(n);
+    case t_F2x:  return rand_F2x(n);
+    case t_Fqx:  return rand_Flx(n,DFLT_qmod);
     case t_Fhx:  return rand_Flx(n,DFLT_hmod);
     case t_Flx:  return rand_Flx(n,DFLT_mod);
     case t_Fl1x: return rand_Flx(n,DFLT_mod1);
     case t_Fl2x: return rand_Flx(n,DFLT_mod2);
+    case t_NFqx: return rand_NFlx(n,DFLT_qmod);
+    case t_NFhx: return rand_NFlx(n,DFLT_hmod);
     case t_NFlx: return rand_NFlx(n,DFLT_mod);
-    case t_FpX:  return rand_FpX(n);
-    case t_NFpX: return rand_NFpX(n);
+    case t_NFl1x: return rand_NFlx(n,DFLT_mod1);
+    case t_NFl2x: return rand_NFlx(n,DFLT_mod2);
+    case t_FpX:    return rand_FpX(n);
+    case t_NFpX:   return rand_NFpX(n);
+    case t_FlxqX:  return rand_FlxqX(n, s->T, s->l);
+    case t_NFlxqX: return rand_NFlxqX(n, s->T, s->l);
+    case t_FpXQX:  return rand_FpXQX(n, s->T);
+    case t_NFpXQX: return rand_NFpXQX(n, s->T);
   }
   return NULL;
 }
 
+static void
+dft_Flxq(speed_param *s)
+{
+  do
+  {
+    s->T = rand_NFlx(10, s->l);
+  } while (!Flx_is_irred(s->T, s->l));
+  s->T[1] = evalvarn(1);
+  s->T = Flx_get_red(s->T, s->l);
+}
+
+static void
+dft_FpXQ(speed_param *s)
+{
+  s->T = rand_NFpX(10);
+  setvarn(s->T, 1);
+  s->T = FpX_get_red(s->T, s->p);
+}
+
 static void
 dftmod(speed_param *s)
 {
   switch (s->type) {
+    case t_Fqx:  s->l=DFLT_qmod; return;
     case t_Fhx:  s->l=DFLT_hmod; return;
     case t_Flx:  s->l=DFLT_mod;  return;
     case t_Fl1x: s->l=DFLT_mod1; return;
     case t_Fl2x: s->l=DFLT_mod2; return;
+    case t_NFqx: s->l=DFLT_qmod;  return;
+    case t_NFhx: s->l=DFLT_hmod;  return;
     case t_NFlx: s->l=DFLT_mod;  return;
+    case t_NFl1x: s->l=DFLT_mod1;  return;
+    case t_NFl2x: s->l=DFLT_mod2;  return;
     case t_FpX:  s->p=LARGE_mod; return;
     case t_NFpX: s->p=LARGE_mod; return;
+    case t_FlxqX:  s->l=DFLT_mod;  dft_Flxq(s); return;
+    case t_NFlxqX: s->l=DFLT_mod;  dft_Flxq(s); return;
+    case t_FpXQX:  s->p=LARGE_mod; dft_FpXQ(s); return;
+    case t_NFpXQX: s->p=LARGE_mod; dft_FpXQ(s); return;
   }
 }
 
@@ -176,6 +263,7 @@ dftmod(speed_param *s)
   return speed_endtime();                \
 }
 
+#define m_menable(s,var,min) (*(s->var)=minss(lg(s->x)-2,s->min))
 #define  m_enable(s,var) (*(s->var)=lg(s->x)-2)/* enable  asymptotically fastest */
 #define m_disable(s,var) (*(s->var)=lg(s->x)+1)/* disable asymptotically fastest */
 
@@ -183,12 +271,14 @@ static void enable(speed_param *s)
 {
   m_enable(s,var); s->enabled = 1;
   if (s->var_disable) m_disable(s,var_disable);
+  if (s->var_enable) m_menable(s,var_enable,var_enable_min);
 }
 
 static void disable(speed_param *s)
 {
   m_disable(s,var); s->enabled = 0;
   if (s->var_disable) m_disable(s,var_disable);
+  if (s->var_enable) m_menable(s,var_enable,var_enable_min);
 }
 
 static double speed_mulrr(speed_param *s)
@@ -228,6 +318,9 @@ static double speed_divrr(speed_param *s)
 static double speed_invmod(speed_param *s)
 { GEN T; TIME_FUN(invmod(s->x, s->y, &T)); }
 
+static double speed_F2x_mul(speed_param *s)
+{ TIME_FUN(F2x_mul(s->x, s->y)); }
+
 static double speed_Flx_sqr(speed_param *s)
 { TIME_FUN(Flx_sqr(s->x, s->l)); }
 
@@ -291,6 +384,67 @@ static double speed_FpX_gcd(speed_param *s)
 static double speed_FpX_extgcd(speed_param *s)
 { GEN u,v; TIME_FUN(FpX_extgcd(s->x, s->y, s->p, &u, &v)); }
 
+static double speed_FlxqX_inv(speed_param *s)
+{ TIME_FUN(FlxqX_invBarrett(s->x, s->T, s->l)); }
+
+
+static double speed_FlxqX_divrem(speed_param *s)
+{
+  GEN r, x = rand_NFlxqX((degpol(s->x)-1)*2, s->T, s->l);
+  TIME_FUN(FlxqX_divrem(x, s->x, s->T, s->l, &r));
+}
+
+static double speed_FlxqX_rem(speed_param *s)
+{
+  GEN x = rand_NFlxqX((degpol(s->x)-1)*2, s->T, s->l);
+  TIME_FUN(FlxqX_rem(x, s->x, s->T, s->l));
+}
+
+static double speed_FlxqXQ_red(speed_param *s) {
+  GEN x = rand_NFlxqX((degpol(s->x)-1)*2, s->T, s->l);
+  GEN q = FlxqX_get_red(s->x, s->T, s->l);
+  TIME_FUN(FlxqX_rem(x, q, s->T, s->l));
+}
+
+static double speed_FlxqX_halfgcd(speed_param *s)
+{ TIME_FUN(FlxqX_halfgcd(s->x, s->y, s->T, s->l)); }
+
+static double speed_FlxqX_extgcd(speed_param *s)
+{ GEN u,v; TIME_FUN(FlxqX_extgcd(s->x, s->y, s->T, s->l, &u, &v)); }
+
+static double speed_FlxqX_gcd(speed_param *s)
+{ TIME_FUN(FlxqX_gcd(s->x, s->y, s->T, s->l)); }
+
+static double speed_FpXQX_inv(speed_param *s)
+{ TIME_FUN(FpXQX_invBarrett(s->x, s->T, s->p)); }
+
+static double speed_FpXQX_divrem(speed_param *s)
+{
+  GEN r, x = rand_NFpXQX((degpol(s->x)-1)*2, s->T);
+  TIME_FUN(FpXQX_divrem(x, s->x, s->T, s->p, &r));
+}
+
+static double speed_FpXQX_rem(speed_param *s)
+{
+  GEN x = rand_NFpXQX((degpol(s->x)-1)*2, s->T);
+  TIME_FUN(FpXQX_rem(x, s->x, s->T, s->p));
+}
+
+static double speed_FpXQXQ_red(speed_param *s) {
+  GEN x = rand_NFpXQX((degpol(s->x)-1)*2, s->T);
+  GEN q = FpXQX_get_red(s->x, s->T, s->p);
+  TIME_FUN(FpXQX_rem(x, q, s->T, s->p));
+}
+
+static double speed_FpXQX_halfgcd(speed_param *s)
+{ TIME_FUN(FpXQX_halfgcd(s->x, s->y, s->T, s->p)); }
+
+static double speed_FpXQX_extgcd(speed_param *s)
+{ GEN u,v; TIME_FUN(FpXQX_extgcd(s->x, s->y, s->T, s->p, &u, &v)); }
+
+static double speed_FpXQX_gcd(speed_param *s)
+{ TIME_FUN(FpXQX_gcd(s->x, s->y, s->T, s->p)); }
+
 /* small coeffs: earlier thresholds for more complicated rings */
 static double speed_RgX_sqr(speed_param *s)
 { TIME_FUN(RgX_sqr(s->x)); }
@@ -322,21 +476,58 @@ static tune_param param[] = {
 {0,   var(LOGAGMCX_LIMIT),         t_REAL,3,0, speed_logcx,0.05},
 {0,   var(AGM_ATAN_LIMIT),         t_REAL,20,0, speed_atan,0.05},
 {GMP, var(INVMOD_GMP_LIMIT),       t_INT, 3,0, speed_invmod},
+{0,   var(F2x_MUL_KARATSUBA_LIMIT),t_F2x,3,0, speed_F2x_mul},
 {0,   var(Flx_MUL_KARATSUBA_LIMIT),t_Flx,5,0, speed_Flx_mul,0,0,&Fmod_MUL_MULII_LIMIT},
 {0,   var(Flx_SQR_KARATSUBA_LIMIT),t_Flx,5,0, speed_Flx_sqr,0,0,&Fmod_SQR_SQRI_LIMIT},
+{0,   var(Flx_MUL_QUARTMULII_LIMIT),t_Fqx,3,0, speed_Flx_mul},
+{0,   var(Flx_SQR_QUARTSQRI_LIMIT), t_Fqx,3,0, speed_Flx_sqr},
 {0,   var(Flx_MUL_HALFMULII_LIMIT),t_Fhx,3,0, speed_Flx_mul},
 {0,   var(Flx_SQR_HALFSQRI_LIMIT), t_Fhx,3,0, speed_Flx_sqr},
 {0,   var(Flx_MUL_MULII_LIMIT),    t_Fl1x,5,0, speed_Flx_mul},
 {0,   var(Flx_SQR_SQRI_LIMIT),     t_Fl1x,5,0, speed_Flx_sqr},
 {0,   var(Flx_MUL_MULII2_LIMIT),   t_Fl2x,5,20000, speed_Flx_mul,0.05},
 {0,   var(Flx_SQR_SQRI2_LIMIT),    t_Fl2x,5,20000, speed_Flx_sqr,0.05},
-{0,   var(Flx_INVBARRETT_LIMIT),  t_NFlx,10,0, speed_Flx_inv,0.05},
+{0,  var(Flx_INVBARRETT_KARATSUBA_LIMIT), t_NFlx,5,20000,
+            speed_Flx_inv,0,0,&Fmod_MUL_MULII_LIMIT,&Flx_MUL_KARATSUBA_LIMIT},
+{0,  var(Flx_INVBARRETT_QUARTMULII_LIMIT), t_NFqx,5,0,
+            speed_Flx_inv,0,0,NULL,&Flx_MUL_QUARTMULII_LIMIT},
+{0,  var(Flx_INVBARRETT_HALFMULII_LIMIT), t_NFhx,5,0,
+            speed_Flx_inv,0,0,NULL,&Flx_MUL_HALFMULII_LIMIT},
+{0,  var(Flx_INVBARRETT_MULII_LIMIT), t_NFl1x,5,0,
+            speed_Flx_inv,0,0,NULL,&Flx_MUL_MULII_LIMIT},
+{0,  var(Flx_INVBARRETT_MULII2_LIMIT),t_NFl2x,5,0,
+            speed_Flx_inv,0,0,NULL,&Flx_MUL_MULII2_LIMIT},
 {0,  var(Flx_DIVREM_BARRETT_LIMIT),t_NFlx,10,0, speed_Flx_divrem,0.05},
 {0,  var(Flx_REM_BARRETT_LIMIT),  t_NFlx,10,0, speed_Flx_rem,0.05},
-{0,  var(Flx_BARRETT_LIMIT),      t_NFlx,10,0, speed_Flxq_red},
-{0,  var(Flx_HALFGCD_LIMIT),       t_Flx,10,0, speed_Flx_halfgcd},
+{0,  var(Flx_BARRETT_KARATSUBA_LIMIT), t_NFlx,5,0,
+            speed_Flxq_red,0,0,&Fmod_MUL_MULII_LIMIT,&Flx_MUL_KARATSUBA_LIMIT},
+{0,  var(Flx_BARRETT_QUARTMULII_LIMIT), t_NFqx,5,0,
+            speed_Flxq_red,0,0,NULL,&Flx_MUL_QUARTMULII_LIMIT},
+{0,  var(Flx_BARRETT_HALFMULII_LIMIT), t_NFhx,5,0,
+            speed_Flxq_red,0,0,NULL,&Flx_MUL_HALFMULII_LIMIT},
+{0,  var(Flx_BARRETT_MULII_LIMIT), t_NFl1x,5,0,
+            speed_Flxq_red,0,0,NULL,&Flx_MUL_MULII_LIMIT},
+{0,  var(Flx_BARRETT_MULII2_LIMIT),t_NFl2x,5,0,
+            speed_Flxq_red,0,0,NULL,&Flx_MUL_MULII2_LIMIT},
+{0,  var(Flx_HALFGCD_KARATSUBA_LIMIT), t_Flx,10,0,
+            speed_Flx_halfgcd,0,0,&Fmod_MUL_MULII_LIMIT,&Flx_MUL_KARATSUBA_LIMIT},
+{0,  var(Flx_HALFGCD_QUARTMULII_LIMIT), t_Fqx,10,0,
+            speed_Flx_halfgcd,0,0,NULL,&Flx_MUL_QUARTMULII_LIMIT},
+{0,  var(Flx_HALFGCD_HALFMULII_LIMIT), t_Fhx,10,0,
+            speed_Flx_halfgcd,0,0,NULL,&Flx_MUL_HALFMULII_LIMIT},
+{0,  var(Flx_HALFGCD_MULII_LIMIT), t_Fl1x,10,0,
+            speed_Flx_halfgcd,0,0,NULL,&Flx_MUL_MULII_LIMIT},
+{0,  var(Flx_HALFGCD_MULII2_LIMIT),t_Fl2x,10,0,
+            speed_Flx_halfgcd,0,0,NULL,&Flx_MUL_MULII2_LIMIT},
 {0,  var(Flx_GCD_LIMIT),           t_Flx,10,0, speed_Flx_gcd,0.1},
 {0,  var(Flx_EXTGCD_LIMIT),        t_Flx,10,0, speed_Flx_extgcd},
+{0,  var(FlxqX_INVBARRETT_LIMIT),t_NFlxqX,10,0, speed_FlxqX_inv,0.05},
+{0,  var(FlxqX_BARRETT_LIMIT),   t_NFlxqX,10,0, speed_FlxqXQ_red,0.05},
+{0,  var(FlxqX_DIVREM_BARRETT_LIMIT), t_NFlxqX,10,0, speed_FlxqX_divrem,0.05},
+{0,  var(FlxqX_REM_BARRETT_LIMIT), t_NFlxqX,10,0, speed_FlxqX_rem,0.05},
+{0,  var(FlxqX_HALFGCD_LIMIT),    t_FlxqX,10,0, speed_FlxqX_halfgcd,0.05},
+{0,  var(FlxqX_GCD_LIMIT),        t_FlxqX,10,0, speed_FlxqX_gcd,0.05},
+{0,  var(FlxqX_EXTGCD_LIMIT),     t_FlxqX,10,0, speed_FlxqX_extgcd,0.05},
 {0,  var(FpX_INVBARRETT_LIMIT),   t_NFpX,10,0, speed_FpX_inv,0.05},
 {0,  var(FpX_DIVREM_BARRETT_LIMIT),t_NFpX,10,0, speed_FpX_divrem,0.05},
 {0,  var(FpX_REM_BARRETT_LIMIT),  t_NFpX,10,0, speed_FpX_rem,0.05},
@@ -344,6 +535,13 @@ static tune_param param[] = {
 {0,  var(FpX_HALFGCD_LIMIT),       t_FpX,10,0, speed_FpX_halfgcd},
 {0,  var(FpX_GCD_LIMIT),           t_FpX,10,0, speed_FpX_gcd,0.1},
 {0,  var(FpX_EXTGCD_LIMIT),        t_FpX,10,0, speed_FpX_extgcd},
+{0,  var(FpXQX_INVBARRETT_LIMIT),t_NFpXQX,10,0, speed_FpXQX_inv,0.05},
+{0,  var(FpXQX_BARRETT_LIMIT),   t_NFpXQX,10,0, speed_FpXQXQ_red,0.05},
+{0,  var(FpXQX_DIVREM_BARRETT_LIMIT), t_NFpXQX,10,0, speed_FpXQX_divrem,0.05},
+{0,  var(FpXQX_REM_BARRETT_LIMIT), t_NFpXQX,10,0, speed_FpXQX_rem,0.05},
+{0,  var(FpXQX_HALFGCD_LIMIT),    t_FpXQX,10,0, speed_FpXQX_halfgcd,0.05},
+{0,  var(FpXQX_GCD_LIMIT),        t_FpXQX,10,0, speed_FpXQX_gcd,0.05},
+{0,  var(FpXQX_EXTGCD_LIMIT),     t_FpXQX,10,0, speed_FpXQX_extgcd,0.05},
 {0,  var(RgX_MUL_LIMIT),           t_FpX, 4,0, speed_RgX_mul},
 {0,  var(RgX_SQR_LIMIT),           t_FpX, 4,0, speed_RgX_sqr},
 };
@@ -422,10 +620,11 @@ diag(const char *format, ...)
   va_list ap;
   va_start(ap, format);
   vfprintf(stderr, format, ap);
+  va_end(ap);
 }
 void
 print_define(const char *name, long value)
-{ printf("#define __%-25s  %5ld\n", name, value); }
+{ printf("#define __%-30s %ld\n", name, value); }
 
 long
 analyze_dat(int final)
@@ -470,6 +669,7 @@ Test(tune_param *param)
   speed_param s;
   long save_var_disable = -1;
   pari_timer T;
+  pari_sp av=avma;
 
   if (param->kernel == AVOID) { print_define(param->name, -1); return; }
 
@@ -478,11 +678,13 @@ Test(tune_param *param)
   DEFAULT(stop_factor, 1.2);
   DEFAULT(max_size, 10000);
   if (param->var_disable) save_var_disable = *(param->var_disable);
+  if (param->var_enable)  s.var_enable_min = *(param->var_enable);
 
   s.type = param->type;
   s.size = param->min_size;
   s.var  = param->var;
   s.var_disable  = param->var_disable;
+  s.var_enable  = param->var_enable;
   dftmod(&s);
   ndat = since_positive = since_change = thresh = 0;
   if (option_trace >= 1)
@@ -500,8 +702,8 @@ Test(tune_param *param)
   {
     pari_sp av=avma;
     double t1, t2, d;
-    s.x = rand_g(s.size, s.type);
-    s.y = rand_g(s.size, s.type);
+    s.x = rand_g(&s);
+    s.y = rand_g(&s);
     t1 = time_fun(param->fun, &s, 0);
     t2 = time_fun(param->fun, &s, 1);
     avma = av;
@@ -558,6 +760,8 @@ Test(tune_param *param)
   print_define(param->name, thresh);
   *(param->var) = thresh; /* set to optimal value for next tests */
   if (param->var_disable) *(param->var_disable) = save_var_disable;
+  if (param->var_enable) *(param->var_enable) = s.var_enable_min;
+  avma = av;
 }
 
 void error(char **argv) {
@@ -585,10 +789,12 @@ main(int argc, char **argv)
   DFLT_mod = 27449;
   LARGE_mod=subis(powuu(3,128),62);
 #ifdef LONG_IS_64BIT
+  DFLT_qmod = 3;
   DFLT_hmod = 257;
   DFLT_mod2 = 281474976710677UL;
 #else
-  DFLT_hmod = 17;
+  DFLT_qmod = 3;
+  DFLT_hmod = 3;
   DFLT_mod1 = 1031UL;
 #endif
   v = new_chunk(argc);
